#!/usr/bin/env python 
# -*- coding: utf-8 -*-
#
# -------------------------------------------------------------------------------------
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------

from gimpfu import *
import decimal
import random
import time
import datetime

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
############################################################################
#

def ChildishEngrave (img, layer, howmany, patfilter) :
    
    pdb.gimp_context_push()

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(img)
    
    sourcename = layer.name
    if '.' in sourcename:
        split = sourcename.split(".", 1)
        sourcename = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    # simplify
    pdb.plug_in_sel_gauss (img, layer, 15.0, 72)
    
    # Get the layer position.
    pos = 0;
    for i in range(len(img.layers)):
        if(img.layers[i] == layer):
            pos = i
            
    # Create a new layer to save the results (otherwise is not possible to undo the operation).
    newLayer = layer.copy()
    newLayer.name = layer.name + " mod"

    img.add_layer(newLayer, pos)
    
    # max saturation
    pdb.gimp_hue_saturation (newLayer, 0, 0, 0, +100)
    
    # change image mode to indexed  

    pdb.gimp_image_convert_indexed (img, 0, 0, howmany, 0, 0, "ignored")
    is_indexed=pdb.gimp_drawable_is_indexed (newLayer) 
    if is_indexed==1:
        hidden_message="OK, converted"
    else:    
        erro_msg ("Image not converted to indexed, exit")
        pdb.gimp_context_pop
        return    

    # get the number of pixels in the colormap (which exists for indexed images)
    # ==========================================================================
    (count, colors)=pdb.gimp_image_get_colormap(img)
    # the count is nr_of_colors * 3, example 8*3=24
    nr_areas=(count/3) 
    #info_msg("nr of colors in the palette: "+str(nr_areas)) 
  
    temp_palette=pdb.gimp_palette_new(pdb.gimp_image_get_name(img))
    
    index=0
    while index<count:
    # Compute the color
        color= (colors[index], colors[index+1], colors[index+2])
    # Add to the filter temporary palette
        pdb.gimp_palette_add_entry(temp_palette, str(index/3), color)
        index+=3 
    # Endwhile index<count
  
    # the temporary palette has been created            
    # ======================================

    # re-set image to RGB to proceed
    # ==============================
    pdb.gimp_image_convert_rgb (img)
    
    bordersLayer = newLayer.copy()
    bordersLayer.name = layer.name + " borders"
    bordersLayer.mode = HARDLIGHT_MODE

    img.add_layer(bordersLayer, pos)
    
    pdb.gimp_context_set_foreground ((0,0,0,255))
    areaNR=0
    area_color=[]   # empty list
    for areaNR in range(nr_areas):
        one_color=pdb.gimp_palette_entry_get_color(temp_palette, areaNR)
        #info_msg ("one_color = "+str(one_color))
        pdb.gimp_image_select_color(img, CHANNEL_OP_REPLACE, layer, one_color)
        (there_is_a_selection, x1, y1, x2, y2) = pdb.gimp_selection_bounds(img)
        
        if there_is_a_selection==1:
            border = int(round((img.width+img.height)/1000.0))
            #info_msg ("border ="+str(border))    
            pdb.gimp_selection_border (img, border)
            pdb.gimp_edit_fill (bordersLayer, FOREGROUND_FILL)

    ##info_msg ("end loop")
    pdb.gimp_selection_none (img)
    pdb.gimp_threshold (bordersLayer, 16, 255)
    pdb.plug_in_colortoalpha (img, bordersLayer, ((255,255,255,255)))

    (np, LPATT) = pdb.gimp_patterns_get_list (patfilter)       
    if np>0:
        rn = random.randrange (np)
        pdb.gimp_context_set_pattern (LPATT[rn])
        #info_msg (str(LPATT[rn]))
        pdb.gimp_edit_fill (newLayer, PATTERN_FILL)
        
    pdb.plug_in_spread (img, bordersLayer, 2, 2)
    
    bordersLayer2 = pdb.gimp_layer_new_from_visible (img, img, layer.name + " borders2")
    bordersLayer2.mode = DARKEN_ONLY_MODE

    img.add_layer(bordersLayer2, pos)
    
    pdb.plug_in_gauss_rle2 (img, bordersLayer2, 5, 5)
        
    pdb.plug_in_spread (img, bordersLayer2, 4, 4)
    
    newLayer.mode = GRAIN_MERGE_MODE
    newLayer.opacity = 80.0
    
    #return()

    finalLayer = pdb.gimp_image_merge_down (img, newLayer, CLIP_TO_BOTTOM_LAYER)
    finalLayer = pdb.gimp_image_merge_down (img, bordersLayer, CLIP_TO_BOTTOM_LAYER)
    finalLayer = pdb.gimp_image_merge_down (img, bordersLayer2, CLIP_TO_BOTTOM_LAYER)
    finalLayer.name = st+" "+sourcename+" childish engraved"
    
    # Close the undo group.
    pdb.gimp_image_undo_group_end(img)
    
    # End process.
    pdb.gimp_context_pop()
    
    return(finalLayer)

register(
    "ChildishEngrave",    "ChildishEngrave",    "ChildishEngrave.",
    "Dinasset",    "Open source (BSD 3-clause license)",    "2017",
    "ChildishEngrave",
    "RGB, RGB*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "colours", "How many colours",  9, (4, 255, 1)), 		#default, (min, max, increment)
      (PF_STRING, "filter", "Filter for wood patterns names"+
                             " \n (if none is found engrave on image):", "wood of some sort"),
    ],
    [
      (PF_DRAWABLE, "outdrawable", "Output drawable", None),
    ],
    ChildishEngrave,
    menu="<Image>/Filters/Diego/Ancillary"
    )

main()
