#!/usr/bin/env python

# Draw_Percentage_Ruler Rel 1
# Created by Tin Tran http://bakon.ca/gimplearn/
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.


# Creates a path/vector that looks like a ruler
# to measure in percentages

import math
from gimpfu import *
#from array import array
import os, string, sys


def python_tt_draw_ruler(image, layer, marklength) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#PUT YOUR CODE HERE 
	
	#grab selection bounding box for calculations later.
	selection = pdb.gimp_selection_bounds(image);
	
	x1 = selection[1];
	y1 = selection[2];
	x2 = selection[3];
	y2 = selection[4];
	
	width = float(abs(x2-x1))
	height = float(abs(y2-y1))

	vectors_name = "Percentage Ruler"
	vectors = pdb.gimp_vectors_new(image,vectors_name)
	pdb.gimp_image_insert_vectors(image,vectors,None,-1)
	
	gimp.progress_init("Drawing Percentage Rulers...")
	closed = 0
	
	for mark in range(0,101):
		gimp.progress_update(float(mark)/100)
		#Determine mark length
		mlength = float(marklength)/4 #regular marks are 1/4 of full mark length
		if mark % 5 == 0: #every 5 marks, make the mlength 1/2 of full mark length
			mlength = float(marklength)/2
		if mark % 10 == 0:
			mlength = marklength
			
		#horizontal rulers top and bottom
		mx1 = float(width)/100 * mark + x1
		mx2 = mx1
		my1 = y1
		my2 = y1 - mlength
		control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
		my1 = y2
		my2 = y2 + mlength
		control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
		#vertical marks
		mx1 = x1
		mx2 = x1 - mlength
		my1 = float(height)/100 * mark + y1
		my2 = my1
		control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
		mx1 = x2
		mx2 = x2 + mlength
		control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
		
		#here, we have control points all calculated depending on method, so we add the points
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)

	#Draw box 
	control_points = ([x1,y1]*3) + ([x2,y1]*3)
	pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
	control_points = ([x2,y1]*3) + ([x2,y2]*3)
	pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
	control_points = ([x1,y2]*3) + ([x2,y2]*3)
	pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
	control_points = ([x1,y1]*3) + ([x1,y2]*3)
	pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
	
	
	pdb.gimp_item_set_visible(vectors,1)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return


register(
	"python_fu_tt_draw_ruler",                           
	"Draws Horizontal and Vertical Ruler in percentages",
	"Draws Horizontal and Vertical Ruler in percentages...",
	"Tin Tran",
	"Tin Tran",
	"2014",
	"<Image>/Filters/Path/Draw Percentage Ruler...",             #Menu path
	"*",
	[
	(PF_SPINNER, "marklength", "Mark Length(Pixels):", 12, (4, 200, 1)),
	],
	[],
	python_tt_draw_ruler)

main()
