#!/usr/bin/env python

# Draw_Protractor Rel 1
# Created by Tin Tran http://bakon.ca/gimplearn/
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.


# Creates a path/vector that looks like a protractor
# to measure angle in degrees

import math
from gimpfu import *
#from array import array
import os, string, sys


def python_tt_draw_protractor(image, layer, marklength) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#PUT YOUR CODE HERE 
	
	#grab selection bounding box for calculations later.
	selection = pdb.gimp_selection_bounds(image);
	
	x1 = selection[1];
	y1 = selection[2];
	x2 = selection[3];
	y2 = selection[4];
	
	width = float(abs(x2-x1))
	height = float(abs(y2-y1))
	
	radius = min(width,height)/2.0
	midx = x1 + (width/2.0)
	midy = y1 + (height/2.0)
	

	vectors_name = "Protractor Marks"
	vectors = pdb.gimp_vectors_new(image,vectors_name)
	pdb.gimp_image_insert_vectors(image,vectors,None,-1)
	
	gimp.progress_init("Drawing Protractor Marks...")
	closed = 0
	
	for mark in range(0,361):
		gimp.progress_update(float(mark)/360)
		#Determine mark length
		mlength = float(marklength)/4 #regular marks are 1/4 of full mark length
		if mark % 5 == 0: #every 5 marks, make the mlength 1/2 of full mark length
			mlength = float(marklength)/2
		if mark % 10 == 0:
			mlength = marklength
			
		
		rad = (mark/360.0) * math.pi * 2
		# draw star in center
		if mark % 45 == 0:
			
			mx1 = midx 
			my1 = midy 
			mx2 = midx + (math.cos(rad) * (mlength))
			my2 = midy + (math.sin(rad) * (mlength))
			
			# draw mark to mark degree
			control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
			pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
		#horizontal rulers top and bottom
		
		mx1 = midx + (math.cos(rad) * radius)
		my1 = midy + (math.sin(rad) * radius)
		mx2 = midx + (math.cos(rad) * (radius + mlength))
		my2 = midy + (math.sin(rad) * (radius + mlength))
		
		# draw mark to mark degree
		control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
		# draw marks to make circle
		if mark > 0: 
			control_points = ([mx1,my1]*3) + ([ox1,oy1]*3)
			pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		ox1 = mx1
		oy1 = my1
		
		
		
	#Draw Circle
	
	
	
	
	pdb.gimp_item_set_visible(vectors,1)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return


register(
	"python_fu_tt_draw_protractor",                           
	"Draws Protractor",
	"Draws Protractor...",
	"Tin Tran",
	"Tin Tran",
	"2014",
	"<Image>/Filters/Path/Draw Protractor...",             #Menu path
	"*",
	[
	(PF_SPINNER, "marklength", "Mark Length(Pixels):", 12, (4, 200, 1)),
	],
	[],
	python_tt_draw_protractor)

main()
