#!/usr/bin/env python

# Extrude_With_Path release 4
# Created by Tin Tran http://bakon.ca/gimplearn/
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# ----------------------------------------------------------------------------------
# Extrude active layer with active path... It's best if you bump map your active layer first.
# 
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: remove debug messages.
# Rel 3: Fixed gradient mapping bug for when we have a really short path.
#        Added gradient reverse
# Rel 4: to work with GIMP 2.10
import math
import string
#import Image
from gimpfu import *
from array import array

PRECISION = 0.0000001
def paint_it(image,outputlayer,size,x,y,cx,cy,color):
	floating_selection = pdb.gimp_edit_paste(outputlayer,1)
	pdb.gimp_floating_sel_to_layer(floating_selection)
	transformed_layer = pdb.gimp_item_transform_2d(floating_selection,cx,cy,size,size,0,x,y)
	pdb.plug_in_colorify(image,transformed_layer,(int(color[0]*255),int(color[1]*255),int(color[2]*255)))
	#pdb.gimp_message("blue" + str(color[2])+ "conv: "+ str(color[2]*255))
	
	pdb.gimp_layer_set_opacity(transformed_layer,color[3]*100)
	#pdb.gimp_message(str(color[3]))
	result_layer = pdb.gimp_image_merge_down(image,transformed_layer,EXPAND_AS_NECESSARY)
	return result_layer
def python_extrude_with_path(image, layer, startpercentage,gradientreverse) :
	activegradient = pdb.gimp_context_get_gradient()
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#.gimp_message("This is sample shell script.")
	#PUT YOUR CODE HERE 
	
	
	#context_brush_size = pdb.gimp_context_get_brush_size()
	#bspacing = context_brush_size * (pathspacing/100.0)
	gimp.progress_init("Extrude with path...")
	
	#make active layer a usable brush
	pdb.plug_in_autocrop_layer(image,layer)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_edit_copy(layer)
	num_brushes,brush_list = pdb.gimp_brushes_get_list("")
	pdb.gimp_context_set_brush(brush_list[0])
	brush_size = max(layer.width,layer.height)
	
	
	pdb.gimp_selection_none(image)
	#create a new layer to work on
	extrudelayer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Extrude Layer",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,extrudelayer,None,-1)
	
	
	
	
	
	#find center of layer to use as ending brush position.
	offset_x,offset_y = pdb.gimp_drawable_offsets(layer)
	end_x = offset_x + (layer.width/2.0)
	end_y = offset_y + (layer.height/2.0)
	
	cx = image.width/2
	cy = image.height/2
	active_vectors = pdb.gimp_image_get_active_vectors(image)
	if active_vectors > -1:
		#brush_at = []
		num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(active_vectors)
		
		for stroke_id in stroke_ids:
			stroke_length = pdb.gimp_vectors_stroke_get_length(active_vectors,stroke_id,PRECISION)
			#get path's ending point
			path_end_x,path_end_y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_id,stroke_length,PRECISION)
			dist = 0.0
			ox,oy,o_size_ratio = None,None,None
			done = False
			onemore = False
			while not done:
				gradient_position = dist/stroke_length #make gradient also steps
				num_color_samples,color = pdb.gimp_gradient_get_custom_samples(activegradient,1,[gradient_position],gradientreverse)
				gimp.progress_update(float(dist)/stroke_length)
				x_point,y_point,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_id,dist,PRECISION)
				size_ratio = (1.0-(startpercentage/100.0))/stroke_length * (dist) + (startpercentage/100.0)
				
				
				
				cur_x,cur_y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_id,dist,PRECISION)
				
				x = (cur_x - path_end_x) + end_x
				y = (cur_y - path_end_y) + end_y
				
				if ox is not None: #if it's define we paint it up to current position
					iter = max(2,max(abs(ox-x),abs(oy-y),abs((o_size_ratio*brush_size)-(size_ratio*brush_size)))*2)
					for i in range(0,int(iter)+1):
						gradient_position = (float(dist-o_dist)/iter * i + (o_dist))/stroke_length #make gradient also steps
						num_color_samples,color = pdb.gimp_gradient_get_custom_samples(activegradient,1,[gradient_position],gradientreverse)
						sr = (size_ratio-o_size_ratio)/iter * i + o_size_ratio
						ix = float(x-ox)/iter * i + ox
						iy = float(y-oy)/iter * i + oy
						extrudelayer = paint_it(image,extrudelayer,sr,ix,iy,cx,cy,color)
						#pdb.gimp_context_set_brush_size(sr)
						#pdb.gimp_paintbrush(extrudelayer,0,2,(ix,iy),PAINT_INCREMENTAL,0)
				
				#paint it, yeah
				extrudelayer = paint_it(image,extrudelayer,size_ratio,x,y,cx,cy,color)
				#pdb.gimp_message(str(color))
				#break
				#pdb.gimp_context_set_brush_size(brush_size*size_ratio)
				#pdb.gimp_paintbrush(extrudelayer,0,2,(x,y),PAINT_INCREMENTAL,0)
				
				ox,oy,o_size_ratio,o_dist = x,y,size_ratio,dist
				
				dist = dist + 1
				if onemore: #already ran one more time quit.
					done = True
				if dist > stroke_length:
					dist = stroke_length
					onemore = True
				
			#end while loop
		#end for loop
	#end if there is a vector defined
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_extrude_with_path",                           
	"Extrude active layer with active path",
	"Extrude active layer with active path...",
	"Tin Tran",
	"Tin Tran",
	"2016",
	"<Image>/Python-Fu/Extrude Active Layer with Active Path...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "yminpercentage", "Starting Brush size percentage:", 100, (1, 500, 1)),
	(PF_TOGGLE, "gradientreverse",   "Reverse Gradient?:", 0),
	],
	[],
	python_extrude_with_path)

main()
