#!/usr/bin/env python

# Follow_Path_With_Perspective_Brush Rel 5 (requested by Rod on gimpchat.com)
# Created by Tin Tran http://bakon.ca/gimplearn/
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# ----------------------------------------------------------------------------------
# Rod requested that script follows path, but set brush size based on Y coordinate.
# 
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Fixed Track Direction bug. Brush angle related issue.
# Rel 3: Added forgoten progress update
# Rel 4: Draw smaller brushes first so that closer brushes(large ones) are shown in front
# Rel 5: Dang it, forgot progress bar again.
import math
import string
#import Image
from gimpfu import *
from array import array

PRECISION = 0.0000001
def python_follow_path_with_perspective_brush(image, layer, yminpercentage, ymaxpercentage, pathspacing, trackdirection) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#.gimp_message("This is sample shell script.")
	#PUT YOUR CODE HERE 
	
	
	context_brush_size = pdb.gimp_context_get_brush_size()
	bspacing = context_brush_size * (pathspacing/100.0)
	gimp.progress_init("Following Path with Perspective Brush...")
	
	
	
	
	
	
	
	active_vectors = pdb.gimp_image_get_active_vectors(image)
	if active_vectors > -1:
		brush_at = []
		num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(active_vectors)
		
		for stroke_id in stroke_ids:
			stroke_length = pdb.gimp_vectors_stroke_get_length(active_vectors,stroke_id,PRECISION)
			dist = 0.0
			while (dist <= stroke_length):
				gimp.progress_update(float(dist)/stroke_length)
				x_point,y_point,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_id,dist,PRECISION)
				size_ratio = ((ymaxpercentage/100.0)-(yminpercentage/100.0))/ image.height * (y_point) + (yminpercentage/100.0)
				
				
				# pdb.gimp_context_set_brush_size(context_brush_size * size_ratio) #set brush size to reflect size change
				if (trackdirection == 1): #set brush angle if we're tracking direction
					if dist + 1 > stroke_length:
						x_point0,y_point0,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_id,dist-1,PRECISION)
						x_point1,y_point1,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_id,dist,PRECISION)
					else:
						x_point0,y_point0,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_id,dist,PRECISION)
						x_point1,y_point1,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_id,dist+1,PRECISION)
					dx = x_point1 - x_point0
					dy = y_point1 - y_point0
					rads = math.atan2(-dy,dx)
					rads %= 2*math.pi
					angle = 360 - math.degrees(rads)
					if angle > 180:
						angle = angle - 360
					angle = math.degrees(math.atan2(dx,dy))
				else:
					angle = 0
					#
				brush_at.append((x_point,y_point,angle,context_brush_size * size_ratio))
				# pdb.gimp_context_set_brush_angle(angle)
				# pdb.gimp_paintbrush(layer,0,2,(x_point,y_point),PAINT_CONSTANT,0)
					
				
				dist = dist + (bspacing * size_ratio)
			#end while loop
		#end for loop
		
		#brush from far (smaller y) to near(y)
		gimp.progress_init("Brushing from far to near...")
		brush_at.sort(key=lambda y: y[3]) #sort based on 4th index starting at zero index which is brush size
		count = 0
		for r in brush_at:
			count+=1
			gimp.progress_update(float(count)/len(brush_at))
			x = r[0]
			y = r[1]
			angle = r[2]
			brush_size = r[3]
			pdb.gimp_context_set_brush_size(brush_size) #set brush size to reflect size change
			pdb.gimp_context_set_brush_angle(angle)
			pdb.gimp_paintbrush(layer,0,2,(x,y),PAINT_CONSTANT,0)
		
	#end if there is a vector defined
		
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_follow_path_with_perspective_brush",                           
	"Follows a path with perspective brush based on Y-coordinate",
	"Follows a path with perspective brush based on Y-coordinate",
	"Tin Tran",
	"Tin Tran",
	"2014",
	"<Image>/Python-Fu/Follow Path with Perspective Brush",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "yminpercentage", "Minimum Brush size percentage (y=0):", 50, (1, 500, 1)),
	(PF_SPINNER, "ymaxpercentage", "Maximum Brush size percentage (y=height of image):", 100, (1, 500, 1)),
	(PF_SPINNER, "pathspacing", "On Path Spacing (percentage of brush size):", 100, (1, 1000, 1)),
	(PF_TOGGLE, "trackdirection",   "Track Direction?:", 0),
	],
	[],
	python_follow_path_with_perspective_brush)

main()
