#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os, sys
import math
from gimpfu import *

# GIMP plugin to create simple path shapes centered in the image or selection.
# (c) MareroQ 2016

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# ------------
#| Change Log |
# ------------
# Rel 0.1: Initial release.
# Rel 0.2: Add shapes: "Pentagon","Star pentagons","Pentagram","Yin-Yang","Petals","Quadrant","Flower".
# Rel 0.3: Add border to layer - requires Changing Image Border.scm. The combined two scripts:
#          by TheUncle2k (http://theuncle2k.deviantart.com/art/Resize-image-to-layers-150439776) "Resize_image_to_layers_by_theuncle2k.scm"
#          and Art Wade (fencepost) "All-layers-to-image-size.scm". Thanks for authors.
#          Add shapes: "Trefoil","Quatrefoil","Cross","Crescent","Pie","Semicircle", "Arrow"   
#          Sorted shapes alphabetical
# Rel 0.4: Add shapes: "Octagon","Triangle Reuleaux (curvilinear)","Flower" divided into two (in and out).
#          Add option shape in selection.
#          Add option inspired by Dinasset "Stroke Color" and "Fill Pattern" created by Tin Tran (http://bakon.ca/gimplearn/viewtopic.php?f=3&t=141#p324). Thanks Trandoductin!.
# Rel 0.5: Add shapes: "Recycle".
#          Included: needed stroke outside the selection.
# Rel 0.6  Add shapes: "Roses compass 4-point"
#          				and "Ruler" created by Tin Tran (Draw Percentage Ruler Script for GIMP: http://bakon.ca/gimplearn/viewtopic.php?f=3&t=139&p=320#p320)
# Rel 0.7  Add shapes: "Dodecagon","Mobius band"
# Rel 0.8  Add shapes: "Recycle symbol" (experimental for Mahvin)
#					   "Binoculars"
#                  		and "Protractor" created by Tin Tran (Draw_Protractor Rel 1: http://bakon.ca/gimplearn/viewtopic.php?f=3&t=175#p512)
# Rel 0.9  Repair of error for "Protractor" and "Ruler" (not centered when no selection).
#          Show a selection, after you run the shape, when you select "In Selection" on the menu.
#          "Protractor" rel 3: Draw marks as a single stroke for speed, outside and inside circle
#          Add shape: "Wilber"
# Rel 1.0 alfa: Show a selection only when you select "In Selection" on the menu (no show for no selection)
#               Add shapes "Grid" (Grid rectangular (c) Ofnuts v0.0 https://sourceforge.net/projects/gimp-path-tools/files/scripts/
#				Add shapes "Pie 1/2 and 7/8"
#               Add a separate option for Mark Length
#               Add fills selection with Active Brush created by Tin Tran (Fill_Selection_With_Brush ver.3 http://bakon.ca/gimplearn/viewtopic.php?f=3&t=177#p522)
#				Shape "Ruler" return to the basic version.
# Rel 1.0 beta: Repair of error for Gimp 2.9 (called deprecated procedure 'gimp_selection_load'.It should call 'gimp-image-select-item' instead!)
#               Add transparency layer (create new) for fill and stroke
# Rel 1.0   	Correction shapes "Recycle" (bad axis)
#           	Add shapes "Gear", "Star" Polygon" created by Jonathan Stipe 2004-2006(necessary script-fu Shape-path.scm)
# Rel 1.0 FR: 	Repair error (Don't stroke all layer)for Stroke Path (using SVG method)
# 		  		Add shapes "Runded Rectangle", WAVE: "Sine", "Triangle", "Square" created by Jonathan Stipe 2004-2006 - necessary script-fu Shape-path.scm
# 				Add shapes "Spirals" created by Samj (Des_spirales.scm 2013  http://samjcreations.blogspot.ca/)- necessary script-fu Shape-path.scm
#         		Add Fill "With Foreground" and "With Background"
#		  		Add Stroke "With active brush and foreground color"
# 				Add a separate option (Color and Line Width) for "Stroke Path (using SVG method)"
# Rel 1.0 End: Repair of error for Grid by Ofnust in Gimp 2.9) (called deprecated procedure 'gimp-image-add-vectors'.It should call 'gimp-image-insert-vectors' instead!)
#              Only one version (common for Gimp 2.8/2.9 - thanks Ofnuts (http://gimpchat.com/viewtopic.php?f=9&t=14857#p206183)
#			   Option "Resize layer" only run interactive
#			   Add shapes: Grid:"Rectangle", "Ellipse", "Hexagon", "Diamond", "Octagon" based on Shapegrid 1_1.py by dd (adaptation code script-fu JMS - James Sambrook)
# Rel 1.1: 	   Add shape: Rounded Rectangle Advanced created by Jonathan Stipe
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#==============================================================================================================================================================================

def newline(x1, y1, x2, y2):
    return [x1,y1]*3+[x2,y2]*3

def shapegrid_sel_ssc(image, layer, gType, sType, xCirc,yCirc, oBorder, gapSpace, sel):

		gimp.context_push()
		image.undo_group_start()

		if sel == False:
			pdb.gimp_selection_all(image)
		selection = pdb.gimp_selection_bounds(image)
		x1 = selection[1];
		y1 = selection[2];
		x2 = selection[3];
		y2 = selection[4];

		temp = pdb.gimp_selection_save(image)
		circDiam = max(int((x2-x1)/xCirc), int((y2-y1)/yCirc))
		s3 = math.sqrt(3)
		circRad = circDiam / 2.0
		tBorder = oBorder * 2

		xGap = 0
		yGap = 0
		inWidth = 0
		inHeight = 0
		theImage = 0
		baseLayer = 0

		xFlag = 0
		yFlag = 0
		xStart = 0.0
		yStart = 0.0

		if gType == 0:
			inWidth = tBorder + (circDiam * xCirc) + (gapSpace * (xCirc - 1))
			inHeight = tBorder + (circDiam * yCirc) + (gapSpace * (yCirc - 1))
		if gType == 1:
			xGap = (s3 * (circRad + (gapSpace * 0.5)))
			yGap = (gapSpace * (yCirc - 1))
			inWidth = (tBorder + circDiam + (xGap * (xCirc - 1)))
			inHeight = (tBorder + circRad + yGap + (gapSpace * 0.5) + (circDiam * yCirc))

		theImage = image
		if (image.width < inWidth) or (image.height < inHeight):        
			pdb.gimp_image_resize(theImage, max(inWidth, image.width), max(inHeight, image.height), 0, 0)
		baseLayer = pdb.gimp_layer_new(theImage, inWidth, inHeight, RGBA_IMAGE,  "Shapegrid", 100, NORMAL_MODE)
		theImage.add_layer(baseLayer)
		pdb.gimp_context_set_foreground((0, 0, 0))
		if gimp.version >= (2,9,0):       
			baseLayer.fill(FILL_TRANSPARENT)	       #GIMP 2.9
		else:
			baseLayer.fill(TRANSPARENT_FILL)           #GIMP 2.8
		sel = pdb.gimp_image_get_selection(theImage)
	#shape code
		xStart = oBorder
		yStart = oBorder    
		if sType == 0:
			pdb.gimp_image_select_rectangle(theImage, CHANNEL_OP_REPLACE, xStart, yStart, circDiam, circDiam)
		if sType == 1:
			pdb.gimp_image_select_ellipse(theImage, CHANNEL_OP_REPLACE,xStart, yStart, circDiam, circDiam)
		if sType == 2:
			pdb.gimp_image_select_polygon(theImage, CHANNEL_OP_REPLACE,12,
				(
				xStart, (yStart + (circDiam * 0.5)),
				(xStart + (circDiam * 0.25)), (yStart + (circDiam * (0.2 / 3))),
				(xStart + (circDiam * 0.75)), (yStart + (circDiam * (0.2 / 3))),
				(xStart + circDiam), (yStart + (circDiam * 0.5)),
				(xStart + (circDiam * 0.75)), (yStart + (circDiam * (1 - (0.2 / 3)))),
				(xStart + (circDiam * 0.25)), (yStart + (circDiam * (1 - (0.2 / 3)))) 
				))
		if sType == 3:
			pdb.gimp_image_select_polygon(theImage, CHANNEL_OP_REPLACE,8,
				(
				xStart, (yStart + (circDiam * 0.5)),
				(xStart + (circDiam * 0.5)), yStart,
				(xStart + circDiam), (yStart + (circDiam * 0.5)),
				(xStart + (circDiam * 0.5)), (yStart + circDiam)
				))
		if sType == 4:
			 pdb.gimp_image_select_polygon(theImage, CHANNEL_OP_REPLACE, 16,
					(
						(xStart + (circDiam / 3)), yStart,
						(xStart + ((circDiam * 2) / 3)), yStart,
						(xStart + circDiam), (yStart + (circDiam / 3)),
						(xStart + circDiam), (yStart + ((circDiam * 2) / 3)),
						(xStart + ((circDiam * 2) / 3)), (yStart + circDiam),
						(xStart + (circDiam / 3)), (yStart + circDiam),
						xStart, (yStart + ((circDiam * 2) / 3)),
						xStart, (yStart + (circDiam / 3))
					))

		xFlag = 0
		yFlag = 0

		if gType == 0:
			xMove = (circDiam + gapSpace)
		   
		elif gType == 1:
			xMove = (s3 * ((circDiam + gapSpace) * 0.5))
		yMove = gapSpace + circDiam

		if gType == 0:
			while (xFlag < xCirc):
				yFlag = 0
				while (yFlag < (yCirc-1)):
					pdb.gimp_edit_bucket_fill(baseLayer, 0, NORMAL_MODE, 100, 255, 0, 0, 0)            
					pdb.gimp_drawable_offset(sel, 1, 0, 0, yMove)
					yFlag += 1
				pdb.gimp_edit_bucket_fill(baseLayer, 0, NORMAL_MODE, 100, 255, 0, 0, 0)
				pdb.gimp_drawable_offset(sel, 1, 0, xMove, (0-(yMove*(yCirc-1))))     
				xFlag +=1
		
		if gType == 1:
			odd = False
			yMax = 0
			while (xFlag < xCirc):
				if (xFlag % 2) == 1:
					odd = True
					yMax = (yCirc - 2)
				else:
					odd = False
					yMax = (yCirc - 1)

				if odd:
					pdb.gimp_drawable_offset(sel, 1, 0, 0, (yMove/2))
				yFlag = 0
				while (yFlag < yMax):
					pdb.gimp_edit_bucket_fill(baseLayer, 0, NORMAL_MODE, 100, 255, 0, 0, 0)            
					pdb.gimp_drawable_offset(sel, 1, 0, 0, yMove)
					yFlag += 1
				pdb.gimp_edit_bucket_fill(baseLayer, 0, NORMAL_MODE, 100, 255, 0, 0, 0)
				pdb.gimp_drawable_offset(sel, 1, 0, xMove, (0-(yMove*yMax)))
				if odd:
					pdb.gimp_drawable_offset(sel, 1, 0, 0, (0-(yMove/2)))
				xFlag +=1

		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,baseLayer)

		pdb.plug_in_sel2path(image, baseLayer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Grid")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Siatka")
		pdb.gimp_item_transform_scale(active_vectors, x1+oBorder, y1+oBorder, x2-oBorder, y2-oBorder)
		pdb.gimp_image_remove_layer(image, baseLayer, run_mode=RUN_NONINTERACTIVE)
		pdb.gimp_image_resize_to_layers( image)

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_item(image, CHANNEL_OP_REPLACE, temp)
		image.remove_channel(temp)

		gimp.context_pop()
		image.undo_group_end()
		pdb.gimp_displays_flush()
def plugin_simple_shapes_centered(image, layer, sel,s, R_1, M_L, H_L, V_L, R_L, fill, fillpattern, H_B, V_B, stroke, strokecolor, strokewidth):

	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	before_vectors = list(image.vectors)
	selection = pdb.gimp_selection_bounds(image)
	x1 = selection[1];
	y1 = selection[2];
	x2 = selection[3];
	y2 = selection[4];
	if sel:
		temp = pdb.gimp_selection_save(image)
		W1 = 2*(x1+(x2 - x1)/2)
		H1 = 2*(y1+(y2 - y1)/2)
		j = x1
		k = y1
		if x2-x1>=y2-y1:
			m = (x1+x2-y2+y1)/2
			v = y1
			W = (y2 - y1)
			H = (y2 - y1)
		else:
			m = x1
			v = (y1+y2-x2+x1)/2
			W = x2 - x1
			H = x2 - x1
	else:
		W1 = pdb.gimp_image_width(image)  #szer
		H1 = pdb.gimp_image_height(image) #wys
		k = 0
		j = 0
		if W1>H1:
			m =(W1-H1)/2
			v = 0
			W = H1
			H = H1

		else:
			m = 0
			v = (H1-W1)/2
			W = W1
			H = W1

	if s==0: # Arrow

		new_vectors = pdb.gimp_vectors_new(image,"Arrow >")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 84,
		(5*W/8 +m, W/8 +v,5*W/8 +m, W/8 +v,5*W/8 +m, W/8 +v,   5*W/8 +m,3*W/8 +v, 5*W/8 +m,3*W/8 +v,5*W/8 +m,3*W/8 +v,
		5*W/8 +m,3*W/8 +v, 5*W/8 +m,3*W/8 +v,5*W/8 +m,3*W/8 +v,   m,3*W/8 +v,m,3*W/8 +v,m,3*W/8 +v,
		m,3*W/8 +v, m,3*W/8 +v, m,3*W/8 +v,   m,5*W/8 +v, m,5*W/8 +v, m,5*W/8 +v,
		m,5*W/8 +v, m,5*W/8 +v, m,5*W/8 +v,   5*W/8 +m,5*W/8 +v, 5*W/8 +m,5*W/8 +v, 5*W/8 +m,5*W/8 +v,
		5*W/8 +m,5*W/8 +v, 5*W/8 +m,5*W/8 +v, 5*W/8 +m,5*W/8 +v,  5*W/8 +m,7*W/8 +v, 5*W/8 +m,7*W/8 +v,5*W/8 +m,7*W/8 +v,
		5*W/8 +m,7*W/8 +v, 5*W/8 +m,7*W/8 +v,5*W/8 +m,7*W/8 +v,   W +m,W/2 +v,W +m,W/2 +v,W +m,W/2 +v,
		W +m,W/2 +v,W +m,W/2 +v,W +m,W/2 +v,   5*W/8 +m, W/8 +v,5*W/8 +m, W/8 +v,5*W/8 +m, W/8 +v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
		new_vectors = pdb.gimp_vectors_new(image,"Arrow ^")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 84,
		(W/2 +m, v,W/2 +m, v,W/2 +m, v,    W/8 +m,3*W/8 +v, W/8 +m,3*W/8 +v,W/8 +m,3*W/8 +v,
		W/8 +m,3*W/8 +v, W/8 +m,3*W/8 +v,W/8 +m,3*W/8 +v,  3*W/8 +m,3*W/8 +v,3*W/8 +m,3*W/8 +v,3*W/8 +m,3*W/8 +v,
		3*W/8 +m,3*W/8 +v,3*W/8 +m,3*W/8 +v,3*W/8 +m,3*W/8 +v,  3*W/8+m,W+v,3*W/8+m,W+v,3*W/8+m,W+v,
		3*W/8+m,W+v,3*W/8+m,W+v,3*W/8+m,W+v,  5*W/8 +m,W +v, 5*W/8 +m,W +v,5*W/8 +m,W +v,
		5*W/8 +m,W +v, 5*W/8 +m,W +v,5*W/8 +m,W +v,  5*W/8 +m,3*W/8 +v, 5*W/8 +m,3*W/8 +v,5*W/8 +m,3*W/8 +v,
		5*W/8 +m,3*W/8 +v, 5*W/8 +m,3*W/8 +v,5*W/8 +m,3*W/8 +v,  7*W/8 +m,3*W/8 +v,7*W/8 +m,3*W/8 +v,7*W/8 +m,3*W/8 +v,
		7*W/8 +m,3*W/8 +v,7*W/8 +m,3*W/8 +v,7*W/8 +m,3*W/8 +v,  W/2 +m, v,W/2 +m, v,W/2 +m, v,), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
		new_vectors = pdb.gimp_vectors_new(image,"Arrow <")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 84,
		(3*W/8 +m,W/8 +v,3*W/8 +m,W/8 +v,3*W/8 +m,W/8 +v,   m, W/2+v,m, W/2+v,m, W/2+v,
		m, W/2+v,m, W/2+v,m, W/2+v,    3*W/8 +m, 7*W/8 +v, 3*W/8 +m, 7*W/8 +v, 3*W/8 +m, 7*W/8 +v, 
		3*W/8 +m, 7*W/8 +v, 3*W/8 +m, 7*W/8 +v,   3*W/8 +m, 5*W/8 +v,3*W/8 +m, 5*W/8 +v,3*W/8 +m, 5*W/8 +v,
		3*W/8 +m,5*W/8 +v, 3*W/8 +m,5*W/8 +v, 3*W/8 +m,5*W/8 +v,   W+m, 5*W/8 +v,W+m, 5*W/8 +v,W+m, 5*W/8 +v,
		W+m, 5*W/8 +v,W+m, 5*W/8 +v,W+m, 5*W/8 +v,   W+m, 3*W/8 +v,W+m, 3*W/8 +v,W+m, 3*W/8 +v,
		W+m, 3*W/8 +v,W+m, 3*W/8 +v,W+m, 3*W/8 +v,  3*W/8 +m,3*W/8 +v, 3*W/8 +m,3*W/8 +v,3*W/8 +m,3*W/8 +v,
		3*W/8 +m,3*W/8 +v,3*W/8 +m,3*W/8 +v,3*W/8 +m,3*W/8 +v, 3*W/8 +m,W/8 +v,3*W/8 +m,W/8 +v,3*W/8 +m,W/8 +v,), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
		new_vectors = pdb.gimp_vectors_new(image,"Arrow v")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 84,
		(3*W/8 +m,v, 3*W/8 +m,v, 3*W/8 +m,v,   3*W/8 +m,5*W/8 +v,3*W/8 +m,5*W/8 +v,3*W/8 +m,5*W/8 +v,
		3*W/8 +m,5*W/8 +v,3*W/8 +m,5*W/8 +v,3*W/8 +m,5*W/8 +v,   W/8 +m,5*W/8 +v,W/8 +m,5*W/8 +v,W/8 +m,5*W/8 +v,
		W/8 +m,5*W/8 +v,W/8 +m,5*W/8 +v,W/8 +m,5*W/8 +v,     W/2 +m,W +v, W/2 +m,W +v, W/2 +m,W +v, 
		W/2 +m,W +v, W/2 +m,W +v, W/2 +m,W +v,  7*W/8 +m,5*W/8 +v, 7*W/8 +m,5*W/8 +v, 7*W/8 +m,5*W/8 +v, 
		7*W/8 +m,5*W/8 +v, 7*W/8 +m,5*W/8 +v,7*W/8 +m,5*W/8 +v,  5*W/8 +m,5*W/8 +v,5*W/8 +m,5*W/8 +v,5*W/8 +m,5*W/8 +v,
		5*W/8 +m,5*W/8 +v,5*W/8 +m,5*W/8 +v,5*W/8 +m,5*W/8 +v,   5*W/8 +m,v,5*W/8 +m,v,5*W/8 +m,v,
		5*W/8 +m,v,5*W/8 +m,v,5*W/8 +m,v,   3*W/8 +m,v, 3*W/8 +m,v,3*W/8 +m,v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
	if s==1: # Axis of symmetry
		new_vectors = pdb.gimp_vectors_new(image,"Axis of symmetry")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 12, newline(W1/2, k, W1/2, H1-k), 0)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 12, newline(j, H1/2, W1-j, H1/2), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
	if s==2: # Binoculars
		f= M_L
		g = f/2
		h = f/4

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,W/4+v-h ,W/2+g ,W/2+g)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.name = active_vectors.name.replace("Selection", "Bin-BL big")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Bin-BL big")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m+f ,W/4+v-h+f ,W/2+g-2*f ,W/2+g-2*f)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.name = active_vectors.name.replace("Selection", "Bin-BL small")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Bin-BL small")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, W/2+m-f+g ,W/4+v-h ,W/2+g ,W/2+g)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.name = active_vectors.name.replace("Selection", "Bin-RL big")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Bin-RL big")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0,  W/2+m+g ,W/4+v-h+f ,W/2+g-2*f ,W/2+g-2*f)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.name = active_vectors.name.replace("Selection", "Bin-RL small")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Bin-RL small")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,W/4+v-h ,W/2+g ,W/2+g)
		pdb.gimp_image_select_ellipse(image, 1, m+f ,W/4+v-h+f ,W/2+g-2*f ,W/2+g-2*f)
		pdb.gimp_image_select_ellipse(image, 0, W/2+m-f+g ,W/4+v-h ,W/2+g ,W/2+g)
		pdb.gimp_image_select_ellipse(image, 1,  W/2+m+g ,W/4+v-h+f ,W/2+g-2*f ,W/2+g-2*f)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Binoculars")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Binoculars")

	if s==3: # Circle
		new_vectors = pdb.gimp_vectors_new(image,"Circle")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(new_vectors ,W/2+m ,W/2+v ,W/2 ,W/2, 0)
		pdb.gimp_item_set_visible(new_vectors, True)

	if s==4: # Crescent

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, W/8+m ,v ,W ,W)
		pdb.gimp_image_select_ellipse(image,1, W/2+m, W/16+v, 7*W/8, 7*H/8)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Crescent")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Crescent")
		pdb.gimp_selection_none(image)
		
	if s==5: # Cross

		new_vectors = pdb.gimp_vectors_new(image,"Cross")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 144,
		(3*W/8 +m, v, 3*W/8 +m, v, 3*W/8 +m, v,    3*W/8 +m,3*W/8 +v, 3*W/8 +m,3*W/8 +v, 3*W/8 +m,3*W/8 +v,
		3*W/8 +m,3*W/8 +v,3*W/8 +m,3*W/8 +v,3*W/8 +m,3*W/8 +v,     m,3*W/8 +v, m,3*W/8 +v, m,3*W/8 +v,
		m,3*W/8 +v, m,3*W/8 +v, m,3*W/8 +v,    m,5*W/8 +v, m,5*W/8 +v, m,5*W/8 +v,
		m,5*W/8 +v, m,5*W/8 +v, m,5*W/8 +v,   3*W/8 +m,5*W/8 +v, 3*W/8 +m,5*W/8 +v, 3*W/8 +m,5*W/8 +v,
		3*W/8 +m,5*W/8 +v, 3*W/8 +m,5*W/8 +v, 3*W/8 +m,5*W/8 +v,   3*W/8 +m,W +v, 3*W/8 +m,W +v, 3*W/8 +m,W +v,
		3*W/8 +m,W +v, 3*W/8 +m,W +v, 3*W/8 +m,W +v,   5*W/8 +m,W +v, 5*W/8 +m,W +v, 5*W/8 +m,W +v,
		5*W/8 +m,W +v, 5*W/8 +m,W +v, 5*W/8 +m,W +v,   5*W/8 +m,5*W/8 +v,5*W/8 +m,5*W/8 +v,5*W/8 +m,5*W/8 +v,
		5*W/8 +m,5*W/8 +v, 5*W/8 +m,5*W/8 +v, 5*W/8 +m,5*W/8 +v,   W+m, 5*W/8+v, W+m, 5*W/8+v, W+m, 5*W/8+v,
		W+m, 5*W/8+v, W+m, 5*W/8+v, W+m, 5*W/8+v,    W+m, 3*W/8+v, W+m, 3*W/8+v, W+m, 3*W/8+v,
		W+m, 3*W/8+v, W+m, 3*W/8+v, W+m, 3*W/8+v,  5*W/8 +m,3*W/8 +v, 5*W/8 +m,3*W/8 +v, 5*W/8 +m,3*W/8 +v,
		5*W/8 +m,3*W/8 +v, 5*W/8 +m,3*W/8 +v, 5*W/8 +m,3*W/8 +v,   5*W/8 +m,v, 5*W/8 +m,v, 5*W/8 +m,v,
		5*W/8 +m,v, 5*W/8 +m,v, 5*W/8 +m,v,   3*W/8 +m, v, 3*W/8 +m, v, 3*W/8 +m, v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
	if s==6: # Diagonals
		new_vectors = pdb.gimp_vectors_new(image,"Diagonals")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 12, newline( j, k, W1-j, H1-k), 0)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 12, newline( W1-j, k, j, H1-k), 0)
		pdb.gimp_item_set_visible(new_vectors, 1)

	if s==7: # Diamond
		new_vectors = pdb.gimp_vectors_new(image,"Diamond")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 48, 
		(W/2+m, v, W/2+m, v, W/2+m, v, W+m, W/2+v, W+m, W/2+v, W+m, W/2+v,
		W+m, W/2+v, W+m, W/2+v, W+m, W/2+v, W/2+m, W+v, W/2+m, W+v, W/2+m, W+v,
		W/2+m, W+v,W/2+m, W+v,W/2+m, W+v,  m, W/2+v, m, W/2+v, m, W/2+v,
		m, W/2+v,m, W/2+v,m, W/2+v,  W/2+m, v,W/2+m, v,W/2+m, v),0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
	if s==8: # Dodecagon
		d = W/2 - math.sqrt(3)*W/4

		new_vectors = pdb.gimp_vectors_new(image,"Dodecagon")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 144,
		(W/2+m, v, W/2+m, v, W/2+m, v,   W/4+m, d+v,W/4+m, d+v,W/4+m, d+v,
		W/4+m, d+v,  W/4+m, d+v,  W/4+m, d+v,   d+m, W/4+v, d+m, W/4+v, d+m, W/4+v,
		 d+m, W/4+v, d+m, W/4+v, d+m, W/4+v,   m, W/2+v, m, W/2+v, m, W/2+v,
		m, W/2+v,m, W/2+v,m, W/2+v,    d +m, 3*W/4 +v,d +m, 3*W/4 +v,d +m, 3*W/4 +v,
		d +m, 3*W/4 +v,d +m, 3*W/4 +v,d +m, 3*W/4 +v,   W/4+m, W-d+v,W/4+m, W-d+v,W/4+m, W-d+v, 
		W/4+m, W-d+v,W/4+m, W-d+v,W/4+m, W-d+v,   W/2+m, W+v,W/2+m, W+v,W/2+m, W+v,
		W/2+m, W+v,W/2+m, W+v,W/2+m, W+v,     3*W/4+m, W-d +v,3*W/4+m, W-d +v,3*W/4+m, W-d +v,
		3*W/4+m, W-d +v,3*W/4+m, W-d +v,3*W/4+m, W-d +v,    W-d+m, 3*W/4+v,W-d+m, 3*W/4 +v,W-d+m, 3*W/4 +v,
		W-d+m, 3*W/4+v,W-d+m, 3*W/4 +v,W-d+m, 3*W/4 +v,   W+m, W/2+v, W+m, W/2+v, W+m, W/2+v,
		W+m, W/2+v, W+m, W/2+v, W+m, W/2+v,   W-d+m, W/4+v,W-d+m, W/4+v,W-d+m, W/4+v,
		W-d+m, W/4+v,W-d+m, W/4+v,W-d+m, W/4+v,  3*W/4 +m, d+v,3*W/4 +m, d+v,3*W/4 +m, d+v,
		3*W/4 +m, d+v,3*W/4 +m, d+v,3*W/4 +m, d+v,  W/2+m, v, W/2+m, v, W/2+m, v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)		
		
	if s==9: # Ellipse
		new_vectors = pdb.gimp_vectors_new(image,"Ellipse")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(new_vectors ,W/2+m ,W/2+v, W/2+m-j ,W/2+v-k, 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
	if s==10: # Flower 1 Out
		e = W/2 - math.sqrt(3)*W/4
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v, W ,W)
		pdb.gimp_image_select_ellipse(image, 3, -W/2+e+m ,-3*W/4+v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fo_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fo_1")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v, W ,W)
		pdb.gimp_image_select_ellipse(image, 3, -W+2*e+m,v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fo_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fo_2")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v,W ,W)
		pdb.gimp_image_select_ellipse(image, 3, -W/2+e+m,3*W/4 + v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fo_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fo_3")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v,W ,W)
		pdb.gimp_image_select_ellipse(image, 3, W/2-e+m,3*W/4 + v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fo_4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fo_4")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v,W ,W)
		pdb.gimp_image_select_ellipse(image, 3, W-2*e+m,v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fo_5")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fo_5")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v,W ,W)
		pdb.gimp_image_select_ellipse(image, 3, W/2-e+m,-3*W/4+v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fo_6")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fo_6")
		pdb.gimp_selection_none(image)

	if s==11: # Flower 2 In
		e = W/2 - math.sqrt(3)*W/4
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, -W/2+e+m ,-W/4 +v ,W ,W)
		pdb.gimp_image_select_ellipse(image, 3, W/2-e+m ,-W/4 + v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fi_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fi_1")
		
		pdb.gimp_selection_none(image)		
		pdb.gimp_image_select_ellipse(image, 0, m ,-W/2+v ,W ,W)
		pdb.gimp_image_select_ellipse(image, 3, -W/2 +e+m ,W/4+v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fi_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fi_2")
	
		pdb.gimp_selection_none(image)		
		pdb.gimp_image_select_ellipse(image, 0, -W/2+e+m ,-W/4 +v ,W ,W)
		pdb.gimp_image_select_ellipse(image, 3, m ,W/2+v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fi_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fi_3")
	
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, -W/2 +e+m ,W/4+v ,W ,W)
		pdb.gimp_image_select_ellipse(image, 3, W/2-e+m ,W/4+v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fi_4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fi_4")
	
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0,m ,W/2+v ,W ,W)
		pdb.gimp_image_select_ellipse(image, 3, W/2-e+m ,-W/4 + v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fi_5")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fi_5")
	
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,-W/2+v ,W ,W)
		pdb.gimp_image_select_ellipse(image, 3, W/2-e+m ,W/4+v ,W ,W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Fi_6")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Fi_6")
		pdb.gimp_selection_none(image)

	if s==12: # GEAR
		oradius=(H_L)*W/200  #Outer Radius=H
		iradius=(V_L)*W/200  #Inner Radius=V
		teeth=M_L            #Number of teeth=M->L
		rotation=R_L
		pdb.script_fu_gear_path_ssc(image, layer, 'Gear', W/2+m, H/2+v, oradius, iradius, teeth, rotation, run_mode=RUN_NONINTERACTIVE)
		
	if s==13: # Grid rectangular (c) Ofnuts v0.0
# Start Grid======================================================================================================================================
#   v0.0: 2013-04-20 first published version
#   v0.1: 2014-01-25 add entry for Offset & Spacing specification  https://sourceforge.net/projects/gimp-path-tools/files/scripts/
		def strokePoints(x1,y1,x2,y2):
			return [x1,y1]*3+[x2,y2]*3
			
		def steps(first,last,count):
			step=(last-first)/(count-1.)
		#	print 'Step(%3.2f,%3.2f,%d)=%3.2f' % (first,last,count,step)
			return [first+(step*x) for x in range(count-1)]+[last]

		class GridPathSelectionRenderer(object):
			if sel == True:
				def __init__(self,image,linesH,linesV):
					self.image=image
					self.linesH=linesH
					self.linesV=linesV
				
					self.minX=0
					self.maxX=0
					self.minY=0
					self.maxY=0
			if sel == False:
				def __init__(self,image,linesH,linesV):
					pdb.gimp_selection_none(image)
					self.image=image
					self.linesH=linesH
					self.linesV=linesV
				
					self.minX=m
					self.maxX=W1+m
					self.minY=v
					self.maxY=H1+v

			def generateVerticalStrokes(self):
				for x in steps(self.minX,self.maxX,self.linesV):
					points=strokePoints(x,self.minY,x,self.maxY)
					pdb.gimp_vectors_stroke_new_from_points(self.path,0, len(points),points,False)
					
			def generateHorizontalStrokes(self):
				for y in steps(self.minY,self.maxY,self.linesH):
					points=strokePoints(self.minX,y,self.maxX,y)
					pdb.gimp_vectors_stroke_new_from_points(self.path,0, len(points),points,False)
			
			def run(self):
				defined,self.minX,self.minY,self.maxX,self.maxY=pdb.gimp_selection_bounds(self.image)
				self.path=pdb.gimp_vectors_new(self.image, 'Grid %d x %d' % (self.linesH,self.linesV))
				if gimp.version >= (2,9,0): 
					pdb.gimp_image_insert_vectors(self.image, self.path, None, 0)
				else:
					pdb.gimp_image_add_vectors(self.image, self.path, 0)
				if self.linesV > 1:
					self.generateVerticalStrokes()
				if self.linesH > 1:
					self.generateHorizontalStrokes()
				self.path.visible=True	
			
		class GridPathRenderer(object):
			def __init__(self,image):
				self.image=image
				
				self.minX=0
				self.maxX=image.width
				self.minY=0
				self.maxY=image.height

			def generateVerticalStrokes(self):
				x=self.minX+self.offsetH
				while x<self.maxX: 
					points=strokePoints(x,self.minY,x,self.maxY)
					pdb.gimp_vectors_stroke_new_from_points(self.path,0, len(points),points,False)
					x+=self.spacingH
					
			def generateHorizontalStrokes(self):
				y=self.minY+self.offsetV
				while y<self.maxY: 
					points=strokePoints(self.minX,y,self.maxX,y)
					pdb.gimp_vectors_stroke_new_from_points(self.path,0, len(points),points,False)
					y+=self.spacingV
			def run(self):
				defined,self.minX,self.minY,self.maxX,self.maxY=pdb.gimp_selection_bounds(self.image)
				self.path=pdb.gimp_vectors_new(self.image, 'Grid %3.1fx%3.1f@%3.1f,%3.1f' % (self.spacingH,self.spacingV,self.offsetH,self.offsetV))
				if gimp.version >= (2,9,0): 
					pdb.gimp_image_insert_vectors(self.image, self.path, None, 0)
				else:
					pdb.gimp_image_add_vectors(self.image, self.path, 0)
				if self.spacingH > 0.:
					self.generateVerticalStrokes()
				if self.spacingV > 0:
					self.generateHorizontalStrokes()
				self.path.visible=True
	#if sel == True:
		linesH = H_L
		linesV = V_L
	try:
		GridPathSelectionRenderer(image,int(linesH),int(linesV)).run()

        except Exception as e:
		print
# End Grid =======================================================================================================================================	

	if s==14: # Grid Rectangle type Rectangular
		shapegrid_sel_ssc(image, layer, 0, 0, H_L, V_L, R_L, M_L, sel)
	if s==15: # Grid Rectangle type Hexagonal
		shapegrid_sel_ssc(image, layer, 1, 0, H_L, V_L, R_L, M_L, sel)
	if s==16: # Grid Ellipse type Rectangular
		shapegrid_sel_ssc(image, layer, 0, 1, H_L, V_L, R_L, M_L, sel)
	if s==17: # Grid Ellipse type Hexagonal
		shapegrid_sel_ssc(image, layer, 1, 1, H_L, V_L, R_L, M_L, sel)
	if s==18: # Grid Hexagon type Rectangular
		shapegrid_sel_ssc(image, layer, 0, 2, H_L, V_L, R_L, M_L, sel)
	if s==19: # Grid Hexagon type Hexagonal
		shapegrid_sel_ssc(image, layer, 1, 2, H_L, V_L, R_L, M_L, sel)
	if s==20: # Grid Diamond type Rectangular
		shapegrid_sel_ssc(image, layer, 0, 3, H_L, V_L, R_L, M_L, sel)
	if s==21: # Grid Diamond type Hexagonal
		shapegrid_sel_ssc(image, layer, 1, 3, H_L, V_L, R_L, M_L, sel)
	if s==22: # Grid Octagon type Rectangular
		shapegrid_sel_ssc(image, layer, 0, 4, H_L, V_L, R_L, M_L, sel)
	if s==23: # Grid Octagon type Hexagonal
		shapegrid_sel_ssc(image, layer, 1, 4, H_L, V_L, R_L, M_L, sel)
	if s==24: # Hexagon
		d = W/2 - math.sqrt(3)*W/4

		new_vectors = pdb.gimp_vectors_new(image,"Hexagon")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 72,
		(W/4+m, d+v,  W/4+m, d+v,  W/4+m, d+v,        W/2+W/4+m,d+v,  W/2+W/4+m, d+v,    W/2+W/4+m, d+v,
		W/4+m, d+v,  W/4+m, d+v,  W/4+m, d+v,       m, W/2 +v, m, W/2 +v,m, W/2 +v,
		m, W/2 +v, m, W/2 +v,m, W/2 +v,               W/4+m, W-d+v,  W/4+m, W-d+v,  W/4+m,W-d+v, 
		W/4+m, W-d+v,  W/4+m, W-d+v,  W/4+m,W-d+v,      W/2+W/4+m, W-d+v, W/2+W/4+m, W-d+v,W/2+W/4+m, W-d+v,
		W/2+W/4+m, W-d+v, W/2+W/4+m, W-d+v,W/2+W/4+m, W-d+v,     W+m, W/2 +v, W+m, W/2 +v,W+m, W/2 +v,
		W+m, W/2 +v, W+m, W/2 +v,W+m, W/2 +v, W/2+W/4+m,d+v,  W/2+W/4+m, d+v,    W/2+W/4+m, d+v,), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
		
	if s==25: # Ice cream Waffle
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_polygon(image, 0, 8, (W/4+m,3*W/8+v,   W/2+m,W+v,    W/2+m, W+v,  3*W/4+m, 3*W/8+v ))
		pdb.gimp_image_select_ellipse(image, 1 ,W/4+m, W/4+v, W/2, H/4)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Waffle")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Wafel")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0 ,W/4+m, W/4+v, W/2, H/4)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Ice_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Lody_1")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_rectangle(image,0, m, v, W, 3*H/8)
		pdb.gimp_image_select_ellipse(image, 3 ,W/4+m, W/8+v, W/2, H/2)
		pdb.gimp_image_select_ellipse(image, 0 ,W/4+m, W/4+v, W/2, H/4)		
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Ice_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Lody_2")		
		
	if s==26: # Mobius band

		b = 3*W/(8*math.tan(math.pi/6))
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_polygon(image, 0, 10, (3*W/8+m,v,   m, b+v,    W/8+m,7*W/8+v,  5*W/8 +m, v,  3*W/8+m,v ))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Mobius band_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Mobius band_1")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_polygon(image, 0, 10, (7*W/8+m,7*W/8+v, 3*W/8+m,v, 5*W/8+m,v, W+m, b+v, 7*W/8+m,7*W/8+v))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Mobius band_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Mobius band_2")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_polygon(image, 0, 10, (m, b+v,  W/8+m,7*W/8+v,  7*W/8+m,7*W/8+v, W+m, b+v, m, b+v))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Mobius band_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Mobius band_3")
		pdb.gimp_selection_none(image)
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_polygon(image, 0, 10, (3*W/8+m,v,   m, b+v,    W/4+m,b+v,  5*W/8 +m, v,  3*W/8+m,v ))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Mobius alternative-3-2-A")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Mobius alternative-3-2-A")		
		
	if s==27: # Octaagon
		a = W*math.sqrt(2)/(2+math.sqrt(2))
		b = a/math.sqrt(2)

		new_vectors = pdb.gimp_vectors_new(image,"Octagon")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 96,(
		b+a+m, v, b+a+m, v,b+a+m, v,  b+m,v,b+m,v,b+m,v,
		b+m,v,b+m,v,b+m,v,   m, b+v, m, b+v,m, b+v,
		m, b+v, m, b+v,m, b+v,  m, a+b+v,m, a+b+v,m, a+b+v,
		m, a+b+v,m, a+b+v,m, a+b+v, b+m,W+v,b+m,W+v,b+m,W+v,
		b+m,W+v,b+m,W+v,b+m,W+v, a+b+m,W+v, a+b+m,W+v,
		a+b+m,W+v, a+b+m,W+v, a+b+m,W+v,  W+m,a+b+v,W+m,a+b+v,W+m,a+b+v,
		W+m,a+b+v,W+m,a+b+v,W+m,a+b+v,   W+m, b+v ,W+m, b+v, W+m, b+v,
		W+m, b+v ,W+m, b+v, W+m, b+v,  b+a+m, v, b+a+m, v,b+a+m, v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)

	if s==28: # Pentagon

		alfa = 36 
		q = 1.0
		r = q*W/2
		x1 = W/2 + r*math.sin((alfa + 72)*math.pi/180)
		y1 = H/2 + r*math.cos((alfa + 72)*math.pi/180)
		x2 = W/2 + r*math.sin((alfa + 144)*math.pi/180)
		y2 = H/2 + r*math.cos((alfa + 144)*math.pi/180)
		x3 = W/2 + r*math.sin((alfa + 216)*math.pi/180)
		y3 = H/2 + r*math.cos((alfa + 216)*math.pi/180)
		x4 = W/2 + r*math.sin((alfa + 288)*math.pi/180)
		y4 = H/2 + r*math.cos((alfa + 288)*math.pi/180)
		x5 = W/2 + r*math.sin((alfa + 360)*math.pi/180)
		y5 = H/2 + r*math.cos((alfa + 360)*math.pi/180)
		new_vectors = pdb.gimp_vectors_new(image,"Pentagon")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 60,(
		x1+m,y1+v,x1+m,y1+v,x1+m,y1+v,  x2+m,y2+v,x2+m,y2+v,x2+m,y2+v,
		x2+m,y2+v,x2+m,y2+v,x2+m,y2+v,  x3+m,y3+v,x3+m,y3+v,x3+m,y3+v,
		x3+m,y3+v,x3+m,y3+v,x3+m,y3+v,  x4+m,y4+v,x4+m,y4+v,x4+m,y4+v,
		x4+m,y4+v,x4+m,y4+v,x4+m,y4+v,  x5+m,y5+v,x5+m,y5+v,x5+m,y5+v,
		x5+m,y5+v,x5+m,y5+v,x5+m,y5+v,  x1+m,y1+v,x1+m,y1+v,x1+m,y1+v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)

	if s==29: # Pentagram

		alfa = 36
		q = 1.0
		r = q*W/2
		x1 = W/2 + r*math.sin((alfa + 72)*math.pi/180)
		y1 = H/2 + r*math.cos((alfa + 72)*math.pi/180)
		x2 = W/2 + r*math.sin((alfa + 144)*math.pi/180)
		y2 = H/2 + r*math.cos((alfa + 144)*math.pi/180)
		x3 = W/2 + r*math.sin((alfa + 216)*math.pi/180)
		y3 = H/2 + r*math.cos((alfa + 216)*math.pi/180)
		x4 = W/2 + r*math.sin((alfa + 288)*math.pi/180)
		y4 = H/2 + r*math.cos((alfa + 288)*math.pi/180)
		x5 = W/2 + r*math.sin((alfa + 360)*math.pi/180)
		y5 = H/2 + r*math.cos((alfa + 360)*math.pi/180)
		new_vectors = pdb.gimp_vectors_new(image,"Pentagram")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 60,(
		x1+m,y1+v,x1+m,y1+v,x1+m,y1+v,  x3+m,y3+v,x3+m,y3+v,x3+m,y3+v,
		x3+m,y3+v,x3+m,y3+v,x3+m,y3+v,  x5+m,y5+v,x5+m,y5+v,x5+m,y5+v,
		x5+m,y5+v,x5+m,y5+v,x5+m,y5+v,  x2+m,y2+v,x2+m,y2+v,x2+m,y2+v,
		x2+m,y2+v,x2+m,y2+v,x2+m,y2+v,  x4+m,y4+v,x4+m,y4+v,x4+m,y4+v,
		x4+m,y4+v,x4+m,y4+v,x4+m,y4+v,  x1+m,y1+v,x1+m,y1+v,x1+m,y1+v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)

	if s==30: #  Petal (butterfly)
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, -W/2+m, v, W, H)
		pdb.gimp_image_select_ellipse(image, 3 ,m, -W/2+v, W, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Petal_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Petal_1")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m, -W/2+v, W, H)
		pdb.gimp_image_select_ellipse(image, 3, W/2+m, v, W, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Petal_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Petal_2")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, W/2+m, v, W, H)
		pdb.gimp_image_select_ellipse(image, 3, m, W/2+v, W, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Petal_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Petal_3")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, -W/2+m, v, W, H)
		pdb.gimp_image_select_ellipse(image, 3, m, W/2+v, W, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Petal_4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Petal_4")
		pdb.gimp_selection_none(image)
		
	if s==31: # Pie 1/2
	
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,3, W/2+m, v, W/2, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/2_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/2_1")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,3, m, W/2+v, W, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/2_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/2_2")		
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,3, m, v, W/2, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/2_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/2_3")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,3, m, v, W, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/2_4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/2_4")		

	if s==32: # Pie 3/4 - 1/4
	
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,1, W/2+m, v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_3/4_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_3/4_1")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,3, W/2+m, v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/4_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/4_1")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image, 1, m, v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_3/4_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_3/4_2")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image, 3, m, v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/4_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/4_2")		
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,1, m, W/2+v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_3/4_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_3/4_3")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,3, m, W/2+v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/4_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/4_3")		
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,1, W/2+m, W/2+v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_3/4_4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_3/4_4")
		pdb.gimp_selection_none(image)
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,3, W/2+m, W/2+v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/4_4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/4_4")
		pdb.gimp_selection_none(image)		
		
	if s==33: # Pie 7/8 - 1/8

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_polygon(image ,3 ,6 ,(W/2+m, v, W/2+m, W/2+v, W+m, v))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/8_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/8_1")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)		
		pdb.gimp_image_select_polygon(image ,1 ,6 ,(W/2+m, v, W/2+m, W/2+v, W+m, v))		
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_7/8_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_7/8_1")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_polygon(image ,3 ,6 ,(W+m, v, W/2+m, W/2+v, W+m, W/2+v))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/8_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/8_2")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)		
		pdb.gimp_image_select_polygon(image ,1 ,6 ,(W+m, v, W/2+m, W/2+v, W+m, W/2+v))	
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_7/8_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_7/8_2")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_polygon(image ,3 ,6 ,(W+m, W+v, W/2+m, W/2+v, W+m, W/2+v))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/8_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/8_3")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)		
		pdb.gimp_image_select_polygon(image ,1 ,6 ,(W+m, W+v, W/2+m, W/2+v, W+m, W/2+v))	
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_7/8_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_7/8_3")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_polygon(image ,3 ,6 ,(W+m, W+v, W/2+m, W/2+v, W/2+m, W+v))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/8_4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/8_4")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)		
		pdb.gimp_image_select_polygon(image ,1 ,6 ,(W+m, W+v, W/2+m, W/2+v, W/2+m, W+v))	
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_7/8_4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_7/8_4")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_polygon(image ,3 ,6 ,(W/2+m, W/2+v, W/2+m, W+v, m, W+v))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/8_5")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/8_5")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)		
		pdb.gimp_image_select_polygon(image ,1 ,6 ,(W/2+m, W/2+v, W/2+m, W+v, m, W+v))	
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_7/8_5")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_7/8_5")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_polygon(image ,3 ,6 ,(m, W/2+v, W/2+m, W/2+v, m, W+v))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/8_6")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/8_6")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)		
		pdb.gimp_image_select_polygon(image ,1 ,6 ,(m, W/2+v, W/2+m, W/2+v, m, W+v))	
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_7/8_6")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_7/8_6")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_polygon(image ,3 ,6 ,(m, v, W/2+m, W/2+v, m, W/2+v))	
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/8_7")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/8_7")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)		
		pdb.gimp_image_select_polygon(image ,1 ,6 ,(m, v, W/2+m, W/2+v, m, W/2+v))	
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_7/8_7")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_7/8_7")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_polygon(image ,3 ,6 ,(W/2+m, v, W/2+m, W/2+v, m, v))	
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_1/8_8")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_1/8_8")
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)		
		pdb.gimp_image_select_polygon(image ,1 ,6 ,(W/2+m, v, W/2+m, W/2+v, m, v))	
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Pie_7/8_8")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Pie_7/8_8")


	if s==34: # Playing cards Clubs

		new_vectors = pdb.gimp_vectors_new(image,"Clubs")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 54,
		(6*W/8 +m,-5*W/64+v, 6*W/8+m,2*W/8+v, 6*W/8+m,-5*W/64+v,
		2*W/8+m,-5*W/64+v,  2*W/8+m,2*W/8+v, -5*W/64+m,2*W/8+v,
		-5*W/64+m,6*W/8+v,  2*W/8+m,6*W/8+v,  3*W/8+m, 6*W/8+v,
		7*W/16+m,11*W/16+v,  W/2+m-W/80,5*W/8+v,  W/2+m,7*W/8+v,
		3*W/8+m,W+v, W/8+m,W+v, 3*W/8+m,W+v,
		5*W/8+m,W+v, 7*W/8+m,W+v, 5*W/8+m,W+v,
		W/2+m,7*W/8+v, W/2+m+W/80,5*W/8+v, 9*W/16+m,11*W/16+v,
		5*W/8+m,6*W/8+v, 6*W/8+m, 6*W/8+v, 69*W/64+m,6*W/8+v,
		 69*W/64+m,2*W/8+v,  6*W/8+m,2*W/8+v,  6*W/8+m,2*W/8+v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
	if s==35: # Playing cards Diamonds

		new_vectors = pdb.gimp_vectors_new(image,"Diamonds")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 30,
		( m+W/2,v+W/8, m+W/2,v, m+W/2,v+W/8,
		m+H/8,v+H/2, m,v+H/2, m+H/8,v+H/2, 
		m+W/2,v+7*W/8, m+W/2,v+H, m+W/2,v+7*W/8,
		m+7*H/8,v+H/2, m+H,v+H/2, m+7*H/8,v+H/2,
		m+W/2,v+W/8, m+W/2,v, m+W/2,v+W/8), 0)
		pdb.gimp_item_set_visible(new_vectors, True)

	if s==36: # Playing cards Hearts

		new_vectors = pdb.gimp_vectors_new(image,"Hearts")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 30,
		( W/2+m,-W/10+v,  W/2+m,W/4+1+v,  W/2+m,-W/10+v,
		m, -W/16+v, m,W/4+v,  m,W/2+v,
		3*W/8+m,5*W/8+v,  W/2+m,W+v,  5*W/8+m,5*W/8+v,
		W+m, W/2+v, W+m,W/4+v,  W+m,-W/16+v,
		W/2+m,-W/10+v,  W/2+m,W/4+1+v,  W/2+m,-W/10+v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)

	if s==37: # Playing cards Spades

		new_vectors = pdb.gimp_vectors_new(image,"Spades")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 48,
		( 5*W/8+m, 2*W/8+v,  W/2+m,v,  3*W/8+m, 2*W/8+v,
		m,2*W/8+v,  m,W/2+v,  m,6*W/8+v,
		2*W/8+m, 7*W/8+v, W/2+m-W/100,5*W/8+v,  W/2+m,7*W/8+v,
		3*W/8+m, W+v,  W/8+m,W+v,  3*W/8+m, W+v,
		5*W/8+m, W+v,  7*W/8+m,W+v,  5*W/8+m, W+v,
		W/2+m, 7*W/8+v, W/2+m+W/100,5*W/8+v,  6*W/8+m, 7*W/8+v,
		W+m,6*W/8+v,  W+m,W/2+v,  W+m,2*W/8+v,
		5*W/8+m, 2*W/8+v,  W/2+m,v,  3*W/8+m, 2*W/8+v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
	if s==38: # POLYGON

		sides=M_L            #Number=M->L
		rotation=R_L
		pdb.script_fu_polygon_path_ssc(image, layer, 'Polygon', W/2+m, H/2+v, W/2, sides, rotation, run_mode=RUN_NONINTERACTIVE)	
		
	if s==39: # Protractor Inside circle
# Start Protractor
#===================================================================================================================================================	
# Rel 1: Initial release.
# Rel 2: Draw marks as a single stroke for speed.
# Rel 3: Add option to draw marks inside circle or outside circle [download: http://bakon.ca/gimplearn/viewtopic.php?f=3&t=175#p512]
# Created by Tin Tran http://bakon.ca/gimplearn/  
# Comments directed to http://gimpchat.com or http://gimpscripts.com

		pdb.gimp_selection_none(image)
		if sel == False:
			x1 = m
			y1 = v
			x2 = W+m
			y2 = H+v
	
		W = float(abs(x2-x1))
		H = float(abs(y2-y1))
		radius = min(W,H)/2.0
		midx = x1 + (W/2.0)
		midy = y1 + (H/2.0)

		marklength = M_L
	
		vectors_name = "Protractor Marks"
		vectors = pdb.gimp_vectors_new(image,vectors_name)
		pdb.gimp_image_insert_vectors(image,vectors,None,-1)
	
		gimp.progress_init("Drawing Protractor Marks...")
		closed = 0
		mark_control_points = []
		for mark in range(0,361):
			gimp.progress_update(float(mark)/360)
			#Determine mark length
			mlength = float(marklength)/4 #regular marks are 1/4 of full mark length
			if mark % 5 == 0: #every 5 marks, make the mlength 1/2 of full mark length
				mlength = float(marklength)/2
			if mark % 10 == 0:
				mlength = marklength
			rad = (mark/360.0) * math.pi * 2
			# draw star in center
			if mark % 45 == 0:
			
				mx1 = midx 
				my1 = midy 
				mx2 = midx + (math.cos(rad) * (mlength))
				my2 = midy + (math.sin(rad) * (mlength))
			
				# draw mark to mark degree
				control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
				pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
			#horizontal rulers top and bottom
		
			mx1 = midx + (math.cos(rad) * radius)
			my1 = midy + (math.sin(rad) * radius)
			mradius = (radius - mlength)
			mx2 = midx + (math.cos(rad) * mradius)
			my2 = midy + (math.sin(rad) * mradius)
		
		# draw mark to mark degree
			mark_control_points = mark_control_points + ([mx1,my1]*3) + ([mx2,my2]*3) + ([mx1,my1]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(mark_control_points),mark_control_points,1)	
		pdb.gimp_item_set_visible(vectors,1)		
		#Draw Circle

	if s==40: # Protractor Outside circle

		pdb.gimp_selection_none(image)
		if sel == False:
			x1 = m
			y1 = v
			x2 = W+m
			y2 = H+v
	
		W = float(abs(x2-x1))
		H = float(abs(y2-y1))
		radius = min(W,H)/2.0
		midx = x1 + (W/2.0)
		midy = y1 + (H/2.0)

		marklength = M_L
	
		vectors_name = "Protractor Marks"
		vectors = pdb.gimp_vectors_new(image,vectors_name)
		pdb.gimp_image_insert_vectors(image,vectors,None,-1)
	
		gimp.progress_init("Drawing Protractor Marks...")
		closed = 0
		mark_control_points = []
		for mark in range(0,361):
			gimp.progress_update(float(mark)/360)
			#Determine mark length
			mlength = float(marklength)/4 #regular marks are 1/4 of full mark length
			if mark % 5 == 0: #every 5 marks, make the mlength 1/2 of full mark length
				mlength = float(marklength)/2
			if mark % 10 == 0:
				mlength = marklength
			rad = (mark/360.0) * math.pi * 2
			# draw star in center
			if mark % 45 == 0:
			
				mx1 = midx 
				my1 = midy 
				mx2 = midx + (math.cos(rad) * (mlength))
				my2 = midy + (math.sin(rad) * (mlength))
			
				# draw mark to mark degree
				control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
				pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
			#horizontal rulers top and bottom
		
			mx1 = midx + (math.cos(rad) * radius)
			my1 = midy + (math.sin(rad) * radius)
			mradius = (radius + mlength)
			mx2 = midx + (math.cos(rad) * mradius)
			my2 = midy + (math.sin(rad) * mradius)
		
		# draw mark to mark degree
			mark_control_points = mark_control_points + ([mx1,my1]*3) + ([mx2,my2]*3) + ([mx1,my1]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(mark_control_points),mark_control_points,1)	
		pdb.gimp_item_set_visible(vectors,1)		
		#Draw Circle
#End Protractor ===============================================================================================================================================
	if s==41: # Quadrant
	
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,3, m, v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Quadrant_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Quadrant_1")
		pdb.gimp_selection_none(image)
		
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,3, W/2+m, v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Quadrant_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Quadrant_2")
		pdb.gimp_selection_none(image)
		
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,3, W/2+m, W/2+v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Quadrant_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Quadrant_3")
		pdb.gimp_selection_none(image)
		
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image, 3, m, W/2+v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Quadrant_4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Quadrant_4")
		pdb.gimp_selection_none(image)
		
	if s==42: # Quatrefoil

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, W/4+m ,v ,W/2 ,W/2)
		pdb.gimp_image_select_ellipse(image, 0, m ,W/4+v ,W/2 ,W/2)
		pdb.gimp_image_select_ellipse(image, 0, W/4+m ,W/2+v ,W/2 ,W/2)
		pdb.gimp_image_select_ellipse(image, 0, W/2+m ,W/4+v ,W/2 ,W/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Quatrefoil-outside")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Quatrefoil-outside")
		pdb.gimp_selection_none(image)
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_rectangle(image,0, m, v, W/2, H)
		pdb.gimp_image_select_ellipse(image, 3, W/4+m ,v ,W/2 ,W/2)
		pdb.gimp_image_select_ellipse(image, 3, m ,W/4+v ,W/2 ,W/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Quatrefoil-inside-1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Quatrefoil-inside-1")
		pdb.gimp_selection_none(image)
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_rectangle(image,0, m, v, W/2, H)
		pdb.gimp_image_select_ellipse(image, 3, m ,W/4+v ,W/2 ,W/2)
		pdb.gimp_image_select_ellipse(image, 3, W/4+ m ,W/2+v ,W/2 ,W/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Quatrefoil-inside-2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Quatrefoil-inside-2")
		pdb.gimp_selection_none(image)
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_rectangle(image,0, W/2+m, v, W/2, H)
		pdb.gimp_image_select_ellipse(image, 3, W/4+m ,W/2+v ,W/2 ,W/2)
		pdb.gimp_image_select_ellipse(image, 3, W/2+ m ,W/4+v ,W/2 ,W/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Quatrefoil-inside-3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Quatrefoil-inside-3")
		pdb.gimp_selection_none(image)		
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_rectangle(image,0, W/2+m, v, W/2, H)
		pdb.gimp_image_select_ellipse(image, 3, W/4+m ,v ,W/2 ,W/2)
		pdb.gimp_image_select_ellipse(image, 3, W/2+ m ,W/4+v ,W/2 ,W/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Quatrefoil-inside-4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Quatrefoil-inside-4")
		pdb.gimp_selection_none(image)
		
	if s==43: # Rectangle
		new_vectors = pdb.gimp_vectors_new(image,"Rectangle")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 30, 
		(j,k, j,k, j,k,    W1-j,k, W1-j,k ,W1-j,k,
		W1-j,H1-k, W1-j,H1-k, W1-j,H1-k,    j,H1-k, j,H1-k, j,H1-k,
		j,k, j,k, j,k), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
	if s==44: # Recycle two arrows

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_rectangle(image,0, W/2 +m, v, W/2, W)
		pdb.gimp_image_select_ellipse(image, 3, m ,v ,W ,W)
		pdb.gimp_image_select_ellipse(image, 1, W/4+m ,W/4+v ,W/2 ,W/2)
		pdb.gimp_image_select_polygon(image, 1, 10, (5*W/8 + m, 5*W/8 + v,  W/2 + m, 5*W/8 + v,  W/2 +m, W+v, 6*W/8 +m, W+v,  5*W/8+m, 5*W/8+v))
		pdb.gimp_image_select_polygon(image, 0, 8, (5*W/8 + m, 5*W/8 + v,  W/2 + m, 7*W/8 + v,  6*W/8 +m, W+v,  5*W/8+m, 5*W/8+v))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Recycle_D")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Recycle_D")
		pdb.gimp_selection_none(image)
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_rectangle(image,0, m, v, W/2, W)
		pdb.gimp_image_select_ellipse(image, 3, m ,v ,W ,W)
		pdb.gimp_image_select_ellipse(image, 1, W/4+m ,W/4+v ,W/2 ,W/2)
		pdb.gimp_image_select_polygon(image, 1, 10, (W/4 + m,v, 3*W/8+ m, 3*W/8 + v,  W/2 +m, 3*W/8+v, W/2 +m, v,  W/4 + m,v))
		pdb.gimp_image_select_polygon(image, 0, 8, (W/4 + m, v,  3*W/8 + m, 3*W/8 + v,  W/2 +m, W/8+v,  W/4 + m, v,))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Recycle_C")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Recycle_C")
		pdb.gimp_selection_none(image)

	if s==45: # Recycle three arrows
		pdb.gimp_selection_none(image)	
		new_vectors = pdb.gimp_vectors_new(image,"Recycle_1")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		R1 = pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 84,(
		40*W/100+m, 39*W/100+v,40*W/100+m, 39*W/100+v,40*W/100+m, 39*W/100+v,      50*W/100+m, 22*W/100+v,50*W/100+m, 22*W/100+v,50*W/100+m, 22*W/100+v,
		50*W/100+m, 22*W/100+v,50*W/100+m, 22*W/100+v,50*W/100+m, 22*W/100+v,       55*W/100+m, 13*W/100+v,55*W/100+m, 13*W/100+v,55*W/100+m, 13*W/100+v,
		55*W/100+m, 13*W/100+v,55*W/100+m, 13*W/100+v,57*W/100+m, 10*W/100+v,       59*W/100+m, 10*W/100+v, 63*W/100+m, 10*W/100+v,63*W/100+m, 10*W/100+v,		
		63*W/100+m, 10*W/100+v,63*W/100+m, 10*W/100+v,66*W/100+m, 10*W/100+v,       36*W/100+m, 10*W/100+v,36*W/100+m, 10*W/100+v,36*W/100+m, 10*W/100+v,
		36*W/100+m, 10*W/100+v,36*W/100+m, 10*W/100+v,33*W/100+m, 10*W/100+v,       31*W/100+m, 11*W/100+v,30*W/100+m, 13*W/100+v,30*W/100+m, 13*W/100+v,
		30*W/100+m, 13*W/100+v,30*W/100+m, 13*W/100+v,30*W/100+m, 13*W/100+v,      21*W/100+m, 28*W/100+v,21*W/100+m, 28*W/100+v,21*W/100+m, 28*W/100+v,
		21*W/100+m, 28*W/100+v, 21*W/100+m, 28*W/100+v,21*W/100+m, 28*W/100+v,     40*W/100+m, 39*W/100+v,40*W/100+m, 39*W/100+v,40*W/100+m, 39*W/100+v,
		), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		pdb.gimp_image_get_active_vectors(image)
		x = pdb.gimp_vectors_copy(new_vectors)
		pdb.gimp_image_insert_vectors(image, x, None, -1)
		pdb.gimp_vectors_stroke_rotate(new_vectors, R1 ,W/2+m ,W/2+v ,-120)
	
		pdb.gimp_image_get_active_vectors(image)
		y = pdb.gimp_vectors_copy(x)
		pdb.gimp_image_insert_vectors(image, y, None, -1)
		pdb.gimp_vectors_stroke_rotate(y, R1 ,W/2+m ,W/2+v ,120)
		
		pdb.gimp_item_set_visible(new_vectors, True)
		
		new_vectors = pdb.gimp_vectors_new(image,"Recycle_2")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		R2 = pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 108,(
		50*W/100+m, 22*W/100+v,50*W/100+m, 22*W/100+v,50*W/100+m, 22*W/100+v,          55*W/100+m, 13*W/100+v,55*W/100+m, 13*W/100+v,55*W/100+m, 13*W/100+v,
		55*W/100+m, 13*W/100+v,55*W/100+m, 13*W/100+v,57*W/100+m, 10*W/100+v,          59*W/100+m, 10*W/100+v, 63*W/100+m, 10*W/100+v,63*W/100+m, 10*W/100+v,
		63*W/100+m, 10*W/100+v,63*W/100+m, 10*W/100+v,66*W/100+m, 10*W/100+v,          68*W/100+m, 10*W/100+v, 70*W/100+m, 13*W/100+v, 70*W/100+m, 13*W/100+v,
		70*W/100+m, 13*W/100+v,70*W/100+m, 13*W/100+v,70*W/100+m, 13*W/100+v,          74*W/100+m, 20*W/100+v,74*W/100+m, 20*W/100+v,74*W/100+m, 20*W/100+v,
		74*W/100+m, 20*W/100+v,74*W/100+m, 20*W/100+v,74*W/100+m, 20*W/100+v,    	   79*W/100+m, 17*W/100+v,79*W/100+m, 17*W/100+v,79*W/100+m, 17*W/100+v,
		79*W/100+m, 17*W/100+v,79*W/100+m, 17*W/100+v,79*W/100+m, 17*W/100+v,          72*W/100+m, 41*W/100+v,72*W/100+m, 41*W/100+v,72*W/100+m, 41*W/100+v,
		72*W/100+m, 41*W/100+v,72*W/100+m, 41*W/100+v,72*W/100+m, 41*W/100+v,          48*W/100+m, 35*W/100+v,48*W/100+m, 35*W/100+v,48*W/100+m, 35*W/100+v,
		48*W/100+m, 35*W/100+v,48*W/100+m, 35*W/100+v,48*W/100+m, 35*W/100+v,          55*W/100+m, 31*W/100+v,55*W/100+m, 31*W/100+v,55*W/100+m, 31*W/100+v,
		55*W/100+m, 31*W/100+v,55*W/100+m, 31*W/100+v,55*W/100+m, 31*W/100+v,          50*W/100+m, 22*W/100+v,50*W/100+m, 22*W/100+v,50*W/100+m, 22*W/100+v
		), 0)
		
		pdb.gimp_item_set_visible(new_vectors, True)
		pdb.gimp_image_get_active_vectors(image)
		x = pdb.gimp_vectors_copy(new_vectors)
		pdb.gimp_image_insert_vectors(image, x, None, -1)
		pdb.gimp_vectors_stroke_rotate(new_vectors, R2 ,W/2+m ,W/2+v ,-120)
		pdb.gimp_selection_none(image)		
	
		pdb.gimp_image_get_active_vectors(image)
		y = pdb.gimp_vectors_copy(x)
		pdb.gimp_image_insert_vectors(image, y, None, -1)
		pdb.gimp_vectors_stroke_rotate(y, R2 ,W/2+m ,W/2+v ,120)
		pdb.gimp_item_set_visible(new_vectors, True)
		pdb.gimp_selection_none(image)
		
	if s==46: # Roses compass

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m,v,  3*W/8+m,3*W/8+v, W/2+m, W/2+v, W/2+m,v))
		pdb.gimp_image_select_polygon(image, 0, 8, (m, W/2+v, 3*W/8+m, 5*W/8+v, W/2+m, W/2+v, m, W/2+v))
		pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m, W/2+v, W/2+m, W+v, 5*W/8 +m, 5*W/8 +v,W/2+m, W/2+v))
		pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m, W/2+v, W+m, W/2+v, 5*W/8 +m, 3*W/8 +v, W/2+m, W/2+v))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Roses_L")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Róża wiatrów L")
		pdb.gimp_selection_none(image)
		
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m,v, W/2+m,W/2 +v, 5*W/8+m,3*W/8+v, W/2+m,v))
		pdb.gimp_image_select_polygon(image, 0, 8, (3*W/8+m,3*W/8+v, m, W/2+v, W/2+m, W/2+v, 3*W/8+m,3*W/8+v))
		pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m, W/2+v, 3*W/8+m,5*W/8+v, W/2+m, W+v, W/2+m, W/2+v,))
		pdb.gimp_image_select_polygon(image, 0, 8, (W/2+m, W/2+v, 5*W/8 +m, 5*W/8 +v, W+m, W/2+v,  W/2+m, W/2+v,))
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Roses_R")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Róża wiatrów P")
		pdb.gimp_selection_none(image)
		
	if s==47: # Ruller
#===================================================================================================================================================
# Start "Draw_Percentage_Ruler Rel 1"
# Created by Tin Tran http://bakon.ca/gimplearn/
# Comments directed to http://gimpchat.com or http://gimpscripts.com

		pdb.gimp_selection_none(image)
	
		W = float(abs(x2-x1))
		H = float(abs(y2-y1))

		marklength = M_L
		vectors_name = "Percentage Ruler"
		vectors = pdb.gimp_vectors_new(image,vectors_name)
		pdb.gimp_image_insert_vectors(image,vectors,None,-1)
	
		gimp.progress_init("Drawing Percentage Rulers...")
		closed = 0
	
		for mark in range(0,101):
			gimp.progress_update(float(mark)/100)
			#Determine mark length
			mlength = float(marklength)/4 #regular marks are 1/4 of full mark length
			if mark % 5 == 0: #every 5 marks, make the mlength 1/2 of full mark length
				mlength = float(marklength)/2
			if mark % 10 == 0:
				mlength = marklength
			
			#horizontal rulers top and bottom
			mx1 = float(W)/100 * mark + x1
			mx2 = mx1
			my1 = y1
			my2 = y1 - mlength
			control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
			pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
			my1 = y2
			my2 = y2 + mlength
			control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
			pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
			#vertical marks
			mx1 = x1
			mx2 = x1 - mlength
			my1 = float(H)/100 * mark + y1
			my2 = my1
			control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
			pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		
			mx1 = x2
			mx2 = x2 + mlength
			control_points = ([mx1,my1]*3) + ([mx2,my2]*3)
			pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)

		#here, we have control points all calculated depending on method, so we add the points
			pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)

		#Draw box 
		control_points = ([x1,y1]*3) + ([x2,y1]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		control_points = ([x2,y1]*3) + ([x2,y2]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		control_points = ([x1,y2]*3) + ([x2,y2]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		control_points = ([x1,y1]*3) + ([x1,y2]*3)
		pdb.gimp_vectors_stroke_new_from_points(vectors,VECTORS_STROKE_TYPE_BEZIER,len(control_points),control_points,closed)
		pdb.gimp_item_set_visible(vectors,1)
# End "Draw_Percentage_Ruler"
#===================================================================================================================================================

	if s==48: # Rounded Rectangle
		if sel==False:
			top=0
			left=0
			bottom=H1
			right=W1
			radiusx=(H_L)*W1/100
			radiusy=(V_L)*H1/100
		if sel==True:	
			top=y1
			left=x1
			bottom=y2
			right=x2
			radiusx=(H_L)*W/100
			radiusy=(V_L)*H/100
		pdb.script_fu_rounded_rectangle_path_ssc(image, layer, 'Rounded Rectangle', top,left, bottom,right,radiusx,radiusy, run_mode=RUN_NONINTERACTIVE)	

	if s==49: # RUNDED RECTANGLE Advanced
		if sel==False:
			top=0
			left=0
			bottom=H1
			right=W1
			radiusx1=(M_L)*W1/100
			radiusy1=(M_L)*H1/100
			radiusx2=(H_L)*W1/100
			radiusy2=(H_L)*H1/100
			radiusx4=(V_L)*W1/100
			radiusy4=(V_L)*H1/100
			radiusx3=(R_L)*W1/100
			radiusy3=(R_L)*H1/100
		if sel==True:
			top=y1
			left=x1
			bottom=y2
			right=x2
			radiusx1=(M_L)*W/100
			radiusy1=(M_L)*H/100
			radiusx2=(H_L)*W/100
			radiusy2=(H_L)*H/100
			radiusx4=(V_L)*W/100
			radiusy4=(V_L)*H/100
			radiusx3=(R_L)*W/100
			radiusy3=(R_L)*H/100
		pdb.script_fu_advanced_rounded_rectangle_path_ssc (image, layer, 'Rounded Rectangle', top,left,bottom,right,radiusx1,radiusy1,radiusx2,radiusy2,radiusx4,radiusy4,radiusx3,radiusy3,run_mode=RUN_NONINTERACTIVE)		
	if s==50: # Semicircle
	
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,1, W/2+m, v, W, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Semicircle_1")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Semicircle_1")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image, 1, -W/2+m, v, W, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Semicircle_2")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Semicircle_2")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,1, m, W/2+v, W, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Semicircle_3")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Semicircle_3")

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, m ,v ,W ,W)
		pdb.gimp_image_select_rectangle(image,1, m, -W/2+v, W, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Semicircle_4")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Semicircle_4")
		pdb.gimp_selection_none(image)

	if s==51: # Spirals 0 Archimedian Spiral 0 Counterclockwise

		Type_de_spirale = 0     # 0 Archimedian Spiral  1 Logarithmic Spiral
		Sens_de_Rotation = 0    # 0 Counterclockwise    1 Clockwise
		X_origine_pourcent= W/2 + m
		Y_origine_pourcent = H/2 + v
		Nombre_de_spirales = 1
		Angle_de_depart = R_L
		Coefficient_Expansion=H_L/M_L
		Nombre_de_tours = M_L
		Echantillons_par_tour=2 + V_L
		pdb.Spirals_Spirales_samj_ssc(image, layer, X_origine_pourcent, Y_origine_pourcent, Nombre_de_spirales, Type_de_spirale, Sens_de_Rotation, Angle_de_depart, Coefficient_Expansion, Nombre_de_tours ,Echantillons_par_tour, run_mode=RUN_NONINTERACTIVE)

	if s==52: # Spirals 0 Archimedian Spiral 1 Clockwise

		Type_de_spirale = 0     # 0 Archimedian Spiral  1 Logarithmic Spiral
		Sens_de_Rotation = 1    # 0 Counterclockwise    1 Clockwise
		X_origine_pourcent= W/2 + m
		Y_origine_pourcent = H/2 + v
		Nombre_de_spirales = 1
		Angle_de_depart = R_L
		Coefficient_Expansion=H_L/M_L
		Nombre_de_tours = M_L
		Echantillons_par_tour=2 + V_L
		pdb.Spirals_Spirales_samj_ssc(image, layer, X_origine_pourcent, Y_origine_pourcent, Nombre_de_spirales, Type_de_spirale, Sens_de_Rotation, Angle_de_depart, Coefficient_Expansion, Nombre_de_tours ,Echantillons_par_tour, run_mode=RUN_NONINTERACTIVE)		
	
	if s==53: # Spirals 0 Archimedian Spiral 1 Clockwise

		Type_de_spirale = 1    # 0 Archimedian Spiral  1 Logarithmic Spiral
		Sens_de_Rotation = 0   # 0 Counterclockwise    1 Clockwise
		X_origine_pourcent= W/2 + m
		Y_origine_pourcent = H/2 + v
		Nombre_de_spirales = 1
		Angle_de_depart = R_L
		Coefficient_Expansion=H_L/M_L
		Nombre_de_tours = M_L
		Echantillons_par_tour=2 + V_L
		pdb.Spirals_Spirales_samj_ssc(image, layer, X_origine_pourcent, Y_origine_pourcent, Nombre_de_spirales, Type_de_spirale, Sens_de_Rotation, Angle_de_depart, Coefficient_Expansion, Nombre_de_tours ,Echantillons_par_tour, run_mode=RUN_NONINTERACTIVE)		
	
	if s==54: # Spirals 0 Archimedian Spiral 1 Clockwise

		Type_de_spirale = 1     # 0 Archimedian Spiral  1 Logarithmic Spiral
		Sens_de_Rotation = 1    # 0 Counterclockwise    1 Clockwise
		X_origine_pourcent= W/2 + m
		Y_origine_pourcent = H/2 + v
		Nombre_de_spirales = 1
		Angle_de_depart = R_L
		Coefficient_Expansion=H_L/M_L
		Nombre_de_tours = M_L
		Echantillons_par_tour=2 + V_L
		pdb.Spirals_Spirales_samj_ssc(image, layer, X_origine_pourcent, Y_origine_pourcent, Nombre_de_spirales, Type_de_spirale, Sens_de_Rotation, Angle_de_depart, Coefficient_Expansion, Nombre_de_tours ,Echantillons_par_tour, run_mode=RUN_NONINTERACTIVE)		
	
	if s==55: # Square
		new_vectors = pdb.gimp_vectors_new(image,"Square")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 30, 
		(m,v, m,v,m,v,  W+m,v, W+m,v ,W+m,v,
		W+m,W+v, W+m,W+v, W+m,W+v,  m,W+v, m,W+v, m,W+v,
		m,v, m,v,m,v,), 0)
		pdb.gimp_item_set_visible(new_vectors, True)

	if s==56: # STAR
	
		oradius=(H_L)*W/200  #Outer Radius=H
		iradius=(V_L)*W/200  #Inner Radius=V
		points=M_L           #Number=M->L
		rotation=R_L
		pdb.script_fu_star_path_ssc(image, layer, 'Star', W/2+m, H/2+v, oradius, iradius, points, rotation, run_mode=RUN_NONINTERACTIVE)	
		
	if s==57: # Star pentagons

		alfa = 36
		q = 1.0
		r = q*W/2
		x1 = W/2 + r*math.sin((alfa + 72)*math.pi/180)
		y1 = H/2 + r*math.cos((alfa + 72)*math.pi/180)
		x2 = W/2 + r*math.sin((alfa + 144)*math.pi/180)
		y2 = H/2 + r*math.cos((alfa + 144)*math.pi/180)
		x3 = W/2 + r*math.sin((alfa + 216)*math.pi/180)
		y3 = H/2 + r*math.cos((alfa + 216)*math.pi/180)
		x4 = W/2 + r*math.sin((alfa + 288)*math.pi/180)
		y4 = H/2 + r*math.cos((alfa + 288)*math.pi/180)
		x5 = W/2 + r*math.sin((alfa + 360)*math.pi/180)
		y5 = H/2 + r*math.cos((alfa + 360)*math.pi/180)
		alfa = 0  # 18 36 54
		q = 0.5*(3-math.sqrt(5))
		r = q*W/2
		xm1 = W/2 + r*math.sin((alfa + 72)*math.pi/180)
		ym1 = H/2 + r*math.cos((alfa + 72)*math.pi/180)
		xm2 = W/2 + r*math.sin((alfa + 144)*math.pi/180)
		ym2 = H/2 + r*math.cos((alfa + 144)*math.pi/180)
		xm3 = W/2 + r*math.sin((alfa + 216)*math.pi/180)
		ym3 = H/2 + r*math.cos((alfa + 216)*math.pi/180)
		xm4 = W/2 + r*math.sin((alfa + 288)*math.pi/180)
		ym4 = H/2 + r*math.cos((alfa + 288)*math.pi/180)
		xm5 = W/2 + r*math.sin((alfa + 360)*math.pi/180)
		ym5 = H/2 + r*math.cos((alfa + 360)*math.pi/180)
		
		new_vectors = pdb.gimp_vectors_new(image,"Star pentagons")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 120,(
		x1+m,y1+v,x1+m,y1+v,x1+m,y1+v,  xm1+m,ym1+v,xm1+m,ym1+v,xm1+m,ym1+v,
		xm1+m,ym1+v,xm1+m,ym1+v,xm1+m,ym1+v,  x5+m,y5+v,x5+m,y5+v,x5+m,y5+v,
		x5+m,y5+v,x5+m,y5+v,x5+m,y5+v,  xm5+m,ym5+v,xm5+m,ym5+v,xm5+m,ym5+v,
		xm5+m,ym5+v,xm5+m,ym5+v,xm5+m,ym5+v,  x4+m,y4+v,x4+m,y4+v,x4+m,y4+v,
		x4+m,y4+v,x4+m,y4+v,x4+m,y4+v,  xm4+m,ym4+v,xm4+m,ym4+v,xm4+m,ym4+v,
		xm4+m,ym4+v,xm4+m,ym4+v,xm4+m,ym4+v,  x3+m,y3+v,x3+m,y3+v,x3+m,y3+v,
		x3+m,y3+v,x3+m,y3+v,x3+m,y3+v,  xm3+m,ym3+v,xm3+m,ym3+v,xm3+m,ym3+v,
		xm3+m,ym3+v,xm3+m,ym3+v,xm3+m,ym3+v,  x2+m,y2+v,x2+m,y2+v,x2+m,y2+v,
		x2+m,y2+v,x2+m,y2+v,x2+m,y2+v,  xm2+m,ym2+v,xm2+m,ym2+v,xm2+m,ym2+v,
		xm2+m,ym2+v,xm2+m,ym2+v,xm2+m,ym2+v,  x1+m,y1+v,x1+m,y1+v,x1+m,y1+v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
	if s==58: # Triangle equilateral
		d = 3*W/4
		e = d/math.sqrt(3)

		new_vectors = pdb.gimp_vectors_new(image,"Triangle equilateral")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 36, 
		(W/2+m,v, W/2+m,v, W/2+m,v,  W/2 + e+m,d+v,  W/2 + e+m,d+v, W/2 + e+m,d+v,
		W/2 +e+m, d+v, W/2 +e+m, d+v, W/2 +e+m, d+v,    W/2 -e+m, d+v, W/2 -e+m, d+v, W/2- e+m, d+v,
		W/2-e+m, d+v, W/2-e+m, d+v, W/2-e+m, d+v,   W/2+m, v, W/2+m, v, W/2+m, v ), 0)
		pdb.gimp_item_set_visible(new_vectors, True)

	if s==59: # Triangle isosceles
		new_vectors = pdb.gimp_vectors_new(image,"Triangle isosceles")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 24, 
		(W1/2,k, W1/2,k, W1/2,k,  W1-j,H1-k, W1-j,H1-k ,W1-j,H1-k,  j, H1-k, j,H1-k, j, H1-k,  W1/2,k, W1/2,k, W1/2,k), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
	if s==60: # "Triangle Reuleaux"
	
		R = W - math.sqrt(3)*W/2
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, -W/2+m ,-W+v ,2*W ,2*W)
		pdb.gimp_image_select_ellipse(image, 3, -W+m ,-R+v ,2*W ,2*W)
		pdb.gimp_image_select_ellipse(image, 3, m ,-R+v ,2*W ,2*W)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Triangle Reuleaux")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Triangle Reuleaux")
		pdb.gimp_selection_none(image)
		
	if s==61: # Trefoil

		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, 3*W/16+m ,v ,5*W/8 ,5*W/8)
		pdb.gimp_image_select_ellipse(image, 0, m ,3*W/8+v ,5*W/8 ,5*W/8)
		pdb.gimp_image_select_ellipse(image, 0, 3*W/8+m ,3*W/8+v ,5*W/8 ,5*W/8)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Trefoil")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Trefoil")
		pdb.gimp_selection_none(image)
		
	if s==62: # Sine Wave
		if sel==False:
			amplitude=(H_L)*H1/200  	 #=H
			wavelength=W1/(M_L)      
			cycles=M_L            	 	 #=M->L
			pdb.script_fu_sine_wave_path_ssc(image, layer, 'Sine Wave', j, H/2+v, amplitude, wavelength, cycles, run_mode=RUN_NONINTERACTIVE)
		if sel==True:
			amplitude=(H_L)*(y2-y1)/200  	 #=H
			wavelength=(x2-x1)/(M_L)     
			cycles=M_L            	 		 #=M->L
			pdb.script_fu_sine_wave_path_ssc(image, layer, 'Sine Wave',j, H/2+v, amplitude, wavelength, cycles, run_mode=RUN_NONINTERACTIVE)		

	if s==63: # Square Wave
		convergence=0
		shearing=R_L/360
		displacement=0
		if sel==False:
			amplitude=(H_L)*H1/100  	 #=H
			wavelength=W1/(M_L)      
			cycles=M_L            	 	 #=M->L
		if sel==True:
			amplitude=(H_L)*(y2-y1)/100  	 #=H
			wavelength=(x2-x1)/(M_L)     
			cycles=M_L            	 		 #=M->L
		pdb.script_fu_square_wave_path_ssc(image, layer, 'Square Wave',j, H/2+v, amplitude, wavelength, cycles, convergence, shearing,displacement, run_mode=RUN_NONINTERACTIVE)			

	if s==64: # Triangle Wave
		shearing=0
		rounding=R_L/1800
		if sel==False:
			amplitude=(H_L)*H1/100  	 #=H
			wavelength=W1/(M_L)      
			cycles=M_L            	 	 #=M->L
			#pdb.script_fu_sine_wave_path_ssc(image, layer, 'Sine Wave', j, H/2+v, amplitude, wavelength, cycles, run_mode=RUN_NONINTERACTIVE)
		if sel==True:
			amplitude=(H_L)*(y2-y1)/100  	 #=H
			wavelength=(x2-x1)/(M_L)     
			cycles=M_L            	 		 #=M->L
		pdb.script_fu_triangle_wave_path_ssc(image, layer, 'Triangle Wave',j, H/2+v, amplitude, wavelength, cycles, shearing, rounding, run_mode=RUN_NONINTERACTIVE)

	if s==65: # Wilber

		new_vectors = pdb.gimp_vectors_new(image,"Wilber-01-head outline")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 156,(
		25*W/100+m, 30*W/100+v, 30*W/100+m, 40*W/100+v, 30*W/100+m, 40*W/100+v,     18*W/100+m, 11*W/100+v, 24*W/100+m, 18*W/100+v, 27*W/100+m, 21*W/100+v,
		24*W/100+m, 18*W/100+v, 24*W/100+m, 18*W/100+v, 27*W/100+m, 21*W/100+v,     35*W/100+m, 31*W/100+v, 44*W/100+m, 35*W/100+v, 44*W/100+m, 35*W/100+v,
		44*W/100+m, 35*W/100+v, 44*W/100+m, 35*W/100+v, 54*W/100+m, 39*W/100+v,     60*W/100+m, 39*W/100+v, 69*W/100+m, 36*W/100+v, 69*W/100+m, 36*W/100+v,
		69*W/100+m, 36*W/100+v, 69*W/100+m, 36*W/100+v, 78*W/100+m, 33*W/100+v,     90*W/100+m, 25*W/100+v, 95*W/100+m, 13*W/100+v, 95*W/100+m, 13*W/100+v,
		95*W/100+m, 13*W/100+v, 95*W/100+m, 13*W/100+v, 96*W/100+m, 10*W/100+v,     98*W/100+m, 18*W/100+v, 97*W/100+m, 29*W/100+v, 97*W/100+m, 29*W/100+v,
		97*W/100+m, 29*W/100+v, 97*W/100+m, 29*W/100+v, 96*W/100+m, 40*W/100+v,     90*W/100+m, 46*W/100+v, 81*W/100+m, 52*W/100+v, 81*W/100+m, 52*W/100+v,
		81*W/100+m, 52*W/100+v, 81*W/100+m, 52*W/100+v, 79*W/100+m, 65*W/100+v,     69*W/100+m, 75*W/100+v,  54*W/100+m, 76*W/100+v,  54*W/100+m, 76*W/100+v,
		54*W/100+m, 76*W/100+v, 54*W/100+m, 76*W/100+v, 60*W/100+m, 73*W/100+v,     62*W/100+m, 70*W/100+v, 63*W/100+m, 67*W/100+v,  63*W/100+m, 67*W/100+v,
		63*W/100+m, 67*W/100+v, 63*W/100+m, 67*W/100+v,  64*W/100+m, 63*W/100+v,    63*W/100+m, 62*W/100+v,  61*W/100+m, 60*W/100+v,  61*W/100+m, 60*W/100+v,
		61*W/100+m, 60*W/100+v,  61*W/100+m, 60*W/100+v,  58*W/100+m, 69*W/100+v,   37*W/100+m, 74*W/100+v,  27*W/100+m, 72*W/100+v, 27*W/100+m, 72*W/100+v,
		27*W/100+m, 72*W/100+v, 27*W/100+m, 72*W/100+v, 24*W/100+m, 72*W/100+v,     18*W/100+m, 70*W/100+v, 18*W/100+m, 70*W/100+v, 18*W/100+m, 70*W/100+v,
		18*W/100+m, 70*W/100+v, 18*W/100+m, 70*W/100+v, 26*W/100+m, 66*W/100+v,     26.5*W/100+m, 57*W/100+v, 21*W/100+m, 49*W/100+v, 21*W/100+m, 49*W/100+v,
		21*W/100+m, 49*W/100+v, 21*W/100+m, 49*W/100+v, 21*W/100+m, 49*W/100+v,     23*W/100+m, 47*W/100+v, 23*W/100+m, 47*W/100+v, 23*W/100+m, 47*W/100+v		), 0)
		pdb.gimp_item_set_visible(new_vectors, True)

		new_vectors = pdb.gimp_vectors_new(image,"Wilber-02-left eye")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(new_vectors ,30*W/100+m ,47*W/100+v ,7*W/100 ,7*W/100, 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
		new_vectors = pdb.gimp_vectors_new(image,"Wilber-03-left eye pupil")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(new_vectors ,31*W/100+m ,49*W/100+v ,4*W/100 ,4*W/100, 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
		new_vectors = pdb.gimp_vectors_new(image,"Wilber-04-left eye pupil highlight")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(new_vectors ,30*W/100+m ,48*W/100+v ,2*W/100 ,2*W/100, 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		
		new_vectors = pdb.gimp_vectors_new(image,"Wilber-05-mouth")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 48,(
		54*W/100+m, 76*W/100+v, 54*W/100+m, 76*W/100+v, 60*W/100+m, 73*W/100+v,     62*W/100+m, 70*W/100+v, 63*W/100+m, 67*W/100+v,  63*W/100+m, 67*W/100+v,
		63*W/100+m, 67*W/100+v, 63*W/100+m, 67*W/100+v,  64*W/100+m, 63*W/100+v,    63*W/100+m, 62*W/100+v,  61*W/100+m, 60*W/100+v,  61*W/100+m, 60*W/100+v,
		61*W/100+m, 60*W/100+v,  61*W/100+m, 60*W/100+v,  58*W/100+m, 69*W/100+v,   38*W/100+m, 74*W/100+v, 27*W/100+m, 72*W/100+v, 27*W/100+m, 72*W/100+v,
		27*W/100+m, 72*W/100+v,  27*W/100+m, 72*W/100+v, 34*W/100+m, 75*W/100+v,    46*W/100+m, 78*W/100+v, 54*W/100+m, 76*W/100+v, 60*W/100+m, 73*W/100+v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)

		new_vectors = pdb.gimp_vectors_new(image,"Wilber-06-nose")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 48,(
		4*W/100+m, 46*W/100+v, 10*W/100+m, 44*W/100+v, 16*W/100+m, 43*W/100+v,       21*W/100+m, 47*W/100+v, 24*W/100+m, 55*W/100+v, 24*W/100+m, 55*W/100+v,
		24*W/100+m, 55*W/100+v, 24*W/100+m, 55*W/100+v, 26*W/100+m, 62*W/100+v,   	 23*W/100+m, 68*W/100+v, 18*W/100+m, 70*W/100+v, 18*W/100+m, 70*W/100+v,
		18*W/100+m, 70*W/100+v,18*W/100+m, 70*W/100+v, 12*W/100+m, 71*W/100+v,       6*W/100+m,67*W/100+v, 4*W/100+m,60*W/100+v,  4*W/100+m, 60*W/100+v,
		4*W/100+m,60*W/100+v,4*W/100+m,60*W/100+v, W/100+m, 52*W/100+v,   4*W/100+m, 46*W/100+v,  10*W/100+m, 44*W/100+v,  17*W/100+m, 43*W/100+v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)		
		
		new_vectors = pdb.gimp_vectors_new(image,"Wilber-07-nose highlight")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(new_vectors ,11*W/100+m ,51*W/100+v ,4*W/100 ,5*W/100, 15)
		pdb.gimp_item_set_visible(new_vectors, True)		

		new_vectors = pdb.gimp_vectors_new(image,"Wilber-08-right eye")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(new_vectors ,47*W/100+m ,48*W/100+v ,10*W/100 ,10*W/100, 0)
		pdb.gimp_item_set_visible(new_vectors, True)

		new_vectors = pdb.gimp_vectors_new(image,"Wilber-09-right eye pupil")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(new_vectors ,49*W/100+m ,51*W/100+v ,5*W/100 ,5*W/100, 0)
		pdb.gimp_item_set_visible(new_vectors, True)

		new_vectors = pdb.gimp_vectors_new(image,"Wilber-10-right eye pupil highlight")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(new_vectors ,48*W/100+m ,50*W/100+v ,2*W/100 ,2*W/100, 0)
		pdb.gimp_item_set_visible(new_vectors, True)

		new_vectors = pdb.gimp_vectors_new(image,"Wilber-11-shadow")
		pdb.gimp_image_insert_vectors(image, new_vectors, None, -1)
		pdb.gimp_vectors_stroke_new_from_points(new_vectors, 0, 60,(
		95*W/100+m, 13*W/100+v, 95*W/100+m, 13*W/100+v, 96*W/100+m, 10*W/100+v,   98*W/100+m, 18*W/100+v, 97*W/100+m, 29*W/100+v, 97*W/100+m, 29*W/100+v,
		97*W/100+m, 29*W/100+v, 97*W/100+m, 29*W/100+v, 96*W/100+m, 40*W/100+v,   90*W/100+m, 46*W/100+v, 81*W/100+m, 52*W/100+v, 81*W/100+m, 52*W/100+v,
		81*W/100+m, 52*W/100+v, 81*W/100+m, 52*W/100+v, 79*W/100+m, 65*W/100+v,   69*W/100+m, 75*W/100+v,  54*W/100+m, 76*W/100+v,  54*W/100+m, 76*W/100+v,
		54*W/100+m, 76*W/100+v, 54*W/100+m, 76*W/100+v, 67*W/100+m, 76*W/100+v,   79*W/100+m, 63*W/100+v,77*W/100+m, 49*W/100+v,77*W/100+m, 49*W/100+v,
		77*W/100+m, 49*W/100+v,77*W/100+m, 49*W/100+v,95*W/100+m, 41*W/100+v,     95*W/100+m, 13*W/100+v, 95*W/100+m, 13*W/100+v, 96*W/100+m, 10*W/100+v), 0)
		pdb.gimp_item_set_visible(new_vectors, True)
		pdb.gimp_selection_none(image)

	if s==66: # Yin-Yang
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_rectangle(image,0, m, v, W/2, H)
		pdb.gimp_image_select_ellipse(image, 3 ,m, v, W, H)
		pdb.gimp_image_select_ellipse(image, 0, W/2-W/4+m, v, W/2, H/2)
		pdb.gimp_image_select_ellipse(image, 1, W/2-W/4+m, H/2+v, W/2, H/2)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Yang")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Yang")
		pdb.gimp_selection_invert(image)
		pdb.gimp_image_select_ellipse(image,3, m, v, W, H)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		pdb.gimp_selection_none(image)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Yin")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Yin")
		pdb.gimp_image_select_ellipse(image, 0, 7*W/16+m, 3*W/16+v, W/8, H/8)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Eye Yang")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Oko Yang")
		pdb.gimp_selection_none(image)
		pdb.gimp_image_select_ellipse(image, 0, 7*W/16+m, 11*W/16+v, W/8, H/8)
		pdb.plug_in_sel2path(image, layer, run_mode=RUN_NONINTERACTIVE)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.visible = True
		active_vectors.name = active_vectors.name.replace("Selection", "Eye Yin")
		active_vectors.name = active_vectors.name.replace("Zaznaczenie", "Oko Yin")
		pdb.gimp_selection_none(image)

	if R_1:
		pdb.script_fu_resize_image_to_layers_by_the_edge(image, layer, 1, 1, 1, 1, 1, run_mode=RUN_INTERACTIVE)

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ by Tin Tran

	# See which vectors are added as new vectors, it's new if it's not in our before_vectors list, we'll add it to our new_vectors list
	pdb.gimp_selection_none(image)
	new_vectors = []
	for vectors in image.vectors:
		if vectors not in before_vectors:
			new_vectors.append(vectors)

	#Loop through each new added vectors
	if fill==1:
		W = pdb.gimp_image_width(image)  #szer
		H = pdb.gimp_image_height(image)
		for vectors in new_vectors:
	
		#Select the vectors, this is like front end's "Path To Selection"
			pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,vectors)
			fill_layer = gimp.Layer(image, "Fill Pattern", W, H, RGBA_IMAGE, 100, NORMAL_MODE)
			image.add_layer(fill_layer, -1)
		#Set context pattern and fill selection with pattern fill

			pdb.gimp_context_set_pattern(fillpattern)
			if gimp.version >= (2,9,0):       
				pdb.gimp_edit_fill(fill_layer,FILL_PATTERN)	      	    #GIMP 2.9
			else:
				pdb.gimp_edit_fill(fill_layer,PATTERN_FILL)             #GIMP 2.8

		# Now we select None so that Stroke path call later will stroke the whole path and not just selected area
			pdb.gimp_selection_none(image)
	    # Stroke path using SVG method.
	if fill==2:
		W = pdb.gimp_image_width(image)  #szer
		H = pdb.gimp_image_height(image)

		for vectors in new_vectors:
	
		#Select the vectors, this is like front end's "Path To Selection"
			pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,vectors)
			brush_layer = gimp.Layer(image, "Fill Brush", W, H, RGBA_IMAGE, 100, NORMAL_MODE)
			image.add_layer(brush_layer, -1)
			selection = pdb.gimp_selection_bounds(image);
			
			x1 = selection[1];
			y1 = selection[2];
			x2 = selection[3];
			y2 = selection[4];
			width = W1
			height = H1
			horizontalspacing = H_B
			verticalspacing = V_B
			context_brush_size = pdb.gimp_context_get_brush_size()
			bwidth = context_brush_size * (horizontalspacing/100.0)
			bheight = context_brush_size * (verticalspacing/100.0)
			gimp.progress_init("Filling Selection with Active Brush...")
			
			for x in range(0,int(W1/bwidth)+1):
				gimp.progress_update(x*1.0/(int(W1/bwidth)+1))
				for y in range(0,int(H1/bheight)+1):
					ix = x1 + (x * bwidth)
					iy = y1 + (y * bheight)
					if pdb.gimp_selection_value(image,ix,iy) > 0:
						pdb.gimp_paintbrush(brush_layer,0,2,(ix,iy),PAINT_CONSTANT,0)
						
	if fill==3:
		W = pdb.gimp_image_width(image)  #szer
		H = pdb.gimp_image_height(image)
		for vectors in new_vectors:
			pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,vectors)
			fill_layer = gimp.Layer(image, "Fill Foreground", W, H, RGBA_IMAGE, 100, NORMAL_MODE)
			image.add_layer(fill_layer, -1)

			pdb.gimp_context_set_pattern(fillpattern)
			if gimp.version >= (2,9,0):       
				pdb.gimp_edit_fill(fill_layer,FILL_FOREGROUND)	       #GIMP 2.9
			else:
				pdb.gimp_edit_fill(fill_layer,FOREGROUND_FILL)         #GIMP 2.8

			pdb.gimp_selection_none(image)

	if fill==4:
		W = pdb.gimp_image_width(image)
		H = pdb.gimp_image_height(image)
		for vectors in new_vectors:
			pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,vectors)
			fill_layer = gimp.Layer(image, "Fill Background", W, H, RGBA_IMAGE, 100, NORMAL_MODE)
			image.add_layer(fill_layer, -1)

			pdb.gimp_context_set_pattern(fillpattern)
			if gimp.version >= (2,9,0):       
				pdb.gimp_edit_fill(fill_layer,FILL_BACKGROUND)	       #GIMP 2.9
			else:
				pdb.gimp_edit_fill(fill_layer,BACKGROUND_FILL)         #GIMP 2.8

			pdb.gimp_selection_none(image)
			
	if stroke==1:
		for vectors in new_vectors:
			pdb.gimp_selection_none(image)
			W = pdb.gimp_image_width(image)
			H = pdb.gimp_image_height(image)
			stroke_b = gimp.Layer(image, "Stroke brush", W, H, RGBA_IMAGE, 100, NORMAL_MODE)
			image.add_layer(stroke_b, -1)
			pdb.gimp_edit_stroke_vectors(stroke_b ,vectors)
#=====================================================================================================================			
	if stroke==2:
		def color_to_hex(color):
			whole_color = (color[0],color[1],color[2]);
			return '#%02x%02x%02x' % whole_color
		def vector_to_line_stroke(image, vector, layer, color="#000000", width=1, capstyle="butt", joinstyle="miter", miterlimit=10, shaperendering="auto"):
			import re, tempfile
			import os, string, sys
			newelements = {
					'stroke': color,
					'stroke-width': width,
					'stroke-linecap': capstyle,
					'stroke-linejoin': joinstyle,
					'stroke-miterlimit': miterlimit,
					'shape-rendering': shaperendering,
					}
			svg = pdb.gimp_vectors_export_to_string(image, vector)
			#fix width and height to be resolution (px/inch)-independent
			svg = re.sub(r'(<svg\s[^>]*\swidth\s*=\s*)\S*"', r'\1"%dpx"' % image.width, svg, flags=re.DOTALL)
			svg = re.sub(r'(<svg\s[^>]*\sheight\s*=\s*)\S*"', r'\1"%dpx"' % image.height, svg, flags=re.DOTALL)
			svg = re.sub(r'(<path\s[^>]*)\sstroke\s*=\s*"black"', r'\1', svg, flags=re.DOTALL)
			svg = re.sub(r'(<path\s[^>]*)\sstroke-width\s*=\s*"1"', r'\1', svg, flags=re.DOTALL)
			svg = re.sub(r'(<path\s)', r'\1' + ''.join([r'%s="%s" ' % i for i in newelements.items()]), svg, flags=re.DOTALL)
			tmpfile = tempfile.TemporaryFile(suffix=".svg")
			
			#gets just the file name to use, added by Tin Tran, tempFile.TemporaryFile above gave write permission error
			filename = os.path.basename(tmpfile.name)
			svgfile =  "~\\.gimp-2.8\\tmp\\" + filename
			svgfile = os.path.expanduser(svgfile)
			tmpfile = open(svgfile,"w")
			#end of addition by Tin Tran
			
			tmpfile.write(svg)
			tmpfile.flush();
			newlayer = pdb.gimp_file_load_layer(image, tmpfile.name)
			tmpfile.close()
			image.add_layer(newlayer) #needs to be added to the image to be able to copy from
			copyname = pdb.gimp_edit_named_copy(newlayer, "stroke")
			image.remove_layer(newlayer)
			floating_sel = pdb.gimp_edit_named_paste(layer, copyname, True)
			pdb.gimp_floating_sel_anchor(floating_sel)
			
			#added by Tin Tran, remove the file when we're done
			os.remove(svgfile)
		for vectors in new_vectors:
			pdb.gimp_selection_none(image)
			W = pdb.gimp_image_width(image)
			H = pdb.gimp_image_height(image)
			stroke = gimp.Layer(image, "Stroke SVG", W, H, RGBA_IMAGE, 100, NORMAL_MODE)
			image.add_layer(stroke, -1)
			vector_to_line_stroke(image,vectors,stroke,color_to_hex(strokecolor),strokewidth,"square","bevel",10,"auto")

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	if sel == True:	
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,temp)
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

register(
        "plugin_simple_shapes_centered",
        "Simple paths shapes centered in the image or selection Ver-1.1.\n==========================================\nThanks to Tin Tran, Ofnuts, TheUncle2k, A.Wade, J.Stipe, Samj, dd.",
        "Simple path shapes centered in the image v.1.1",
        "MrQ",
        "GPL",
        "2016",
        "<Image>/Filters/Path/Simple shapes centered...",
        "RGB*, GRAY*",
        [
		#(PF_IMAGE, "image", "Input image", None),
		#(PF_VECTORS, "drawable", "Input drawable", None),
		(PF_BOOL, "sel", ("In selection? "), False),
		(PF_OPTION, "s", "Shape:", 65, 
		[
		 "Arrow [Paths:4]",																#0
		 "Axis of symmetry",															#1
		 "Binoculars [Paths:1+4]\nM->L = [Outer radius - Inner radius](pix)",			#2
		 "Circle",																		#3
		 "Crescent",																	#4
		 "Cross",																		#5
		 "Diagonals",																	#6
		 "Diamond",																		#7
		 "Dodecagon",																	#8
		 "Ellipse",																		#9
		 "Flower Out [Paths:6]", 														#10
		 "Flower In [Paths:6]", 														#11
		 "GEAR (c) J.Stipe\nM->L = Number of teeth (min.3)    R = Rotation\nH = Outer Radius(%)           V = Inner Radius(%)",	#12
		 "Grid rectangular (c) Ofnuts \nH = Horizontal lines\nV = Vertical lines",		#13
		 "Grid Rectangle type Rectangular\nM->L = Gap between shapes R = Outer border\nH = X shapes V = Y shapes",	#14
		 "Grid Rectangle type Hexagonal\nM->L = Gap between shapes R = Outer border\nH = X shapes V = Y shapes",	#15
		 "Grid Ellipse type Rectangular\nM->L = Gap between shapes R = Outer border\nH = X shapes V = Y shapes",	#16
		 "Grid Ellipse type Hexagonal\nM->L = Gap between shapes R = Outer border\nH = X shapes V = Y shapes",		#17
		 "Grid Hexagon type Rectangular\nM->L = Gap between shapes R = Outer border\nH = X shapes V = Y shapes",	#18
		 "Grid Hexagon type Hexagonal\nM->L = Gap between shapes R = Outer border\nH = X shapes V = Y shapes",		#19
		 "Grid Diamond type Rectangular\nM->L = Gap between shapes R = Outer border\nH = X shapes V = Y shapes",	#20
		 "Grid Diamond type Hexagonal\nM->L = Gap between shapes R = Outer border\nH = X shapes V = Y shapes",		#21
		 "Grid Octagon type Rectangular\nM->L = Gap between shapes R = Outer border\nH = X shapes V = Y shapes",	#22
		 "Grid Octagon type Hexagonal\nM->L = Gap between shapes R = Outer border\nH = X shapes V = Y shapes",		#23
		 "Hexagon",																		#24
		 "Ice cream Waffle", 															#25
		 "Mobius band [Paths:3+1]",														#26
		 "Octagon",																		#27
		 "Pentagon",																	#28
		 "Pentagram",																	#29
		 "Petals [Paths:4]",															#30
		 "Pie 1/2 [Paths:4]",															#31
		 "Pie 1/4 [Paths:4+4]",        													#32
		 "Pie 1/8 [Paths:8+8]",        													#33
		 "Playing cards: Clubs",														#34
		 "Playing cards: Diamonds",														#35
		 "Playing cards: Hearts",														#36
		 "Playing cards: Spades",														#37
		 "POLYGON (c) J.Stipe\nM->L = Sides (min.3)  R = Rotation",						#38
		 "Protractor in circle (c) Tin Tran \nM->L = Mark Length (pix)",  				#39
		 "Protractor out circle (c) Tin Tran \nM->L = Mark Length (pix)", 				#40
		 "Quadrant [Paths:4]",															#41
		 "Quatrefoil [Paths:1+4]",														#42
		 "Rectangle",																	#43
		 "Recycle (two arrows) [Paths:2]",												#44
		 "Recycle (three arrows) [Paths:3+3]",   										#45
		 "Roses compass 4-point [Paths:2]",      										#46
		 "Ruler (c) Tin Tran \nM->L = Mark Length (pix)",								#47
		 "RUNDED RECTANGLE (c) J.Stipe\nH = radius-x (%)  V = radius-y (%)",			#48
		 "RUNDED RECTANGLE Advanced (c) J.Stipe\nRadius(%): M-L = Top-Left  H = Top-Right\n            V = Bottom-Left R = Bottom-Right",  #49
		 "Semicircle [Paths:4]", 														#50
		 "SPIRALS Archimedian Counterclockwise (c) Samj\nM_L = Number of turns   H = Coefficient Expansion\nV = Samples Per Revolution    R = Departure Angle", #51
		 "SPIRALS Archimedian Clockwise (c) Samj\nM_L = Number of turns   H = Coefficient Expansion\nV = Samples Per Revolution    R = Departure Angle", 		#52
		 "SPIRALS Logarithmic Counterclockwise (c) Samj\nM_L = Number of turns   H = Coefficient Expansion\nV = Samples Per Revolution    R = Departure Angle", #53
		 "SPIRALS Logarithmic Clockwise (c) Samj\nM_L = Number of turns   H = Coefficient Expansion\nV = Samples Per Revolution    R = Departure Angle", 		#54
		 "Square",																		#55
		 "STAR (c) J.Stipe\nM->L = Number of points (min.3)    R = Rotation\nH = Outer Radius(%)           V = Inner Radius(%)",#56
		 "Star pentagons",																#57
		 "Triangle equilateral",														#58
		 "Triangle isosceles", 															#59
		 "Triangle Reuleaux (curvilinear)", 											#60
		 "Trefoil",																		#61
		 "WAVE SINE (c) J.Stipe\nM->L = cycles  H = amplitude (%)",						#62
		 "WAVE SQUARE (c) J.Stipe\nM->L = cycles  H = amplitude (%) R = inclination",	#63
		 "WAVE TRIANGLE (c) J.Stipe\nM->L = cycles  H = amplitude (%) R = rounding",	#64
		 "Wilber [Paths:11]",															#65
		 "Yin-Yang [Paths:4]",															#66
		]),
		(PF_BOOL, "resize_1", ("Resize layer (run  interactive\nChanging Image Border.scm)? "), False),
		(PF_SPINNER, "M_L", "'M->L' Mark Length (pix)", 10, (-100, 1000, 1)),
		(PF_SPINNER, "H_L", "'H' Horizontal", 10, (-100, 100, 1)),
		(PF_SPINNER, "V_L", "'V' Vertical", 10, (-100, 100, 1)),
		(PF_SPINNER, "R_L", "'R' Rotation (angle)", 0, (-360, 360, 1)),
		(PF_OPTION, "fill", "FILL (path to selection)?:", 0, ["Don't fill", "With Pattern", "With Active Brush [%]", "With Foreground", "With Background"]),
		(PF_PATTERN, "fillpattern", "Fill Pattern:", 0),
		(PF_SPINNER, "H_B", "Brush Horizontal Spacing (%):", 100, (1, 500, 1)),
		(PF_SPINNER, "V_B", "Brush Vertical Spacing (%):", 100, (1, 500, 1)),
		(PF_OPTION, "stroke", "STROKE?:", 0, ["Don't stroke", "With active brush and foreground color", "Stroke Path (using SVG method) ? "]),		
		(PF_COLOR, "strokecolor",  "Stroke Path - Color:",  (0,0,0)),
		(PF_SPINNER, "strokewidth", "Stroke Path - Line Width (pix):", 6, (1, 200, 1))
		],
		[],
		plugin_simple_shapes_centered)

main()