#!/usr/bin/env python

# Sugar bag Rel 1
# Created by Tin Tran (http://bakon.ca/gimplearn)
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.

#import math
#import string
#import Image
from gimpfu import *
#from array import array

def python_sugar_bag(image, layer, shadow, border, color, inner, borderwidth):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	pdb.gimp_selection_none(image)
	
	pdb.gimp_selection_none(image)
	pdb.gimp_layer_resize_to_image_size(layer)
	
	#create shadow layer and shift it
	shadow_layer = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,shadow_layer,None,1)
	pdb.gimp_context_set_foreground(shadow)
	pdb.gimp_layer_set_lock_alpha(shadow_layer,True)
	pdb.gimp_edit_fill(shadow_layer,FOREGROUND_FILL)
	pdb.gimp_layer_set_lock_alpha(shadow_layer,False)
	pdb.gimp_layer_set_offsets(shadow_layer,borderwidth*3.5,borderwidth*3.5)
	pdb.gimp_item_set_name(shadow_layer,"Shadow")
	
	#grow selection and cut away
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_selection_grow(image,borderwidth*1.5)
	pdb.gimp_edit_clear(shadow_layer)
	
	#now fill original layer with border color
	pdb.gimp_selection_none(image)
	pdb.gimp_context_set_foreground(border)
	pdb.gimp_layer_set_lock_alpha(layer,True)
	pdb.gimp_edit_fill(layer,FOREGROUND_FILL)
	pdb.gimp_layer_set_lock_alpha(layer,False)
	pdb.gimp_item_set_name(shadow_layer,"Border")
	
	color_layer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'color',100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,color_layer,None,0)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_selection_shrink(image,borderwidth)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if non_empty:
		pdb.gimp_context_set_foreground(color)
		pdb.gimp_edit_fill(color_layer,FOREGROUND_FILL)
	inner_layer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'inner',100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,inner_layer,None,0)
	pdb.gimp_selection_shrink(image,borderwidth*2.5)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if non_empty:
		pdb.gimp_context_set_foreground(inner)
		pdb.gimp_edit_fill(inner_layer,FOREGROUND_FILL)
	
	pdb.gimp_selection_none(image)

	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_sugar_bag",                           
	"Creates Sugar Bag effect on text/shape",
	"Creates Sugar Bag effect on text/shape",
	"Tin Tran",
	"Tin Tran",
	"May 2017",
	"<Image>/Python-Fu/Sugar Bag...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_COLOR, "shadow",  "Shadow color:",  (255,255,255)),
	(PF_COLOR, "border",  "Border color:",  (136,46,35)),
	(PF_COLOR, "color",  "Main color:",  (169,62,70)),
	(PF_COLOR, "inner",  "Inner color:",  (110,38,42)),
	(PF_SPINNER, "borderwidth", "Border Width(pixels):", 2.0, (0.1, 30.0, 0.1)),
	#(PF_SPINNER, "shift_pixels", "Shift Pixels:", 5, (1, 100, 1)),
	#(PF_COLOR, "oldcolor",  "Replace Color:",  (43,198,255)),
	#(PF_COLOR, "newcolor",  "With New Color:",  (255,198,0)),
	#(PF_SPINNER, "x1", "x1:", 0, (0, 100, 0.5)),
	#(PF_SPINNER, "y1", "y1:", 100, (0, 100, 0.5)),
	#(PF_SPINNER, "x2", "x2:", 100, (0, 100, 0.5)),
	#(PF_SPINNER, "y2", "y2:", 0, (0, 100, 0.5)),
	#(PF_COLOR, "black",  "Black point color",  (0,0,0) PF_FILE, "infilename", "Temp Filepath", "/Default/Path"),
	#(PF_COLOR, "white",  "White point color",  (255,255,255) ),
	#(PF_COLOR, "gray",  "Gray point color",  (128,128,128) )
	#(PF_FILE, "infilename", "Temp Filepath", "/Default/Path")
	#(PF_DIRNAME, "source_directory", "Source Directory", "") for some reason, on my computer when i(Tin) use PF_DIRNAME the pythonw.exe would crash
	],
	[],
	python_sugar_bag)

main()
