#!/usr/bin/env python
# coding=utf-8
# -*- encoding: utf-8 -*-
# abs_pixel.py rel 3
# Created by Tin Tran
# Comments directed to http://gimplearn.net

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Pencil instead of brush for speed.
# Rel 3: Trying using path and stroke path instead of paintbrush for speed to get around delay bug in 2.10
# ======================== AUTOCOMPLETE HELP STARTS =============================
# ========================= while in atom editor can safely be removed
# try:
# 	#Try to set a variable to pdb, if it works then we're inside GIMP execution
# 	test = pdb
# except:
# 	#If it fails we're on our own so we'll use this for autocomplete
# 	try:
# 		import mypdb as pdb #import this file so that we bet pdb.* autocomplete
# 		from myenums import * #import this file so that we have ENUMS autocomplete
# 	except:
# 		pass
# ======================= AUTOCOMPLETE HELP ENDS ================================
import random
from gimpfu import *
import math
import re, tempfile
import os, string, sys
#Copied this from ofn-text-along-path.py returns angle in radians
#main method

#===== methods to be used when we want to stroke a path. Tin Tran did not write vector_to_line_stroke, I found it on the net somewhere 
# and made minor changes to it.

tmpfile = tempfile.TemporaryFile(suffix=".svg")
filename = os.path.basename(tmpfile.name)
def color_to_hex(color):
	whole_color = (color[0],color[1],color[2]);
	return '#%02x%02x%02x' % whole_color
def vector_to_line_stroke(image, vector, layer, color="#000000", width=1, capstyle="butt", joinstyle="miter", miterlimit=10, shaperendering="auto"):
    newelements = {
            'stroke': color,
            'stroke-width': width,
            'stroke-linecap': capstyle,
            'stroke-linejoin': joinstyle,
            'stroke-miterlimit': miterlimit,
			'shape-rendering': shaperendering,
            }
    svg = pdb.gimp_vectors_export_to_string(image, vector)
    #fix width and height to be resolution (px/inch)-independent
    svg = re.sub(r'(<svg\s[^>]*\swidth\s*=\s*)\S*"', r'\1"%dpx"' % image.width, svg, flags=re.DOTALL)
    svg = re.sub(r'(<svg\s[^>]*\sheight\s*=\s*)\S*"', r'\1"%dpx"' % image.height, svg, flags=re.DOTALL)
    svg = re.sub(r'(<path\s[^>]*)\sstroke\s*=\s*"black"', r'\1', svg, flags=re.DOTALL)
    svg = re.sub(r'(<path\s[^>]*)\sstroke-width\s*=\s*"1"', r'\1', svg, flags=re.DOTALL)
    svg = re.sub(r'(<path\s)', r'\1' + ''.join([r'%s="%s" ' % i for i in newelements.items()]), svg, flags=re.DOTALL)
    
	
    #gets just the file name to use, added by Tin Tran, tempFile.TemporaryFile above gave write permission error
    
    svgfile =  "~\\" + filename
    svgfile = os.path.expanduser(svgfile)
    tmpfile = open(svgfile,"w")
	#end of addition by Tin Tran
	
    tmpfile.write(svg)
    tmpfile.flush();
    newlayer = pdb.gimp_file_load_layer(image, tmpfile.name)
    tmpfile.close()
    image.add_layer(newlayer) #needs to be added to the image to be able to copy from
    copyname = pdb.gimp_edit_named_copy(newlayer, "stroke")
    image.remove_layer(newlayer)
    floating_sel = pdb.gimp_edit_named_paste(layer, copyname, True)
    pdb.gimp_floating_sel_anchor(floating_sel)
	
    #added by Tin Tran, remove the file when we're done
    os.remove(svgfile)
#=====================================================================================================================
def my_line(image,vectors,layer,color,width,coords):
	#create new stroke
	stroke_id = pdb.gimp_vectors_stroke_new_from_points(vectors,0,12,coords[0:2]+coords[0:2]+coords[0:2]+coords[2:4]+coords[2:4]+coords[2:4],0)
	#draw it
	vector_to_line_stroke(image,vectors,layer,color_to_hex(color),width,"square","bevel",10,"auto")
	#remove stroke when done	
	pdb.gimp_vectors_remove_stroke(vectors,stroke_id)

def python_abs_pixels_tt(image, layer):
	points = []	
	midx = layer.width/2.0
	midy = layer.height/2.0
	pdb.gimp_message("Reading pixels...")
	for iy in range(0,layer.height):
		for ix in range(0,layer.width):
			pixel = layer.get_pixel(ix,iy)
			x = ix - midx
			y = iy - midy
			z = int((pixel[2]+pixel[1]+pixel[0])/3.0)
			c = abs(x)**2 + abs(y)**2
			points.append([int(pixel[0]),int(pixel[1]),int(pixel[2]),x,y,z,c])
	pdb.gimp_message("Done reading pixels")
	pdb.gimp_message("Sorting depths")
	points.sort(key=lambda x:(1000000000 -  x[6]))
	pdb.gimp_message("Sorting complete")

	pdb.gimp_message("Creating new image to work with")
	nwidth = layer.width*3
	nheight = layer.height*3
	new_image = pdb.gimp_image_new(nwidth, nheight, RGB)
	pdb.gimp_display_new(new_image)
	work_layer = pdb.gimp_layer_new(new_image,nwidth,nheight,RGBA_IMAGE,"Black",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(new_image, work_layer, None, 0)
	pdb.gimp_context_set_foreground((0,0,0))
	pdb.gimp_edit_fill(work_layer,FOREGROUND_FILL)
	work_layer = pdb.gimp_layer_new(new_image,nwidth,nheight,RGBA_IMAGE,"Abstract Pixels",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(new_image, work_layer, None, 0)
	starting = 85.0
	rangez = 255.0 - starting
	minz = starting
	midxto = nwidth/2.0
	midyto = nheight/2.0
	pdb.gimp_message("Start Calculating and drawing...")
	count = 0
	prev_z = points[0][5]
	#pdb.gimp_context_set_brush_size(1)
	pdb.gimp_message("Done setting brush size")
	vectors = pdb.gimp_vectors_new(new_image,"temp vectors")
	for point in points:
		#current_z = point[5]
		#if current_z < prev_z:
		#	prev_z = current_z
		#	pdb.gimp_drawable_update(work_layer,0,0,nwidth,nheight)
		#	work_layer = pdb.gimp_layer_new(new_image,nwidth,nheight,RGBA_IMAGE,str(current_z),100,NORMAL_MODE)
		#	pdb.gimp_image_insert_layer(new_image, work_layer, None, 0)
		count += 1
		mapz = (((255-point[5]) / 255.0) * (rangez) + minz)/255.0
		fromx = point[3] + midxto
		fromy = point[4] + midyto
		mapx = (point[3]/mapz + midxto)
		mapy = (point[4]/mapz + midyto)
		brush_size = 1.0/mapz
		#pdb.gimp_message(str (mapx) + ":" + str(mapy)) 
		#work_layer.set_pixel(mapx,mapy,4,point[0:3]+[255])
		#pdb.gimp_context_set_brush_size(brush_size)
		#pdb.gimp_context_set_foreground((point[0],point[1],point[2]))
		my_line(new_image,vectors,work_layer,(point[0],point[1],point[2]),brush_size,[fromx,fromy,mapx,mapy])
		#pdb.gimp_paintbrush_default(work_layer,4,[fromx,fromy,mapx,mapy])
		#pdb.gimp_pencil(work_layer,4,[fromx,fromy,mapx,mapy])
		#pdb.gimp_paintbrush(work_layer,0,4,[fromx,fromy,mapx,mapy],PAINT_CONSTANT,0)
		#pdb.gimp_message("Pencil done")
		#pdb.gimp_drawable_set_pixel(work_layer,mapx,mapy,4,point[0:3]+[255])
		#pdb.gimp_message("pixel putted")
		if count % 1000 == 0:
			pdb.gimp_message(str(count)+"/"+str(len(points)))
			pdb.gimp_drawable_update(work_layer,0,0,nwidth,nheight)
			pdb.gimp_displays_flush()
	pdb.gimp_drawable_update(work_layer,0,0,nwidth,nheight)
	pdb.gimp_displays_flush()
	pdb.gimp_message("Complete!")
	#return

register(
	"python_fu_abs_pixels_tt",
	"Abstract Pixel Layer",
	"Abstract Pixel layer",
	"Tin Tran",
	"Tin Tran",
	"September 2018",
	"<Image>/Python-Fu/Abs Pixels...",             #Menu path
	"RGB*, GRAY*",
	[
	#(PF_SPINNER, "px_height", "Height of each tile image:", 100, (1, 1000, 50)),
	],
	[],
	python_abs_pixels_tt)

main()
