#!/usr/bin/env python

# Beads Look Rel 2
# Created by Tin Tran https://gimplearn.net
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: added interpolation option for scaling (as sometimes "None" is better than "Cubic" depending on image/taste)
# Rel 3: Added Delta-E Matching method
import math
import string
import copy
#import Image
from gimpfu import *
from array import array
stitch_dimension = 30

def indexed_color(arr):
	indexed = []
	for i in range(0,len(arr)/3):
		indexed.append((arr[i*3+0],arr[i*3+1],arr[i*3+2]))
	return indexed
def flatten_color(ind):
	flat = []
	for i in range(0,len(ind)):
		flat.append(ind[i][0])
		flat.append(ind[i][1])
		flat.append(ind[i][2])
	return flat
def rgb2lab(rgb):
    r = rgb[0] / 255.0
    g = rgb[1] / 255.0
    b = rgb[2] / 255.0

    r = ((r + 0.055) / 1.055)**2.4 if (r > 0.04045) else (r / 12.92);
    g = ((g + 0.055) / 1.055)**2.4 if (g > 0.04045) else (g / 12.92);
    b = ((b + 0.055) / 1.055)**2.4 if (b > 0.04045) else (b / 12.92);

    x = (r * 0.4124 + g * 0.3576 + b * 0.1805) / 0.95047;
    y = (r * 0.2126 + g * 0.7152 + b * 0.0722) / 1.00000;
    z = (r * 0.0193 + g * 0.1192 + b * 0.9505) / 1.08883;

    x = x**(1/3.0) if (x > 0.008856) else (7.787 * x) + 16/116.0;
    y = y**(1/3.0) if (y > 0.008856) else (7.787 * y) + 16/116.0;
    z = z**(1/3.0) if (z > 0.008856) else (7.787 * z) + 16/116.0;

    return [(116.0 * y) - 16.0, 500.0 * (x - y), 200.0 * (y - z)]	
def deltaE(labA, labB):
    deltaL = labA[0] - labB[0];
    deltaA = labA[1] - labB[1];
    deltaB = labA[2] - labB[2];
    c1 = (labA[1] * labA[1] + labA[2] * labA[2])**0.5;
    c2 = (labB[1] * labB[1] + labB[2] * labB[2])**0.5;
    deltaC = c1 - c2;
    deltaH = deltaA * deltaA + deltaB * deltaB - deltaC * deltaC;
    deltaH = 0 if (deltaH < 0) else (deltaH)**0.5;
    sc = 1.0 + 0.045 * c1;
    sh = 1.0 + 0.015 * c1;
    deltaLKlsl = deltaL / (1.0);
    deltaCkcsc = deltaC / (sc);
    deltaHkhsh = deltaH / (sh);
    i = deltaLKlsl * deltaLKlsl + deltaCkcsc * deltaCkcsc + deltaHkhsh * deltaHkhsh;
    return 0 if (i < 0) else (i)**0.5;	
def python_beads_look_tt(image, layer, num_colors, color_dithering, interpolation, match_method, hor_stitches, aida_file,inHama,inNabbi,inPerler,inArtkal,exTransp,exPearl,exMetal,exGlow,exGlitter):
	#DMC information [DMC,Name,RGB,distance] distance is to be determined/calculated later and used to sort for closest match color
	ALL_BEADS = [['H01','White',(236,237,237),0,'Hama',''],['H02','Cream',(240,232,185),0,'Hama',''],['H03','Yellow',(240,185,1),0,'Hama',''],['H04','Orange',(230,79,39),0,'Hama',''],['H05','Red',(182,49,54),0,'Hama',''],['H06','Pink',(225,136,159),0,'Hama',''],['H07','Purple',(105,74,130),0,'Hama',''],['H08','Dark Blue (Blue)',(44,70,144),0,'Hama',''],['H09','Light Blue',(48,92,176),0,'Hama',''],['H10','Green',(37,104,71),0,'Hama',''],['H11','Light green',(73,174,137),0,'Hama',''],['H12','Brown',(83,65,55),0,'Hama',''],['H13','Transparent Red',(192,36,53),0,'Hama','Transp'],['H16','Transparent Green',(55,184,118),0,'Hama','Transp'],['H17','Grey',(131,136,138),0,'Hama',''],['H18','Black',(46,47,49),0,'Hama',''],['H19','Clear',(216,210,206),0,'Hama','Transp'],['H20','Reddish Brown',(127,51,42),0,'Hama',''],['H21','Light Brown',(165,105,63),0,'Hama',''],['H22','Dark Red',(165,45,54),0,'Hama',''],['H24','Translucent Purple',(104,62,154),0,'Hama','Transp'],['H25','Translucent Brown',(135,89,61),0,'Hama','Transp'],['H26','Flesh',(222,155,144),0,'Hama',''],['H27','Beige',(222,180,139),0,'Hama',''],['H28','Army (Dark Green)',(54,63,56),0,'Hama',''],['H29','Claret',(185,57,94),0,'Hama',''],['H30','Burgundy',(89,47,56),0,'Hama',''],['H31','Turquoise',(103,151,174),0,'Hama',''],['H32','Neon Pink (Fucsia)',(255,32,141),0,'Hama',''],['H33','Cerise',(255,57,86),0,'Hama',''],['H34','Neon Yellow',(229,239,19),0,'Hama','Transp'],['H35','Neon Red',(255,40,51),0,'Hama','Transp'],['H36','Neon Blue',(35,83,176),0,'Hama','Transp'],['H37','Neon Green',(6,183,60),0,'Hama','Transp'],['H38','Neon Orange',(253,134,0),0,'Hama','Transp'],['H39','Fluorescent Yellow',(241,242,28),0,'Hama',''],['H40','Fluorescent Orange',(254,99,11),0,'Hama',''],['H41','Fluorescent Blue',(38,89,178),0,'Hama',''],['H42','Fluorescent Green',(12,189,81),0,'Hama',''],['H43','Pastel Yellow',(240,234,55),0,'Hama',''],['H44','Pastel Red',(238,105,114),0,'Hama',''],['H45','Pastel Purple',(136,109,185),0,'Hama',''],['H46','Pastel Blue',(98,158,215),0,'Hama',''],['H47','Pastel Green',(131,203,112),0,'Hama',''],['H48','Pastel Pink',(207,112,183),0,'Hama',''],['H49','Azure',(73,152,188),0,'Hama',''],['N01','Black',(58,61,65),0,'Nabbi',''],['N02','Dark Brown',(80,68,59),0,'Nabbi',''],['N03','Brown Medium',(90,62,54),0,'Nabbi',''],['N04','Maroon (Wine Red)',(129,53,71),0,'Nabbi',''],['N05','Caramel',(167,98,36),0,'Nabbi',''],['N06','Tan (Beige)',(173,150,126),0,'Nabbi',''],['N07','Sand (Skin)',(238,177,130),0,'Nabbi',''],['N08','Ash (Ash Grey',(141,139,127),0,'Nabbi',''],['N09','Hunter Green',(47,74,57),0,'Nabbi',''],['N10','Light Grey',(211,203,203),0,'Nabbi',''],['N11','Purple',(100,69,145),0,'Nabbi',''],['N12','Ivory',(226,208,191),0,'Nabbi',''],['N13','Orange (Clear Orange)',(243,96,27),0,'Nabbi',''],['N14','Yellow',(249,202,0),0,'Nabbi',''],['N15','White',(244,244,243),0,'Nabbi',''],['N16','Green',(41,122,59),0,'Nabbi',''],['N17','Bright Blue (Blue)',(59,117,203),0,'Nabbi',''],['N18','Light Rose (Light Pink)',(225,180,171),0,'Nabbi',''],['N19','Red (Light Red)',(223,38,56),0,'Nabbi',''],['N20','Light Brown',(181,139,105),0,'Nabbi',''],['N21','Lt Yellow',(245,236,141),0,'Nabbi',''],['N22','Lime (pearl Green)',(72,175,79),0,'Nabbi',''],['N23','Med Blue (pastel blue)',(113,163,230),0,'Nabbi',''],['N24','Lavender',(182,160,219),0,'Nabbi',''],['N25','Pink (old rose)',(238,106,151),0,'Nabbi',''],['N26','Peach (Light orange)',(252,168,121),0,'Nabbi',''],['N27','Chocolate',(135,95,82),0,'Nabbi',''],['N28','Sky (light blue)',(167,198,241),0,'Nabbi',''],['N29','Gold/Orange (Pearl Orange)',(238,149,39),0,'Nabbi',''],['N30','Kiwi (Olive)',(199,191,94),0,'Nabbi',''],['P01','White',(241,241,241),0,'Perler',''],['P02','Cream',(224,222,169),0,'Perler',''],['P03','Yellow',(236,216,0),0,'Perler',''],['P04','Orange',(237,97,32),0,'Perler',''],['P05','Red',(191,46,64),0,'Perler',''],['P06','Bubblegum',(221,102,155),0,'Perler',''],['P07','Purple',(96,64,137),0,'Perler',''],['P08','Dark Blue',(43,63,135),0,'Perler',''],['P09','Light Blue',(51,112,192),0,'Perler',''],['P10','Dark Green',(28,117,62),0,'Perler',''],['P100','Pearl Coral',(249,126,121),0,'Perler','Pearl'],['P101','Pearl Light Blue',(122,174,162),0,'Perler','Pearl'],['p102','Pearl Green',(132,183,145),0,'Perler','Pearl'],['P103','Pearl Yellow',(202,192,51),0,'Perler','Pearl'],['P104','Pearl Light Pink',(215,168,162),0,'Perler','Pearl'],['P105','Silver',(119,123,129),0,'Perler','Metal'],['P11','Light Green',(86,186,159),0,'Perler',''],['P12','Brown',(81,57,49),0,'Perler',''],['P17','Grey',(138,141,145),0,'Perler',''],['P18','Black',(46,47,50),0,'Perler',''],['P20','Rust',(140,55,44),0,'Perler',''],['P21','Light Brown',(129,93,52),0,'Perler',''],['P33','Peach',(238,186,178),0,'Perler',''],['P35','Tan',(188,147,113),0,'Perler',''],['P38','Magenta',(242,42,123),0,'Perler',''],['P47','Neon Yellow',(220,224,2),0,'Perler','Transp'],['P48','Neon Orange',(255,119,0),0,'Perler','Transp'],['P49','Neon Green',(1,158,67),0,'Perler','Transp'],['P50','Neon Pink',(255,57,145),0,'Perler','Transp'],['P52','Pastel Blue',(83,144,209),0,'Perler',''],['P53','Pastel Green',(118,200,130),0,'Perler',''],['P54','Pastel Lavender',(138,114,193),0,'Perler',''],['P56','Pastel Yellow',(254,248,117),0,'Perler',''],['P57','Chedder',(241,170,12),0,'Perler',''],['P58','Toothpaste',(147,200,212),0,'Perler',''],['P59','Hot Coral',(255,56,81),0,'Perler',''],['P60','Plum',(162,75,156),0,'Perler',''],['P61','Kiwi Lime',(108,190,19),0,'Perler',''],['P62','Cyan (aka Turquoise)',(43,137,198),0,'Perler',''],['P63','Blush',(255,130,133),0,'Perler',''],['P70','Periwinkle Blue',(100,124,190),0,'Perler',''],['P75','Glow Green',(190,198,150),0,'Perler','Glow'],['P79','Light Pink',(246,179,221),0,'Perler',''],['P80','Bright Green',(79,173,66),0,'Perler',''],['P81','Light Gray',(177,181,178),0,'Perler',''],['P179','Evergreen',(53,83,67),0,'Perler',''],['P82','Lavender',(173,152,212),0,'Perler',''],['P83','Pink',(228,72,146),0,'Perler',''],['P85','Gold',(187,118,52),0,'Perler','Metal'],['P88','Raspberry',(165,48,97),0,'Perler',''],['P90','Butterscotch',(212,132,55),0,'Perler',''],['P91','Parrot Green',(6,124,129),0,'Perler',''],['P92','Dark Grey',(77,81,86),0,'Perler',''],['P93','Blueberry Cream',(130,151,217),0,'Perler',''],['P96','Cranapple',(128,50,69),0,'Perler',''],['P97','Prickly Pear',(189,218,1),0,'Perler',''],['P98','Sand',(228,182,144),0,'Perler',''],['S01/R01','White',(233,231,230),0,'Artkal',''],['S02/R02','Burning Sand',(254,144,143),0,'Artkal',''],['S03/R03','Tangerine',(254,164,2),0,'Artkal',''],['S04/R04','Orange',(253,97,73),0,'Artkal',''],['S05/R05','Tall Poppy',(221,52,68),0,'Artkal',''],['S06/R06','Raspberry pink',(235,102,190),0,'Artkal',''],['S07/R07','Gray 2',(142,143,147),0,'Artkal',''],['S08/R08','Emerald',(82,204,106),0,'Artkal',''],['S09/R09','Dark Green',(42,93,98),0,'Artkal',''],['S10/R10','Baby Blue',(97,168,223),0,'Artkal',''],['S11/R11','Dark Blue',(40,68,146),0,'Artkal',''],['S12/R12','Pastel Lavender',(137,98,173),0,'Artkal',''],['S13/R13','Black',(31,31,33),0,'Artkal',''],['S14/R14','Sandstorm',(239,221,6),0,'Artkal',''],['S15/R15','Redwood',(124,69,69),0,'Artkal',''],['S16/R16','Brown',(120,88,78),0,'Artkal',''],['S17/R17','light brown',(89,71,67),0,'Artkal',''],['S18/R18','Sand',(206,155,134),0,'Artkal',''],['S19/R19','Bubblegum',(237,174,172),0,'Artkal',''],['S20/R20','Green',(84,145,96),0,'Artkal',''],['S21/R21','Pastel Green',(132,197,81),0,'Artkal',''],['S22/R22','Purple',(85,72,137),0,'Artkal',''],['S23/R23','Royal Purple',(125,102,173),0,'Artkal',''],['S24/R24','True Blue',(57,109,188),0,'Artkal',''],['S25/R25','Hot Pink',(254,91,186),0,'Artkal',''],['S26/R26','Magenta',(254,66,137),0,'Artkal',''],['S27/R27','Yellow',(240,206,1),0,'Artkal',''],['S28/R28','Lily Pink',(223,159,205),0,'Artkal',''],['S29/R29','Pastel Yellow',(240,226,79),0,'Artkal',''],['S30/R30','Shadow Green',(161,198,206),0,'Artkal',''],['S31/R31','Sea Mist',(171,218,202),0,'Artkal',''],['S32/R32','Beeswax',(248,197,131),0,'Artkal',''],['S33/R33','Maverick',(192,174,219),0,'Artkal',''],['S35/R35','Mona Lisa',(249,188,193),0,'Artkal',''],['S36/R36','Old Pink',(200,103,130),0,'Artkal',''],['S37/R37','Blue Green',(132,199,198),0,'Artkal',''],['S38/R38','burgundy',(161,48,83),0,'Artkal',''],['S39/R39','Yellow Orange',(255,131,75),0,'Artkal',''],['S40/R40','Carnation Pink',(253,145,214),0,'Artkal',''],['S41/R41','Copper',(157,91,46),0,'Artkal','Metal'],['S42/R42','Silver',(103,107,115),0,'Artkal','Metal'],['S43/R43','Gray 3',(102,103,106),0,'Artkal',''],['S44/R44','Sky Blue',(164,200,232),0,'Artkal',''],['S45/R45','Medium Turquoise',(63,152,158),0,'Artkal',''],['S46/R46','Bright Green',(168,228,42),0,'Artkal',''],['S47/R47','Marigold',(156,116,60),0,'Artkal',''],['S48/R48','Corn',(246,191,5),0,'Artkal',''],['S49/R49','Mulberry Wood',(120,49,90),0,'Artkal',''],['S50/R50','Mandys Pink',(253,164,126),0,'Artkal',''],['S51/R51','Spring Sun',(243,217,182),0,'Artkal',''],['S52/R52','Picasso',(239,229,109),0,'Artkal',''],['S53/R53','Turquoise',(101,147,223),0,'Artkal',''],['S54/R54','Light Blue',(26,117,208),0,'Artkal',''],['S55/R57','Pistachio',(160,213,137),0,'Artkal',''],['S56/R59','Outrageous Orange',(254,127,80),0,'Artkal',''],['S57/R63','Buccaneer',(152,78,67),0,'Artkal',''],['S58/R64','Paprika',(172,44,68),0,'Artkal',''],['S60/R66','Lavender',(166,117,191),0,'Artkal',''],['S61/R68','Key Lemon Pie',(187,197,40),0,'Artkal',''],['S62/R69','Green Tea',(56,112,97),0,'Artkal',''],['S63/R70','Metallic Gold',(105,96,70),0,'Artkal',''],['S64/R55','Black Rock',(54,61,92),0,'Artkal',''],['S65/R58','Canary',(247,248,57),0,'Artkal',''],['S66/R70','Blaze Orange',(253,81,67),0,'Artkal',''],['S67/R61','Vanilla',(226,193,178),0,'Artkal',''],['S68','Tan',(175,150,102),0,'Artkal',''],['S69','Mine Shaft',(67,72,78),0,'Artkal',''],['S70','Dark Algae',(146,183,1),0,'Artkal',''],['S71','Jade Green',(49,131,50),0,'Artkal',''],['S72','Light Sea Blue',(119,191,209),0,'Artkal',''],['S73','Steel blue',(91,159,199),0,'Artkal',''],['S74','Azure',(64,144,192),0,'Artkal',''],['S75','Dark Steel Blue',(56,121,166),0,'Artkal',''],['S76','Sea Blue',(59,135,171),0,'Artkal',''],['S77','Ghost White',(211,211,208),0,'Artkal',''],['S78','Ash Gray',(189,189,187),0,'Artkal',''],['S79','Gray 1',(163,167,169),0,'Artkal',''],['S80','Dark Olive',(124,137,52),0,'Artkal',''],['S81','Deer',(184,138,98),0,'Artkal',''],['S82','Clay',(145,106,77),0,'Artkal',''],['S83','Sienna',(174,103,53),0,'Artkal',''],['S84','Deep Chestnut',(165,87,97),0,'Artkal',''],['S85','Red Wine',(140,65,74),0,'Artkal',''],['S86','Goldenrod',(233,181,4),0,'Artkal',''],['S87','Coral Red',(250,97,106),0,'Artkal',''],['S88','Dark Pink',(217,59,128),0,'Artkal',''],['S89','Gray 4',(88,92,99),0,'Artkal',''],['S90','Pastel Orange',(251,183,73),0,'Artkal',''],['S91','Brunswick Green',(52,64,66),0,'Artkal',''],['SG1/RG1','Glow Yellow',(198,192,180),0,'Artkal','Glow'],['SG2/RG2','Glow Pink',(223,171,188),0,'Artkal','Glow'],['SG3/RG3','Glow Blue',(111,173,193),0,'Artkal','Glow'],['SL1/RL1','Glitter Blue',(54,135,178),0,'Artkal','Glitter'],['SL2/RL2','Glitter Green',(17,124,82),0,'Artkal','Glitter'],['SL3/RL3','Glitter Red',(194,41,51),0,'Artkal','Glitter'],['SL4/RL4','Glitter Yellow',(205,189,0),0,'Artkal','Glitter'],['SN1','Neon Orange',(255,116,5),0,'Artkal',''],['SN2','Neon Green',(1,198,62),0,'Artkal',''],['SN3','Neon Yellow',(242,243,60),0,'Artkal',''],['SN4','Neon Rose',(255,71,122),0,'Artkal',''],['SP1/RP1','Pearlescent Green',(147,202,135),0,'Artkal','Pearl'],['SP2/RP2','Pearlescent Tangerine',(254,127,84),0,'Artkal','Pearl'],['SP3/RP3','Pearlescent Orange',(240,161,73),0,'Artkal','Pearl'],['SP4/RP4','Pearlescent Pink',(220,158,169),0,'Artkal','Pearl'],['SP5/RP5','Pearlescent Red',(214,60,83),0,'Artkal','Pearl'],['SP6/RP6','Pearlescent Blue',(44,78,151),0,'Artkal','Pearl'],['SP7/RP7','Pearlescent Purple',(100,66,126),0,'Artkal','Pearl'],['SP8','Pearlescent White',(224,219,202),0,'Artkal','Pearl'],['ST1/RT1','Transparent',(199,188,184),0,'Artkal','Transp'],['ST2/RT2','Transparent Pink',(249,84,152),0,'Artkal','Transp'],['ST3/RT3','Transparent Tangering',(255,124,69),0,'Artkal','Transp'],['ST4/RT4','Transparent Yellow',(218,223,61),0,'Artkal','Transp'],['ST5','transparent green',(15,174,88),0,'Artkal','Transp'],['ST6','Transparent Purple',(132,119,179),0,'Artkal','Transp']]
	#DMC information [DMC,Name,RGB,distance] distance is to be determined/calculated later and used to sort for closest match color
	#We'll call it DMC since it uses the same method as matching stitching thread colors, but here we're doing beads so it's really 
	#the same
	DMC = []
	#add to DMC to treat it like thread color if conditions are met.
	for bead in ALL_BEADS:
		isHama,isNabbi,isPerler,isArtkal = bead[4]=='Hama',bead[4]=='Nabbi',bead[4]=='Perler',bead[4]=='Artkal'
		isTransp,isPearl,isMetal,isGlow,isGlitter = bead[5]=='Transp',bead[5]=='Pearl',bead[5]=='Metal',bead[5]=='Glow',bead[5]=='Glitter'
		if ((isHama and inHama) or (isNabbi and inNabbi) or (isPerler and inPerler) or (isArtkal and inArtkal)) and \
			(bead[5]=='' or (isTransp and not exTransp) or (isPearl and not exPearl)or (isMetal and not exMetal) \
			or (isGlow and not exGlow) or (isGlitter and not exGlitter)):
			DMC.append(bead)
	SYM = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '1A', '1B', '1C', '1D', '1E', '1F', '20', '21', '22', '23', '24', '25', '26', '27', '28', '29', '2A', '2B', '2C', '2D', '2E', '2F', '30', '31', '32', '33', '34', '35', '36', '37', '38', '39', '3A', '3B', '3C', '3D', '3E', '3F', '40', '41', '42', '43', '44', '45', '46', '47', '48', '49', '4A', '4B', '4C', '4D', '4E', '4F', '50', '51', '52', '53', '54', '55', '56', '57', '58', '59', '5A', '5B', '5C', '5D', '5E', '5F', '60', '61', '62', '63', '64', '65', '66', '67', '68', '69', '6A', '6B', '6C', '6D', '6E', '6F', '70', '71', '72', '73', '74', '75', '76', '77', '78', '79', '7A', '7B', '7C', '7D', '7E', '7F', '80', '81', '82', '83', '84', '85', '86', '87', '88', '89', '8A', '8B', '8C', '8D', '8E', '8F', '90', '91', '92', '93', '94', '95', '96', '97', '98', '99', '9A', '9B', '9C', '9D', '9E', '9F', 'A0', 'A1', 'A2', 'A3', 'A4', 'A5', 'A6', 'A7', 'A8', 'A9', 'AA', 'AB', 'AC', 'AD', 'AE', 'AF', 'B0', 'B1', 'B2', 'B3', 'B4', 'B5', 'B6', 'B7', 'B8', 'B9', 'BA', 'BB', 'BC', 'BD', 'BE', 'BF', 'C0', 'C1', 'C2', 'C3', 'C4', 'C5', 'C6', 'C7', 'C8', 'C9', 'CA', 'CB', 'CC', 'CD', 'CE', 'CF', 'D0', 'D1', 'D2', 'D3', 'D4', 'D5', 'D6', 'D7', 'D8', 'D9', 'DA', 'DB', 'DC', 'DD', 'DE', 'DF', 'E0', 'E1', 'E2', 'E3', 'E4', 'E5', 'E6', 'E7', 'E8', 'E9', 'EA', 'EB', 'EC', 'ED', 'EE', 'EF', 'F0', 'F1', 'F2', 'F3', 'F4', 'F5', 'F6', 'F7', 'F8', 'F9', 'FA', 'FB', 'FC', 'FD', 'FE', 'FF']
	stitched_file = aida_file
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#make a new image of active layer
	new_image = pdb.gimp_image_new(layer.width,layer.height,RGB)
	new_display = pdb.gimp_display_new(new_image)
	#copy layer to new image
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy,None,0)
	
	#scale it down based on horizontal stitches
	scale = float(hor_stitches)/layer.width
	pdb.gimp_context_set_interpolation(interpolation) 
	pdb.gimp_image_scale(new_image,hor_stitches,int(layer.height*scale))
	#reduce number of colors
	pdb.gimp_convert_indexed(new_image,color_dithering,MAKE_PALETTE,num_colors,FALSE,FALSE,"")
	
	#get color map
	num_bytes,colormap = pdb.gimp_image_get_colormap(new_image)
	#converts it to indexed tuples
	colormap = indexed_color(colormap)
	
	dmcmap = []
	#match color to DMCs
	for c in range(0,len(colormap)):
		#grab RGB info to calculate distance.
		R = colormap[c][0]
		G = colormap[c][1]
		B = colormap[c][2]
		for d in range(0,len(DMC)):
			if match_method == 0: #Perceptive distance calculation
				DMC[d][3] = ((R-DMC[d][2][0])*0.3)**2 + ((G-DMC[d][2][1])*0.59)**2 + ((B-DMC[d][2][2])*0.11)**2
			elif match_method == 1: #Regular distance calculation
				DMC[d][3] = ((R-DMC[d][2][0]))**2 + ((G-DMC[d][2][1]))**2 + ((B-DMC[d][2][2]))**2
			elif match_method==2: #Delta-E
				DMC[d][3] = deltaE(rgb2lab((R,G,B)),rgb2lab(DMC[d][2]))	
		DMC.sort(key=lambda x: x[3])
		#add first DMC (closest match) to dmcmap.
		dmcmap.append(DMC[0][2])
	#get unique colors to go through to pick later.
	uniquecolors = list(set(dmcmap))
	dmcmap = flatten_color(dmcmap)
	pdb.gimp_image_set_colormap(new_image,len(dmcmap),dmcmap)
	
	#scale our image so that each stitch is stitch_dimension large
	new_width = new_image.width * stitch_dimension
	new_height = new_image.height * stitch_dimension
	pdb.gimp_context_set_interpolation(INTERPOLATION_NONE) #possible TODO: this could be an option.
	pdb.gimp_image_scale(new_image,new_width,new_height)
	
	#converts back to RGB so we can work with other colors.
	pdb.gimp_image_convert_rgb(new_image)
	#add white layer with opacity so that black stitch layers would stand out above.
	
	#pdb.gimp_context_set_pattern("Clipboard") #below calls set pattern to "Clipboard" even in different languages where it's not called "Clipboard"
	pdb.gimp_context_set_pattern(pdb.gimp_patterns_list('')[1][0])
	#pdb.gimp_drawable_fill(wavy_layer,PATTERN_FILL)
	
	#settings to set before using select_color
	pdb.gimp_context_set_antialias(True)
	pdb.gimp_context_set_feather(False)
	pdb.gimp_context_set_feather_radius(0,0)
	pdb.gimp_context_set_sample_merged(False)
	pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_COMPOSITE)
	pdb.gimp_context_set_sample_threshold(0)
	pdb.gimp_context_set_sample_transparent(True)
	
	#loads aida png file
	aida_image = pdb.file_png_load(aida_file,aida_file)
	aida_layer = pdb.gimp_image_get_active_layer(aida_image)
	pdb.gimp_selection_all(aida_image)
	pdb.gimp_edit_copy(aida_layer)
	
	aida_cloth_layer = pdb.gimp_layer_new(new_image,new_image.width,new_image.height,RGBA_IMAGE,"aida cloth",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(new_image,aida_cloth_layer,None,1)
	pdb.gimp_selection_none(new_image)
	pdb.gimp_edit_fill(aida_cloth_layer,PATTERN_FILL)
	
	#deletes aida image
	pdb.gimp_image_delete(aida_image)
	
	stitched_image = pdb.file_png_load(stitched_file,stitched_file)
	stitched_layer = pdb.gimp_image_get_active_layer(stitched_image)
	pdb.gimp_selection_all(stitched_image)
	pdb.gimp_edit_copy(stitched_layer)
	
	stitched_cloth_layer = pdb.gimp_layer_new(new_image,new_image.width,new_image.height,RGBA_IMAGE,"stitched overlay",100,OVERLAY_MODE)
	pdb.gimp_image_insert_layer(new_image,stitched_cloth_layer,None,0)
	pdb.gimp_selection_none(new_image)
	pdb.gimp_edit_fill(stitched_cloth_layer,PATTERN_FILL)
	
	pdb.gimp_image_select_item(new_image,CHANNEL_OP_REPLACE,layer_copy)
	pdb.gimp_selection_invert(new_image)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(new_image)
	if non_empty:
		pdb.gimp_edit_clear(stitched_cloth_layer)
	
	#deletes stitched png image
	pdb.gimp_image_delete(stitched_image)
	
	pdb.gimp_selection_none(new_image)
	'''darken_only_copy = pdb.gimp_layer_new_from_drawable(aida_cloth_layer,new_image)
	pdb.gimp_image_insert_layer(new_image,darken_only_copy,None,1)
	pdb.gimp_item_set_name(darken_only_copy,"aida cloth darken only mode")
	pdb.gimp_layer_set_mode(darken_only_copy,DARKEN_ONLY_MODE)
	pdb.gimp_brightness_contrast(darken_only_copy,0,65)
	'''
	#gimp.progress_init("Rendering stich patterns...")
	#gimp.progress_update(1.0)
	
		
	pdb.gimp_selection_none(new_image)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_beads_look_tt",                           
	"Generates a beads/perler look",
	"Generates a beads/perler look",
	"Tin Tran",
	"Tin Tran",
	"March 2018",
	"<Image>/Python-Fu/Beads Look...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "num_colors","# of Colors:", 8, (2, 256, 1)),
	(PF_OPTION, "color_dithering",   "Color _dithering:", 0, ["None","Floyd-Steinberg(Normal)","Floyd-Steinberg(Reduce color bleeding)","Positioned"]), # initially 0th is choice
	(PF_OPTION, "interpolation", "Interpolation (Used for Scaling):", 2, ["None","Linear","Cubic","NoHalo","LoHalo"]),
	(PF_OPTION, "match_method",   "Color Match method:", 0, ["Perceptive","Regular","Delta-E"]), # initially 0th is choice
	(PF_SPINNER, "hor_stitches", "# of Beads (Horizontally):", 100, (1, 20000, 10)),
	(PF_FILENAME, "aida_file", "beads.png:", ""),
	#(PF_FILENAME, "stitched_file", "stitched.png:", ""),
	(PF_TOGGLE, "inHama",   "Include Hama Beads:", 1), 
	(PF_TOGGLE, "inNabbi",   "Include Nabbi Beads:", 0),
	(PF_TOGGLE, "inPerler",   "Include Perler Beads:", 0),
	(PF_TOGGLE, "inArtkal",   "Include Artkal Beads:", 1),
	(PF_TOGGLE, "exTransp",   "Exclude Special Transp Beads:", 1),
	(PF_TOGGLE, "exPearl",   "Exclude Special Pearl Beads:", 1),
	(PF_TOGGLE, "exMetal",   "Exclude Special Metal Beads:", 1),
	(PF_TOGGLE, "exGlow",   "Exclude Special Glow Beads:", 1),
	(PF_TOGGLE, "exGlitter",   "Exclude Special Glitter Beads:", 1),
	],
	[],
	python_beads_look_tt)

main()
