#!/usr/bin/env python

# Bismuth Rel 1
# Steps described here -> http://gimpchat.com/viewtopic.php?f=23&t=8795
# Created by Tin Tran (https://gimplearn.net)
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.

#import math
#import string
#import Image
from gimpfu import *
#from array import array
#import os


def python_bismuth(image, layer, zoom) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#grab coordinates y1,y2 for gradient much later
	pdb.plug_in_autocrop_layer(image,layer)
	offset_x,y1 = pdb.gimp_drawable_offsets(layer)
	y2 = y1 + layer.height
	
	#resize layer to image size
	pdb.gimp_layer_resize_to_image_size(layer)
	pdb.gimp_selection_none(image)
	
	
	#4 Duplicate text layer merge down to background
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,layer_copy,None,1)
	background_layer = pdb.gimp_image_merge_down(image,layer_copy,EXPAND_AS_NECESSARY)
	#5. Gaussian blur the background layer by 10. Then turn the layer invisible. This will be the bump map
	# for the first bump. Rename the layer bump 1 for clarity.
	pdb.plug_in_gauss(image,background_layer,10.0,10.0,1)
	pdb.gimp_item_set_visible(background_layer,False)
	pdb.gimp_item_set_name(background_layer,"bump 1")
	bump1 = background_layer
	#6. On the text layer, which in my case is named Bismuth #1, lock the alpha lock.
	pdb.gimp_layer_set_lock_alpha(layer,1)
	#7. Make this color in the color selection, html notation 808080. Which is exactly half grey and ideal
	#for bump mapping awesomeness.
	pdb.gimp_context_set_foreground((128,128,128))
	#8. Fill the text with this color, then remove the alpha lock. (click the little box again)
	pdb.gimp_edit_fill(layer,FOREGROUND_FILL)
	pdb.gimp_layer_set_lock_alpha(layer,False)
	#9. Duplicate the text layer once, then alpha to selection on the copy.
	#10. Shrink the selection by 4, invert selection and hit the delete key. Deselect all.
	inlay = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,inlay,None,0)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,inlay)
	pdb.gimp_selection_shrink(image,4)
	pdb.gimp_selection_invert(image)
	pdb.gimp_edit_clear(inlay)
	pdb.gimp_selection_none(image)
	#11. Duplicate the inlay text layer and lock the alpha channel on it.
	#12. Fill it with white.
	inlay_copy = pdb.gimp_layer_new_from_drawable(inlay,image)
	pdb.gimp_image_insert_layer(image,inlay_copy,None,0)
	pdb.gimp_layer_set_lock_alpha(inlay_copy,1)
	pdb.gimp_edit_fill(inlay_copy,WHITE_FILL)
	#13. Make a new layer under it and fill it with black. Name this layer bump 2.
	bump2 = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,bump2,None,1)
	pdb.gimp_context_set_foreground((0,0,0))
	pdb.gimp_edit_fill(bump2,FOREGROUND_FILL)
	pdb.gimp_item_set_name(bump2,"bump 2")
	#14. Merge the white down onto the black.
	bump2 = pdb.gimp_image_merge_down(image,inlay_copy,EXPAND_AS_NECESSARY)
	#15. Gaussian blur this bump 2 layer by 5.
	pdb.plug_in_gauss(image,bump2,5.0,5.0,1)
	#16. Move it down and turn it invisible. This is our second bump map.
	pdb.gimp_image_lower_item(image,bump2)
	pdb.gimp_image_lower_item(image,bump2)
	pdb.gimp_item_set_visible(bump2,False)
	#17. Now duplicate the top text layer. The one we just shrank. Alpha to selection, shrink selection by 1.
	inlay_copy = pdb.gimp_layer_new_from_drawable(inlay,image)
	pdb.gimp_image_insert_layer(image,inlay_copy,None,0)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,inlay_copy)
	pdb.gimp_selection_shrink(image,1)
	#18. Invert selection and hit delete button on your keyboard.
	pdb.gimp_selection_invert(image)
	pdb.gimp_edit_clear(inlay_copy)
	pdb.gimp_selection_none(image)
	
	#19. Now duplicate the text layer we just made and lock the alpha on it again. Then fill it with white.
	inlay_copy_copy = pdb.gimp_layer_new_from_drawable(inlay_copy,image)
	pdb.gimp_image_insert_layer(image,inlay_copy_copy,None,0)
	pdb.gimp_layer_set_lock_alpha(inlay_copy_copy,1)
	pdb.gimp_edit_fill(inlay_copy_copy,WHITE_FILL)
	
	#20. Now make a new black layer right under the 3rd text layer. Then merge the white text down to the
	#black layer and name it bump 3.
	bump3 = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,bump3,None,1)
	pdb.gimp_context_set_foreground((0,0,0))
	pdb.gimp_edit_fill(bump3,FOREGROUND_FILL)
	pdb.gimp_item_set_name(bump3,"bump 3")
	bump3 = pdb.gimp_image_merge_down(image,inlay_copy_copy,EXPAND_AS_NECESSARY)
	
	#21. Gaussian Blur the merged bump 3 layer by 20. Then sort it to the top of all the bump layers and
	#turn it invisible.
	pdb.plug_in_gauss(image,bump3,20,20,1)
	pdb.gimp_image_lower_item(image,bump3)
	pdb.gimp_image_lower_item(image,bump3)
	pdb.gimp_image_lower_item(image,bump3)
	pdb.gimp_item_set_visible(bump3,False)
	#22. Invert the colors of bump 2, black text on white back. This will be important for the bumps.
	pdb.gimp_invert(bump2)
	
	#23. Now, at the top of the stack, make a new black layer and a new white layer.
	black = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,black,None,0)
	pdb.gimp_context_set_foreground((0,0,0))
	pdb.gimp_edit_fill(black,FOREGROUND_FILL)
	pdb.gimp_item_set_name(black,"black")
	white = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,white,None,0)
	pdb.gimp_context_set_foreground((255,255,255))
	pdb.gimp_edit_fill(white,FOREGROUND_FILL)
	pdb.gimp_item_set_name(white,"white")
	#24. With the white layer selected, go to Filters >Artistic > Cubism and use the settings from the image.
	pdb.plug_in_cubism(image,white,6.5,1.6,0)
	#25. Then merge it down onto the black layer
	Cubism = pdb.gimp_image_merge_down(image,white,EXPAND_AS_NECESSARY)
	#26. Rename the layer to Cubism, then move it down between the text layers and the map layers. Turn
	# off visibility.
	pdb.gimp_image_lower_item(image,Cubism)
	pdb.gimp_image_lower_item(image,Cubism)
	pdb.gimp_image_lower_item(image,Cubism)
	pdb.gimp_item_set_visible(Cubism,False)
	pdb.gimp_item_set_name(Cubism,"Cubism")
	# 27. Now we'll make the colors. Make a new image.
	color_image = pdb.gimp_image_new(640,480,RGB)
	new_display = pdb.gimp_display_new(color_image)
	color_layer = pdb.gimp_layer_new(color_image,640,480,RGBA_IMAGE,"color",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(color_image,color_layer,None,0)
	#28. On this new image, go to Filters>Render>Environment Map.
	pdb.script_fu_environment_map(color_image,color_layer,0,0,0,0,"Full saturation spectrum CW",1,4.3,4.3,0,1,0,0,0)
	color_layer = pdb.gimp_image_get_active_layer(color_image)
	#29. Now back to the original image, make a grey layer (808080) between Cubism and Text 1 layer.
	texture = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,texture,None,3)
	pdb.gimp_context_set_foreground((128,128,128))
	pdb.gimp_edit_fill(texture,FOREGROUND_FILL)
	pdb.gimp_item_set_name(texture,"Texture")
	#30. With this layer selected, go to Filters>Light and Shadow> Lighting Effects. 
	pdb.plug_in_lighting(image,texture,Cubism,color_layer,True,True,0,1,(255,255,255),-1.0,-1.0,1.0,-1.2,-1.29,1.0,0.2,0.5,0.5,27,0,True,False,False)
	#31. Use the move tool, slide the texture around to find the best colors and such and then alpha select
	#from the bottom text layer. Invert the selection and delete the excess.
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	pdb.gimp_selection_invert(image)
	pdb.gimp_edit_clear(texture)
	pdb.gimp_selection_none(image)
	#32. Now we'll do the bump maps for the bevels. Turn off all the eyes except for the color texture we
	#just made and the first text layer.
	pdb.gimp_item_set_visible(inlay,False)
	pdb.gimp_item_set_visible(inlay_copy,False)
	#33. Select the Text 1 layer.
	#34. Bump Map with the following settings.
	pdb.plug_in_bump_map(image,layer,bump1,135,36.15,4,0,0,0,0,True,False,0)
	#35. Turn on the eye on the second text layer and select it.
	pdb.gimp_item_set_visible(inlay,True)
	#36. Bump this layer with the same settings only use bump 2 layer as the map.
	pdb.plug_in_bump_map(image,inlay,bump2,135,36.15,4,0,0,0,0,True,False,0)
	#37. On to the last text layer at the top. Turn on the eye.
	pdb.gimp_item_set_visible(inlay_copy,True)
	#38. Duplicate it and alpha lock it
	inlay_copy_copy = pdb.gimp_layer_new_from_drawable(inlay_copy,image)
	pdb.gimp_image_insert_layer(image,inlay_copy_copy,None,0)
	pdb.gimp_layer_set_lock_alpha(inlay_copy_copy,1)
	pdb.gimp_context_set_foreground((255,255,255))
	pdb.gimp_context_set_background((0,0,0))
	#39. With the Gradient Tool, foreground white, background black, stroke a linear gradient from the
	# bottom straight up.
	pdb.gimp_edit_blend(inlay_copy_copy,FG_BG_RGB_MODE,NORMAL_MODE,GRADIENT_LINEAR,100,0,REPEAT_NONE,False,False,1,0,True,0,y2,0,y1)
	#40. Bump map the text layer with bump 3 layer as the map.
	pdb.plug_in_bump_map(image,inlay_copy,bump3,135,62.35,7,0,0,0,0,True,False,0)
	#41. And since I'm dumb and didn't invert the map layer I'm going to sink the inlay instead of having it
	# pillow up by inverting the colors of the layer.
	pdb.gimp_invert(inlay_copy)
	#42. Then we'll take the gradient layer and grain merge it at 50% opacity
	pdb.gimp_layer_set_opacity(inlay_copy_copy,50)
	#43. Turn off the eyes on the texture layer and make sure only the text layers are visible. Then merge all
	#visible layers.
	pdb.gimp_item_set_visible(texture,False)
	gray_text = pdb.gimp_image_merge_visible_layers(image,EXPAND_AS_NECESSARY)
	#44. Now duplicate the grey text layer and slide the texture layer between the two. Set the texture mode
	#to grain merge at 75% opacity and the top grey layer to overlay.
	gray_text_copy = pdb.gimp_layer_new_from_drawable(gray_text,image)
	pdb.gimp_image_insert_layer(image,gray_text_copy,None,0)
	pdb.gimp_image_raise_item(image,texture)
	pdb.gimp_item_set_visible(texture,True)
	pdb.gimp_layer_set_mode(texture,GRAIN_MERGE_MODE)
	pdb.gimp_layer_set_opacity(texture,75)
	pdb.gimp_layer_set_mode(gray_text_copy,OVERLAY_MODE)
	#45. We want this to pop more though. New layer from visible at the very top.
	visible = pdb.gimp_layer_new_from_visible(image,image,"visible")
	pdb.gimp_image_insert_layer(image,visible,None,0)
	#46. Gaussian Blur by 2. Set layer mode to overlay
	pdb.plug_in_gauss(image,visible,2.0,2.0,1)
	pdb.gimp_layer_set_mode(visible,OVERLAY_MODE)
	#47. Duplicate the original text layer, move the copy to the very top, set layer mode to grain merge and
	#opacity to 36%.
	gray_text_copy2 = pdb.gimp_layer_new_from_drawable(gray_text,image)
	pdb.gimp_image_insert_layer(image,gray_text_copy2,None,0)
	pdb.gimp_layer_set_mode(gray_text_copy2,GRAIN_MERGE_MODE)
	pdb.gimp_layer_set_opacity(gray_text_copy2,36)
	#48. When you're happy with the way things are, merge visible layers.
	bismuth_text = pdb.gimp_image_merge_visible_layers(image,EXPAND_AS_NECESSARY)
	#=========== DONE TEXT THE FOLLOW IS FOR THE ZOOM EFFECT ================================================
	if zoom == 1:
		#49. Now to make the zoom, duplicate the text layer again.
		bismuth_copy = pdb.gimp_layer_new_from_drawable(bismuth_text,image)
		pdb.gimp_image_insert_layer(image,bismuth_copy,None,0)
		#50. Motion blur zoom with the settings from the image. Make sure Blur outward is unchecked. Align
		#the X and Y settings so it's nice and in the middle of the letter. You can fiddle with it from the preview.
		#It should look somewhat like that.
		pdb.plug_in_mblur_inward(image,bismuth_copy,2,55,0,image.width/2,image.height/2)
		#51. Duplicate and merge down the zoomed layer about 5 times. Maybe 6. You want it to be 100%
		#opaque. This is what it should look like.
		for i in range(0,6):
			motion_copy = pdb.gimp_layer_new_from_drawable(bismuth_copy,image)
			pdb.gimp_image_insert_layer(image,motion_copy,None,0)
			bismuth_copy = pdb.gimp_image_merge_down(image,motion_copy,EXPAND_AS_NECESSARY)
		#52. Duplicate the zoomed layer and desaturate it with the luminosity button selected.
		desaturate_motion = pdb.gimp_layer_new_from_drawable(bismuth_copy,image)
		pdb.gimp_image_insert_layer(image,desaturate_motion,None,0)
		pdb.gimp_desaturate_full(desaturate_motion,DESATURATE_LUMINOSITY)
		#53. Apply chrome curves kind of like these ones here to the grey zoom layer.
		pdb.gimp_curves_spline(desaturate_motion,HISTOGRAM_VALUE,14,[0,0,30,9,53,161,93,15,150,223,195,28,255,255])
		#54. Make a new layer at the top and render some plasma. Use the following settings or any seed you
		#like that has colors that complement the text. I went with one that has a lot of blue. Make sure you slide
		#turbulence all the way to the right though.
		plasma = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"plasma",100,OVERLAY_MODE)
		pdb.gimp_image_insert_layer(image,plasma,None,0)
		pdb.plug_in_plasma(image,plasma,1748050271,7.0)
		#55. Set the layer mode to overlay and merge the plasma layer down.
		plasma = pdb.gimp_image_merge_down(image,plasma,EXPAND_AS_NECESSARY)
		#56. Set the layer mode to hard light and the opacity to 60%.
		pdb.gimp_layer_set_mode(plasma,HARDLIGHT_MODE)
		pdb.gimp_layer_set_opacity(plasma,60)
		#57. Should look at something like this right now. If you like what you see, merge the layer down.
		motion = pdb.gimp_image_merge_down(image,plasma,EXPAND_AS_NECESSARY)
		#58. Move the text layer on top of the zoom layer.
		pdb.gimp_image_raise_item(image,bismuth_text)
		#manual steps of painting skipped #59, 60
		
		#61. Merge the text layer down to the zoom layer, duplicate it and lock the alpha channel, then fill it
		#with black.
		bismuth_fulltext = pdb.gimp_image_merge_down(image,bismuth_text,EXPAND_AS_NECESSARY)
		bismuth_shadow = pdb.gimp_layer_new_from_drawable(bismuth_fulltext,image)
		pdb.gimp_image_insert_layer(image,bismuth_shadow,None,1)
		pdb.gimp_layer_set_lock_alpha(bismuth_shadow,1)
		pdb.gimp_context_set_foreground((0,0,0))
		pdb.gimp_edit_fill(bismuth_shadow,FOREGROUND_FILL)
		#62. Unlock the alpha channel and gaussian blur by 18.
		pdb.gimp_layer_set_lock_alpha(bismuth_shadow,0)
		pdb.plug_in_gauss(image,bismuth_shadow,18.0,18.0,1)
		pdb.gimp_layer_set_offsets(bismuth_shadow,5,15)
		pdb.gimp_layer_resize_to_image_size(bismuth_shadow)
		pdb.gimp_item_set_name(bismuth_shadow,"Bismuth Shadow")
	
	pdb.gimp_display_delete(new_display)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_bismuth",                           
	"Create Bismuth Text Effect",
	"Create Bismuth Text Effect...",
	"Tin Tran",
	"Tin Tran",
	"2014",
	"<Image>/Python-Fu/Bismuth...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_TOGGLE, "zoom",   "Apply Zoom and Shadow:", 1),
	],
	[],
	python_bismuth)

main()
