#!/usr/bin/env python

# bring_in_animation.py Rel 2
# Created by Tin Tran
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Fix bug where it was using active path and not the selected bringin path, now it actually uses bringin path properly.
from gimpfu import *
import random
def python_tt_bring_in_animation(image, layer, steps, bringvector, drawhand, handlayer, penvector) : #FUNCTION DEFINITION
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	pdb.gimp_selection_none(image)
	#creates a working layer
	
	
	#Get piece's location deatails
	piece_offset_x,piece_offset_y = pdb.gimp_drawable_offsets(layer)
	#get location of pen vector, first point only
	num_strokes_piece, stroke_ids_piece = pdb.gimp_vectors_get_strokes(bringvector)
	piece_type,piece_num_points,piece_controlpoints,piece_closed = pdb.gimp_vectors_stroke_get_points(bringvector,stroke_ids_piece[0])
	#get last point as the absolute location
	absolute_piece_x,absolute_piece_y = piece_controlpoints[-4:-2]


	#if draw hand move hand layer on very top
	if drawhand == 1:
		pdb.gimp_image_raise_item_to_top(image,handlayer)
		hand_offset_x,hand_offset_y = pdb.gimp_drawable_offsets(handlayer)
		#get location of pen vector, first point only
		num_strokes_pen, stroke_ids_pen = pdb.gimp_vectors_get_strokes(penvector)
		pen_type,pen_num_points,pen_controlpoints,pen_closed = pdb.gimp_vectors_stroke_get_points(penvector,stroke_ids_pen[0])
		absolute_pen_x,absolute_pen_y = pen_controlpoints[2:4]

	#new image used for animation
	new_image = pdb.gimp_image_new(image.width,image.height,RGB)
	new_display = pdb.gimp_display_new(new_image)

	#get active vector and work with it
	vectors = bringvector
	num_strokes, stroke_ids = pdb.gimp_vectors_get_strokes(vectors)

	#get total stroke length
	total_length = 0
	for i in range(0,num_strokes):
		stroke_id = stroke_ids[i] #current stroke
		current_length = pdb.gimp_vectors_stroke_get_length(vectors,stroke_id,0.01)
		total_length += current_length


	step_length = float(total_length)/steps #length per step
	for i in range(0,num_strokes):
		stroke_id = stroke_ids[i] #current stroke
		current_length = pdb.gimp_vectors_stroke_get_length(vectors,stroke_id,0.01)
		current_at = 0.01
		x,y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,current_at,0.01)
		oldx,oldy = x,y
		while current_at < current_length-0.01:
			#x,y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,current_at,0.01)	
			#pdb.gimp_paintbrush_default(work_layer,4,[oldx,oldy,x,y]) #paint to new point
			next_at = (current_at + step_length)
			if next_at > current_length-0.01:
				next_at = current_length-0.01
			this_at = current_at
			while this_at < next_at:
				nox,noy,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,this_at,0.01)	
				this_at += 1
				if this_at > current_length-0.01:
					this_at = current_length-0.01
				nx,ny,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,this_at,0.01)	
				#Do like the paint paths in steps but dont' paint anything, only moving animation around, comment below
				#pdb.gimp_paintbrush_default(work_layer,4,[nox,noy,nx,ny])
			#pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
			oldx,oldy = x,y
			current_at += step_length
			#drop shadow
			#if dropshadow == 1:
			#	pdb.script_fu_drop_shadow(image,work_layer,shadow_offset_x,shadow_offset_y,shadow_blur_radius,shadow_color,shadow_opacity,1)	


			#move layer/piece to location to make it look like it's being moved by hand
			move_offset_x = nx - absolute_piece_x + piece_offset_x
			move_offset_y = ny - absolute_piece_y + piece_offset_y
			pdb.gimp_layer_set_offsets(layer,move_offset_x,move_offset_y)


			#moves the hand layer into position of last drawn stroke
			if drawhand == 1:
				move_offset_x = nx - absolute_pen_x + hand_offset_x
				move_offset_y = ny - absolute_pen_y + hand_offset_y
				pdb.gimp_layer_set_offsets(handlayer,move_offset_x,move_offset_y)

			new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"painted")
			pdb.gimp_image_insert_layer(new_image,new_layer,None,0)

			#if drawhand and last frame create an extra frame with out hand(by moving it out of frame)
			if drawhand == 1 and current_at >= current_length-0.01:
				pdb.gimp_layer_set_offsets(handlayer,image.width,image.height)
				#creates extra frame
				new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"painted")
				pdb.gimp_image_insert_layer(new_image,new_layer,None,0)

			#delete shadow for next fresh next round
			#if dropshadow == 1:
			#	pdb.gimp_image_remove_layer(image,image.layers[1])
			
	#if we drew hand, move hand back to original position, just in case use wants to rerun script it's at starting state 		
	if drawhand == 1:
		pdb.gimp_layer_set_offsets(handlayer,hand_offset_x,hand_offset_y)

	#move it to what it was before we did anything so user can run script again if they wish	
	pdb.gimp_layer_set_offsets(layer,piece_offset_x,piece_offset_y)
	#restore to what it was before we ran script
	
	#YOUR CODE ENDS ========================
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	#return

register(
	"python_fu_tt_bring_in_animation",
	"Bring in animation with optional hand",
	"Bring in animation with optional ha`nd",
	"Tin Tran",
	"Tin Tran",
	"July 2019",
	"<Image>/Python-Fu/Bring in Animation...",
	"*",      # Create a new image, don't work on an existing one
	[ 
	#INPUT
	(PF_SPINNER, "steps", "frames:", 50, (10,5000,1)),
	(PF_VECTORS, "bringvector", "Bring-in vector:",None),
	(PF_TOGGLE, "drawhand", "Draw hand:" , 0),
	(PF_LAYER, "handlayer", "Hand layer (Used if Draw hand):",None),
	(PF_VECTORS, "penvector", "Pen location vector (Used if Draw hand):",None),
	#INPUT ENDS
	],
	[],
	python_tt_bring_in_animation)

main()