#!/usr/bin/env python

# budget_spectacular.py Release 2
# Created by Tin Tran http://gimplearn.net
# This plug-in was requested by Mapsking at https://gimplearn.net/viewtopic.php?p=27452#p27452
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Add Alpha-lock all layers (default-YES) for easy manual recoloring after plug-in finishes.
# Rel 3: Doesn't require an image to be opened. Can execute plug-in anytime. And you enter squares per row squares per column instead of total squares.
from gimpfu import *

def python_tt_budget_spectacular(total_cost,squares_row,squares_column,top_text,top_text_color,grid_color,arrow_color_1,arrow_color_2,label_color_1,label_color_2,alpha_lock): #FUNCTION DEFINITION
	#pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#YOUR CODE BEGINS=======================
	width,height = 2550,3300
	#Create new image with white background layer
	new_image = pdb.gimp_image_new(width,height,RGB)
	new_display = pdb.gimp_display_new(new_image)
	white_background = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,"White background",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,white_background,None,0)
	#pdb.gimp_edit_fill(white_background,FILL_WHITE)
	#commenting out above and using below blend is faster on GIMP 2.10
	pdb.gimp_context_set_foreground((255,255,255))
	pdb.gimp_context_set_background((255,255,255))
	pdb.gimp_edit_blend(white_background,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,0,0,1,1)
	#font 118, offset shadow (7,7) of color (85,85,85)
	fontname = 'Sans-serif'
	#top text shadow
	pdb.gimp_context_set_foreground((85,85,85))
	text_shadow = pdb.gimp_text_fontname(new_image,None,0,0,top_text,0,TRUE,118,0,fontname)
	pdb.gimp_layer_set_offsets(text_shadow,(width-text_shadow.width)/2+7,42+7)
	pdb.gimp_item_set_name(text_shadow,"Top Text Shadow")
	#top text 
	pdb.gimp_context_set_foreground(top_text_color)
	text= pdb.gimp_text_fontname(new_image,None,0,0,top_text,0,TRUE,118,0,fontname)
	pdb.gimp_layer_set_offsets(text,(width-text.width)/2,42)
	pdb.gimp_item_set_name(text,"Top Text")
	#add an underline for top text
	top_text_underline = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,"Top text underline",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,top_text_underline,None,0)
	pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,(width-text.width)/2,42+text.height,text.width,5)
	
	#pdb.gimp_edit_fill(top_text_underline,FILL_FOREGROUND)
	#commenting out above and using below blend is faster on GIMP 2.10
	pdb.gimp_context_set_background(top_text_color)
	pdb.gimp_edit_blend(top_text_underline,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,0,0,1,1)
	
	#add grid
	grid = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,"Grid",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,grid,None,0)
	pdb.gimp_context_set_foreground(grid_color)
	#squares = int(math.ceil(squares/20.0) * 20)
	grid_x_tiles = int(squares_row)
	grid_y_tiles = int(squares_column)
	squares = grid_x_tiles*grid_y_tiles #new spec by x*y instead of squares.
	square_dim = 92
	grid_width = grid_x_tiles * square_dim
	grid_height = grid_y_tiles * square_dim
	pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,(width-grid_width)/2-2,42+text.height+5+50-2,grid_width+5,grid_height+5)
	pdb.plug_in_grid(new_image,grid,5,square_dim,42+text.height+5+50,grid_color,255,5,square_dim,(width-grid_width)/2,grid_color,255,0,0,0,grid_color,255)
	#add label
	label_width = 1544
	label_height = 190
	grid_bottom = 42+text.height+5+50-2+grid_height+5
	label = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,"Label",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,label,None,0)
	pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,(width-label_width)/2,grid_bottom+34,label_width,label_height)
	pdb.script_fu_selection_rounded_rectangle(new_image,label,25,0)
	pdb.gimp_context_set_foreground((0,0,0))
	#pdb.gimp_edit_fill(label,FILL_FOREGROUND)
	#commenting out above and using below blend is faster on GIMP 2.10
	pdb.gimp_context_set_background((0,0,0))
	pdb.gimp_edit_blend(label,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,0,0,1,1)
	
	#add label gradient
	label_gradient = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,"Label Gradient",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,label_gradient,None,0)
	pdb.gimp_selection_shrink(new_image,6)
	pdb.gimp_context_set_foreground(label_color_1)
	pdb.gimp_context_set_background(label_color_2)
	pdb.gimp_edit_blend(label_gradient,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,\
		(width-label_width)/2              ,grid_bottom+34+grid_height/2,\
		(width-label_width)/2 + label_width,grid_bottom+34+grid_height/2)
	#add arrow
	arr_refx = width/2  #arrow reference x, we use this point to draw arrow (center of label)
	arr_refy = grid_bottom+34+label_height/2 #arrow reference y (is just center of label)
	#set coordinates for arrow
	x1 = arr_refx - 163
	y1 = arr_refy - 32
	x2 = arr_refx - 54
	y2 = y1
	x3 = x2
	y3 = arr_refy - 63
	x4 = arr_refx + 58
	y4 = arr_refy
	x5 = x2
	y5 = arr_refy + 63
	x6 = x2
	y6 = arr_refy + 32
	x7 = x1
	y7 = y6
	grid_bottom = 42+text.height+5+50-2+grid_height+5
	arrow = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,"Arrow",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,arrow,None,0)
	pdb.gimp_image_select_polygon(new_image,CHANNEL_OP_REPLACE,14,[x1,y1,x2,y2,x3,y3,x4,y4,x5,y5,x6,y6,x7,y7])
	pdb.gimp_context_set_foreground((0,0,0))
	#pdb.gimp_edit_fill(arrow,FILL_FOREGROUND)
	#commenting out above and using below blend is faster on GIMP 2.10
	pdb.gimp_context_set_background((0,0,0))
	pdb.gimp_edit_blend(arrow,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,0,0,1,1)
	
	pdb.gimp_selection_shrink(new_image,5)
	pdb.gimp_context_set_foreground(arrow_color_1)
	pdb.gimp_context_set_background(arrow_color_2)
	arrow_gradient = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,"Arrow Gradient",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,arrow_gradient,None,0)
	pdb.gimp_edit_blend(arrow_gradient,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,\
		x1,y4,\
		x4,y4)
	#Total cost over number of squares texts
	pdb.gimp_context_set_foreground((0,0,0))
	font_refy = arr_refy
	fontsize = 56
	text = "$" + format(int(total_cost),",")
	text_cost = pdb.gimp_text_fontname(new_image,None,0,0,text,0,TRUE,fontsize,0,fontname)
	text = str(squares) + " squares"
	text_squares = pdb.gimp_text_fontname(new_image,None,0,0,text,0,TRUE,fontsize,0,fontname)
	font_refx = width/2 - label_width/2 + max(text_cost.width,text_squares.width)/2 + 50 
	pdb.gimp_layer_set_offsets(text_cost,font_refx-text_cost.width/2,font_refy-text_cost.height)
	pdb.gimp_layer_set_offsets(text_squares,font_refx-text_squares.width/2,font_refy)
	#Total cost underline/divide
	text_cost_underline = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,"Total Cost underline",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,text_cost_underline,None,0)
	line_width = max(text_cost.width,text_squares.width)
	pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,font_refx-line_width/2,font_refy,line_width,5)
	#pdb.gimp_edit_fill(text_cost_underline,FILL_FOREGROUND)
	#commenting out above and using below blend is faster on GIMP 2.10
	pdb.gimp_context_set_background((0,0,0))
	pdb.gimp_edit_blend(text_cost_underline,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0,REPEAT_NONE,FALSE,FALSE,3,0.2,FALSE,0,0,1,1)
	#Each square text and each square worth text.
	value_per_square = float(total_cost)/squares
	text = "Each square"
	text_each_square = pdb.gimp_text_fontname(new_image,None,0,0,text,0,TRUE,fontsize,0,fontname)
	text = "Equals: $" + "{0:,.2f}".format(round(math.ceil(value_per_square*100)/100.0,2))
	text_equals = pdb.gimp_text_fontname(new_image,None,0,0,text,0,TRUE,fontsize,0,fontname)
	font_refx = width/2 + label_width/2 - max(text_each_square.width,text_equals.width)/2 - 50 
	pdb.gimp_layer_set_offsets(text_each_square,font_refx-text_each_square.width/2,font_refy-text_each_square.height)
	pdb.gimp_layer_set_offsets(text_equals,font_refx-text_equals.width/2,font_refy)
	
	#if user chose to lock alpha, we lock all layers alpha.
	if alpha_lock == 1:
		for layer in new_image.layers:
			pdb.gimp_layer_set_lock_alpha(layer,1)

	#YOUR CODE ENDS ========================
	pdb.gimp_context_pop()
	#pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_budget_spectacular",
	"Create a Savings Grid (Budget Spectacular)",
	"Create a Savings Grid (Budget Spectacular)...",
	"Tin Tran",
	"Tin Tran",
	"June 2019",
	"<Toolbox>/Python-Fu/Budget Spectacular...",
	"",      # Create a new image, don't work on an existing one
	[ 
	#INPUT BEGINS
	(PF_SPINNER, "total_cost", "Total Cost($):", 5000, (10, 50000, 1)),
	(PF_SPINNER, "squares_row", "Squares per row ([width] max 20):", 20, (1, 20, 1)),
	(PF_SPINNER, "squares_column", "Squares per column ([height] max 30):", 15, (1, 30, 1)),
	(PF_STRING, "top_text", "Top Text:", "DISNEY VACATION!!!"),
	(PF_COLOR, "top_text_color", "Top Text Color:", (185, 228, 247)),
	(PF_COLOR, "grid_color", "Grid Color", (176, 102, 148)),
	(PF_COLOR, "arrow_color_1", "Arrow Color 1:", (220, 116, 162)),
	(PF_COLOR, "arrow_color_2", "Arrow Color 2:", (218, 221, 109)),
	(PF_COLOR, "label_color_1", "Label Color 1 (Where cost is shown):", (179, 226, 245)),
	(PF_COLOR, "label_color_2", "Label Color 2 (Where cost is shown):", (254, 251, 204)),
	(PF_TOGGLE, "alpha_lock", "Alpha-lock all layers (so they can be easily colored):", 1),
	#(PF_OPTION, "arrow_side", "Arrows Ends:", SIDE_END, SIDE_NAMES),
	#(PF_TOGGLE, "arrow_close", "Arrows Close:", 0),
	#(PF_SPINNER, "border_width", "Border Width (bevelled width):", 10, (0, 500, 1)),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_blur_radius", "Shadow Blur Radius:", 15, (0,1024,1)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 100, (0,100,1)),
	#INPUT ENDS
	],
	[],
	python_tt_budget_spectacular)

main()