#!/usr/bin/env python

# character-layers-from-text.py
# Tries to break recreate text layer with individual layers of characters (intended to be used with font jiggle)
# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Put every layers inside a layer group (requested by Blighty) and creates a new image for friendliness.
import math
import string
#import Image
from gimpfu import *
from array import array
import sys
import random
def python_character_layers_from_text(image, textlayer, justify):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	text = pdb.gimp_text_layer_get_text(textlayer)
	
	font = pdb.gimp_text_layer_get_font(textlayer)
	font_size,unit = pdb.gimp_text_layer_get_font_size(textlayer)

	new_image = pdb.gimp_image_new(image.width,image.height,RGB)
	new_display = pdb.gimp_display_new(new_image) #creates a display for our new image
	new_layer_group = pdb.gimp_layer_group_new(new_image)
	pdb.gimp_image_insert_layer(new_image,new_layer_group,None,0)
	pdb.gimp_item_set_name(new_layer_group,"Character Layers from Text")

	offset_x,offset_y = pdb.gimp_drawable_offsets(textlayer)

	lines = text.split("\n")
	starting_y = offset_y
	for line in lines:
		textlayers = []
		totalwidth = 0
		for i in range(0,len(line)):
			char = line[i]
			if ((char == " ") and (i > 0) and (i < len(line)-1)): #if space and it's between 2 chararacters we treat it special
				before = line[i-1]
				space = line[i]
				after = line[i+1]
				special = before + space + after
				special_layer = pdb.gimp_text_fontname(new_image,None,0,0,special,0,TRUE,font_size,unit,font)
				before_layer = pdb.gimp_text_fontname(new_image,None,0,0,before,0,TRUE,font_size,unit,font)
				after_layer = pdb.gimp_text_fontname(new_image,None,0,0,after,0,TRUE,font_size,unit,font)
				layer_height = special_layer.height
				layer_width = special_layer.width - before_layer.width - after_layer.width
				text_layer = pdb.gimp_layer_new(new_image,layer_width,layer_height,RGBA_IMAGE,"Space",100,LAYER_MODE_NORMAL)
				pdb.gimp_image_insert_layer(new_image,text_layer,new_layer_group,0)
				pdb.gimp_image_remove_layer(new_image,before_layer)
				pdb.gimp_image_remove_layer(new_image,after_layer)
				pdb.gimp_image_remove_layer(new_image,special_layer)
			else:
				text_layer = pdb.gimp_text_fontname(new_image,None,0,0,char,0,TRUE,font_size,unit,font)
				pdb.gimp_image_reorder_item(new_image,text_layer,new_layer_group,0) #puts it inside our layer group for organization
			totalwidth += text_layer.width #increment total width to calculate for placement and justify.
			size=100
			fontname="Arial"
			width, height, ascent, descent = pdb.gimp_text_get_extents_fontname(char, size, 0, fontname)
			#pdb.gimp_layer_resize_to_image_size(text_layer)
			wt = pdb.gimp_drawable_width(char)
			ht = pdb.gimp_drawable_height(char)
			angle_slope=25
			corner = math.fabs(angle_slope)			
			rad =  corner*0.01745329252
			an = ht*math.tan(rad)
			pdb.gimp_item_transform_perspective(char, an, 0,   wt + an, 0,   wt, ht,  0, ht)
			textlayers.append(text_layer)
		if (justify == 0): #left justify
			starting_x = offset_x
		elif (justify == 1): #centered
			starting_x = offset_x + (textlayer.width/2) - totalwidth/2
		elif (justify == 2): #right justify
			starting_x = offset_x + textlayer.width - totalwidth
		for text_layer in textlayers: #loop through and move it into its placement
			pdb.gimp_layer_set_offsets(text_layer,starting_x,starting_y)
			starting_x += text_layer.width
			
		starting_y += text_layer.height #increment it for next round/line
	#textlayer.visible = False	

	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_character_layers_from_text",
	"Character layers from Text",
	"Character layers from Text",
	"Tin Tran",
	"Tin Tran",
	"December 2019",
	"<Image>/Python-Fu/Character Layers from Text...",             #Menu path
	"RGB*, GRAY*",
	[
	(PF_OPTION,"justify",   "Justified:", 0, ["Left Justified","Centered","Right Justified"]),
	#(PF_SPINNER, "xscale", "Scale X Percentage:", 100, (50, 200, 1)),
	#(PF_COLOR, "toalpha", "Color to Make alpha (to turn image into silhoutte):", (255, 255, 255)),
	#(PF_COLOR, "light", "Icon Color:", (255, 156, 71)),
	#(PF_COLOR, "dark", "Icon Slightly Darker Color:", (183, 110, 51)),
	],
	[],
	python_character_layers_from_text)


main()
