#!/usr/bin/env python

# circle_map.py Rel 2
# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Added option to map to square as well.
import math
import string
#import Image
from gimpfu import *
from array import array
import sys
def python_circle_map(image, layer, option) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	width = min(layer.width,layer.height)
	if not pdb.gimp_drawable_has_alpha(layer):
		pdb.gimp_layer_add_alpha(layer)
	pdb.gimp_layer_scale(layer,width,width,FALSE)
	midx = width/2.0
	midy = width/2.0
	new_layer = pdb.gimp_layer_new(image,width,width,RGBA_IMAGE,'circle map',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,new_layer,None,0)
	coords = []
	for y in range(0,int(width)):
		for x in range(0,int(width)):
			#get the color at that rectangle coord
			if option == 0: #mapping to circle
				num_channels,pixel = pdb.gimp_drawable_get_pixel(layer,x,y)
			#convert to unit rectangle
			x1 = (x-midx)/midx
			y1 = (y-midy)/midy
			#map to unit circle
			x_ = x1 * (1 - (y1*y1)/2)**0.5
			y_ = y1 * (1 - (x1*x1)/2)**0.5
			#scale it back
			x_ = int(x_ * midx + midx)
			y_ = int(y_ * midy + midy)
			if option == 0: #mapping to circle
				pdb.gimp_drawable_set_pixel(new_layer,x_,y_,num_channels,pixel)
			else: #mapping to square, save coordinates to work in reverse
				coords.append([x_,y_,x,y])
		if option == 0:	#mapping to circle	
			pdb.gimp_progress_update(float(y)/width)
	if option == 1: #mapping to square, do work here
		currenti = 0
		totalpixels = len(coords)
		for coord in coords:
			num_channels,pixel = pdb.gimp_drawable_get_pixel(layer,coord[0],coord[1])
			pdb.gimp_drawable_set_pixel(new_layer,coord[2],coord[3],num_channels,pixel)	
			currenti += 1
			pdb.gimp_progress_update(float(currenti)/totalpixels)
	pdb.gimp_drawable_update(new_layer,0,0,width,width)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_circle_map",
	"Maps image to/from circle",
	"Maps image to/from circle",
	"Tin Tran",
	"Tin Tran",
	"August 2019",
	"<Image>/Python-Fu/Circle map...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_OPTION,"option",   "Mapping:", 0, ["To Circle","To Square"]),
	],
	[],
	python_circle_map)

main()
