#!/usr/bin/env python
# coding=utf-8
# -*- encoding: utf-8 -*-
# collage_based_on_paths.py Rel 7
# Created by Tin Tran
# Comments directed to http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Introduced other paths below the top most path that define torn paper shapes
# Rel 3: This version doesn't use path as our torn paper is random alignment anyways. So all paths are used for paper torn shapes
# Rel 4: for loop instead of infinite loop, should perform better.
import random
from gimpfu import *
import math
import time
brush_spacing = 88;
#Copied this from ofn-text-along-path.py returns angle in radians
def computeOrientedSlope(dx,dy,slope): #This is a better version than below computeThetaWithSlope function.
	if abs(slope) > 100000: # very vertical
		return math.atan2(dy,dx) # no perfect, but properly oriented
		# keep dx/dy signs but give then the same ratio as in slope
	return math.atan2(math.copysign(slope,dy),math.copysign(1,dx))
#just returns distance square as we don't care to get distance but just degree to compare them
def distance(p0, p1):
	return (p0[0] - p1[0])*(p0[0] - p1[0]) + (p0[1] - p1[1])*(p0[1] - p1[1])

#reads points and angles at each point into points array
def read_points_angles(image,layer,spacing,offset_angle):
	points = []
	if len(image.vectors) < 1:
		pdb.gimp_message("You must define a path/vector")
		raise Exception('You must define a path/vector')
	vectors = image.vectors[0]
	strokes = vectors.strokes
	for stroke in strokes: #loop through all strokes
		stroke_length = stroke.get_length(0.01) - 1.1 #subtract 1.1 so we can get the point at +1 and its' still valid
		sections = int(stroke_length/spacing)
		for i in range(0,sections+1): #loop from 0 to sections
			dist1 = i * spacing
			dist2 = dist1 + 1 #1
			x1,y1,slope1,valid1 = stroke.get_point_at_dist(dist1,0.01)
			x2,y2,slope2,valid2 = stroke.get_point_at_dist(dist2,0.01)
			rad_angle = computeOrientedSlope(x2-x1, y2-y1, slope1)
			angle = math.degrees(rad_angle)
			angle = (angle + offset_angle + random.randrange(0,30) - 15) % 360
			if (angle > 180): #make it fit between -180 and 180
				angle = -180 + (angle-180)
			points.append([[x1,y1],angle])
			#pdb.gimp_message(str(points[len(points)-1]))
	return points
#main method
#simply rotate selection by degrees if there is a selection
def rotate_selection(image,degrees):
	selection = pdb.gimp_image_get_selection(image)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if non_empty == TRUE:
		swidth = x2-x1; sheight = y2-y1;
		pdb.gimp_item_transform_rotate(selection,math.radians(degrees),FALSE,x1+swidth/2,y1+sheight/2)
#move selection with center to x,y coordinate		
def move_selection(image,mx,my):
	selection = pdb.gimp_image_get_selection(image)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if non_empty == TRUE:
		swidth = x2-x1; sheight = y2-y1;
		pdb.gimp_item_transform_translate(selection,mx-(x1+swidth/2),my-(y1+sheight/2))
#scale selection only passing width so that it'll keep aspect ratio of the selection shape
def scale_selection(image,percentage):
	selection = pdb.gimp_image_get_selection(image)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if non_empty == TRUE:
		swidth = float(x2-x1); sheight = float(y2-y1);
		newwidth = image.width * percentage; newheight = image.height * percentage;
		pdb.gimp_item_transform_scale(selection,0,0,newwidth,newheight)
def flip_selection(image):
	selection = pdb.gimp_image_get_selection(image)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if non_empty == TRUE:
		swidth = float(x2-x1); sheight = float(y2-y1);
		pdb.gimp_item_transform_flip(selection,x1+swidth/2,y1,x1+swidth/2,y2)

def python_collage_based_on_paths_2(image, layer, width, length, offset_angle):
	brush_spacing = 88*layer.width/6000 #if it was 6000 wide we use 88 so we'll adjust to width for smaller images
	if len(image.layers)==3:
		collage_layer = image.layers[0]
		layer = image.layers[1]; #Just in case we don't have right layer selected
		texture_layer = image.layers[2] #this can be changed but we'll just bottom layer for now
	else:
		layer = image.layers[0]; #Just in case we don't have right layer selected
		texture_layer = image.layers[1] #this can be changed but we'll just bottom layer for now
	pdb.gimp_context_push()
	#points = read_points_angles(image,layer,brush_spacing,offset_angle) #brush_space default to 5 for simplicity for now
	random_angle = 1;
	hwidth = width/2
	hlength = length/2
	qwidth = width/4
	qlength = length/4
	if len(image.layers)==2:
		collage_layer = pdb.gimp_layer_new(image, layer.width, layer.height, RGBA_IMAGE, "Collage", 100, NORMAL_MODE)
		pdb.gimp_image_insert_layer(image, collage_layer, None, 0)

	dp = 0; #when this reaches 100 we flush result so we can see in front end
	piecenum = 0
	starttime = time.time()
	#while 1==1:
	#SELECT ALL THEN SUBTRACT CONTENT FROM collage layer to get points/locations to draw randomly
	pdb.gimp_selection_all(image)
	pdb.gimp_image_select_item(image,CHANNEL_OP_SUBTRACT,collage_layer)
	selyupper = layer.height/brush_spacing #loop times
	selxupper = layer.width/brush_spacing
	draw_points = [];
	for sely in range(0,selyupper):
		for selx in range(0,selxupper):
			selxlocation = selx*brush_spacing
			selylocation = sely*brush_spacing
			value = pdb.gimp_selection_value(image,selxlocation,selylocation)
			if value > 0: #if it's in selection
				draw_points.append([selxlocation,selylocation])
	draw_points = sorted(draw_points,key=lambda x:random.randrange(1000000)-500000) #randomize the list
	pdb.gimp_message(str(len(draw_points)) + " POSSIBLE locations to work on...")
	for while_ in range(0,len(draw_points)):
		dp += 1;
		#pick a random point
		#x = random.randrange(0,layer.width)
		#y = random.randrange(0,layer.height)
		#draw_point = [x,y]
		x = draw_points[while_][0]
		y = draw_points[while_][1]
		draw_point = [x,y];

		#draw_point = draw_points[dp]

		shortestdist = 10000000
		if random_angle == 0:
			for p in range(0,len(points)):
				point = points[p]
				dist = distance(draw_point, point[0])
				if dist < shortestdist:
					shortestdist = dist
					draw_angle = point[1] #grab angle
		else:
			draw_angle = random.randrange(0,360);			
		printerror = 1
		
		try:
			#randomly set coordinates to be offset within a square that is half the width
			# x1 = x-qwidth+random.randrange(0,hwidth)
			# x3 = x-qwidth+random.randrange(0,hwidth)
			# x2 = x+width-qwidth+random.randrange(0,hwidth)
			# x4 = x+width-qwidth+random.randrange(0,hwidth)

			# y1 = y-qwidth+random.randrange(0,hwidth)
			# y2 = y-qwidth+random.randrange(0,hwidth)
			# y3 = y+length-qwidth+random.randrange(0,hwidth)
			# y4 = y+length-qwidth+random.randrange(0,hwidth)


			# #myradians = math.atan2(targetY-gunY, targetX-gunX)
			# a1 = math.degrees(math.atan2(y1-y,x1-x))
			# a2 = math.degrees(math.atan2(y2-y,x2-x))
			# a3 = math.degrees(math.atan2(y3-y,x3-x))
			# a4 = math.degrees(math.atan2(y4-y,x4-x))

			# #rotate it by draw angle and possible offset_angle that isn't zero
			# a1 += draw_angle + offset_angle
			# a2 += draw_angle + offset_angle
			# a3 += draw_angle + offset_angle
			# a4 += draw_angle + offset_angle

			# #calculate distances so we can rotate it with cos and sin
			# d1 = ((x1-x)**2 + (y1-y)**2)**0.5
			# d2 = ((x2-x)**2 + (y2-y)**2)**0.5
			# d3 = ((x3-x)**2 + (y3-y)**2)**0.5
			# d4 = ((x4-x)**2 + (y4-y)**2)**0.5

			# #to radians so we can use cos and sin
			# a1 = math.radians(a1)
			# a2 = math.radians(a2)
			# a3 = math.radians(a3)
			# a4 = math.radians(a4)

			# #actual calculation to get rotated points
			# x1 = math.cos(a1) * d1 + x
			# y1 = math.sin(a1) * d1 + y
			# x2 = math.cos(a2) * d2 + x
			# y2 = math.sin(a2) * d2 + y
			# x3 = math.cos(a3) * d3 + x
			# y3 = math.sin(a3) * d3 + y
			# x4 = math.cos(a4) * d4 + x
			# y4 = math.sin(a4) * d4 + y

			# #first select the intended shape
			# pdb.gimp_image_select_polygon(image,CHANNEL_OP_REPLACE,8,[x1,y1,x2,y2,x4,y4,x3,y3])

			# NEW INSTEAD OF SELECTION RANDOM RECTANGLE WE SELECT RANDOM PAPER TORN SHAPE
			# below selects a random path to selection ignoring the topmost path because that is our path that defines to follow along
			#rvector = image.vectors[random.randrange(0,len(image.vectors)-1)+1]
			#new version doesn't need a defined path so use all paths for torn shapes
			rvector = image.vectors[random.randrange(0,len(image.vectors))]
			#pdb.gimp_path_to_selection(image,rvector,CHANNEL_OP_REPLACE,TRUE,FALSE,0,0)
			pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,rvector)

			#make our selection scale it according to 300 (based on testing of 6000px image)
			#rotate to a desired angle acoording to draw_angle of top most path
			#move selection so that center of selection is the random x,y coordinate we chose randomly to do work on
			if random.randrange(0,100) > 49: #randomly flip selection to get even more variety of shapes
				flip_selection(image)
			scale_selection(image,width/300.0)
			rotate_selection(image,draw_angle+offset_angle)
			move_selection(image,x,y)

			#subtract the alpha of layer
			pdb.gimp_image_select_item(image,CHANNEL_OP_SUBTRACT,collage_layer)
			
			#if we have a selection then do work
			non_empty,sx1,sy1,sx2,sy2 = pdb.gimp_selection_bounds(image)
			if non_empty == TRUE: #WE HAVE SELECTION
				piecenum += 1
				pdb.gimp_message("Working on piece #" + str(piecenum))
				timetakensofar = time.time()-starttime
				pdb.gimp_message("Time taken so far:" + str(timetakensofar/60) + " minutes!")
				swidth = sx2-sx1; sheight = sy2-sy1;
				#copy content from layer to paste pattern on texture
				pdb.gimp_edit_copy(layer)
				sarray = []
	
				#set up before we start moving things and measuring closeness to black
				base_diff_layer = pdb.gimp_layer_new_from_drawable(texture_layer,image)
				pdb.gimp_image_insert_layer(image,base_diff_layer,None,0)
				pattern_diff_layer = pdb.gimp_layer_new(image, layer.width, layer.height, RGBA_IMAGE, "Pattern Layer", 100, NORMAL_MODE)
	
				pdb.gimp_image_insert_layer(image,pattern_diff_layer,None,0)
				#select all image to fill pattern
	
				pdb.gimp_selection_none(image)
				num_patterns,pattern_list = pdb.gimp_patterns_list("") #just get the pattern list
				#this is so that when it's in another language we just the first pattern
				#which is the "Clipboard Image" pattern in English
				pdb.gimp_context_set_pattern(pattern_list[0]) 
				pdb.gimp_drawable_edit_fill(pattern_diff_layer,FILL_PATTERN)
	
				diff_layer = pdb.gimp_layer_new_from_drawable(texture_layer,image)
				pdb.gimp_image_insert_layer(image,diff_layer,None,0)
				pdb.gimp_layer_set_mode(diff_layer,LAYER_MODE_DIFFERENCE)
				for flip in range(0,2):
					if flip > 0:
						base_diff_layer = pdb.gimp_drawable_transform_flip_simple(base_diff_layer,0,TRUE,0,FALSE)
						diff_layer = pdb.gimp_drawable_transform_flip_simple(diff_layer,0,TRUE,0,FALSE)
					for rot in range(-1,3):
						if rot > -1: #if it's not the first time we rotate it
							base_diff_layer = pdb.gimp_drawable_transform_rotate_simple(base_diff_layer,rot,TRUE,0,0,FALSE)
							diff_layer = pdb.gimp_drawable_transform_rotate_simple(diff_layer,rot,TRUE,0,0,FALSE)
						for yo in range(0,(sy2-sy1)/brush_spacing):
							for xo in range(0,(sx2-sx1)/brush_spacing):
								x_offset = xo * brush_spacing
								y_offset = yo * brush_spacing
								#offset here wrap around
								pdb.gimp_drawable_offset(base_diff_layer,TRUE,OFFSET_WRAP_AROUND,x_offset,y_offset)
								pdb.gimp_drawable_offset(diff_layer,TRUE,OFFSET_WRAP_AROUND,x_offset,y_offset)

								#create a visible result to sample
								visible = pdb.gimp_layer_new_from_visible(image,image,'difference visible')
								pdb.gimp_image_insert_layer(image,visible,None,0)

								for yi in range(0,texture_layer.height/sheight):
									for xi in range(0,texture_layer.width/swidth):
										#select the rectangle we're looking at
										pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,xi*swidth,yi*sheight,swidth,sheight)	
										# R,std_dev,median,pixels,count,percentile = pdb.gimp_histogram(visible,HISTOGRAM_RED,0,255)
										# G,std_dev,median,pixels,count,percentile = pdb.gimp_histogram(visible,HISTOGRAM_GREEN,0,255)
										# B,std_dev,median,pixels,count,percentile = pdb.gimp_histogram(visible,HISTOGRAM_BLUE,0,255)
										# close_to_black = 0.21*R + 0.72*G + 0.07*B #might need to change this
										#trying just Luminance for more speed
										luminance,std_dev,median,pixels,count,percentile = pdb.gimp_histogram(visible,HISTOGRAM_LUMINANCE,0,255)
										#close_to_black = luminance
										sarray.append([luminance,xi,yi,x_offset,y_offset,rot,flip])
								pdb.gimp_image_remove_layer(image,visible) #remove it so we can do next check
								#pdb.gimp_item_delete(visible)
								pdb.gimp_selection_none(image)
								#offset here wrap around - UNDO previous sets
								pdb.gimp_drawable_offset(base_diff_layer,TRUE,OFFSET_WRAP_AROUND,-x_offset,-y_offset)
								pdb.gimp_drawable_offset(diff_layer,TRUE,OFFSET_WRAP_AROUND,-x_offset,-y_offset)
						rotundo = -1;	
						if rot == 0: #90
							rotundo = 2 #270 + 90 = 360
						elif rot == 1: #180
							rotundo = 1 #180 + 180 = 360
						elif rot == 2: #270
							rotundo = 0 #90	+ 270 = 360
						#UNDO rotation if it was rotate
						if rotundo > -1:	
							base_diff_layer = pdb.gimp_drawable_transform_rotate_simple(base_diff_layer,rotundo,TRUE,0,0,FALSE)
							diff_layer = pdb.gimp_drawable_transform_rotate_simple(diff_layer,rotundo,TRUE,0,0,FALSE)
					if flip > 0: #DO AGAIN TO UNDO
						base_diff_layer = pdb.gimp_drawable_transform_flip_simple(base_diff_layer,0,TRUE,0,FALSE)
						diff_layer = pdb.gimp_drawable_transform_flip_simple(diff_layer,0,TRUE,0,FALSE)			
				#here we sort	
				sarray = sorted(sarray,key=lambda x:x[0]) #sort from closest to black
				xi = sarray[0][1]; yi = sarray[0][2]; #the position we want to put it in to get closest to black
				x_offset = sarray[0][3]; y_offset = sarray[0][4];
				rot = sarray[0][5];	flip = sarray[0][6]
				#if flip then flip
				if flip > 0: #DO FLIP if it needs it when flip == 1
					base_diff_layer = pdb.gimp_drawable_transform_flip_simple(base_diff_layer,0,TRUE,0,FALSE)
					diff_layer = pdb.gimp_drawable_transform_flip_simple(diff_layer,0,TRUE,0,FALSE)
				#if rot > -1 means it has been rotated then we rotate it	
				if rot > -1:
					base_diff_layer = pdb.gimp_drawable_transform_rotate_simple(base_diff_layer,rot,TRUE,0,0,FALSE)
					diff_layer = pdb.gimp_drawable_transform_rotate_simple(diff_layer,rot,TRUE,0,0,FALSE)
				
				#set offset to be closest to black
				pdb.gimp_drawable_offset(base_diff_layer,TRUE,OFFSET_WRAP_AROUND,x_offset,y_offset)
				pdb.gimp_drawable_offset(diff_layer,TRUE,OFFSET_WRAP_AROUND,x_offset,y_offset)

				#intersect to get the shape back
				pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,xi*swidth,yi*sheight,swidth,sheight)
				pdb.gimp_image_select_item(image,CHANNEL_OP_INTERSECT,pattern_diff_layer)
				#edit copy from base_diff_layer
				pdb.gimp_edit_copy(base_diff_layer)

				# #Copied so now we remove all working layers
				# pdb.gimp_image_remove_layer(image,base_diff_layer)
				# pdb.gimp_image_remove_layer(image,pattern_diff_layer)
				# pdb.gimp_image_remove_layer(image,diff_layer)

				#now paste it and move it to the right location
				floating_sel = pdb.gimp_edit_paste(collage_layer,TRUE)
				pdb.gimp_floating_sel_to_layer(floating_sel)
				pdb.gimp_layer_set_offsets(floating_sel,sx1,sy1) #where the selection was

				#merge with working results
				collage_layer = pdb.gimp_image_merge_down(image,floating_sel,CLIP_TO_BOTTOM_LAYER)
				#safety check remove, we should only have 3 bottom layers remaining
				while len(image.layers)>3: #while more than 3 layers
					pdb.gimp_image_remove_layer(image,image.layers[0]); #keep deleting the top one
		except Exception, e:
			if printerror == 1:
				pdb.gimp_message("Errored:" + str(e) + "...continuing")
				printerror = 0
			pass
		if dp % 100 == 0:
			pdb.gimp_displays_flush()
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	pdb.gimp_message("Complete!")
	#return

register(
	"python_fu_collage_based_on_paths_2",
	"Collage based on paths on the working image",
	"Collage based on paths on the working image",
	"Tin Tran",
	"Tin Tran",
	"July 13 2022",
	"<Image>/Python-Fu/Collage Based on Paths 2...",             #Menu path
	"RGB*, GRAY*",
	[
	#(PF_BRUSH, "brush", "Brush:", 0),
	(PF_SPINNER, "width", "Estimated Piece Width:", 300, (10, 1000, 1)),
	(PF_SPINNER, "length", "Estimated Piece Length:", 800, (10, 1000, 1)),
	(PF_SPINNER, "offset_angle", "Offset angle (90 is default because we default a collage shape generally vertically):", 90, (0,360, 1)),
	#(PF_TOGGLE, "random_angle",   "Use random angle (when doing generally round blotches):", 0),
	#(PF_SPINNER, "px_height", "Height of each tile image:", 100, (1, 1000, 50)),
	],
	[],
	python_collage_based_on_paths_2)

main()
