#!/usr/bin/env python

# color-cast-reduction.py
# Created by David Marsden
#
# Based on Krikor-color-cast-reduction
# Created by Tin Tran https://gimplearn.net
#
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
#
from gimpfu import *

def python_color_cast_reduction(image,layer) :
	
	has_alpha = pdb.gimp_drawable_has_alpha(layer)
	
	if has_alpha == TRUE:		
		
		channel_A = pdb.gimp_channel_new_from_component(image,CHANNEL_ALPHA,"A")								# alpha channel copy		
		pdb.gimp_image_insert_channel(image,channel_A,None,0)													# insert alpha channel copy		
		pdb.gimp_edit_copy(channel_A)																			# copy channel to re-insert later		
		pdb.gimp_image_remove_channel(image,channel_A)															# delete channel - no longer needed
		pdb.gimp_image_set_active_layer(image, layer) 															# make layer active
		pdb.gimp_layer_flatten(layer)																			# remove alpha channel		
		
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,layer_copy,None,0)	
	
	pdb.plug_in_pixelize2(image,layer_copy,layer_copy.width,layer_copy.height)									# blur pixelize with image width and image height
	
	pdb.gimp_drawable_invert(layer_copy,FALSE) 																	# colors - invert FALSE for Linear
	
	pdb.gimp_layer_set_mode(layer_copy,LAYER_MODE_OVERLAY)														# layer mode to overlay
	
	layer_copy1 = pdb.gimp_layer_new_from_drawable(layer,image)													# add layer to become contrast mask
	pdb.gimp_image_insert_layer(image,layer_copy1,None,0)
	
	pdb.gimp_drawable_desaturate(layer_copy1,DESATURATE_LUMINANCE)												# desaturate layer
	
	pdb.gimp_layer_set_mode(layer_copy1,LAYER_MODE_OVERLAY)														# overlay mode	
	pdb.gimp_layer_set_opacity(layer_copy1, 50.0)																# opacity will change contrast
	
	pdb.gimp_image_set_active_layer(image, layer)																# make original layer active	
	
	if has_alpha == TRUE:
		pdb.gimp_layer_add_alpha(layer)
		
	############## how do I paste the original Alpha layer contents into the new Alpha layer I've just made??? #####################

register(
	"python_fu_color_cast_reduction",
	"Color Cast Reduction",
	"Color Cast Reduction",
	"David Marsden",
	"David Marsden",
	"July 2022",
	"<Image>/Python-Fu/Color Cast Reduction...",             													#Menu path
	"RGB*, GRAY*, INDEXED*", 
	[],
	[],
	python_color_cast_reduction)

main()
