#!/usr/bin/env python

# colorchips.py Release 3
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Fix bug for Issabella.
# Rel 3: Color chips on separate layers (requested by Issabella).
from gimpfu import *
import random
#from array import array
#import sys

def python_tt_colorchips(image, layer, colors):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#core code goes here/below
	new_image = pdb.gimp_image_duplicate(image)
	active_layer = pdb.gimp_image_get_active_layer(new_image)
	new_display = pdb.gimp_display_new(new_image)
	pdb.gimp_image_convert_indexed(new_image,CONVERT_DITHER_NONE,CONVERT_PALETTE_GENERATE,colors,FALSE,TRUE,"")
	num_bytes,colormap = pdb.gimp_image_get_colormap(new_image)
	num_bytes_per_color = int(num_bytes/colors)
	pdb.gimp_context_set_sample_threshold(0.0)
	pdb.gimp_image_convert_rgb(new_image)
	for i in range(0,int(colors)):
		new_layer = pdb.gimp_layer_new(new_image,new_image.width,new_image.height,RGBA_IMAGE,"chip layer",100,LAYER_MODE_NORMAL)
		pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
		startindex = i*num_bytes_per_color
		color = (colormap[startindex],colormap[startindex+1],colormap[startindex+2])
		pdb.gimp_image_select_color(new_image,CHANNEL_OP_REPLACE,active_layer,color)
		non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(new_image)
		if non_empty:
			mean,std_dev,median,pixels,count,percentile = pdb.gimp_drawable_histogram(active_layer,HISTOGRAM_VALUE,0,1)
			side = count**0.5
			x = random.randrange(x1,max(x1+1,int(x2 - side)))
			y = random.randrange(y1,max(y1+1,int(y2 - side)))
			pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,x,y,side,side)
			pdb.gimp_context_set_foreground(color)
			pdb.gimp_edit_fill(new_layer,FILL_FOREGROUND)
		
		
	#UNCOMMENT BELOW WHEN DONE
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_colorchips",                           
	"Creates Color Chips of active layer",
	"Creates Color chips of active layer",
	"Tin Tran",
	"Tin Tran",
	"Feb 2019",
	"<Image>/Python-Fu/Color Chips..",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "colors", "Number of colors(to turn to indexed to apply chips to):", 10, (2, 256, 1)),
	#(PF_SPINNER, "xjitter", "X Jitter (pixels):", 50, (1, 2000, 1)),
	#(PF_SPINNER, "yjitter", "Y Jitter (pixels):", 50, (1, 2000, 1)),	
	],
	[],
	python_tt_colorchips)

main()
