#!/usr/bin/env python

# convert_scanned_tiff.py Release 3
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Allow user to specify border width and color 
#        (color will be converted to grayscale color because image is grayscale tif image and we want to preserve that).
# Rel 3: Straightens cropped area (really slow).
from gimpfu import *
import random
import math
import os
#from array import array
#import sys
def python_tt_convert_scanned_tiff(path,padding_width,padding_color):
	converted_append = "_crop_"
	tifs = []
	#path = "/home/tony/pictures"
	valid_images = [".tif"]
	pdb.gimp_context_set_background((0,0,0)) #so that when we rotate we get black background.
	for f in os.listdir(path):
		ext = os.path.splitext(f)[1]
		if ext.lower() not in valid_images or converted_append in f:
			continue
		tifs.append(os.path.join(path,f))
	pdb.gimp_context_set_foreground(padding_color)
	for tif in tifs:
		image = pdb.file_tiff_load(tif,tif)
		layer = pdb.gimp_image_get_active_layer(image)
		pdb.gimp_image_crop(image,layer.width-(77*2),layer.height-(77*2),77,77) #crop around border of 77px to get rid of possible artifacts
		pdb.gimp_fuzzy_select(layer,5,5,42,CHANNEL_OP_REPLACE,TRUE,FALSE,0,TRUE) #fuzzy select in the corner (5,5)
		pdb.gimp_selection_grow(image,10) #grow selection 10
		pdb.gimp_selection_invert(image) # invert it to have scanned page selected
		pdb.gimp_selection_grow(image,10) #grow selection 10 back so that it selects all possible whites of page
		
		#straighten image and merge down
		pdb.python_fu_tt_straighten(image,layer)
		layer = pdb.gimp_image_merge_down(image, image.layers[0],EXPAND_AS_NECESSARY)

		#repeat fuzzy select to get page
		pdb.gimp_fuzzy_select(layer,5,5,42,CHANNEL_OP_REPLACE,TRUE,FALSE,0,TRUE) #fuzzy select in the corner (5,5)
		pdb.gimp_selection_grow(image,10) #grow selection 10
		pdb.gimp_selection_invert(image) # invert it to have scanned page selected
		pdb.gimp_selection_grow(image,10) #grow selection 10
		

		non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
		pdb.gimp_image_crop(image,x2-x1,y2-y1,x1,y1)

		#add border before exporting it
		pdb.gimp_image_resize(image,image.width+padding_width*2,image.height+padding_width*2,padding_width,padding_width)
		background = pdb.gimp_layer_new(image,image.width,image.height,GRAYA_IMAGE,"background to create border",100,LAYER_MODE_NORMAL)
		pdb.gimp_image_insert_layer(image,background,None,1)
		pdb.gimp_selection_none(image)
		pdb.gimp_edit_fill(background,FILL_FOREGROUND)
		layer = pdb.gimp_image_merge_down(image,layer,EXPAND_AS_NECESSARY)
		#pdb.gimp_display_new(image)
		#export it to ..._crop_.tif
		cropped_filename = tif.replace(".tif","_crop_.tif")
		pdb.file_tiff_save(image,layer,cropped_filename,cropped_filename,0)
		pdb.gimp_message(str(tif) + " converted to " + str(cropped_filename))
		pdb.gimp_image_delete(image)

	#42.7 threshold grow 10

	#return

register(
	"python_tt_convert_scanned_tiff",                           
	"Convert Scanned Tiffs",
	"Convert Scanned Tiffs",
	"Tin Tran",
	"Tin Tran",
	"May 2019",
	"<Toolbox>/Python-Fu/Convert Scanned Tifs...",             #Menu path
	"", 
	[
	(PF_DIRNAME, "path", "Directory of .tif files:", 0),
	(PF_SPINNER, "padding_width", "Padding Border Width (pixels):", 115, (10, 1000, 1)),
	(PF_COLOR, "padding_color", "Padding Color", (255,255,255)),
	#(PF_SPINNER, "colors", "Number of colors:", 8, (2, 255, 1)),
	#(PF_SPINNER, "simplify_size", "Simplify Size (Width):", 100, (10, 1000, 1)),
	#(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	#(PF_TOGGLE, "keep_aspect", "Keep Aspect Ratio of Border:", 1),
	#(PF_TOGGLE, "mirror","Mirror/Reflect Borders:", 1),
	#(PF_TOGGLE, "drop_shadow","Drop Shadow:", 1),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_blur", "Shadow Blur Radius:", 15, (0, 1000, 1)),
	],
	[],
	python_tt_convert_scanned_tiff)

main()
