#!/usr/bin/env python

# copies-along-path.py
# Created by Tin Tran
# designed to make copies of dents (or rasterized shapes) on a layer along the path to satisfy the designing of Teneriffe Lace Looms described here: https://gimplearn.net/viewtopic.php?f=9&t=3352
# Comments directed to http://gimplearn.net or http://gimp-forum.net or http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.

#  Instructions:
# To use the copies-along-path.py plugin, follow these simple steps:
# 1. Create a new layer and draw a shape.
# 2. Convert the shape into a path.
# 3. On another layer, draw a tooth/dent at the top facing downwards.
# 4. Note the base location of the dent and where you'd like the center point to be.
# 5. Run the plugin, enter the dent's location and the center point's location, choose any desired options, and click OK to run.
#6. Voila! You'll now have a layer with the dents rotated and positioned correctly along the path.

import math
import random
#import Image
from gimpfu import *
import time
import os

class Point:
	x = 0
	y = 0

#code below by ofnuts, copied from ofn-text-along-path.py
def computeOrientedSlope(dx,dy,slope): #returns min = -PI max = +PI.
	if abs(slope) >100000: # very vertical
		return math.atan2(dy,dx) # no perfect, but properly oriented
    # keep dx/dy signs but give then the same ratio as in slope
	return math.atan2(math.copysign(slope,dy),math.copysign(1,dx))
def angle_(dx,dy):
	return math.atan2(float(dy), float(dx))
edgescheck = [];
def python_copies_along_path(image, layer, el_x, el_y, copies, rotate_option, copies_option, center_x, center_y):
	active_vectors = pdb.gimp_image_get_active_vectors(image)
	num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(active_vectors)
	precision = 0.01
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,active_vectors)
	pathpoints = []
	length = pdb.gimp_vectors_stroke_get_length(active_vectors,stroke_ids[0],precision)
	samplelength = 1.0
	shortestdist = 100000 #find points and closest point at the same time
	shorestk = -1
	shortestangle = -1
	for k in range(0,int(length/samplelength)):
		pdb.gimp_progress_update(k/float(length))
		dist = k * 1 + samplelength #0.01 just so we have at least something to subtract from to get the dx,dy for figuring out the angle
		xpoint1,ypoint1,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_ids[0],dist+0.001,precision)
		xpoint,ypoint,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_ids[0],dist,precision)
		angle = computeOrientedSlope(xpoint-xpoint1,ypoint-ypoint1,slope)
		if valid == TRUE:
			pathpoints.append([xpoint,ypoint,angle])
			dist = ((xpoint-el_x)**2 + (ypoint-el_y)**2)**0.5
			if dist < shortestdist:
				shortestdist = dist 
				shortestk = k
				shortestangle = angle
				offsetx = xpoint-el_x
				offsety = ypoint-el_y
				centeroffsetangle = angle_(ypoint-center_y,xpoint-center_x) #do reverse subtract to get angle from point to center
	copies = int(copies)
	rotate_option = int(rotate_option)
	copies_option = int(copies_option)
	minangle = 350;
	maxangle = -350;
	#merge_downlayer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,"work-layer",100,LAYER_MODE_NORMAL)
	#pdb.gimp_image_insert_layer(image,merge_downlayer,None,0)
	for copy in range(1,copies+1):
		newlayer = pdb.gimp_layer_new_from_drawable(layer,image)
		pdb.gimp_image_insert_layer(image,newlayer,None,0)
		indexk = int(int(length/samplelength)*copy/copies+shortestk)%int(length/samplelength)
		if copies_option == 0: #spread evenly along path
			if (rotate_option == 0): #in relation to path
				thispoint = pathpoints[indexk]
				thisangle = thispoint[2] - shortestangle
				item = pdb.gimp_item_transform_rotate(newlayer,thisangle,FALSE,el_x,el_y)
				item = pdb.gimp_item_transform_translate(item,thispoint[0]-pathpoints[shortestk][0],thispoint[1]-pathpoints[shortestk][1])
			elif (rotate_option == 1): #in relation fo center point
				#thisangle = (math.pi * 2.0 /  copies * copy) - centeroffsetangle
				# closestangle = 100000
				# savek = -1
				# for k in range(0,len(pathpoints)):
				# 	findangle = pathpoints[k][2]
				# 	diffangle = abs(findangle - thisangle)
				# 	if (diffangle < closestangle):
				# 		closestangle = diffangle
				# 		savek = k 
				#here we havce savek pointing to closest angle point
				thispoint = pathpoints[indexk]
				thisangle = shortestangle - angle_(thispoint[1]-center_y,thispoint[0]-center_x) 
				
				item = pdb.gimp_item_transform_rotate(newlayer,thisangle,FALSE,el_x,el_y)
				item = pdb.gimp_item_transform_translate(item,thispoint[0]-pathpoints[shortestk][0],thispoint[1]-pathpoints[shortestk][1])
			elif (rotate_option == 2): #no rotation
				thispoint = pathpoints[indexk]
				item = pdb.gimp_item_transform_translate(item,thispoint[0]-pathpoints[shortestk][0],thispoint[1]-pathpoints[shortestk][1])
		if copies_option == 1: #spread evenly along center angle
			thisanglesub = (math.pi * 2.0 * copy /  copies)%(2*math.pi)-math.pi
			minangle = min(thisanglesub,minangle)
			maxangle = max(thisanglesub,maxangle)
			closestangle = 100000
			savek = -1
			for k in range(0,len(pathpoints)):
				#findangle = pathpoints[k][2]
				findangle = angle_(pathpoints[k][1]-center_y,pathpoints[k][0]-center_x)
				minangle = min(findangle,minangle)
				maxangle = max(findangle,maxangle)
				diffangle = abs(findangle - thisanglesub)
				if (diffangle <= closestangle):
					closestangle = diffangle
					savek = k 
			if (rotate_option == 0): #in relation to path
				thispoint = pathpoints[savek]
				thisangle = thispoint[2] - shortestangle
				#thisangle = thispoint[2] - shortestangle
				item = pdb.gimp_item_transform_rotate(newlayer,thisangle,FALSE,el_x,el_y)
				item = pdb.gimp_item_transform_translate(item,thispoint[0]-pathpoints[shortestk][0],thispoint[1]-pathpoints[shortestk][1])				
			elif (rotate_option == 1): #in relation to center point
				#here we havce savek pointing to closest angle point
				thispoint = pathpoints[savek]
				#Rthisangle = centeroffsetangle - angle_(thispoint[1]-center_y,thispoint[0]-center_x) 
				thisangle = shortestangle -  angle_(pathpoints[savek][1]-center_y,pathpoints[savek][0]-center_x)
				item = pdb.gimp_item_transform_rotate(newlayer,thisangle,FALSE,el_x,el_y)
				item = pdb.gimp_item_transform_translate(item,thispoint[0]-pathpoints[shortestk][0],thispoint[1]-pathpoints[shortestk][1])
			elif (rotate_option == 2):
				thispoint = pathpoints[savek]
				item = pdb.gimp_item_transform_translate(item,thispoint[0]-pathpoints[shortestk][0],thispoint[1]-pathpoints[shortestk][1])
		is_floating = pdb.gimp_layer_is_floating_sel(item)
		if is_floating == TRUE:		
			pdb.gimp_floating_sel_to_layer(item)
		merge_downlayer = pdb.gimp_image_merge_down(image,item,EXPAND_AS_NECESSARY)
	

	#@pdb.gimp_selection_shrink(image,1) #shrink it so we don't do outline
	# fontpoints = [];
	# halffontwidth = font_width/2.0
	# new_vectors = pdb.gimp_vectors_new(image,"Single Line Font")
	# gimpVectorsBezierStroke = gimp.VectorsBezierStroke
	# incangle = 90.0/360*math.pi*2.0

	# pathpoints = []
	# averages = []
	# x1 = [100000] * num_strokes
	# x2 = [-100000] * num_strokes
	# y1 = [100000] * num_strokes
	# y2 = [-100000] * num_strokes
	# for z in range(0,num_strokes):
	# 	length = pdb.gimp_vectors_stroke_get_length(active_vectors,stroke_ids[z],precision)
	# 	#dist = 0;
	# 	#pathpoints =  []
	# 	fp = [];
	# 	fontpoints = [];
		
	# 	pathpoints.append([])
	# 	averx = 0
	# 	avery = 0
	# 	c = 0
	# 		angle = computeOrientedSlope(xpoint1-xpoint,ypoint1-ypoint,slope)
	# 		#midx = xpoint+math.cos(angle+incangle)*halffontwidth
	# 		#midy = ypoint+math.sin(angle+incangle)*halffontwidth
	# 		#angle = math.pi*2.0*angle
	# 		x1[z] = min(xpoint,x1[z])
	# 		x2[z] = max(xpoint,x2[z])
	# 		y1[z] = min(xpoint,y1[z])
	# 		y2[z] = max(xpoint,y2[z])
	# 		pathpoints[z].append([xpoint,ypoint,z,dist,angle]);
	# 		averx += xpoint
	# 		avery += ypoint
	# 		c+=1
	# 	averx /= c 
	# 	avery /= c 
	# 	averages.append([averx,avery])
	# #the below helps put all the strokes that belong to a letter together so we can deal with them in group
	# for z in range(len(pathpoints)-2,-1,-1):
	#  	for y in range(len(pathpoints)-1,z,-1):
	#  		# dist = ((averages[z][0]-averages[y][0])**2+(averages[z][1]-averages[y][1])**2)**0.5
	#  		# if dist < font_width * 1.5: #it's part of the same letter
	#  		if (((x1[y] >= x1[z] and x1[y] <= x2[z]) and (x2[y] >= x1[z] and x2[y] <= x2[z])) and ((y1[y] >= y1[z] and y1[y] <= y2[z]) and (y2[y] >= y1[z] and y2[y] <= y2[z]))):
	#  			pathpoints[z] += pathpoints[y][0:]
	#  			del pathpoints[y]
	# # for z in range(len(pathpoints)):
	# # 	new_vectors = pdb.gimp_vectors_new(image,"Single Line Font")
	# # 	addpath = []
	# # 	for i in range(0,len(pathpoints[z])-1):
	# # 		addpath += [pathpoints[z][i][0],pathpoints[z][i][1]]*3
	# # 	faststroke = gimpVectorsBezierStroke(new_vectors,addpath,0)		
	# # 	pdb.gimp_image_insert_vectors(image,new_vectors,None,0);	


	# largeNumber = 1000000
	# font_width = 7.2 #manually set
	# for z in range(len(pathpoints)):
	# 	curpath = pathpoints[z]
	# 	curpathcount = len(curpath)
	# 	candidates = []
	# 	x1 = 1000000
	# 	x2 = -1000000
	# 	y1 = 1000000
	# 	y2 = -1000000
	# 	for i in range(0,curpathcount):
	# 		#get border rectangle
	# 		x1 = min(x1,curpath[i][0])
	# 		x2 = max(x2,curpath[i][0])
	# 		y1 = min(y1,curpath[i][1])
	# 		y2 = max(y2,curpath[i][1])
	# 	#select the whole text intersected with our border to get just the character we're looking at
	# 	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,active_vectors)
	# 	pdb.gimp_image_select_rectangle(image,CHANNEL_OP_INTERSECT,x1,y1,x2-x1,y2-y1)
	# 	non_empty = TRUE #do at least once
	# 	while non_empty == TRUE:
	# 		pdb.gimp_selection_shrink(image,1) #shrink by 1 pixel each time
	# 		non_empty,x1_,y1_,x2_,y2_ = pdb.gimp_selection_bounds(image)
	# 		if non_empty == TRUE:
	# 			#loop over the whole selection value
	# 			for y in range(y1_,y2_):
	# 				for x in range(x1_,x2_):
	# 					thispixel = pdb.gimp_selection_value(image,x,y)
	# 					if (thispixel > 0):
	# 						leftpixel = pdb.gimp_selection_value(image,x-1,y)
	# 						leftpixel2 = pdb.gimp_selection_value(image,x-2,y)
	# 						if leftpixel == 0 or leftpixel2 == 0:
	# 							rightpixel = pdb.gimp_selection_value(image,x+1,y)
	# 							rightpixel2 = pdb.gimp_selection_value(image,x+2,y)
	# 							if rightpixel == 0 or rightpixel2 == 0:
	# 								candidates.append([float(x),float(y)])
	# 								pdb.gimp_image_select_rectangle(image,CHANNEL_OP_SUBTRACT,x-2,y,5,1)
	# 						toppixel = pdb.gimp_selection_value(image,x,y-1)
	# 						toppixel2 = pdb.gimp_selection_value(image,x,y-2)
	# 						if toppixel == 0 or toppixel2 == 0:
	# 							bottompixel = pdb.gimp_selection_value(image,x,y+1)
	# 							bottompixel2 = pdb.gimp_selection_value(image,x,y+2)
	# 							if bottompixel == 0 or bottompixel2 == 0:
	# 								candidates.append([float(x),float(y)])
	# 								pdb.gimp_image_select_rectangle(image,CHANNEL_OP_SUBTRACT,x,y-2,1,5)
				
	# 	pointsToAdd = candidates;
	# 	# for i in range(0,curpathcount): #just try adding points which are closest candidates
	# 	# 	pdb.gimp_progress_update(i/(float(curpathcount)))
	# 	# 	shortestdist = 1000000
	# 	# 	savej = -1
	# 	# 	thispoint = curpath[i]
	# 	# 	for j in range(0,len(candidates)):
	# 	# 		dist = ((candidates[j][0]-thispoint[0])**2 + (candidates[j][1]-thispoint[1])**2)**0.5
	# 	# 		if dist < shortestdist:
	# 	# 			shortestdist = dist
	# 	# 			savej = j
	# 	# 	#at this point savej should point to closest candiate for point
	# 	# 	if savej > -1:
	# 	# 		pointsToAdd.append(candidates[savej])

				
	# 	# addpath = []
	# 	# for i in range(0,len(pointsToAdd)-1):
	# 	# 	pdb.gimp_progress_update(i/(float(len(pointsToAdd)+1)))
	# 	# 	addpath += pointsToAdd[i]*3
	# 	# faststroke = gimpVectorsBezierStroke(new_vectors,addpath,0)
				
	# 	edgescheck = [];

	# 	p = []
	# 	for u in range(0,len(pointsToAdd)):
	# 	 	closest = 1000000
	# 	 	savej = -1
	# 	 	savei = -1
	# 	 	for i in range(0,len(pointsToAdd)):
	# 	 		for j in range(0,len(pointsToAdd)):
	# 	 			if (i!=j):
	# 	 				dist = ((pointsToAdd[i][0] - pointsToAdd[j][0])**2 + (pointsToAdd[i][1] - pointsToAdd[j][1])**2)**0.5
	# 	 				if (canadd([i,j]) == 1) and (dist < closest):
	# 	 					savei = i;
	# 	 					savej = j;
	# 	 					closest = dist
	# 	 	if savej > -1:
	# 	 		addcheck([savei,savej])
	# 	 		p.append([savei,savej])
	# 	addpath = []
	# 	for i in range(0,len(p)):
	# 	 	pdb.gimp_progress_update(j/(float(len(p)+1)))
	# 	 	addpath = pointsToAdd[p[i][0]]*3 + pointsToAdd[p[i][1]]*3
	# 	 	faststroke = gimpVectorsBezierStroke(new_vectors,addpath,0)
	# pdb.gimp_image_insert_vectors(image,new_vectors,None,0);
			
	
register(
	"python_fu_copies_along_path",
	"Make copies along path",
	"Make copies along Path",
	"Tin Tran",
	"Tin Tran",
	"Jan 27,2023",
	"<Image>/Python-Fu/Copies Along Path",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "el_x", "Element X location:", 100, (0,4000,1)),
	(PF_SPINNER, "el_y", "Element Y location:", 100, (0,4000,1)),
	(PF_SPINNER, "copies", "# of Copies:", 8, (1,500,1)),
	(PF_OPTION,"rotate_option",   "Rotate Elements:", 0, ["In Relation to Path","In Relation to Defined Center","No Rotation"]), # initially 0th is choice
	(PF_OPTION,"copies_option",   "Make Copies:", 0, ["Spread Evenly Along Path","Spread Evenly Along Angles in Relation to Defined Center"]), # initially 0th is choice
	(PF_SPINNER, "center_x", "Define Center X:", 100, (0,4000,1)),
	(PF_SPINNER, "center_y", "Define Center Y:", 200, (0,4000,1)),
	# (PF_SPINNER, "pathspeed", "Minimum Path-speed:", 1.0, (0.01,100, 0.01)),
	# (PF_SPINNER, "maxpathspeed", "Maximum Path-speed:", 1.0, (0.01,100, 0.01)),
	# (PF_TOGGLE, "rotate", "Randomly rotate tiles:", 0),
	# (PF_TOGGLE, "flip", "Randomly flip tiles:", 0),
	],
	[],
	python_copies_along_path)

main()
