﻿#!/usr/bin/env python

# custom_railing.py Rel 1
# Created by Tin Tran http://gimplearn.net
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Try to use 3 points instead of 4 points for better aligned result.
# Rel 3: Add option to toggle between using 3 or 4 points. Because when the railing isn't straight we want to use 4 points.
import math
import string
#import Image
from gimpfu import *

def python_custom_railing_tt(image, layer, xcffilename, use3points):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#loads railing definition file
	dummy = 0
	railing_img = pdb.gimp_xcf_load(dummy,xcffilename,xcffilename)
	railing_layer = pdb.gimp_image_get_active_layer(railing_img)

	#creates resulting image.
	result_img = pdb.gimp_image_new(railing_layer.width,railing_layer.height,RGB)
	result_display = pdb.gimp_display_new(result_img)
	result_layer = pdb.gimp_layer_new_from_drawable(railing_layer,result_img)
	pdb.gimp_image_insert_layer(result_img,result_layer,None,0)

	pieces = len(railing_img.vectors)
	vectors = railing_img.vectors[::-1]
	
	for i in range(0,len(vectors)):
		current_vector = vectors[i]
		numstrokes,src_strokeids = pdb.gimp_vectors_get_strokes(current_vector)
		src_stroke = src_strokeids[0] #just work with first stroke, not sure about multiple strokes
		vector_type, num_points, controlpoints, closed = pdb.gimp_vectors_stroke_get_points(current_vector,src_stroke)
		px1,py1 = controlpoints[2],controlpoints[3] #top left
		px2,py2 = controlpoints[8],controlpoints[9] #top right
		px3,py3 = controlpoints[14],controlpoints[15] #bottom right
		#we'll calculate point 4 based on the first 3 points.
		if use3points == 0:
			px4,py4 = controlpoints[20],controlpoints[21] #bottom left
		#pdb.gimp_message(str(controlpoints))

		#copy that section over to new image
		width = layer.width*1.0/pieces
		pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,i*width,0,width,layer.height)
		pdb.gimp_edit_copy(layer)
		floating_sel = pdb.gimp_edit_paste(result_layer,TRUE)
		if (pdb.gimp_layer_is_floating_sel(floating_sel)):
			pdb.gimp_floating_sel_to_layer(floating_sel)
		if use3points == 0:
			newly_mapped_item = pdb.gimp_item_transform_perspective(floating_sel,px1,py1,px2,py2,px4,py4,px3,py3)
		#try to use 3 points instead of 4 points.
		else:
			newly_mapped_item = pdb.gimp_item_transform_perspective(floating_sel,px1,py1,px2,py2,px3-(px2-px1),py1+(py3-py2),px3,py3)




	#we're done delete railing image
	pdb.gimp_image_delete(railing_img)

	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_custom_railing_tt",                           
	"Paint/Paste railing of active layer over a railing definition file",
	"Paint/Paste railing of active layer over a railing definition file",
	"Tin Tran",
	"Tin Tran",
	"May 2017",
	"<Image>/Python-Fu/Custom Railing...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_FILENAME, "xcffilename", ".xcf railing file:", 0),
	(PF_TOGGLE, "use3points",   "Use 3 Points instead of 4 (Calculate 4th points):", 1), 
	],
	[],
	python_custom_railing_tt)

main()
