#!/usr/bin/env python

# eagle-mode.py
# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Speedy version.
# Rel 3: Show a dialog window to show progress updates.
# Rel 4: Speed Up.
#import Image
from gimpfu import *
import random
import math
import os
from array import array #needed for array pixel access

def python_eagle_mode_tt(image,layer,unused) :
	pdb.gimp_image_undo_group_start(image)
	
	#creates a new image of same size and create a display for it.
	if not pdb.gimp_drawable_has_alpha(layer):
		pdb.gimp_layer_add_alpha(layer);
	#

	nwidth = layer.width*3
	nheight = layer.height*3
	new_image = pdb.gimp_image_new(nwidth,nheight,RGB)
	new_display = pdb.gimp_display_new(new_image)
	#new_image = image
	new_layer = pdb.gimp_layer_new(new_image,nwidth,nheight,RGBA_IMAGE,"2x Eagle Mode",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,new_layer,None,0)

	#fast pixel operations
	srcRgn = layer.get_pixel_rgn(0, 0, layer.width, layer.height, False, False) #read only
	dstRgn = new_layer.get_pixel_rgn(0, 0, new_layer.width, new_layer.height, True, True) #update to pixels
	p_size_source = len(srcRgn[0,0])
	p_size_dest = len(dstRgn[0,0])
	src_pixels = array("B", srcRgn[0:layer.width, 0:layer.height]) #source
	dest_pixels = array("B", dstRgn[0:new_layer.width, 0:new_layer.height]) #destination pixel

	pixels = []
	#Read pixels
	num_channels = 4
	pdb.gimp_progress_update(float(0)/layer.width)
	pdb.gimp_progress_set_text("Reading Pixels")
	show = 0
	test_progress = layer.width/10
	src_pos_init = -p_size_source
	y_inc = layer.width*p_size_source
	for x in range(0,layer.width):
		show += 1
		if (show == test_progress):
			pdb.gimp_progress_update(float(x)/layer.width)
			show = 0
		row = []
		src_pos_init += p_size_source
		src_pos = src_pos_init
		for y in range(0,layer.height):
			pixel = src_pixels[src_pos: src_pos + p_size_source]
			src_pos += y_inc
			row.append(pixel)
		pixels.append(row)
	pdb.gimp_progress_set_text("Drawing Pixels")
	show = 0
	test_progress = layer.width/10
	#dest_pos = x*3*p_size_dest + 3*y*new_layer.width*p_size_dest
	dest_x_inc = 3*p_size_dest
	dest_x = -dest_x_inc

	#3*0*new_layer.width*p_size_dest #start
	#3*new_layer.width*p_size_dest
	dest_y_inc = 3*new_layer.width*p_size_dest
	dest_y_inc_unit = new_layer.width*p_size_dest
	dest_y = -dest_y_inc
	for x in range(0,layer.width):
		dest_x += dest_x_inc
		show += 1
		if (show == test_progress):
			pdb.gimp_progress_update(float(x)/layer.width)
			show = 0
		dest_y = -dest_y_inc
		for y in range(0,layer.height):
			dest_y += dest_y_inc			
			E = pixels[x][y]
			p1=E; p2=E; p3=E; p4=E; p5=E; p6=E; p7=E; p8=E; p9=E;
			#ABC
			#DEF
			#GHI
			if (x>0) and (y>0):	A = pixels[x-1][y-1]	
			if (y>0): B = pixels[x][y-1]
			if (y>0) and (x<layer.width-1): C = pixels[x+1][y-1]
			if (x>0): D = pixels[x-1][y]
			if (x<layer.width-1): F = pixels[x+1][y]
			if (x>0) and (y<layer.height-1): G = pixels[x-1][y+1]
			if (y<layer.height-1): H = pixels[x][y+1]
			if (x<layer.width-1) and (y<layer.height-1): I = pixels[x+1][y+1]
			#123
			#456
			#789

			#ABC
			#DEF
			#GHI
			if (x>0) and (y>0) and (x<layer.width-1):
				if D==B and D!=H and B!=F: p1=D
			if (x>0) and (y>0) and (x<layer.width-1) and (y<layer.height-1):
				if (D==B and D!=H and B!=F and E!=C) or (B==F and B!=D and F!=H and E!=A): p2=B
				if B==F and B!=D and F!=H: p3=F
				if (H==D and H!=F and D!=B and E!=A) or (D==B and D!=H and B!=F and E!=G): p4=D
				if (B==F and B!=D and F!=H and E!=I) or (F==H and F!=B and H!=D and E!=C): p6=F
				if H==D and H!=F and D!=B: p7=D
				if (F==H and F!=B and H!=D and E!=G) or (H==D and H!=F and D!=B and E!=I): p8=H
				if F==H and F!=B and H!=D: p9=F		
			p5=E
			

			
			dest_pos = dest_x + dest_y                          #add no unit
			dest_pixels[dest_pos : dest_pos + p_size_dest] = p1
			
			dest_pos = dest_pos+p_size_dest
			dest_pixels[dest_pos : dest_pos + p_size_dest] = p2
			
			dest_pos = dest_pos+p_size_dest
			dest_pixels[dest_pos : dest_pos + p_size_dest] = p3
			
			dest_pos = dest_x+dest_y+dest_y_inc_unit            #add 1 unit
			dest_pixels[dest_pos : dest_pos + p_size_dest] = p4
			
			dest_pos = dest_pos+p_size_dest
			dest_pixels[dest_pos : dest_pos + p_size_dest] = p5
			
			dest_pos = dest_pos+p_size_dest
			dest_pixels[dest_pos : dest_pos + p_size_dest] = p6
			
			dest_pos = dest_x+dest_y+dest_y_inc_unit+dest_y_inc_unit #add 2 unit
			dest_pixels[dest_pos : dest_pos + p_size_dest] = p7
			
			dest_pos = dest_pos+p_size_dest
			dest_pixels[dest_pos : dest_pos + p_size_dest] = p8
			
			dest_pos = dest_pos+p_size_dest
			dest_pixels[dest_pos : dest_pos + p_size_dest] = p9
			
					
					
	#flush to new_layer data			
	dstRgn[0:new_layer.width, 0:new_layer.height] = dest_pixels.tostring() 	
	new_layer.flush()
	new_layer.merge_shadow(True)
	new_layer.update(0, 0,new_layer.width,new_layer.height)
	pdb.gimp_displays_flush()

	#pdb.gimp_drawable_update(new_layer,0,0,nwidth,nheight)
	#pdb.gimp_displays_flush()	
	
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_eagle_mode_tt",
	"3x Eagle mode filter",
	"3x Eagle mode filter",
	"Tin Tran",
	"Tin Tran",
	"May 2020",
	"<Image>/Python-Fu/Eagle Mode 3x...",             #Menu path
	"RGB*, GRAY*, INDEXED*", 
	[
	#(PF_DRAWABLE, "shape", "Shape layer (with Alpha):", None),
	#(PF_TOGGLE, "mergedown",   "Merge Down layers:", 1),
	(PF_OPTION,"option",   "Unused Input:", 0, ["To show Progress Bar"]),
	],
	[],
	python_eagle_mode_tt)

main()
