#!/usr/bin/env python

# expand_map.py Rel 1
# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
import math
import string
#import Image
from gimpfu import *
from array import array
import sys
def python_expand_map(image, layer, cx, cy, scale) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	if not pdb.gimp_drawable_has_alpha(layer):
		pdb.gimp_layer_add_alpha(layer)
	#pdb.gimp_layer_scale(layer,width,width,FALSE)
	right = layer.width - cx
	left = cx 
	bottom = layer.height - cy
	top = cy
	maxx = max(right,left)
	maxy = max(top,bottom)
	radius = (maxx**2 + maxy**2)**0.5
	new_layer = pdb.gimp_layer_new(image,int(radius)*3,int(radius)*3,RGBA_IMAGE,'expand map',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,new_layer,None,0)
	for y in range(0,int(layer.height)):
		for x in range(0,int(layer.width)):
			#get the color at that rectangle coord
			num_channels,pixel = pdb.gimp_drawable_get_pixel(layer,x,y)
			#convert to unit rectangle
			x1 = float(x-cx)
			y1 = float(y-cy)
			r1 = (x1**2 + y1**2)**0.5 #calculate current radius
			r2 = radius - ((radius - r1)/scale) #calculate new radius
			#scale1 = r1/r2 #calculate scale to fit into new radius
			if r1 > 0:
				x1 = (x1/r1) * r2
				y1 = (y1/r1) * r2
			#cast them to integers
				x_ = int(x1 + radius*1.5)
				y_ = int(y1 + radius*1.5)
				if x_ >= 0 and x_ <= new_layer.width and y_ >= 0 and y_ <= new_layer.height:
					try:
						pdb.gimp_drawable_set_pixel(new_layer,x_,y_,num_channels,pixel)
					except:
						pass	
		pdb.gimp_progress_update(float(y)/layer.height)
	pdb.gimp_drawable_update(new_layer,0,0,new_layer.width,new_layer.height)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_expand_map",
	"Maps image to expand center to have an opened center",
	"Maps image to expand center to have an opened center",
	"Tin Tran",
	"Tin Tran",
	"August 2019",
	"<Image>/Python-Fu/Expand map...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "cx", "Center X:", 100, (1, 4000, 1)),
	(PF_SPINNER, "cy", "Center Y:", 100, (1, 4000, 1)),
	(PF_SPINNER, "scale", "Scale down times:", 2, (1, 20, 1)),
	#(PF_OPTION,"option",   "Mapping:", 0, ["To Circle","To Square"]),
	],
	[],
	python_expand_map)

main()
