#!/usr/bin/env python

# fade_frames.py Release 11
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Home 
# Rel 3: Custom fifer.info where . is the house zooming in.
# Rel 4: Custom One Good Turn transition
# Rel 5: Custom One Good Turn rotate transition
# Rel 6: Custom One Good Turn zoom transition
# Rel 7: Custom One Good Turn star wars transition
# Rel 8: Custom One Good Turn starwars solid color
# Rel 9: Page Curl effect
# Rel 10: Page Curl effect with which corner option
# Rel 11: 180 rotation transition from top (first) layer into 2nd layer.
from gimpfu import *
import random
import math
import os

def python_tt_fade_frames(image, layer, frames, outdir, filename):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#core code goes here/below


	pdb.gimp_progress_init("Exporting 'One'...",None)
	#p_count = pcount(p)
	layer = image.layers[0]
	layer2 = image.layers[1]
	pdb.gimp_item_set_visible(layer,FALSE)
	pdb.gimp_item_set_visible(layer2,FALSE)
	#pdb.gimp_item_set_visible(One,FALSE)

	steps = (int(frames)+1)/2
	#offsetstep = (layer.width + One.width)/steps
	anglestep = 90.0/steps 
	radius = layer.width/2.0
	for frame in range(0,steps+1):
		pdb.gimp_progress_update(float(frame)/steps)
		angle = (frame*anglestep)
		x = math.cos(math.radians(angle))*radius
		y = math.sin(math.radians(angle))*radius/6.0 #make it a fake ellipse
		#make a copy and manipulate it instead of fading
		pdb.gimp_item_set_visible(layer,TRUE)
		layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
		pdb.gimp_image_insert_layer(image,layer_copy,None,0)
		pdb.gimp_item_set_visible(layer,FALSE)
		#offsetx,offsety = pdb.gimp_drawable_offsets(One)
		pdb.gimp_selection_none(image)
		pdb.gimp_item_transform_perspective(layer_copy,layer.width/2.0-x,0+y,layer.width/2.0+x,0-y,layer.width/2.0-x,layer.height-y,layer.width/2.0+x,layer.height+y)
		#float_sel = pdb.gimp_item_transform_perspective(layer_copy,layer.width/2.0-x,0+y,layer.width/2.0+x,0-y,layer.width/2.0-x,layer.height-y,layer.width/2.0+x,layer.height+y)
		#pdb.gimp_floating_sel_anchor(float_sel)

		#pdb.gimp_layer_set_opacity(layer,opacity)
		visible = pdb.gimp_layer_new_from_visible(image,image,"visible")
		pdb.gimp_image_insert_layer(image,visible,None,0)
		digit = "000" + str(frame)
		digit = digit[-3:]
		fullname = os.path.join(outdir,filename+digit+".png")
		pdb.file_png_save_defaults(image,visible,fullname,fullname)
		pdb.gimp_image_remove_layer(image,visible) #done with it so we remove it
		pdb.gimp_image_remove_layer(image,layer_copy)
		#pdb.gimp_layer_set_offsets(One,offsetx+offset_left_right,0)
		#pdb.gimp_layer_set_offsets(One,+offset_left_right,0)
	old_steps = steps
	steps = (int(frames)+1)/2
	#offsetstep = (layer.width + One.width)/steps
	anglestep = 90.0/steps 
	radius = layer.width/2.0	
	for frame in range(0,steps+1):
		pdb.gimp_progress_update(float(frame)/steps)
		angle = (frame*anglestep)-90
		x = math.cos(math.radians(angle))*radius
		y = math.sin(math.radians(angle))*radius/6.0 #make it a fake ellipse
		#make a copy and manipulate it instead of fading
		pdb.gimp_item_set_visible(layer2,TRUE)
		layer_copy = pdb.gimp_layer_new_from_drawable(layer2,image)
		pdb.gimp_image_insert_layer(image,layer_copy,None,0)
		pdb.gimp_item_set_visible(layer2,FALSE)
		#offsetx,offsety = pdb.gimp_drawable_offsets(One)
		#offsetx,offsety = pdb.gimp_drawable_offsets(One)
		#pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,One)
		
		#pdb.gimp_selection_invert(image)
		#non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
		#if non_empty:
		#	pdb.gimp_edit_clear(layer_copy)
		pdb.gimp_selection_none(image)
		pdb.gimp_item_transform_perspective(layer_copy,layer.width/2.0-x,0+y,layer.width/2.0+x,0-y,layer.width/2.0-x,layer.height-y,layer.width/2.0+x,layer.height+y)
		#float_sel = pdb.gimp_item_transform_perspective(layer_copy,layer.width/2.0-x,0+y,layer.width/2.0+x,0-y,layer.width/2.0-x,layer.height-y,layer.width/2.0+x,layer.height+y)
		#pdb.gimp_floating_sel_anchor(float_sel)

		#pdb.gimp_layer_set_opacity(layer,opacity)
		visible = pdb.gimp_layer_new_from_visible(image,image,"visible")
		pdb.gimp_image_insert_layer(image,visible,None,0)
		digit = "000" + str(frame+old_steps+1)
		digit = digit[-3:]
		fullname = os.path.join(outdir,filename+digit+".png")
		pdb.file_png_save_defaults(image,visible,fullname,fullname)
		pdb.gimp_image_remove_layer(image,visible) #done with it so we remove it
		pdb.gimp_image_remove_layer(image,layer_copy)
	
	#desaturate
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	#return

register(
	"python_fu_tt_fade_frames",                           
	"Fade from 1 frame to another",
	"Fade from 1 frame to another",
	"Tin Tran",
	"Tin Tran",
	"May 2019",
	"<Image>/Python-Fu/Fade Frames...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "frames", "Number of frames:", 250, (2, 1000, 1)),
	(PF_DIRNAME, "outdir", "Output Directory:", 0),
	(PF_STRING, "filename", ".png Filename (Prefix):", "foo"), 
	#(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	#(PF_TOGGLE, "keep_aspect", "Keep Aspect Ratio of Border:", 1),
	#(PF_TOGGLE, "mirror","Mirror/Reflect Borders:", 1),
	#(PF_TOGGLE, "drop_shadow","Drop Shadow:", 1),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_blur", "Shadow Blur Radius:", 15, (0, 1000, 1)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 60, (0, 100, 1)),
	],
	[],
	python_tt_fade_frames)

main()