#!/usr/bin/env python

# gimper_bevel_rectangles Rel 1
# Created by Tin Tran
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
#import math
#import string
#import Image
import random
from gimpfu import *
#from array import array

def python_gimper_bevel_rectangles(image, layer, rectangles,min_width_per,max_width_per,min_height_per,max_height_per,bevel_thickness,bevel_times) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	min_width = layer.width * min_width_per / 100
	max_width = layer.width * max_width_per / 100
	min_height = layer.height * min_height_per / 100
	max_height = layer.height * max_height_per / 100
	for i in range(0,int(rectangles)):
		#determine random width and height within percentage range given
		#and top left corner of selection (x,y)
		width = int(random.randrange(0,int(max_width-min_width)) + min_width)
		height = int(random.randrange(0,int(max_height-min_height)) + min_height)
		x = random.randrange(0,int(layer.width - width))
		y = random.randrange(0,int(layer.height - height))
		#make selection and copy it and paste it as new layers
		pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,x,y,width,height)
		pdb.gimp_edit_copy(layer)
		floating_sel = pdb.gimp_edit_paste(layer,False)
		pdb.gimp_floating_sel_to_layer(floating_sel)
		#add bevel (bevel_times) number of times
		for j in range(0,int(bevel_times)):
			pdb.script_fu_add_bevel(image,floating_sel,bevel_thickness,False,False)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_gimper_bevel_rectangles",                           
	"Creates random bevel rectangles from active layer",
	"Creates random bevel rectangles from active layer",
	"Tin Tran",
	"Tin Tran",
	"April 2017",
	"<Image>/Python-Fu/Gimper Bevel Rectangles...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "rectangles", "# of Rectangles:", 8, (1, 100, 1)),
	(PF_SPINNER, "min_width_per", "Minimum Width Percentage:", 20, (5, 100, 1)),
	(PF_SPINNER, "max_width_per", "Maximum Width Percentage (must be greater than Minimum):", 60, (5, 100, 1)),
	(PF_SPINNER, "min_height_per", "Minimum Height Percentage:", 20, (5, 100, 1)),
	(PF_SPINNER, "max_height_per", "Maximum Height Percentage (must be greater than Minimum):", 60, (5, 100, 1)),
	(PF_SPINNER, "bevel_thickness", "Bevel Thickness:", 30, (0, 30, 1)),
	(PF_SPINNER, "bevel_times", "Bevel times (how many times to run filter on each rectangle:", 3, (1, 30, 1)),
	],
	[],
	python_gimper_bevel_rectangles)

main()
