#!/usr/bin/env python

# handwriting-inspired-font.py
# idea inspired by hand-writing
# Created by Tin Tran https://gimplearn.net
# Comments directed to http://gimplearn.net, http://gimp-forum.net, or gimpchat.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.

#import math
import random
#import string
#import Image
from gimpfu import *
#from array import array
#import sys
#========================================
def python_handwriting_inspired_font_tt(image,layer,fontname,size,minheight,minwidth,horshearmin,horshearmax,vertshearmin,vertshearmax,text):
	#pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#get Linebreak height, use Capital A and another one with A newline then capital A to find out line height
	A = pdb.gimp_text_fontname(image,None,0,0,"A",0,TRUE,size,0,fontname)
	A_A = pdb.gimp_text_fontname(image,None,0,0,"A\nA",0,TRUE,size,0,fontname)
	lineheight = A_A.height - A.height #this gives value to increase new line by pixels
	#remove these A layers because we don't need them anymore now we have lineheight
	pdb.gimp_image_remove_layer(image,A)
	pdb.gimp_image_remove_layer(image,A_A)

	#create a new layer to merge all our drawn text on.
	worklayer = pdb.gimp_layer_new(image,10,10,RGBA_IMAGE,'handwriting-inspired',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,worklayer,None,0)

	lines = text.split("\n") #split up by newlines as we'll be drawing them line by line
	x = 0; y = 0; #start drawing at 0,0 (top left) 
	for i in range(0,len(lines)):
		line = lines[i] #get the current line of text
		previous_char = "" #default a previous_char as empty_string, 
		                   #this is used later for when there is a character to handle kerning
		for j in range(0,len(line)):
			char = line[j]
			char_t = pdb.gimp_text_fontname(image,None,0,0,char,0,TRUE,size,0,fontname)
			prev_and_char_t = pdb.gimp_text_fontname(image,None,0,0,previous_char + char,0,TRUE,size,0,fontname)
			# offset = prev_and_char_t.width - char_t
			x = x+prev_and_char_t.width-char_t.width
			pdb.gimp_image_remove_layer(image,prev_and_char_t) #don't need this anymore remove
			pdb.gimp_layer_set_offsets(char_t,x,y)
			width = char_t.width #save width and height before scaling
			height = char_t.height
			#scaling in place
			new_width = width * (random.randint(minwidth*10,1000)/1000.0)
			new_height = height * (random.randint(minheight*10,1000)/1000.0)
			char_t = pdb.gimp_item_transform_scale(char_t,0,0,new_width,new_height)
			#move it such that it's still center/inplace
			pdb.gimp_layer_set_offsets(char_t,x+(width-new_width)/2.0,y+(height-new_height)/2.0)
			#now do the horizontal and vertical shearing inplace
			char_t = pdb.gimp_item_transform_shear(char_t,ORIENTATION_HORIZONTAL,random.randint(horshearmin,horshearmax))
			char_t = pdb.gimp_item_transform_shear(char_t,ORIENTATION_VERTICAL,random.randint(vertshearmin,vertshearmax))

			worklayer = pdb.gimp_image_merge_down(image,char_t,EXPAND_AS_NECESSARY)
			previous_char = char #set this at the end for next character kerning

		y+=lineheight;x=0;	 #set for new line


	
	pdb.gimp_context_pop()
	#pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

register(
	"python_fu_handwriting_inspired_font_tt",                           
	"Draw font with variations to look like handwriting",
	"Draw font with variations to look like handwriting",
	"Tin Tran",
	"Tin Tran",
	"November 26th, 2021",
	"<Image>/Python-Fu/Handwriting Inspired Font...",             #Menu path
	"RGB*, GRAY*",
	[
	(PF_FONT, "fontname", "Font (Name):", "Comforter Brush"),
	(PF_SPINNER, "size", "Font Size(pixels):", 50, (10, 1000, 1)),
	(PF_SPINNER, "minheight", "Min Height(percentage):", 80, (30, 100, 1)),
	(PF_SPINNER, "minwidth", "Min Width(percentage):", 80, (30, 100, 1)),
	(PF_SPINNER, "horshearmin", "Horizontal Shear Min (magnitude):", -20, (-100, 0, 1)),
	(PF_SPINNER, "horshearmax", "Horizontal Shear Max (magnitude):", 5, (0, 100, 1)),
    (PF_SPINNER, "vertshearmin", "Vertical Shear Min (magnitude):", -15, (-100, 0, 1)),
	(PF_SPINNER, "vertshearmax", "Vertical Shear Max (magnitude):", 15, (0, 100, 1)),
	(PF_TEXT, "text", "Multiline text:", "Gimp Learn\nHandwriting-Inspired\nFont"),
	#(PF_SPINNER, "spacing", "Letter Spacing (before scaling text to fit on path):", 0, (-1000, 1000, 1)),
	],
	[],
	python_handwriting_inspired_font_tt)

main()