#!/usr/bin/env python

# layer_into_selection_vertically.py Release 1
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
from gimpfu import *
#from array import array
#import sys
def python_tt_layer_into_selection_vertically(image, layer):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#core code goes here/below
	
	#make a copy of layer so we can layer to image size it.
	#layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
	#pdb.gimp_image_insert_layer(image,layer_copy,None,0)

	#create a result layer to save our work to
	result_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"result_layer",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,result_layer,None,0)

	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if not non_empty:
		pdb.gimp_message("Must have an active selection")
	else:
		saved_selection = pdb.gimp_selection_save(image)
		sel_width = x2 - x1
		sel_height = y2 - y1
		pdb.gimp_progress_init("Reading Selection Info.",None)
		for x in range(x1,x2):
			pdb.gimp_progress_update((x-x1)*1.0/min(1.0,sel_width))
			#intersect to find a slice
			pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_selection)
			pdb.gimp_image_select_rectangle(image,CHANNEL_OP_INTERSECT,x,0,1,image.height)
			_non_empty,_x1,_y1,_x2,_y2 = pdb.gimp_selection_bounds(image)
			if _non_empty: #if there is a selected area we record the slice and work with it
				vert_slice = [_y1,_y2]
				#select the whole slice			
				pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,x,0,1,image.height)
				_non_empty,_x1,_y1,_x2,_y2 = pdb.gimp_selection_bounds(image)
				if _non_empty:
					pdb.gimp_edit_copy(layer)
					floating_sel = pdb.gimp_edit_paste(result_layer,TRUE)
					pdb.gimp_floating_sel_to_layer(floating_sel)
					scaled_item = pdb.gimp_item_transform_scale(floating_sel,x,vert_slice[0],x+1,vert_slice[1])
					pdb.gimp_floating_sel_to_layer(scaled_item)
					result_layer = pdb.gimp_image_merge_down(image,scaled_item,EXPAND_AS_NECESSARY)
					result_layer = pdb.gimp_image_merge_down(image,result_layer,EXPAND_AS_NECESSARY)
				
		#we're done return the selection as before and delete it from channel	
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_selection)
		pdb.gimp_image_remove_channel(image,saved_selection)

	#UNCOMMENT BELOW WHEN DONE
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_tt_layer_into_selection_vertically",                           
	"Fit Layer Into Selection Vertically Using Transforms",
	"Fit Layer Into Selection Vertically Using Transforms",
	"Tin Tran",
	"Tin Tran",
	"March 2019",
	"<Image>/Python-Fu/Layer Into Selection Vertically",             #Menu path
	"RGB*, GRAY*", 
	[
	#(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	#(PF_TOGGLE, "keep_aspect", "Keep Aspect Ratio of Border", 1),
	#(PF_TOGGLE, "scale_last","Scale Last Line (Sometimes it's too short, you don't want to scale it):", 1),
	],
	[],
	python_tt_layer_into_selection_vertically)

main()