#!/usr/bin/env python

# magic-eye.py
# Created by Tin Tran http://gimplearn.net
# Tried to translate from C code found at http://sites.inka.de/sites/bunsen/alex/english/gimp/magic.99.html
# From this request: (https://www.gimp-forum.net/Thread-sterograph) by denzjos
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Friendlier Progress Bar Updates
# Rel 3: Speed Up.
# Rel 4: More speed up
from gimpfu import *
import random
import math
import time
from array import array #needed for array pixel access

#from array import array
#import sys
#========================================

# ------------------------------------------------------------------------	
def python_tt_magic_eye(image, layer, texture, strip_width, depth,fromleft,updown):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()

	# Always add alpha to work with full alpha so we don't get different pixel depths/size
	if not pdb.gimp_drawable_has_alpha(layer):
		pdb.gimp_layer_add_alpha(layer);
	if not pdb.gimp_drawable_has_alpha(texture):
		pdb.gimp_layer_add_alpha(texture);
	# Create new image (same size as our input layer [grayscale map]) to work on
	new_image = pdb.gimp_image_new(layer.width,layer.height,RGB)
	new_display = pdb.gimp_display_new(new_image)
	# Create new working layer and insert it to new_image, this is the one layer we'll be working on
	new_layer = pdb.gimp_layer_new(new_image,layer.width,layer.height,RGBA_IMAGE,'magic-eye layer',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,new_layer,None,0);

	# Fast pixel access ---------------------------------------------
	textureRgn = texture.get_pixel_rgn(0, 0, texture.width, texture.height, False, False) #read only range
	sourceRgn = layer.get_pixel_rgn(0, 0, layer.width, layer.height, False, False) #read only range
	dstRgn = new_layer.get_pixel_rgn(0, 0, new_layer.width, new_layer.height, True, True) #update/writable to pixels here
	p_size = len(dstRgn[0,0])
	texture_pixels = array("B", textureRgn[0:texture.width, 0:texture.height]) #texture pixels
	source_pixels = array("B", sourceRgn[0:layer.width, 0:layer.height]) #texture pixels
	dest_pixels = array("B", dstRgn[0:new_layer.width, 0:new_layer.height]) #destination pixel
	# --------------------------------------------------------------- Fast pixel access ends

	# work with ints, so it doesn't fail calls later
	strip_width = int(strip_width)
	depth = int(depth)

	# Here is where the work is done ----------------------------------------------------------------
	# First, copy the background/texture to the new image mod-ing by strip_width so we get the strip repeated on new image
	pdb.gimp_progress_update(float(0)/10)
	pdb.gimp_progress_set_text("Phase 1. Copy Texture Chunks")
	# import timeit
	# start_time = timeit.default_timer()
	# code you want to evaluate
	
	#addition = 0*texture.width * p_size
	addition_add = texture.width*p_size
	addition_init = -addition_add
	addition = addition_init
	additiony_add = layer.width*p_size
	additiony_init = -additiony_add
	additiony = additiony_init
	for y in range(0,layer.height):
		if (y % 20==0):
			pdb.gimp_progress_update(float(y)/layer.height)
		#addition = y*texture.width * p_size
		addition += addition_add    #moved out of loop
		additiony += additiony_add  #moved out of loop
		#additiony = y*layer.width*p_size 
		dest_pos = additiony #initialize
		for x in range(0,layer.width):
			#gets pixel fast way
			#src_pos = (y*texture.width+(x%strip_width))*p_size #x is mod-ed to get repeating pattern
			src_pos = addition+(x%strip_width)*p_size #x is mod-ed to get repeating pattern
			pixel = texture_pixels[src_pos:src_pos+p_size]
			#sets pixel fast way
			# dest_pos = (y*layer.width+x)*p_size
			dest_pixels[dest_pos:dest_pos+p_size] = pixel
			dest_pos += p_size;
			#num_channels,pixel = pdb.gimp_drawable_get_pixel(texture,x%strip_width,y)
			#pdb.gimp_drawable_set_pixel(new_layer,x,y,num_channels,pixel)
	up_down = int(updown)   #default as 1 TODO use as input
	from_left = int(fromleft)

	# up_down = 1; from_left = 1; #this I can view with cross eye technique
	# up_down = 1; from_left = 0; #starts in middle

	if (up_down == 1):
		_up_down = -1
	else:
		_up_down = 1
	pdb.gimp_progress_set_text("Phase 1b. Precalculate depth");
	height = [];
	hoehevalue =[];
	for i in range(0,256):
		if (i%20==0):
			pdb.gimp_progress_update(float(i)/(256))
		hoehe = i * depth / 255	
		#height.append(hoehe)
		hoehevalue.append(hoehe*_up_down*p_size)

	pdb.gimp_progress_set_text("Phase 2. Mapping Depth")
	if (from_left == 1): #FROM LEFT side
		src_pos_j_p_size = strip_width*p_size-p_size
		src_pos_addition = layer.width * p_size #constant
		dest_get_init = -p_size
		dest_get_addition = layer.width * p_size #constant
		dest_set_addition = layer.width * p_size #constant
		dest_set_y_init = strip_width*p_size-p_size
		for j in range(strip_width,layer.width):
			if (j % 20 == 0):
				pdb.gimp_progress_update(float(j-strip_width)/(layer.width-strip_width))
			src_pos_j_p_size += p_size
			src_pos = src_pos_j_p_size #initialize;
			#dest_get = y*layer.width+(j-strip_width)*p_size
			#((y*layer.width+j-strip_width)) * p_size
			dest_get_init += p_size
			#dest_get += (j-strip_width)*p_size
			dest_get = dest_get_init
			#dest_set_y = j*p_size
			dest_set_y_init += p_size
			dest_set_y = dest_set_y_init
			for y in range(0,layer.height):
				#src_pos = (y*layer.width+j)*p_size
				#get hoehe_pixel, don't ask me what hoehe means it just looks like it does something with depth
				#hoehe = hoehe_pixel[0] * depth / 255;
				hoehe_pixel = source_pixels[src_pos:src_pos+p_size]
				#hoehe = height[hoehe_pixel[0]]; #use precalculated instead of having to redo division every time 
				#dest_get = ((y*layer.width+j-strip_width)) * p_size
				#if (dest_get >= 0 and dest_get < layer.height*layer.width+layer.width): #error out of range prevention
				pixel = dest_pixels[dest_get:dest_get+p_size]
				#dest_set = (y*layer.width+j-hoehe*_up_down) * p_size
				dest_set = dest_set_y - hoehevalue[hoehe_pixel[0]]
				dest_set_y += dest_set_addition
				#if (dest_set >= 0 and dest_set < layer.height*layer.width+layer.width): #error out of range prevention
				dest_pixels[dest_set:dest_set+p_size] = pixel
				#set it at the end.
				src_pos += src_pos_addition #do it at the end
				dest_get += dest_get_addition
	else:	#FROM MIDDLE runs two loops, one loop for each half
		loop_point = int(layer.width/2.0/strip_width*strip_width)
		loop_length = layer.width-strip_width
		src_pos_j_p_size = loop_point*p_size-p_size
		src_pos_addition = layer.width * p_size #constant
		dest_get_init = (loop_point-strip_width) * p_size - p_size
		dest_get_addition = layer.width * p_size #constant
		dest_set_addition = layer.width * p_size #constant
		dest_set_y_init = loop_point*p_size-p_size
		for j in range(loop_point,layer.width):
			if (j % 20==0):
				pdb.gimp_progress_update(float(j-loop_point)/loop_length)
			src_pos_j_p_size += p_size
			src_pos = src_pos_j_p_size #initialize;
			dest_get_init += p_size
			dest_get = dest_get_init   #initalize;
			dest_set_y_init += p_size
			dest_set_y = dest_set_y_init #initalize
			for y in range(0,layer.height):
				#src_pos = (y*layer.width+j)*p_size
				#get hoehe_pixel, don't ask me what hoehe means it just looks like it does something with depth
				#hoehe_pixel = source_pixels[src_pos:src_pos+p_size] 
				#hoehe = hoehe_pixel[0] * depth / 255;
				#hoehe = height[hoehe_pixel[0]]
				#dest_get = ((y*layer.width+j-strip_width)) * p_size
				pixel = dest_pixels[dest_get:dest_get+p_size]
				#if (dest_get >= 0 and dest_get < layer.height*layer.width+layer.width): #error out of range prevention
				hoehe_pixel = source_pixels[src_pos:src_pos+p_size] 
				#hoehe = height[hoehe_pixel[0]]
				#dest_set = (y*layer.width+j-hoehe*_up_down) * p_size
				
				dest_set = dest_set_y - hoehevalue[hoehe_pixel[0]]
				dest_set_y += dest_set_addition
				
				#	if (dest_set >= 0 and dest_set < layer.height*layer.width+layer.width): #error out of range prevention
				dest_pixels[dest_set:dest_set+p_size] = pixel
				src_pos += src_pos_addition #do it at the end
				dest_get += dest_get_addition
		#Pre calculating things before 2nd loop it's going in negative direction		
		src_pos_j_p_size = loop_point*p_size+p_size
		src_pos_addition = layer.width * p_size #constant
		dest_get_init = loop_point * p_size + p_size
		dest_get_addition = layer.width * p_size #constant
		dest_set_addition = layer.width * p_size #constant
		dest_set_y_init = loop_point*p_size+p_size
		dest_set_strip = strip_width * p_size #constant
		for j in range(loop_point,strip_width,-1):
			if (j % 20==0):
				pdb.gimp_progress_update(float(layer.width-j)/loop_length)
			src_pos_j_p_size -= p_size
			src_pos = src_pos_j_p_size #initialize
			dest_get_init -= p_size
			dest_get = dest_get_init #initialize
			#dest_get = j * p_size
			dest_set_y_init -= p_size
			dest_set_y = dest_set_y_init #initialize
			#dest_set = ((y*layer.width+j+hoehe*_up_down-strip_width)) * p_size
			#dest_set = ((j*p_size-strip_width*p_size)
			for y in range(0,layer.height):
				#src_pos = (y*layer.width+j)*p_size
				#get hoehe_pixel, don't ask me what hoehe means it just looks like it does something with depth
				hoehe_pixel = source_pixels[src_pos:src_pos+p_size] 
				#hoehe = hoehe_pixel[0] * depth / 255;
				#hoehe = height[hoehe_pixel[0]];
				#dest_get = (y*layer.width+j) * p_size
				#if (dest_get >= 0 and dest_get < layer.height*layer.width+layer.width): #error out of range prevention
				pixel = dest_pixels[dest_get:dest_get+p_size]
				#dest_set = ((y*layer.width+j+hoehe*_up_down-strip_width)) * p_size
				dest_set = dest_set_y + hoehevalue[hoehe_pixel[0]] - dest_set_strip
				dest_set_y += dest_set_addition
				#	if (dest_set >= 0 and dest_set < layer.height*layer.width+layer.width): #error out of range prevention
				dest_pixels[dest_set:dest_set+p_size] = pixel			
				src_pos += src_pos_addition #do it at the end
				dest_get += dest_get_addition
	# -----------------------------------------------------------------------------------------------
	# elapsed = timeit.default_timer() - start_time
	# pdb.gimp_message(elapsed);
	# # fast pixel updates to whole image at every end ---------------
	dstRgn[0:new_layer.width, 0:new_layer.height] = dest_pixels.tostring() 	
	new_layer.flush()
	new_layer.merge_shadow(True)
	new_layer.update(0, 0,new_layer.width,new_layer.height)
	pdb.gimp_displays_flush()
	# fast pixel updates ends --------------------------------------


	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	#return

register(
	"python_fu_tt_magic_eye",                           
	"Maps a Magic Eye",
	"Maps a Magic Eye",
	"Tin Tran",
	"Tin Tran",
	"May 2021",
	"<Image>/Python-Fu/AAA Magic Eye...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_DRAWABLE, "texture", "Texture (Layer):", None),	
	(PF_SPINNER, "strip_width", "Stripe Width:", 80, (1, 300, 1)),
	(PF_SPINNER, "depth", "Depth:", 15, (1, 100, 1)),
	(PF_TOGGLE, "fromleft", "From Left/Middle (YES/NO):", 0),
	(PF_TOGGLE, "updown", "Cross-Eye/Stereo (YES/NO):", 1),
	#(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	#(PF_TOGGLE, "keep_aspect", "Keep Aspect Ratio of Border:", 1),
	#(PF_TOGGLE, "mirror","Mirror/Reflect Borders:", 1),
	#(PF_TOGGLE, "drop_shadow","Drop Shadow:", 1),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_blur", "Shadow Blur Radius:", 15, (0, 1000, 1)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 60, (0, 100, 1)),
	],
	[],
	python_tt_magic_eye)

main()
