#!/usr/bin/env python

# make_icon.py Rel 3
# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Instead of making color to alpha, clear and invert fill with black instead.
# Rel 3: Change HSV blend to RGB blend so you don't get the rainbow bug.
import math
import string
#import Image
from gimpfu import *
from array import array
import sys
def python_make_icon(image, layer, toalpha, light, dark):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	pdb.gimp_selection_none(image)

	if not pdb.gimp_drawable_has_alpha(layer):
		pdb.gimp_layer_add_alpha(layer)
	pdb.gimp_item_set_name(layer,"Silhoutte")

	#pdb.plug_in_colortoalpha(image,layer,toalpha)

	#set alpha lock to fill it with black
	#pdb.gimp_layer_set_lock_alpha(layer,TRUE)
	pdb.gimp_context_set_antialias(TRUE)
	pdb.gimp_context_set_feather(FALSE)
	pdb.gimp_context_set_sample_merged(FALSE)
	pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_COMPOSITE)
	pdb.gimp_context_set_sample_threshold(0.5)
	pdb.gimp_context_set_sample_transparent(TRUE)
	#num_channels,pixel = pdb.gimp_drawable_get_pixel(layer,1,1)
	#color = (pixel[0],pixel[1],pixel[2])
	pdb.gimp_image_select_color(image,CHANNEL_OP_REPLACE,layer,toalpha)
	pdb.gimp_edit_clear(layer)
	pdb.gimp_selection_invert(image)
	pdb.gimp_context_set_foreground((0,0,0))
	pdb.gimp_edit_fill(layer,FILL_FOREGROUND)
	#unset alpha lock	
	#pdb.gimp_layer_set_lock_alpha(layer,FALSE)	

	#draw
	buttonlayer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"button",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,buttonlayer,None,1)
	pdb.gimp_selection_all(image)
	pdb.script_fu_selection_rounded_rectangle(image,buttonlayer,25,FALSE)
	pdb.gimp_context_set_foreground(light)
	pdb.gimp_context_set_background(dark)
	pdb.gimp_edit_blend(buttonlayer,BLEND_FG_BG_RGB,LAYER_MODE_NORMAL,GRADIENT_LINEAR,100,0,FALSE,FALSE,FALSE,3,0.2,FALSE,image.width/2,0,image.width/2,image.height)

	#add layer effects for icon and button face
	pdb.gimp_selection_none(image)
	pdb.python_layerfx_inner_shadow(image,layer,(0,0,0),75,0,0,3,1,45.0,8,90.0,9.0,FALSE)
	pdb.python_layerfx_drop_shadow(image,layer,(255,255,255),75.0,0,0.0,5,10.0,2,90,4.0,TRUE,FALSE)
	pdb.python_layerfx_outer_glow(image,layer,(0,0,0),22,0,0.0,3,0.0,1,TRUE,FALSE)
	pdb.python_layerfx_inner_shadow(image,buttonlayer,(0,0,0),45,0,0,3,1,0.0,10,-90.0,3.0,FALSE)
	pdb.python_layerfx_inner_shadow(image,buttonlayer,(255,255,255),45,0,0,4,1,35.0,4,100,5.0,FALSE)

	pdb.gimp_layer_set_opacity(layer,83)

	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_make_icon_tt",
	"Creates an Icon out off image",
	"Creates an Icon out off image",
	"Tin Tran",
	"Tin Tran",
	"October 2019",
	"<Image>/Python-Fu/Make Icon...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_COLOR, "toalpha", "Color to Make alpha (to turn image into silhoutte):", (255, 255, 255)),
	(PF_COLOR, "light", "Icon Color:", (255, 156, 71)),
	(PF_COLOR, "dark", "Icon Slightly Darker Color:", (183, 110, 51)),
	],
	[],
	python_make_icon)

main()
