#!/usr/bin/env python

# match_squares.py Rel 1
# This plug-in will try to compose selected layer with collage paper layer
# It'll random pick a square and try to match up with closest match then mark it as used so it doesn't use it again

# Created by Tin Tran http://gimplearn.net
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Faster version.
#import string
#import Image
import random
import math
#from array import array
import sys
from gimpfu import *

def python_tt_match_squares(image, layer, paper_layer, dimension):

	#Add alpha to paper_layer if it doesn't have one.
	#TODO create image to add paper layer to.
	#Uncessary as we'll load image and remove them later after each download
	if not paper_layer.has_alpha:
		pdb.gimp_layer_add_alpha(paper_layer)
	
	#round to nearest multiples of dimension
	x_tiles = int(image.width/dimension)
	y_tiles = int(image.height/dimension)

	width = x_tiles * dimension
	height = y_tiles * dimension 

	x_paper_tiles =  int(paper_layer.width/dimension)
	y_paper_tiles =  int(paper_layer.height/dimension)
	paper_width = x_paper_tiles * dimension
	paper_height = y_paper_tiles * dimension 

	greater_width = max(width,paper_width)
	greater_height = max(height,paper_height)
	#pdb.gimp_image_crop(image,greater_width,greater_height,0,0)
	pdb.gimp_image_resize(image,greater_width,greater_height,0,0)
	
	compare_layer = pdb.gimp_layer_new(image,image.width,image.height,RGB_IMAGE,"compare",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,compare_layer,None,0);
	
	work_layer = pdb.gimp_layer_new(image,image.width,image.height,RGB_IMAGE,"collage",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,work_layer,None,0);
	pdb.gimp_item_set_visible(work_layer,FALSE);

	locs = [] #image locations
	for y in range(0,y_tiles):
		for x in range(0,x_tiles):
			locs.append([x,y])

	unused = [] #collage image locations left over to search
	for y in range(0,y_paper_tiles):
		for x in range(0,x_paper_tiles):
			unused.append([x,y])
	
	
	total_tiles = len(locs)
	locs.sort(key=lambda x: random.randrange(0,total_tiles)-(total_tiles/2))
	#unused.sort(key=lambda x: random.randrange(0,total_tiles)-(total_tiles/2))
	#pdb.gimp_item_set_visible(paper_layer,FALSE);
	total_checks = (total_tiles+1)*(total_tiles/2)
	check_count = 0
	num_patterns, pattern_list = pdb.gimp_patterns_list("") #get pattern list
	pdb.gimp_context_set_pattern(pattern_list[0]) #use the first one which is Clipboard in all languages.
	for i in range(0,total_tiles):
		#create paper layer here from the net of same size as our image
		#paper_layer = pdb.gimp_image_get_active_layer(random_image)

		
		#progress update
		pdb.gimp_item_set_visible(work_layer,FALSE);
		loc = locs[i] #without deleting

		#copy loc content and fill compare layer with it
		pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,loc[0]*dimension,loc[1]*dimension,dimension,dimension)
		pdb.gimp_edit_copy(layer)
		pdb.gimp_selection_all(image)
		pdb.gimp_edit_fill(compare_layer,FILL_PATTERN)
		#makes copy of paper layer
		collage_paper = pdb.gimp_layer_new_from_drawable(paper_layer,image)
		pdb.gimp_image_insert_layer(image,collage_paper,None,0)
		pdb.gimp_item_set_visible(collage_paper,TRUE)
		pdb.gimp_layer_set_mode(collage_paper,LAYER_MODE_DIFFERENCE)
		visible_layer = pdb.gimp_layer_new_from_visible(image,image,"visible")
		pdb.gimp_image_insert_layer(image,visible_layer,None,0)
		sortme = []
		pdb.gimp_progress_update(float(i)/total_tiles)
		pdb.gimp_progress_set_text("Squares Searching...")
		for j in range(0,len(unused)):
			#check_count += 1
			#pdb.gimp_progress_update(float(check_count)/total_checks)
			unuse = unused[j]
			#xoffset = (loc[0] - unuse[0]) * dimension
			#yoffset = (loc[1] - unuse[1]) * dimension
			#pdb.gimp_layer_set_offsets(collage_paper,xoffset,yoffset)
			pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,unuse[0]*dimension,unuse[1]*dimension,dimension,dimension)
			r, _, _, _, _, _ = pdb.gimp_drawable_histogram(visible_layer,HISTOGRAM_RED,0,1)
			g, _, _, _, _, _ = pdb.gimp_drawable_histogram(visible_layer,HISTOGRAM_GREEN,0,1)
			b, _, _, _, _, _ = pdb.gimp_drawable_histogram(visible_layer,HISTOGRAM_BLUE,0,1)
			COLOR_VALUE = r+g+b;
			
			sortme.append([COLOR_VALUE,j]) #save the rgb sum from black and index to unused.
		pdb.gimp_image_remove_layer(image,visible_layer)	
		sortme.sort(key=lambda x:x[0]) #first item is now closest to black
		pdb.gimp_item_set_visible(work_layer,TRUE);
		j = sortme[0][1]; #first item
		unuse = unused[j];
		#reselect that best condition
		xoffset = (loc[0] - unuse[0]) * dimension
		yoffset = (loc[1] - unuse[1]) * dimension
		pdb.gimp_layer_set_offsets(collage_paper,xoffset,yoffset)
		pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,loc[0]*dimension,loc[1]*dimension,dimension,dimension)
		#invert clear and merge down
		pdb.gimp_selection_invert(image)
		pdb.gimp_edit_clear(collage_paper)
		work_layer = pdb.gimp_image_merge_down(image,collage_paper,CLIP_TO_BOTTOM_LAYER)
		#del unused[j] #delele this item to not use it again
		#delete image that contains paper layer here.
		#pdb.gimp_image_delete(random_image) #we're done delete random image from memory
	pdb.gimp_displays_flush()

register(
	"python_fu_tt_match_squares",                           
	"Match up squares from collage layer until finish",
	"Match up squares from collage layer until finish",
	"Tin Tran",
	"Tin Tran",
	"July 2020",
	"<Image>/Python-Fu/AA Match Squares...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_DRAWABLE, "paper_layer", "(Collage) Paper Layer (to use):", None),
	(PF_SPINNER, "dimension", "Dimension (pixels):", 50, (10, 1000, 1)),
	],
	[],
	python_tt_match_squares)

main()
