#!/usr/bin/env python

# matrix_rain.py Rel 3
# Created by Tin Tran 
# Comments directed to http://gimplearn.net
# Roughly based on https://www.gimplearn.net/viewtopic.php/Matrix-Effect-Script-Request?f=18&t=1696
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Add an addition flipped result at 40% opacity for more depth.
# Rel 3: Use New Overlay-mode for GIMP 2.10

from gimpfu import *
import random
fontname = "Courier New";
chars = ' !"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~'
def getweirdchars():
	global chars
	for i in range(128,1280):
		ch = ""
		ch += chr(i)
		if ch.isprintable():
		    chars += ch
def getrandomtext():
	s = ""
	while (len(s)<25000):
		s += chars[random.randrange(0,len(chars))]
		if (random.randrange(0,160)==0):
			s += "\n"
	
	#pdb.gimp_message(s)
	return s
def python_matrix_rain_tt(image, layer):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#pdb.gimp_progress_init("Processing...",None)
	#pdb.gimp_progress_update(float(intensity)/255.0)


        #creates a portrait image, we'll rotate it later since we can't create vertical text in GIMP
	new_image = pdb.gimp_image_new(1080,1920,0)
	new_display = pdb.gimp_display_new(new_image)
	#creates black filled layer
	black_layer = pdb.gimp_layer_new(new_image,new_image.width,new_image.height,1,"Black layer",100,0)
	pdb.gimp_image_insert_layer(new_image,black_layer,None,0)
	pdb.gimp_context_set_foreground((0,0,0))
	pdb.gimp_edit_fill(black_layer,0) #FOREGROUND-FILL
	#creates 1st text layer
	text1_layer = pdb.gimp_text_layer_new(new_image,getrandomtext(),fontname,8,0)
	pdb.gimp_image_insert_layer(new_image,text1_layer,None,0)
	pdb.gimp_text_layer_resize(text1_layer,new_image.width,new_image.height)
	pdb.gimp_text_layer_set_color(text1_layer,(78,255,115))
	pdb.gimp_text_layer_set_justification(text1_layer,0) #TEXT-JUSTIFY-LEFT
	pdb.gimp_text_layer_set_letter_spacing(text1_layer,0) #VA setting in Photoshop I think
    #creates 2nd text layer
	text2_layer = pdb.gimp_text_layer_new(new_image,getrandomtext(),fontname,10,0)
	pdb.gimp_image_insert_layer(new_image,text2_layer,None,0)
	pdb.gimp_text_layer_resize(text2_layer,new_image.width,new_image.height)
	pdb.gimp_text_layer_set_color(text2_layer,(78,255,115))
	pdb.gimp_text_layer_set_justification(text2_layer,0) #TEXT-JUSTIFY-LEFT
	pdb.gimp_text_layer_set_letter_spacing(text2_layer,10) #VA setting in Photoshop I think
	pdb.gimp_text_layer_set_line_spacing(text2_layer,10)
	#creates 3rd text layer
	text3_layer = pdb.gimp_text_layer_new(new_image,getrandomtext(),fontname,12,0)
	pdb.gimp_image_insert_layer(new_image,text3_layer,None,0)
	pdb.gimp_text_layer_resize(text3_layer,new_image.width,new_image.height)
	pdb.gimp_text_layer_set_color(text3_layer,(78,255,115))
	pdb.gimp_text_layer_set_justification(text3_layer,0) #TEXT-JUSTIFY-LEFT
	pdb.gimp_text_layer_set_letter_spacing(text3_layer,25) #VA setting in Photoshop I think
	pdb.gimp_text_layer_set_line_spacing(text3_layer,25)
	#creates cloud layer to use as masks for text2 and text3 layers
	cloud_layer = pdb.gimp_layer_new(new_image,new_image.width,new_image.height,1,"Cloud layer used for masking",100,0)
	pdb.gimp_image_insert_layer(new_image,cloud_layer,None,0)
	pdb.gimp_context_set_foreground((255,255,255))
	pdb.gimp_edit_fill(cloud_layer,0) #FOREGROUND-FILL
	pdb.script_fu_difference_clouds(new_image,cloud_layer)
	cloud_layer = pdb.gimp_image_get_layer_by_name(new_image,"Cloud layer used for masking")
	#copy it to use as mask
	pdb.gimp_selection_all(new_image)
	pdb.gimp_edit_copy(cloud_layer)
	#create mask for textlayer2
	mask2 = pdb.gimp_layer_create_mask(text2_layer,0)
	pdb.gimp_layer_add_mask(text2_layer,mask2)
	floating_sel = pdb.gimp_edit_paste(mask2,True)
	pdb.gimp_floating_sel_anchor(floating_sel)
	#rotate 180 to have a different mask
	cloud_layer = pdb.gimp_item_transform_rotate_simple(cloud_layer,1,True,0,0)
	pdb.gimp_floating_sel_anchor(cloud_layer)
	cloud_layer = pdb.gimp_image_get_layer_by_name(new_image,"Cloud layer used for masking")
	#copy it to use as mask
	pdb.gimp_selection_all(new_image)
	pdb.gimp_edit_copy(cloud_layer)
	#create mask for textlayer2
	mask3 = pdb.gimp_layer_create_mask(text3_layer,0)
	pdb.gimp_layer_add_mask(text3_layer,mask3)
	floating_sel2 = pdb.gimp_edit_paste(mask3,True)
	pdb.gimp_floating_sel_anchor(floating_sel2)
	#random dots on the masks
	pdb.gimp_context_set_brush_size(360)
	pdb.gimp_context_set_brush("2. Hardness 050")
	pdb.gimp_context_set_foreground((0,0,0))
	for i in range(0,15):
		pdb.gimp_paintbrush(mask2,0,2,[random.randrange(0,new_image.width),random.randrange(0,new_image.height)],0,0)
		pdb.gimp_paintbrush(mask3,0,2,[random.randrange(0,new_image.width),random.randrange(0,new_image.height)],0,0)
	#apply all masks
	pdb.gimp_layer_remove_mask(text2_layer,0)
	pdb.gimp_layer_remove_mask(text3_layer,0)
	#merge down text layers
	merged = pdb.gimp_image_merge_down(new_image,text3_layer,0)
	merged = pdb.gimp_image_merge_down(new_image,merged,0)
	#remove cloud now, we're done with it, copy and motion blur the merged result
	pdb.gimp_image_remove_layer(new_image,cloud_layer)
	merged_copy = pdb.gimp_layer_new_from_drawable(merged,new_image)
	pdb.gimp_image_insert_layer(new_image,merged_copy,None,0)
	pdb.plug_in_mblur(new_image,merged_copy,0,15,0,0,0)
	#white lines layer
	whitelines_layer = pdb.gimp_layer_new(new_image,new_image.width,new_image.height,1,"White Lines",100,5)
	pdb.gimp_image_insert_layer(new_image,whitelines_layer,None,0)
	pdb.gimp_context_set_foreground((255,255,255))
	pdb.gimp_context_set_brush_size(20)
	shortest = 80
	longest = 600
	#draw white lines randomly
	for i in range(0,180):
		y = random.randrange(0,new_image.height)
		x = random.randrange(-shortest,new_image.width)
		length = random.randrange(0,longest-shortest)+shortest
		pdb.gimp_paintbrush(whitelines_layer,0,4,[x,y,x+length,y],0,0)
	#make a copy of whiteline layer and rotate it 180
	whitelines_copy = pdb.gimp_layer_new_from_drawable(whitelines_layer,new_image)
	pdb.gimp_image_insert_layer(new_image,whitelines_copy,None,0)
	floating_sel = pdb.gimp_item_transform_rotate_simple(whitelines_copy,1,True,0,0)
	pdb.gimp_floating_sel_anchor(floating_sel)
	#set overlay mode on white lines layers
	if pdb.gimp_version()[0:3] == "2.8":
		pdb.gimp_layer_set_mode(whitelines_layer,5) #5 is OVERLAY MODE
		pdb.gimp_layer_set_mode(whitelines_copy,5)
	elif pdb.gimp_version()[0:4] == "2.10":
		pdb.gimp_layer_set_mode(whitelines_layer,LAYER_MODE_OVERLAY) #5 is OVERLAY MODE
		pdb.gimp_layer_set_mode(whitelines_copy,LAYER_MODE_OVERLAY)
	#make visible from result
	visible = pdb.gimp_layer_new_from_visible(new_image,new_image,"Visible Matrix Rain")
	pdb.gimp_image_insert_layer(new_image,visible,None,0)
	if pdb.gimp_version()[0:3] == "2.8":
		pdb.gimp_levels(visible,0,0,167,1.0,0,255)
		pdb.gimp_brightness_contrast(visible,50,100)
	elif pdb.gimp_version()[0:4] == "2.10":
		pdb.gimp_brightness_contrast(visible,0,40)
	#finalize by rotating whole image by 90 degrees
	pdb.gimp_image_rotate(new_image,0)
	#add additional flipped copy for more depth
	visible_copy = pdb.gimp_layer_new_from_drawable(visible,new_image)
	pdb.gimp_image_insert_layer(new_image,visible_copy,None,0)
	pdb.gimp_item_transform_flip_simple(visible_copy,0,True,0)
	pdb.gimp_layer_set_opacity(visible_copy,40)
	pdb.gimp_layer_set_mode(visible_copy,7) #addition


	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_matrix_rain_tt",                           
	"Creates Matrix-Rain Image",
	"Creates Matrix-Rain Image",
	"Tin Tran",
	"Tin Tran",
	"August 2018",
	"<Image>/Python-Fu/Matrix Rain",             #Menu path
	"RGB*, GRAY*", 
	[
	#(PF_GRADIENT, "gradient", "Background Gradient:", 0),
	#(PF_TOGGLE, "reverse",   "Reverse Gradient:", 0),
	#(PF_SPINNER, "slice_angle", "Rotate angle(before slicing):", -5, (-180, 180, 1)),
	#(PF_SPINNER, "slice_shift", "Slice Shift(pixels):", -30, (-100, 100, 1)),
	#(PF_SPINNER, "text_angle", "Rotate angle(when complete):", -8, (-180, 180, 1)),
	],
	[],
	python_matrix_rain_tt)

main()
