#!/usr/bin/env python

# Morpheus_automate Rel 6
# Created by Tin Tran http://bakon.ca/gimplearn/
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Added ability to select color for Bar and Font, and all other colors inverted value of color
# Rel 3: Fixed y placement calculation bug.
# Rel 4: Open .tiff files as well as .tif files
# Rel 5: Flatten image after opening file as file might have mutiple layers.
# Rel 6: Custom for Morpheus according to his professor.
import math
import string
#import Image
from gimpfu import *
from array import array
import os
cm_per_inch = 2.54

def python_morpheus_automate(image, layer, inputdir, tifoutputdir, jpgoutputdir, a, color) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#pdb.gimp_message("This is sample shell script.")
	#PUT YOUR CODE HERE 
	#images = list(gimp.image_list())
	layername = pdb.gimp_item_get_name(layer)
	tiffilename = os.path.splitext(layername)[0] + ".tif"
	jpgfilename = os.path.splitext(layername)[0] + ".jpg"
	
	#pdb.gimp_message(tiffilename)
	#pdb.gimp_message(jpgfilename)
	
	tfilename = os.path.join(tifoutputdir,tiffilename)
	jfilename = os.path.join(jpgoutputdir,jpgfilename)
	
	selection = pdb.gimp_selection_bounds(image);
	
	x1 = selection[1];
	y1 = selection[2];
	x2 = selection[3];
	y2 = selection[4];
	
	pdb.gimp_image_set_resolution(image,cm_per_inch*a,cm_per_inch*a)
	pdb.gimp_image_crop(image,x2-x1,y2-y1,x1,y1)
	
	#saves file as a tiff file name
	pdb.file_tiff_save(image,layer,tfilename,tfilename,0)
	
	inverted_color = (255 - color[0], 255-color[1], 255-color[2])
	map_legend_image,theDisplay = pdb.python_fu_map_scale_legend(image,layer,float(8)/286 * image.height,a,a,1,"cm",0,1,inverted_color,color,\
		inverted_color,"Times New Roman,",float(13)/286 * image.height,color,inverted_color)
		
	pdb.gimp_selection_all(map_legend_image)
	pdb.gimp_edit_copy_visible(map_legend_image)
	
	floating_selection = pdb.gimp_edit_paste(layer,1)
	pdb.gimp_floating_sel_to_layer(floating_selection)
	
	pdb.gimp_layer_translate(floating_selection,\
		-(layer.width/2) + ((float(32)/640) * layer.width) + (floating_selection.width/2),\
		+(layer.height/2) - ((float(15)/286) * layer.height) - (floating_selection.height/2))
	
	#delete scale legend image now that we have it
	pdb.gimp_display_delete(theDisplay)
	pdb.gimp_image_delete(map_legend_image)
	
	final_layer = pdb.gimp_image_merge_down(image,floating_selection,EXPAND_AS_NECESSARY)
	pdb.file_jpeg_save(image,final_layer,jfilename,jfilename,1,0,1,1,"Created with GIMP",2,1,0,0)
	
	pdb.gimp_image_clean_all(image)
	found = False
	for name in os.listdir(inputdir):
		if name.lower() == layername.lower():
			found = True
		if ((found) and (os.path.splitext(name)[1].lower() == ".tif" or os.path.splitext(name)[1].lower() == ".tiff") and (name.lower() != layername.lower())):
			loadfilename = os.path.join(inputdir,name)
			nextimage = pdb.file_tiff_load(loadfilename,loadfilename)
			pdb.gimp_display_new(nextimage)
			pdb.gimp_image_flatten(nextimage)
			#sets name so that we can open next image and so on...
			activelayer = pdb.gimp_image_get_active_layer(nextimage)
			pdb.gimp_item_set_name(activelayer,name)
			break
	
	#image = image #gimp.image_list()[0]
	#drawable = layer #image.layers[0]
	#for x in range(10):
	#	for y in range(10):
	#		pdb.gimp_drawable_set_pixel(drawable,x+10,y+10,3,(0,0,0))
	#pdb.gimp_drawable_update(drawable,0,0,drawable.width,drawable.height)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_morpheus_automate",                           
	"Automate steps described by Morpheus",
	"Automate steps described by Morpheus...",
	"Tin Tran",
	"Tin Tran",
	"2014",
	"<Image>/Python-Fu/Morpheus Automate",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_DIRNAME, "inputdir", "Input Directory(Where original files are located):", "C:"),
	(PF_DIRNAME, "tifoutputdir", ".tif Output Directory(Where .tif files will be saved):", "C:"),
	(PF_DIRNAME, "jpgoutputdir", ".jpg Output Directory(Where .jpg files will be saved):", "C:"),
	(PF_SPINNER, "a", "Pixels (A):", 100, (1, 5000, 0.1)),
	(PF_COLOR, "color", "Bar and Font color:", (255, 255, 255)),
	],
	[],
	python_morpheus_automate)

main()
