#!/usr/bin/env python

# multiple_zooms.py Rel 5
# Created by Tin Tran http://gimplearn.net
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: allow base opacity change.
# Rel 3: allow center coordinate specification
# Rel 5: UTF-8 instead of UTF-8 BOM
import math
import string
#import Image
from gimpfu import *

def python_tt_multiple_zooms(image, layer, zooms, frames, maxscale, base_opacity, center_X, center_Y):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#creates new image
	new_image = pdb.gimp_image_new(image.width,image.height,RGB)
	new_display = pdb.gimp_display_new(new_image)

	scale_fraction = (maxscale-100)*1.0/frames/zooms/100
	opacity_fraction = base_opacity/frames/zooms
	for f in range(0,int(frames)):
		new_layers = []
		for z in range(0,int(zooms)):
			currentzoom = (z * frames) + f
			new_layer = pdb.gimp_layer_new_from_drawable(layer,image)
			pdb.gimp_image_insert_layer(image,new_layer,None,0)
			new_width = layer.width * (1+scale_fraction*currentzoom)
			new_height = layer.height * (1+scale_fraction*currentzoom)
			offsetX = (center_X * (scale_fraction*currentzoom))
			offsetY = (center_Y * (scale_fraction*currentzoom))
			new_opacity = max(0,base_opacity - currentzoom * opacity_fraction)
			pdb.gimp_layer_scale(new_layer,new_width,new_height,FALSE)
			pdb.gimp_layer_set_offsets(new_layer,-offsetX,-offsetY)
			pdb.gimp_layer_set_opacity(new_layer,new_opacity)
			new_layers.append(new_layer)
		visible_layer = pdb.gimp_layer_new_from_visible(image,new_image,"frame" + str(f+1))
		pdb.gimp_image_insert_layer(new_image,visible_layer,None,0)
		for new_layer in new_layers:
			pdb.gimp_image_remove_layer(image,new_layer)

	pdb.gimp_selection_none(image)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_multiple_zooms",                           
	"Multiple zooms effect",
	"Multiple zooms effect",
	"Tin Tran",
	"Tin Tran",
	"June 2018",
	"<Image>/Python-Fu/Multiple Zooms...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "zooms","# of Zooms:", 7, (1, 20, 1)),
	(PF_SPINNER, "frames","# of Frames:", 3, (1, 20, 1)),
	(PF_SPINNER, "maxscale","Max Scale Percentage:", 200, (5,1000,1)),
	(PF_SPINNER, "base_opacity","Base Opacity:", 30, (5, 50, 1)),
	(PF_SPINNER, "center_X","Center X coordinate:", 500, (-5000, 5000, 0.1)),
	(PF_SPINNER, "center_Y","Center Y coordinate:", 500, (-5000, 5000, 0.1)),
	],
	[],
	python_tt_multiple_zooms)

main()
