#!/usr/bin/env python

# Nelo Plaid Background Rel 2
# Created by Tin Tran http://bakon.ca/gimplearn/
# Comments directed to http://gimplearn.com or http://gimp-forum.net or http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Not use hardcoded "Clipboard" but get it from patterns list.
import math
import string
#import Image
from gimpfu import *
from array import array
import sys
def python_nelo_plaid_background(image, layer, depth, expand) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#expand image
	width = image.width
	height = image.height
	pdb.gimp_image_resize(image,width+expand*2,height+expand*2,expand,expand)
	
	#creates new layers to fill below all other layers
	newlayer1 = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Nelo Plaid 1",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,newlayer1,None,len(image.layers))
	
	newlayer2 = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"Nelo Plaid 2",50,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,newlayer2,None,len(image.layers)-1)

	#creates smooth palette but not show it.
	smoothimage,smoothlayer = pdb.plug_in_smooth_palette(image,layer,256,64,depth,0)
	
	#work with clipboard
	#pdb.gimp_context_set_pattern("Clipboard")
	pdb.gimp_context_set_pattern(pdb.gimp_patterns_list('')[1][0])
	
	#copy pattern to clipboard
	pdb.gimp_selection_all(smoothimage)
	pdb.gimp_edit_copy(smoothlayer)
	
	#fill Nelo plaid 1
	pdb.gimp_drawable_fill(newlayer1,PATTERN_FILL)
	 
	#rotate smooth by 90 degrees
	pdb.gimp_image_rotate(smoothimage,0) 
	
	#copy pattern to clipboard
	pdb.gimp_selection_all(smoothimage)
	pdb.gimp_edit_copy(smoothlayer)
	
	#fill Nelo plaid 2
	pdb.gimp_drawable_fill(newlayer2,PATTERN_FILL)
	
	#deletes smooth image (from memory)
	pdb.gimp_image_delete(smoothimage)
	

	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_nelo_plaid_background",                           
	"Creates Nelo Plaid Background using active layer as smooth palette",
	"Creates Nelo Plaid Background using active layer as smooth palette",
	"Tin Tran",
	"Tin Tran",
	"2017",
	"<Image>/Python-Fu/Nelo Plaid Background ...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "depth", "Smooth Palette Search Depth:", 50, (1, 1024, 1)),
	(PF_SPINNER, "expand", "Expand Border(pixels):", 100, (0, 2048, 1)),
	],
	[],
	python_nelo_plaid_background)

main()
