#!/usr/bin/env python

# paint_paths_in_steps.py Rel 8
# Created by Tin Tran
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Follow path closer to pixel accuracy
# Rel 3: Drop shadow option
# Rel 4: Animate hand option with hand-layer input and pen-location vectors and animates hand around like as if it's drawing it
# Rel 5: You can choose to paint a drawable which can be a layer mask to reveal drawing underneath.
# Rel 6: Only create frame without hand at the very end frame (not when there's still paths to draw so hand doesn't disappear in middle of animation)
# Rel 7: Options to add animation of transitioning IN (from outside of whiteboard), BETWEEN shapes, and OUT (of whiteboard at the end).
# Rel 8: Fix bug for when user choose drop shadow.
from gimpfu import *
import random
def python_tt_paint_paths_in_steps(image, layer, steps, dropshadow, shadow_offset_x,shadow_offset_y,shadow_blur_radius,shadow_color,shadow_opacity, drawhand, handlayer, penvector, paintexisting, paintdrawable, transin,transmid,transout) : #FUNCTION DEFINITION
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	pdb.gimp_selection_none(image)
	#creates a working layer
	if paintexisting == 1:
		work_layer = pdb.gimp_layer_get_mask(paintdrawable)
	else:
		work_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'work layer',100,LAYER_MODE_NORMAL)
		pdb.gimp_image_insert_layer(image,work_layer,None,0)

	#if draw hand move hand layer on very top
	if drawhand == 1:
		pdb.gimp_image_raise_item_to_top(image,handlayer)
		hand_offset_x,hand_offset_y = pdb.gimp_drawable_offsets(handlayer)
		#get location of pen vector, first point only
		num_strokes_pen, stroke_ids_pen = pdb.gimp_vectors_get_strokes(penvector)
		pen_type,pen_num_points,pen_controlpoints,pen_closed = pdb.gimp_vectors_stroke_get_points(penvector,stroke_ids_pen[0])
		absolute_pen_x,absolute_pen_y = pen_controlpoints[2:4]

	#new image used for animation
	new_image = pdb.gimp_image_new(image.width,image.height,RGB)
	new_display = pdb.gimp_display_new(new_image)

	#get active vector and work with it
	vectors = pdb.gimp_image_get_active_vectors(image)
	num_strokes, stroke_ids = pdb.gimp_vectors_get_strokes(vectors)

	#get total stroke length
	total_length = 0
	for i in range(0,num_strokes):
		stroke_id = stroke_ids[i] #current stroke
		current_length = pdb.gimp_vectors_stroke_get_length(vectors,stroke_id,0.01)
		total_length += current_length


	step_length = float(total_length)/steps #length per step
	
	#TRANSITION IN
	if (drawhand == 1) and (transin == 1): #if user choose to transition in we animate hand into first shape
		#get starting point of first path
		start_type,start_num_points,start_controlpoints,start_closed = pdb.gimp_vectors_stroke_get_points(vectors,stroke_ids[0])
		start_x,start_y = start_controlpoints[2:4]
		#starts from outside where the layer would be right aligned and it's below the height
		out_x = image.width - handlayer.width + (absolute_pen_x - hand_offset_x)
		out_y = image.height + (absolute_pen_y - hand_offset_y)
		difx,dify = (out_x - start_x),(out_y - start_y)
		move_dist = (difx**2 + dify**2)**0.5
		#make it steps such that we get double the regular speed
		steps = move_dist/(step_length*2.0) #get number of steps based on steplength so animation looks same speed as the rest of it
		step = 0
		while step < steps:
			step += 1.0
			if step > steps:
				step = steps
			at_proportion = step/steps 	
			nx = out_x - (at_proportion * difx)
			ny = out_y - (at_proportion * dify)	
			
			move_offset_x = nx - absolute_pen_x + hand_offset_x
			move_offset_y = ny - absolute_pen_y + hand_offset_y
			pdb.gimp_layer_set_offsets(handlayer,move_offset_x,move_offset_y)
			
			new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"painted")
			pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
	has_shadow = False		
	for i in range(0,num_strokes):
		stroke_id = stroke_ids[i] #current stroke
		current_length = pdb.gimp_vectors_stroke_get_length(vectors,stroke_id,0.01)
		current_at = 0.01
		x,y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,current_at,0.01)
		#TRANSITION MID (animation of hand in between shapes)
		if (drawhand == 1) and (transmid == 1) and (i > 0): #if user choose to transition in we animate hand into first shape
				#get starting point of first path
			#start_type,start_num_points,start_controlpoints,start_closed = pdb.gimp_vectors_stroke_get_points(vectors,stroke_ids[i])
			#start_x,start_y = start_controlpoints[-4:-2]
			#starts from outside where the layer would be right aligned and it's below the height
			#start_type,start_num_points,start_controlpoints,start_closed = pdb.gimp_vectors_stroke_get_points(vectors,stroke_ids[i+1])
			#out_x,out_y = start_controlpoints[2:4]
			start_x, start_y = nx, ny
			out_x, out_y = x,y
			#out_x = image.width - handlayer.width + (absolute_pen_x - hand_offset_x)
			#out_y = image.height + (absolute_pen_y - hand_offset_y)
			difx,dify = (out_x - start_x),(out_y - start_y)
			move_dist = (difx**2 + dify**2)**0.5
			#make it steps such that we get double the regular speed
			steps = move_dist/(step_length*2.0) #get number of steps based on steplength so animation looks same speed as the rest of it
			step = 0.0
			while step < steps:
				step += 1.0
				if step > steps:
					step = steps
				at_proportion = step/steps 	
				mx = start_x + (at_proportion * difx)
				my = start_y + (at_proportion * dify)	
				
				move_offset_x = mx - absolute_pen_x + hand_offset_x
				move_offset_y = my - absolute_pen_y + hand_offset_y
				pdb.gimp_layer_set_offsets(handlayer,move_offset_x,move_offset_y)
				
				new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"painted")
				pdb.gimp_image_insert_layer(new_image,new_layer,None,0)

		oldx,oldy = x,y
		while current_at < current_length-0.01:
			
			#x,y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,current_at,0.01)	
			#pdb.gimp_paintbrush_default(work_layer,4,[oldx,oldy,x,y]) #paint to new point
			next_at = (current_at + step_length)
			if next_at > current_length-0.01:
				next_at = current_length-0.01
			this_at = current_at
			while this_at < next_at:
				nox,noy,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,this_at,0.01)	
				this_at += 1
				if this_at > current_length-0.01:
					this_at = current_length-0.01
				nx,ny,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vectors,stroke_id,this_at,0.01)	
				pdb.gimp_paintbrush_default(work_layer,4,[nox,noy,nx,ny])
			#pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
			oldx,oldy = x,y
			current_at += step_length
			
			#delete shadow here so that if we move pen around we still have shadow until we get here
			if dropshadow == 1 and has_shadow:
				pdb.gimp_image_remove_layer(image,image.layers[shadow_position])
				has_shadow = False

			#drop shadow
			if dropshadow == 1:
				pdb.script_fu_drop_shadow(image,work_layer,shadow_offset_x,shadow_offset_y,shadow_blur_radius,shadow_color,shadow_opacity,1)	
				shadow_position = pdb.gimp_image_get_layer_position(image,work_layer)+1
				has_shadow = True

			#moves the hand layer into position of last drawn stroke
			if drawhand == 1:
				move_offset_x = nx - absolute_pen_x + hand_offset_x
				move_offset_y = ny - absolute_pen_y + hand_offset_y
				pdb.gimp_layer_set_offsets(handlayer,move_offset_x,move_offset_y)

			new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"painted")
			pdb.gimp_image_insert_layer(new_image,new_layer,None,0)

			#if drawhand and last frame create an extra frame with out hand(by moving it out of frame)
			#if drawhand == 1 and current_at >= current_length-0.01:
				#pdb.gimp_layer_set_offsets(handlayer,image.width,image.height)
				#creates extra frame
				#new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"painted")
				#pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
			

			#delete shadow for next fresh next round
			
	#TRANSITION OUT (of hand out of white board)
	if (drawhand == 1) and (transout == 1): #if user choose to transition in we animate hand into first shape
		#get starting point of first path
		#start_type,start_num_points,start_controlpoints,start_closed = pdb.gimp_vectors_stroke_get_points(vectors,stroke_ids[len(stroke_ids)-1])
		start_x,start_y = nx, ny
		#starts from outside where the layer would be right aligned and it's below the height
		out_x = image.width - handlayer.width + (absolute_pen_x - hand_offset_x)
		out_y = image.height + (absolute_pen_y - hand_offset_y)
		difx,dify = (out_x - start_x),(out_y - start_y)
		move_dist = (difx**2 + dify**2)**0.5
		#make it steps such that we get double the regular speed
		steps = move_dist/(step_length*2.0) #get number of steps based on steplength so animation looks same speed as the rest of it
		step = 0.0
		while step < steps:
			step += 1.0
			if step > steps:
				step = steps
			at_proportion = step/steps 	
			mx = start_x + (at_proportion * difx)
			my = start_y + (at_proportion * dify)	
			
			move_offset_x = mx - absolute_pen_x + hand_offset_x
			move_offset_y = my - absolute_pen_y + hand_offset_y
			pdb.gimp_layer_set_offsets(handlayer,move_offset_x,move_offset_y)
			
			new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"painted")
			pdb.gimp_image_insert_layer(new_image,new_layer,None,0)

	#if we drew hand, move hand back to original position, just in case use wants to rerun script it's at starting state 		
	if drawhand == 1:
		#create a frame without hand first, so we show full image without hand in last frame
		pdb.gimp_layer_set_offsets(handlayer,image.width,image.height)
		#creates extra frame
		new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"painted")
		pdb.gimp_image_insert_layer(new_image,new_layer,None,0)

		pdb.gimp_layer_set_offsets(handlayer,hand_offset_x,hand_offset_y)
	#restore to what it was before we ran script
	if paintexisting == 1:
		pass #TODO maybe restore it to before painting on it. 
	else:
		#delete shadow here so that if we move pen around we still have shadow until we get here
		if dropshadow == 1 and has_shadow:
			pdb.gimp_image_remove_layer(image,image.layers[shadow_position])
			has_shadow = False
		pdb.gimp_image_remove_layer(image,work_layer)			

	#YOUR CODE ENDS ========================
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	#return

register(
	"python_fu_tt_paint_paths_in_steps",
	"Paint the paths in steps",
	"Paint the paths in steps",
	"Tin Tran",
	"Tin Tran",
	"June 2019",
	"<Image>/Python-Fu/Paint Paths in Steps...",
	"*",      # Create a new image, don't work on an existing one
	[ 
	#INPUT
	(PF_SPINNER, "steps", "frames:", 50, (10,5000,1)),
	(PF_TOGGLE, "dropshadow", "Drop Shadow:", 0),
	(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 10, (-4096,4096,0.1)),
	(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 10, (-4096,4096,0.1)),
	(PF_SPINNER, "shadow_blur_radius", "Shadow Blur Radius:", 5, (0,1024,0.1)),
	(PF_COLOR, "shadow_color", "Shadow Color:", (0,0,0)),
	(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 50, (0,100,0.1)),
	(PF_TOGGLE, "drawhand", "Draw hand:" , 0),
	(PF_LAYER, "handlayer", "Hand layer (Used if Draw hand):",None),
	(PF_VECTORS, "penvector", "Pen location vector (Used if Draw hand):",None),
	(PF_TOGGLE, "paintexisting", "Paint on Existing Mask:" , 0),
	(PF_LAYER, "paintexisting", "Layer of Existing Mask:", None),
	(PF_TOGGLE, "transin", "Transition Hand Into Whiteboard at start (if Draw hand):" , 0),
	(PF_TOGGLE, "transmid", "Transition Hand Between Paths/Shapes (if Draw hand):" , 0),
	(PF_TOGGLE, "transout", "Transition Hand Out of Whiteboard at end (if Draw hand):" , 0),
	#INPUT ENDS
	],
	[],
	python_tt_paint_paths_in_steps)

main()