#!/usr/bin/env python

# palette_from_tile_grid.py Rel 1
# Created by Tin Tran
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
from gimpfu import *
import math
#from inspect import formatargspec, getfullargspec

def python_palette_from_tile_grid(image, layer, palette_name, xtiles, ytiles) : #FUNCTION DEFINITION
	global apply_layers
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#YOUR CODE BEGINS=======================
	actual_name = pdb.gimp_palette_new(palette_name)
	xtiles = int(xtiles)
	ytiles = int(ytiles)
	if xtiles <= 64:
		pdb.gimp_palette_set_columns(actual_name,xtiles)
	else:
		pdb.gimp_palette_set_columns(actual_name,64)
	tilewidth = layer.width*1.0/xtiles
	tileheight = layer.height*1.0/ytiles

	for yi in range(0,ytiles):
		for xi in range(0,xtiles):
			x = xi * tilewidth + (tilewidth/2.0)
			y = yi * tileheight + (tileheight/2.0)
			color = pdb.gimp_color_picker(image,layer,x,y,FALSE,TRUE,3)
			pdb.gimp_palette_add_entry(actual_name,"x" + str(xi+1) + "y" + str(yi+1),color)
	#YOUR CODE ENDS ========================
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_palette_from_tile_grid",
	"Create palette from tile grid",
	"Create palette from tile grid...",
	"Tin Tran",
	"Tin Tran",
	"May 2019",
	"<Image>/Python-Fu/Palette From Tile Grid...",
	"*",      # Create a new image, don't work on an existing one
	[ 
	#INPUT BEGINS
	(PF_STRING, "palette_name", "Palette Name:", "Custom Palette"),
	(PF_SPINNER, "xtiles", "Number of tiles (horizontally [xtiles]):", 5, (1, 100, 1)),
	(PF_SPINNER, "ytiles", "Number of tiles (vertically [ytiles]):", 5, (1, 100, 1)),
	#(PF_OPTION, "layergroup","Choose on what layers", 1, ["ALL LAYERS","VISIBLE LAYERS","NOT-VISIBLE LAYERS","LINKED LAYERS","NOT-LINKED LAYERS"]),
	#(PF_COLOR, "color","Color To Alpha", (255,255,255)),
	#(PF_STRING, "code", "Code to Run:", "pdb.script_fu_drop_shadow(image,layer,4,4,15,(0,0,0),60,False)"),
	#(PF_OPTION, "arrow_side", "Arrows Ends:", SIDE_END, SIDE_NAMES),
	#(PF_TOGGLE, "arrow_close", "Arrows Close:", 0),
	#(PF_SPINNER, "border_width", "Border Width (bevelled width):", 10, (0, 500, 1)),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_blur_radius", "Shadow Blur Radius:", 15, (0,1024,1)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 100, (0,100,1)),
	#INPUT ENDS
	],
	[],
	python_palette_from_tile_grid)

main()
