#!/usr/bin/env python
# coding=utf-8
# -*- encoding: utf-8 -*-
# paper-weave-2.py Rel 1
# Created by Tin Tran
# Comments directed to http://gimplearn.net
# Inspired by https://warpedfibers.com/weaving-with-paper/ where weaving is not alternating but randomly
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
import random
from gimpfu import *
import math

#main method
def python_paper_weave_2_tt(image, layer, hor, vert, rand, everyN, randN,odds):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	hor = int(hor)
	vert = int(vert)
	rand = int(rand)
	
	yoffset = layer.height/hor #how many y pixels we offset to get next strip
	xoffset = layer.width/vert

	
	#create horizontal strips
	pdb.gimp_selection_none(image) #start with no selection
	y1 = 0; y2 = 0;
	horistrips = []
	for i in range(1,hor+1):
		if (i == (hor)):
			y3 = layer.height
			y4 = layer.height
		else:	
			y3 = i*yoffset
			y4 = i*yoffset
			if (rand==1):
				y3 += random.randrange(0,yoffset*2/3)-yoffset/3
				y4 += random.randrange(0,yoffset*2/3)-yoffset/3
		#if (i % 2==0): #alternating logic
		segs = [0,y1,layer.width,y2,layer.width,y4,0,y3]
		horistrips.append(segs) #save polygon to select later
		#pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,segs)
		#for next time	
		y1 = y3
		y2 = y4
	#pdb.plug_in_sel2path(image,layer)
	#create vertical strips
	x1 = 0; x2 = 0;
	vertstrips = []
	for i in range(1,vert+1):
		if (i == (vert)):
			x3 = layer.width
			x4 = layer.width
		else:	
			x3 = i*xoffset
			x4 = i*xoffset
			if (rand==1):
				x3 += random.randrange(0,xoffset*2/3)-xoffset/3
				x4 += random.randrange(0,xoffset*2/3)-xoffset/3
		#if (i % 2==0): #alternating logic
		segs = [x1,0,x2,layer.height,x4,layer.height,x3,0]
		vertstrips.append(segs) #save polygon to select later
		#pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,segs)
		#for next time	
		x1 = x3
		x2 = x4
	#let's make a notion that 0 is a cell where horizontal strip on top and 1 is celll where vertical strip on top
	#now we create grid to indicate which is on top
	grid = [];
	randN = int(randN)
	everyN = int(everyN)

	for y in range(0,hor): #loop through horizontal strips
		thisrow = [];

		for x in range(0,vert): #loop through vertical strips which gives us cells
			if (randN == 1): #use odds
				if (x == 0): #first cell in row 
					if (random.randrange(0,100) < 50): #just random 50/50 horizontal/vertical
						thiscell = 0; #make horizontal strip on top
					else:
						thiscell = 1; #make vertical strip on top
					thisrow.append(thiscell); #add it to the strip	
				else: #any cell after first cell on row, we have a previous cell to reference
					if (random.randrange(0,1000) < odds): #switch/weave 1/3 (or 33.3%) of the time TODO might want to change this odds number later.
						thiscell = abs(thisrow[x-1]-1); #weave/switch
					else:
						thiscell = thisrow[x-1]; #the rest of the time we just continue with previous cell's configuration (or no weaving/switching)
					thisrow.append(thiscell); #add it to thte strip
			else: #use everyN /switch every N
				if (x == 0): #first cell in row 
					switchcheck = y % (everyN * 2)
					if (switchcheck < everyN):
						thiscell = 0; #make horizontal strip on top
					else:
						thiscell = 1; #make vertical strip on top
					thisrow.append(thiscell); #add it to the strip	
				else: #any cell after first cell on row, we have a previous cell to reference
					switchcheck = (switchcheck + 1) % everyN
					if (switchcheck == 0): #switch/weave 1/3 (or 33.3%) of the time TODO might want to change this odds number later.
						thiscell = abs(thisrow[x-1]-1); #weave/switch
					else:
						thiscell = thisrow[x-1]; #the rest of the time we just continue with previous cell's configuration (or no weaving/switching)
					thisrow.append(thiscell); #add it to thte strip
		grid.append(thisrow);
	
	
	bottom_layer = image.layers[1] #we'll make bottom layer horizontal strips

	#weave layer
	weave_layer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'Paper weave 2',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,weave_layer,None,0)

	#
	line_layer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'Outline',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,line_layer,None,0)

	#select bottom layer to copy and paste to weave layer
	for y in range(0,hor):
		thisrow = grid[y];
		
		pdb.gimp_selection_none(image);
		has_selection = 0;
		for x in range(0,vert):
			if (thisrow[x] == 0): #if horizontal strip on top select the vert to intersect it with the horizontal afterwards
				pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,vertstrips[x])
				has_selection = 1;
		pdb.gimp_image_select_polygon(image,CHANNEL_OP_INTERSECT,8,horistrips[y])
		if has_selection == 1:
			pdb.gimp_edit_copy(bottom_layer);
			float_sel = pdb.gimp_edit_paste(weave_layer,FALSE)
			pdb.gimp_floating_sel_anchor(float_sel)
	
	#draw outline for horizontal strips
	for y in range(0,hor):
		thisrow = grid[y];
		
		pdb.gimp_selection_none(image);
		has_selection = 0;
		for x in range(0,vert):
			if (thisrow[x] == 0): #if horizontal strip on top select the vert to intersect it with the horizontal afterwards
				pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,vertstrips[x])
				has_selection = 1;
		pdb.gimp_image_select_polygon(image,CHANNEL_OP_INTERSECT,8,horistrips[y])
		if (has_selection == 1):
			pdb.gimp_selection_border(image,1.0)
			pdb.gimp_context_set_foreground((0,0,0)) #fill border with black
			pdb.gimp_edit_fill(line_layer,FILL_FOREGROUND)

	#draw outline for vertical strips		
	for x in range(0,vert):
		pdb.gimp_selection_none(image);
		has_selection = 0;
		for y in range(0,hor):
			if (grid[y][x] == 1): #if horizontal strip on top select the vert to intersect it with the horizontal afterwards
				pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,horistrips[y])
				has_selection = 1;
		pdb.gimp_image_select_polygon(image,CHANNEL_OP_INTERSECT,8,vertstrips[x])
		if (has_selection == 1):
			pdb.gimp_selection_border(image,1.0)
			pdb.gimp_context_set_foreground((0,0,0)) #fill border with black
			pdb.gimp_edit_fill(line_layer,FILL_FOREGROUND)		

	shadow_layer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'Shadow',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,shadow_layer,None,0)

	min_dimension = min(layer.width,layer.height)
	max_strips = max(vert,hor)
	border = max(2,min_dimension/max_strips/30.0)

	#Create intermediate shadow layers for each strip
	for y in range(0,hor):
		thisrow = grid[y];
		pdb.gimp_selection_none(image);
		intershadow_layer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'interShadow',100,LAYER_MODE_NORMAL)
		pdb.gimp_image_insert_layer(image,intershadow_layer,None,0)
		has_selection = 0;
		for x in range(0,vert):
			if (thisrow[x] == 0): #if horizontal strip on top select the vert to intersect it with the horizontal afterwards
				pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,vertstrips[x])
				has_selection = 1;
		pdb.gimp_selection_border(image,border/2.0)		
		pdb.gimp_selection_grow(image,border/2.0)
		pdb.gimp_image_select_polygon(image,CHANNEL_OP_INTERSECT,8,horistrips[y])
		if (has_selection == 1):
			pdb.gimp_context_set_foreground((0,0,0)) #fill border with black
			pdb.gimp_edit_fill(intershadow_layer,FILL_FOREGROUND)
			pdb.gimp_selection_none(image)
			pdb.plug_in_gauss(image,intershadow_layer,border,border,1)
		#reselect it again but invert it and clear	
		pdb.gimp_selection_none(image);
		has_selection = 0;
		for x in range(0,vert):
			if (thisrow[x] == 0): #if horizontal strip on top select the vert to intersect it with the horizontal afterwards
				pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,vertstrips[x])
				has_selection = 1;
		pdb.gimp_image_select_polygon(image,CHANNEL_OP_INTERSECT,8,horistrips[y])
		if (has_selection == 1):
			pdb.gimp_selection_invert(image)
			pdb.gimp_edit_clear(intershadow_layer) #clear after inverting so we're left with shadows of that strip
		shadow_layer = pdb.gimp_image_merge_down(image,intershadow_layer,CLIP_TO_BOTTOM_LAYER) #merge down to keep the shadow

	#Create intermediate shadow layers for each vertical strip	
	for x in range(0,vert):
		pdb.gimp_selection_none(image);
		intershadow_layer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'interShadow',100,LAYER_MODE_NORMAL)
		pdb.gimp_image_insert_layer(image,intershadow_layer,None,0)
		has_selection = 0;
		for y in range(0,hor):
			if (grid[y][x] == 1):
				pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,horistrips[y])
				has_selection = 1;
		pdb.gimp_selection_border(image,border/2.0)		
		pdb.gimp_selection_grow(image,border/2.0)
		pdb.gimp_image_select_polygon(image,CHANNEL_OP_INTERSECT,8,vertstrips[x])
		if (has_selection == 1):
			pdb.gimp_context_set_foreground((0,0,0)) #fill border with black
			pdb.gimp_edit_fill(intershadow_layer,FILL_FOREGROUND)
			pdb.gimp_selection_none(image)
			pdb.plug_in_gauss(image,intershadow_layer,border,border,1)
		#reselect it again but invert it and clear	
		pdb.gimp_selection_none(image);
		has_selection = 0;
		for y in range(0,hor):
			if (grid[y][x] == 1):
				pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,horistrips[y])
				has_selection = 1;
		pdb.gimp_image_select_polygon(image,CHANNEL_OP_INTERSECT,8,vertstrips[x])
		if (has_selection == 1):
			pdb.gimp_selection_invert(image)
			pdb.gimp_edit_clear(intershadow_layer) #clear after inverting so we're left with shadows of that strip
		shadow_layer = pdb.gimp_image_merge_down(image,intershadow_layer,CLIP_TO_BOTTOM_LAYER) #merge down to keep the shadow	

	#at the end reduce opacity to 40%	
	pdb.gimp_layer_set_opacity(shadow_layer,40)	
	pdb.gimp_selection_none(image)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	pdb.gimp_message("Complete!")
	#return

register(
	"python_fu_paper_weave_2_tt",
	"Creates paper weave 2 of 2 layers",
	"Creates paper weave 2 of 2 layers",
	"Tin Tran",
	"Tin Tran",
	"July 29 2022",
	"<Image>/Python-Fu/Paper Weave 2...",             #Menu path
	"RGB*, GRAY*",
	[
	(PF_SPINNER, "hor", "Horizontal Strips:", 10, (3, 1000, 1)),
	(PF_SPINNER, "vert", "Vertical Strips:", 10, (3, 1000, 1)),
	(PF_TOGGLE, "rand",   "Randomize Cut Angles:", 0),
	(PF_SPINNER, "everyN", "Weave Every n cells:", 3, (1,100, 1)),
	(PF_TOGGLE, "rand2",   "Randomize n:", 1),
	(PF_SPINNER, "odds", "Odds of Weaving (out of 1000 if Random n is YES):", 666, (10,1000, 1)),
	#(PF_SPINNER, "px_height", "Height of each tile image:", 100, (1, 1000, 50)),
	],
	[],
	python_paper_weave_2_tt)

main()
