#!/usr/bin/env python
# coding=utf-8
# -*- encoding: utf-8 -*-
# paper-weave.py Rel 1
# Created by Tin Tran
# Comments directed to http://gimplearn.net
# Inspired by grade 2 school project and reminded by this post on reddit.com https://www.reddit.com/gallery/w9z9mt.
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Shadow layer added for even more convincing of a weave.
# Rel 3: Even more realistic shadows.
# Rel 4: Even more realistic shadows. With two shadow layers merged into one so that you simply play with shadows opacity afterwards.
import random
from gimpfu import *
import math

#main method
def python_paper_weave_tt(image, layer, hor, vert, rand):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	hor = int(hor)
	vert = int(vert)
	rand = int(rand)
	
	yoffset = layer.height/hor #how many y pixels we offset to get next strip
	xoffset = layer.width/vert

	
	#create horizontal strips
	pdb.gimp_selection_none(image) #start with no selection
	y1 = 0; y2 = 0;
	for i in range(1,hor+1):
		if (i == (hor)):
			y3 = layer.height
			y4 = layer.height
		else:	
			y3 = i*yoffset
			y4 = i*yoffset
			if (rand==1):
				y3 += random.randrange(0,yoffset*2/3)-yoffset/3
				y4 += random.randrange(0,yoffset*2/3)-yoffset/3
		if (i % 2==0): #alternating logic
			segs = [0,y1,layer.width,y2,layer.width,y4,0,y3]
			pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,segs)
		#for next time	
		y1 = y3
		y2 = y4
	pdb.plug_in_sel2path(image,layer)
	#create vertical strips
	pdb.gimp_selection_none(image) #start with no selection
	x1 = 0; x2 = 0;
	for i in range(1,vert+1):
		if (i == (vert)):
			x3 = layer.width
			x4 = layer.width
		else:	
			x3 = i*xoffset
			x4 = i*xoffset
			if (rand==1):
				x3 += random.randrange(0,xoffset*2/3)-xoffset/3
				x4 += random.randrange(0,xoffset*2/3)-xoffset/3
		if (i % 2==0): #alternating logic
			segs = [x1,0,x2,layer.height,x4,layer.height,x4,0]
			pdb.gimp_image_select_polygon(image,CHANNEL_OP_ADD,8,segs)
		#for next time	
		x1 = x3
		x2 = x4	
	pdb.plug_in_sel2path(image,layer)

	bottom_layer = image.layers[1]
	
	weave_layer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'paper weave',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,weave_layer,None,0)

	#first go
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,image.vectors[0])
	pdb.gimp_image_select_item(image,CHANNEL_OP_INTERSECT,image.vectors[1])
	pdb.gimp_edit_copy(bottom_layer)
	float_sel = pdb.gimp_edit_paste(weave_layer,TRUE)
	pdb.gimp_floating_sel_anchor(float_sel)

	#second go
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,image.vectors[0])
	pdb.gimp_selection_invert(image)
	pdb.gimp_image_select_item(image,CHANNEL_OP_SUBTRACT,image.vectors[1])
	pdb.gimp_edit_copy(bottom_layer)
	float_sel = pdb.gimp_edit_paste(weave_layer,TRUE)
	pdb.gimp_floating_sel_anchor(float_sel)	

	min_dimension = min(layer.width,layer.height)
	max_strips = max(vert,hor)
	border = min_dimension/max_strips/30.0
	
	shadow_layer1 = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'shadow',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,shadow_layer1,None,0)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,image.vectors[len(image.vectors)-1]) #bottom most layer
	pdb.gimp_selection_border(image,border/2.0)
	pdb.plug_in_sel2path(image,layer) #turn it into path
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,weave_layer)
	pdb.gimp_image_select_item(image,CHANNEL_OP_INTERSECT,image.vectors[0]) #intersect with vectors we just created
	pdb.gimp_selection_grow(image,border/2.0)
	pdb.gimp_context_set_foreground((0,0,0)) #fill border with black
	pdb.gimp_edit_fill(shadow_layer1,FILL_FOREGROUND)
	pdb.gimp_selection_none(image)
	pdb.plug_in_gauss(image,shadow_layer1,border,border,1)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,weave_layer)
	pdb.gimp_selection_invert(image) #invert to get other squares
	pdb.gimp_edit_clear(shadow_layer1)

	shadow_layer2 = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'shadow',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,shadow_layer2,None,0)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,image.vectors[len(image.vectors)-2]) #second bottom most layer
	pdb.gimp_selection_border(image,border/2.0)
	pdb.plug_in_sel2path(image,layer) #turn it into path
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,weave_layer)
	pdb.gimp_selection_invert(image) #invert to get other squares
	pdb.gimp_image_select_item(image,CHANNEL_OP_INTERSECT,image.vectors[0]) #intersect with vectors we just created
	pdb.gimp_selection_grow(image,border/2.0)
	pdb.gimp_context_set_foreground((0,0,0)) #fill border with black
	pdb.gimp_edit_fill(shadow_layer2,FILL_FOREGROUND)
	pdb.gimp_selection_none(image)
	pdb.plug_in_gauss(image,shadow_layer2,border,border,1)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,weave_layer)
	pdb.gimp_edit_clear(shadow_layer2)	
	
	shadow_layer = pdb.gimp_image_merge_down(image,shadow_layer2,CLIP_TO_BOTTOM_LAYER)
	pdb.gimp_layer_set_opacity(shadow_layer,40)

	pdb.gimp_selection_none(image)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	pdb.gimp_message("Complete!")
	#return

register(
	"python_fu_paper_weave_tt",
	"Creates paper weave of 2 layers",
	"Creates paper weave of 2 layers",
	"Tin Tran",
	"Tin Tran",
	"July 28 2022",
	"<Image>/Python-Fu/Paper Weave...",             #Menu path
	"RGB*, GRAY*",
	[
	(PF_SPINNER, "hor", "Horizontal Strips:", 10, (3, 1000, 1)),
	(PF_SPINNER, "vert", "Vertical Strips:", 10, (3, 1000, 1)),
	#(PF_SPINNER, "offset_angle", "Offset angle (angle to offset from path's angles):", 0, (0,360, 1)),
	(PF_TOGGLE, "rand",   "Randomize Cut Angles:", 0),
	#(PF_SPINNER, "px_height", "Height of each tile image:", 100, (1, 1000, 50)),
	],
	[],
	python_paper_weave_tt)

main()
