#!/usr/bin/env python

# path-to-circle-map.py Release 1
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Speed enhancement
from gimpfu import *
import random
import math
from array import array #needed for array pixel access

#from array import array
#import sys
#========================================
#Copied this from ofn-text-along-path.py
def computeOrientedSlope(dx,dy,slope): #This is a better version than below computeThetaWithSlope function.
	if abs(slope) > 100000: # very vertical
		return math.atan2(dy,dx) # no perfect, but properly oriented
        # keep dx/dy signs but give then the same ratio as in slope
	return math.atan2(math.copysign(slope,dy),math.copysign(1,dx))
#Copied this below function from ofnuts text-along-path-0.4.py
'''def computeThetaWithSlope(fromPoint,toPoint,slope):
	dX=toPoint[0]-fromPoint[0]
	dY=toPoint[1]-fromPoint[1]

	if math.fabs(dX) < .001:
		theta=math.copysign(math.pi/2,dY)
	else:
		theta=math.atan(slope)
		if dX<0:
			theta=theta+math.pi
	return theta
'''
#========================================
# Tin Tran's function to calculate angle given points --------------------
def myangle(x1,y1,x2,y2):
	dx = float(x2-x1)
	if dx == 0:
		dx = 0.0001
	dy = float(y2-y1)
	slope = dy/dx
	return computeOrientedSlope(dx,dy,slope)
def computeOrientedSlope(dx,dy,slope): #This is a better version than below computeThetaWithSlope function.
	if abs(slope) > 100000: # very vertical
		return math.atan2(dy,dx) # no perfect, but properly oriented
        # keep dx/dy signs but give then the same ratio as in slope
	return math.atan2(math.copysign(slope,dy),math.copysign(1,dx))

# ------------------------------------------------------------------------	
def python_tt_path_to_circle_map(image, layer, points):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	if not pdb.gimp_drawable_has_alpha(layer):
		pdb.gimp_layer_add_alpha(layer);
	#
	points = int(points)

	#work with active path
	active_vectors = pdb.gimp_image_get_active_vectors(image)
	num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(active_vectors)
	stroke_id = stroke_ids[0] #use first stroke since we're only expecting one close stroke
	stroke_length = pdb.gimp_vectors_stroke_get_length(active_vectors,stroke_id,0.01)
	sumx = 0.0;
	sumy = 0.0;
	section_length = stroke_length/points;
	arr = []
	cx = 0.0;
	cy = 0.0;	
	for i in range(0,points):
		this_length = i*section_length
		x,y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_id,this_length,0.01)
		sumx += x; sumy += y;
		angle = 0;
		arr.append([x,y,0,0,0]) #x,y,angle,length,diff
	cx = sumx/points
	cy = sumy/points
	for i in range(0,len(arr)):
		x = arr[i][0]
		y = arr[i][1]
		angle = myangle(cx,cy,x,y)
		dangle = angle * 180 / math.pi
		if (dangle < 0):
			dangle = dangle + 360
		
		arr[i][2] = dangle
		length = ((x-cx)**2 + (y-cy)**2)**0.5
		arr[i][3] = length
	s = sorted(arr, key=lambda ar: -ar[3])
	radius = s[0][3] #get longest length and use radius after sorting decending
	new_width = int(math.ceil(radius)*2)
	new_height = int(new_width)
	dcx = math.ceil(radius) #draw(d) cx,cy middle of new image's width and height
	dcy = dcx 
	new_image = pdb.gimp_image_new(new_width,new_height,RGB)
	display = pdb.gimp_display_new(new_image)
	new_layer = pdb.gimp_layer_new(new_image,new_width,new_height,RGBA_IMAGE,"circle map",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,new_layer,None,0)
	# =============== fast pixel access-----------
	srcRgn = layer.get_pixel_rgn(0, 0, layer.width, layer.height, False, False) #read only
	dstRgn = new_layer.get_pixel_rgn(0, 0, new_layer.width, new_layer.height, True, True) #update to pixels
	p_size_source = len(srcRgn[0,0])
	p_size_dest = len(dstRgn[0,0])
	src_pixels = array("B", srcRgn[0:layer.width, 0:layer.height]) #source
	dest_pixels = array("B", dstRgn[0:new_layer.width, 0:new_layer.height]) #destination pixel
	# =============== fast pixel access-------------
	for iy in range(0,new_height):
		pdb.gimp_progress_update(float(iy)/new_height)
		for ix in range(0,new_width):
			length = ((ix-dcx)**2 + (iy-dcy)**2)**0.5
			if (length <= radius): #it's without our circle draw it
				angle = myangle(dcx,dcy,ix,iy)
				dangle = angle * 180 / math.pi
				if (dangle < 0):
					dangle = dangle + 360
				for i in range(0,len(arr)):
					arr[i][4] = abs(arr[i][2]-dangle) #absolute difference
				s = sorted(arr, key=lambda ar: ar[4])	#sort by difference
				this_ar = s[0]
				this_ratio = length/radius
				getx = (this_ar[0]-cx)*this_ratio+cx
				gety = (this_ar[1]-cy)*this_ratio+cy
				#num_chan,pixel = pdb.gimp_drawable_get_pixel(layer,int(getx),int(gety))
				src_pos = (int(getx) + int(gety)*layer.width) * p_size_source
				pixel = src_pixels[src_pos: src_pos + p_size_source]
				#pdb.gimp_drawable_set_pixel(new_layer,ix,iy,num_chan,pixel)
				dest_pos = (ix+ iy * new_layer.width)*p_size_dest
				dest_pixels[dest_pos : dest_pos + p_size_dest] = pixel

	#flush to new_layer data			
	dstRgn[0:new_layer.width, 0:new_layer.height] = dest_pixels.tostring() 	
	new_layer.flush()
	new_layer.merge_shadow(True)
	new_layer.update(0, 0,new_layer.width,new_layer.height)
	pdb.gimp_displays_flush()
	# pdb.gimp_message("cx:" + str(cx))	
	# pdb.gimp_message("cy:" + str(cy))

	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	#return

register(
	"python_fu_tt_path_to_circle_map",                           
	"Maps a closed path shape to circle",
	"Maps a closed path shape to circle",
	"Tin Tran",
	"Tin Tran",
	"May 2021",
	"<Image>/Python-Fu/A Path To Circle Map...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "points", "Path Sample Points:", 100, (100, 5000, 1)),
	#(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	#(PF_TOGGLE, "keep_aspect", "Keep Aspect Ratio of Border:", 1),
	#(PF_TOGGLE, "mirror","Mirror/Reflect Borders:", 1),
	#(PF_TOGGLE, "drop_shadow","Drop Shadow:", 1),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_blur", "Shadow Blur Radius:", 15, (0, 1000, 1)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 60, (0, 100, 1)),
	],
	[],
	python_tt_path_to_circle_map)

main()
