#!/usr/bin/env python

# path-y-to-bw-gradient.py
# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
from gimpfu import *

def python_path_y_to_bw_gradient(image,layer,vector,gradientname,segments):
	#pdb.gimp_image_undo_group_start(image)
	#pdb.gimp_context_push()
	segments = int(segments) #make it an integer
	numStrokes,strokeids = pdb.gimp_vectors_get_strokes(vector)
	firstStroke = strokeids[0]
	#pathLength is in pixels
	pathLength = pdb.gimp_vectors_stroke_get_length(vector,firstStroke,0.01)
	samples = segments*3; #change this value to get less/more accurate
	points = []
	for i in range(0,samples+1):
		dist = i*(pathLength/samples)
		x,y,slope,valid = pdb.gimp_vectors_stroke_get_point_at_dist(vector,firstStroke,dist,0.01)
		if (valid == TRUE):
			points.append([x,y])
	#here we have samples+1 points
	gradient = pdb.gimp_gradient_new(gradientname)
	pdb.gimp_context_set_gradient(gradient)
	#segments = 1024; #max 1024
	pdb.gimp_gradient_segment_range_split_uniform(gradient,0,0,segments)
	
	#calculate x coords
	points.sort(key = lambda x: x[0]) #sort by x coordinates
	minx = points[0][0]; maxx = points[len(points)-1][0];
	xlen = maxx - minx;
	
	#calculate y coords
	points.sort(key = lambda y: y[1]) #sort by y coordinates
	miny = points[0][1]; maxy = points[len(points)-1][1];
	ylen = maxy - miny;
	#go through and calculate color
	for i in range(0,segments):
		leftx = minx +  ( i    * (xlen/segments));
		rightx = minx + ((i+1) * (xlen/segments));
		points.sort(key = lambda x: abs(x[0]-leftx)) #sort by x diff from leftx
		lefty = points[0][1]
		points.sort(key = lambda x: abs(x[0]-rightx)) #sort by x diff from rightx
		righty = points[0][1]
		leftcv = int(round((maxy-lefty)/ylen * 255))
		rightcv = int(((maxy-righty)/ylen * 255))
		leftcolor = (leftcv,leftcv,leftcv)
		rightcolor = (rightcv,rightcv,rightcv)
		pdb.gimp_gradient_segment_set_left_color(gradient,i,leftcolor,100)
		pdb.gimp_gradient_segment_set_right_color(gradient,i,rightcolor,100)
	opacity = 100


	#pdb.gimp_context_pop()
	#pdb.gimp_image_undo_group_end(image)
	#pdb.gimp_displays_flush()


register(
	"python_path_y_to_bw_gradient",
	"Turns Path-Y-Coords into relative Black & White gradient",
	"Turns Path-Y-Coords into relative Black & White gradient",
	"Tin Tran",
	"Tin Tran",
	"July 2020",
	"<Image>/Python-Fu/Path-Y-To-BW-Gradient...",             #Menu path
	"RGB*, GRAY*, INDEXED*", 
	[
	#(PF_SPINNER, "border_radius", "Border-radius:", 50, (1, 1000, 5)),
	#(PF_GRADIENT, "gradient", "Gradient:", 0),
	#(PF_OPTION,"option",   "Mapping:", 0, ["To Circle","To Square"]),
	(PF_VECTORS, "vector", "Path/Vector (to convert from):", None),
	(PF_STRING, "gradient_name", "Gradient Name (to create from Path):", "Path-Y-Gradient"),
	(PF_SPINNER, "segments", "Gradient Segments:", 64, (2, 1024, 1)),
	],
	[],
	python_path_y_to_bw_gradient)

main()
