#!/usr/bin/env python

# perspective_border.py Release 4
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Put image on top of frame and resize frame to image size (cleaner).
# Rel 3: Fix error in mirror mode for top and bottom border
# Rel 4: Added option to drop shadow (suggested by Issabella).
from gimpfu import *
import random
#from array import array
#import sys
def python_tt_perspective_border(image, layer, frame_percent, keep_aspect, mirror, drop_shadow, shadow_offset_x,\
	shadow_offset_y,shadow_blur,shadow_opacity):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#core code goes here/below
	new_image = pdb.gimp_image_duplicate(image)
	active_layer = pdb.gimp_image_get_active_layer(new_image)
	new_display = pdb.gimp_display_new(new_image)
	
	frame_width = int(frame_percent/2.0/100.0 * layer.width)
	frame_height = int(frame_percent/2.0/100.0 * layer.height)
	if keep_aspect == 0: #if we're not keeping aspect ratio then make frame border 
	                      #the constant thickness of thinner of the two (width and height)
		min_border = min(frame_width,frame_height)
		frame_width = min_border
		frame_height = min_border

	new_width = layer.width+2*frame_width
	new_height = layer.height+2*frame_height
	pdb.gimp_image_resize(new_image,new_width,new_height,frame_width,frame_height)

	#reference points around the image
	x1,x2,y1,y2 = frame_width+1, new_width - frame_width-1, frame_height+1, new_height - frame_height-1

	#left border
	layer_copy = pdb.gimp_layer_copy(active_layer,TRUE)
	pdb.gimp_image_insert_layer(new_image,layer_copy,None,0)
	if mirror == 0:
		pdb.gimp_item_transform_perspective(layer_copy,0,0-1, x1,y1-1, 0,new_height+1, x1,y2+1)
	else:
		pdb.gimp_item_transform_perspective(layer_copy,x1,y1-1, 0,0-1, x1,y2+1, 0,new_height+1)

	#right border
	layer_copy = pdb.gimp_layer_copy(active_layer,TRUE)
	pdb.gimp_image_insert_layer(new_image,layer_copy,None,0)
	if mirror == 0:
		pdb.gimp_item_transform_perspective(layer_copy,x2,y1-1, new_width,0-1, x2,y2+1, new_width,new_height+1)
	else:
		pdb.gimp_item_transform_perspective(layer_copy,new_width,0-1, x2,y1-1, new_width,new_height+1, x2,y2+1)

	#top border
	layer_copy = pdb.gimp_layer_copy(active_layer,TRUE)
	pdb.gimp_image_insert_layer(new_image,layer_copy,None,0)
	if mirror == 0:
		pdb.gimp_item_transform_perspective(layer_copy,0,0, new_width,0, x1,y1, x2,y1)
	else:
		pdb.gimp_item_transform_perspective(layer_copy,x1,y1, x2,y1, 0,0, new_width,0)
	#bottom border
	layer_copy = pdb.gimp_layer_copy(active_layer,TRUE)
	pdb.gimp_image_insert_layer(new_image,layer_copy,None,0)
	if mirror == 0:
		pdb.gimp_item_transform_perspective(layer_copy,x1,y2, x2,y2, 0,new_height, new_width,new_height)
	else:
		pdb.gimp_item_transform_perspective(layer_copy,0,new_height, new_width,new_height, x1,y2, x2,y2)

	#merge all borders together
	merged_layer = pdb.gimp_image_merge_down(new_image,layer_copy,EXPAND_AS_NECESSARY)
	merged_layer = pdb.gimp_image_merge_down(new_image,merged_layer,EXPAND_AS_NECESSARY)
	merged_layer = pdb.gimp_image_merge_down(new_image,merged_layer,EXPAND_AS_NECESSARY)
	
	#frame to image size
	pdb.gimp_layer_resize_to_image_size(merged_layer)

	#cut it to reveal all of image
	pdb.gimp_image_select_item(new_image,CHANNEL_OP_REPLACE,active_layer)
	pdb.gimp_edit_cut(merged_layer)
	pdb.gimp_selection_none(new_image)

	#drop shadow
	if drop_shadow == 1:
		pdb.script_fu_drop_shadow(new_image,merged_layer,shadow_offset_x,shadow_offset_y,shadow_blur,(0,0,0),\
			shadow_opacity,0)
	#UNCOMMENT BELOW WHEN DONE
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_perspective_border",                           
	"Creates perspective_borderpective border of active layer",
	"Creates Perspective border of active layer",
	"Tin Tran",
	"Tin Tran",
	"March 2019",
	"<Image>/Python-Fu/Perspective Border...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	(PF_TOGGLE, "keep_aspect", "Keep Aspect Ratio of Border:", 1),
	(PF_TOGGLE, "mirror","Mirror/Reflect Borders:", 1),
	(PF_TOGGLE, "drop_shadow","Drop Shadow:", 1),
	(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 2, (-1000, 1000, 1)),
	(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 2, (-1000, 1000, 1)),
	(PF_SPINNER, "shadow_blur", "Shadow Blur Radius:", 15, (0, 1000, 1)),
	(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 60, (0, 100, 1)),
	],
	[],
	python_tt_perspective_border)

main()