#!/usr/bin/env python

# Picture to Line Art Rel 1
# Created by Tin Tran http://bakon.ca/gimplearn
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
import math
import string
#import Image
from gimpfu import *
from array import array

def python_picture_to_line_art(image, layer) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#Step 1:
	#Duplicate the image/layer.
	layer_copy = pdb.gimp_layer_new_from_drawable(layer, image)
	pdb.gimp_image_insert_layer(image, layer_copy, None, 0)
	#Step 2:
	#On the top-most layer, go to 
	#Colors - Desaturate - Lightness
	pdb.gimp_desaturate_full(layer_copy,DESATURATE_LIGHTNESS)
	#Step 3:
	#On the same image/layer apply 
	#Colors - Levels - Edit these settings as Curves
	#0,0
	#157,110
	#255,255
	pdb.gimp_curves_spline(layer_copy,HISTOGRAM_VALUE,6,[0,0,157,110,255,255])
	#Duplicate the top-most image/layer.
	layer_copy2 = pdb.gimp_layer_new_from_drawable(layer_copy, image)
	pdb.gimp_image_insert_layer(image, layer_copy2, None, 0)
	#Step 4:
	#Invert the top-most layer Colors - Invert.
	pdb.gimp_invert(layer_copy2)
	#Change the layer mode to Dodge.
	pdb.gimp_layer_set_mode(layer_copy2,DODGE_MODE)
	#Go to Filters - Blur - Gaussian Blur and set both Horizontal and Vertical to 15.0 px.
	pdb.plug_in_gauss(image,layer_copy2,15.0,15.0,1)
	#Step 5:
	#Edit Copy Visible the top resulting image/layer, and paste as a new layer. Change the layer mode to Burn. Save the results of the final step. 
	visible_layer = pdb.gimp_layer_new_from_visible(image,image,"visible")
	pdb.gimp_image_insert_layer(image, visible_layer, None, 0)
	pdb.gimp_layer_set_mode(visible_layer,BURN_MODE)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_picture_to_line_art",                           
	"Turns a picture into line art",
	"Turns a picture into line art",
	"Tin Tran",
	"Tin Tran",
	"2014",
	"<Image>/Python-Fu/Picture To Line Art",             #Menu path
	"RGB*, GRAY*", 
	[
	#(PF_COLOR, "oldcolor",  "Replace Color:",  (43,198,255)),
	#(PF_COLOR, "newcolor",  "With New Color:",  (255,198,0)),
	#(PF_SPINNER, "x1", "x1:", 0, (0, 100, 0.5)),
	#(PF_SPINNER, "y1", "y1:", 100, (0, 100, 0.5)),
	#(PF_SPINNER, "x2", "x2:", 100, (0, 100, 0.5)),
	#(PF_SPINNER, "y2", "y2:", 0, (0, 100, 0.5)),
	#(PF_COLOR, "black",  "Black point color",  (0,0,0) PF_FILE, "infilename", "Temp Filepath", "/Default/Path"),
	#(PF_COLOR, "white",  "White point color",  (255,255,255) ),
	#(PF_COLOR, "gray",  "Gray point color",  (128,128,128) )
	#(PF_FILE, "infilename", "Temp Filepath", "/Default/Path")
	#(PF_DIRNAME, "source_directory", "Source Directory", "") for some reason, on my computer when i(Tin) use PF_DIRNAME the pythonw.exe would crash
	],
	[],
	python_picture_to_line_art)

main()
