#!/usr/bin/env python

# random_rip.py Rel 1
# goal here is to turn a current selection into something that looks like random paper ripped edges of selection.
# Created by Tin Tran
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
from gimpfu import *
import math,random
#========================================
#Copied this from ofn-text-along-path.py
def computeOrientedSlope(dx,dy,slope): #This is a better version than below computeThetaWithSlope function.
	if abs(slope) > 100000: # very vertical
		return math.atan2(dy,dx) # no perfect, but properly oriented
        # keep dx/dy signs but give then the same ratio as in slope
	return math.atan2(math.copysign(slope,dy),math.copysign(1,dx))
#========================================
def python_random_rip(image, layer, numpoints, maxstray) : #FUNCTION DEFINITION
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#YOUR CODE BEGINS=======================
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if non_empty:
		pdb.plug_in_sel2path(image,layer)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(active_vectors)
		#create new vector
		new_vectors = pdb.gimp_vectors_new(image,"random rip")
		pdb.gimp_image_insert_vectors(image,new_vectors,None,0)
		for i in range(0,num_strokes):
			new_points = []
			length = pdb.gimp_vectors_stroke_get_length(active_vectors,stroke_ids[i],0.01)
			length_fraction = float(length-0.1)/numpoints #so that we can use the last point (dist) + 0.1 to find point to and it'd equal to length
			radius = 0;
			for l in range(0,int(numpoints+1)):
				dist = l*length_fraction+0.05
				x1,y1,slope1,valid1 = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_ids[i],dist-0.05,0.01)
				x2,y2,slope2,valid2 = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_ids[i],dist,0.01)
				x3,y3,slope3,valid3 = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,stroke_ids[i],dist+0.05,0.01)
				slope = computeOrientedSlope(x3-x1,y3-y1,slope2)
				workslope = slope+math.pi/2
				stray_determinant = float(random.randrange(int(-maxstray*10),int(maxstray*10)))/10
				if stray_determinant >= radius:
					radius += random.randrange(0,4)
				else:
					radius -= random.randrange(0,4)
				
				x = x2 + math.cos(workslope)*radius
				y = y2 + math.sin(workslope)*radius
				new_points += ([x,y]*3)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors,0,len(new_points),new_points,0)
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,new_vectors)
		pdb.gimp_image_remove_vectors(image,new_vectors)
		pdb.gimp_image_remove_vectors(image,active_vectors)
	else:
		pdb.gimp_message("You need to make a selection")
	#YOUR CODE ENDS ========================
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_random_rip",
	"Turns Selection into random paper ripped effect of a selection",
	"Turns Selection into random paper ripped effect of a selection...",
	"Tin Tran",
	"Tin Tran",
	"Oct 2017",
	"<Image>/Python-Fu/Random Rip...",
	"*",      # Create a new image, don't work on an existing one
	[ 
	#INPUT BEGINS
	(PF_SPINNER, "numpoints", "Number of points (to randomly shift per stroke to create rough edges):", 100, (10, 5000, 1)),
	(PF_SPINNER, "maxstray", "Max Stray Pixels:", 20, (1, 100, 1)),
	#(PF_STRING, "code", "Code to Run:", "pdb.script_fu_drop_shadow(image,layer,4,4,15,(0,0,0),60,False)"),
	#(PF_OPTION, "arrow_side", "Arrows Ends:", SIDE_END, SIDE_NAMES),
	#(PF_TOGGLE, "arrow_close", "Arrows Close:", 0),
	#(PF_SPINNER, "border_width", "Border Width (bevelled width):", 10, (0, 500, 1)),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_blur_radius", "Shadow Blur Radius:", 15, (0,1024,1)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 100, (0,100,1)),
	#INPUT ENDS
	],
	[],
	python_random_rip)

main()
