#!/usr/bin/env python

# randomly_remove_pixels.py Rel 2.8.2
# Created by Tin Tran
# Comments directed to https://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Added option to erase (transparent pixels)
# Rel 2.8.1: Handle 2.8 and 2.10 versions.
# Rel 2.8.2: Actually remove pixels from selection rather than from rectangle selection bounds.
from gimpfu import *
import random
def python_tt_randomly_remove_pixels(image, layer, color, numpixels, erase) : #FUNCTION DEFINITION
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()

	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	pixels = []
	pdb.gimp_progress_init("Scanning selection...",None)
	for y in range(y1,y2):
		pdb.gimp_progress_update((y-y1)*1.0/(y2-y1))
		for x in range(x1,x2):
			if pdb.gimp_selection_value(image,x,y) > 0:
				pixels.append([x,y])
	count = 0
	#pdb.gimp_message(len(pixels))
	pixels = sorted(pixels,key=lambda x: random.randrange(len(pixels)))
	#pdb.gimp_message(len(pixels))
	pdb.gimp_context_set_brush("1. Pixel")
	pdb.gimp_context_set_brush_size(1)
	new_image = pdb.gimp_image_new(image.width,image.height,RGB)
	new_display = pdb.gimp_display_new(new_image)
	#pdb.gimp_message(len(pixels))
	pdb.gimp_context_set_foreground(color)
	version = pdb.gimp_version()
	if version.startswith("2.8."):
		is_2_8 = True
	elif version.startswith("2.10."):
		is_2_8 = False
	else:
		pdb.gimp_message("Unrecognized GIMP version! Defaulting for 2.10")
		is_2_8 = False
	if erase==1:
		if is_2_8:
			mask = pdb.gimp_layer_create_mask(layer,ADD_WHITE_MASK)
		else:
			mask = pdb.gimp_layer_create_mask(layer,ADD_MASK_WHITE)
		pdb.gimp_layer_add_mask(layer,mask)
	total_pixels = len(pixels)
	total_count = 0
	pdb.gimp_progress_init("Removing pixels...",None)
	while len(pixels) > 0:
		total_count+=1
		pdb.gimp_progress_update(1.0*total_count/total_pixels)
		#pdb.gimp_message(count)
		count += 1
		if erase==1:
			pdb.gimp_pencil(mask,2,pixels[0])#do it twice so that its completely transparent.
		else:
			pdb.gimp_pencil(layer,2,pixels[0])
		pixels = pixels[1:]
		if count == numpixels or len(pixels) == 0:
			count = 0
			new_layer = pdb.gimp_layer_new_from_visible(image,new_image,"visible")
			pdb.gimp_image_insert_layer(new_image,new_layer,None,0)

	#YOUR CODE ENDS ========================
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_randomly_remove_pixels",
	"Randomly remove pixels in selection until it's all gone while creating new layers of steps",
	"Randomly remove pixels in selection until it's all gone while creating new layers of steps",
	"Tin Tran",
	"Tin Tran",
	"March 2019",
	"<Image>/Python-Fu/Create New/Randomly Remove Pixels (Animate)...",
	"*",      # Create a new image, don't work on an existing one
	[ 
	#INPUT 
	(PF_COLOR, "color", "Color to erase with:", (0, 0, 0) ),
	(PF_SPINNER, "numpixels", "Pixels to remove each time:", 100, (1, 5000, 1)),
	(PF_TOGGLE, "erase", "Erase (transparent)", 0),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 6, (-4096,4096,1)),
	#(PF_SPINNER, "shadow_blur_radius", "Shadow Blur Radius:", 15, (0,1024,1)),
	#(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 100, (0,100,1)),
	#INPUT ENDS
	],
	[],
	python_tt_randomly_remove_pixels)

main()