#!/usr/bin/env python

# reflect.py Rel 3
# Tries to reflect using Spherical Emap (This is a completely experimental script)
# Created by Tin Tran http://gimplearn.net/gimplearn/
# Comments directed to http://gimplearn.net or http://gimp-forum.net or http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Fixed Aspect ratio. Use maximum dimension to calculate offsets for both x and y coords.
# Rel 3: Speed up.
#import Image
from gimpfu import *
from array import array

def pixel_val(pixel):
	return sum(pixel[0:3])/3.0
def get_pixel(x,y,image_width,p_size,src_pixels) :
	src_pos = (x + image_width * y) * p_size
	return (src_pixels[src_pos: src_pos + p_size])
def getpixel(x,y,src_pixels):
	pix = src_pixels[x,y]
	return map(ord,pix)
def setpixel(x,y, dest_pixels,pixel):
	dest_pixels[x,y] = chr(pixel[0]) + chr(pixel[1]) + chr(pixel[2]) + chr(pixel[3])
def python_tt_reflect(image, layer, spherical_emap_png,):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	
	if ((spherical_emap_png[-3:]).lower() == "png"):
		emap_img = pdb.file_png_load(spherical_emap_png,spherical_emap_png)
	elif ((spherical_emap_png[-3:]).lower() == "jpg"):
		emap_img = pdb.file_jpeg_load(spherical_emap_png,spherical_emap_png)
	else:
		pdb.gimp_message("Spherical Emap must be png/jpg file")
		emap_img = False
		
	#if loaded emap_img png/jpg, we run code
	if emap_img:
		
		emap_layer = pdb.gimp_image_get_active_layer(emap_img)
		result_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"reflect",100,NORMAL_MODE)
		pdb.gimp_image_insert_layer(image,result_layer,None,0)
		#loop through all pixels except border pixels, since we'll use neighboring pixels to calculate slope
		base = 4.0
		
		srcRgn = layer.get_pixel_rgn(0, 0, layer.width, layer.height,False, False)
		src_pixels = array("B", srcRgn[0:layer.width, 0:layer.height])
		emapRgn = emap_layer.get_pixel_rgn(0, 0, emap_layer.width, emap_layer.height,False, False)
		emap_pixels = array("B", emapRgn[0:emap_layer.width, 0:emap_layer.height])
		
		p_size = len(srcRgn[0,0])
		pdb.gimp_message("pixel size: " + str(p_size))
		destRgn = result_layer.get_pixel_rgn(0,0, result_layer.width, result_layer.height, True, False)
		
		#reusable variables for speed improvement.
		emap_layer_width_over_2 = emap_layer.width/2.0
		emap_layer_width_over_4 = emap_layer.width/4.0
		max_dimension = max(layer.width,layer.height)
		max_dimension_over_2 = max_dimension/2.0
		layer_width_over_2 = layer.width/2.0
		layer_height_over_2 = layer.height/2.0
		max_dist = ((256.0/base)**2 + (256.0/base)**2)**0.5
		
		gimp.progress_init("Reading pixels height values...")
		pixels = []
		for y in range(0,result_layer.height):
			pdb.gimp_progress_update((y-1.0)/(result_layer.height-2))
			row = []
			for x in range(0,result_layer.width):
				row.append(getpixel(x,y,srcRgn))
			pixels.append(row)
		
		gimp.progress_init("Calculating vertical slopes...")
		vert_slopes = []
		for y in range(0,result_layer.height - 1):
			pdb.gimp_progress_update((y-1.0)/(result_layer.height-2))
			row = []
			for x in range(0, result_layer.width):
				vert_slope = (pixel_val(pixels[y][x]) - pixel_val(pixels[y+1][x]))/base
				row.append(vert_slope)
			vert_slopes.append(row)
			
		gimp.progress_init("Calculating horizontal slopes...")
		hori_slopes = []
		for y in range(0,result_layer.height):
			pdb.gimp_progress_update((y-1.0)/(result_layer.height-2))
			row = []
			for x in range(0, result_layer.width-1):
				hori_slope = (pixel_val(pixels[y][x]) - pixel_val(pixels[y][x+1]))/base
				row.append(hori_slope)
			hori_slopes.append(row)
			
		gimp.progress_init("Rendering Reflect form Emap...")
		for y in range(1,result_layer.height - 1):
			pdb.gimp_progress_update((y-1.0)/(result_layer.height-2))
			for x in range(1, result_layer.width - 1):
				x_diff = (hori_slopes[y][x-1] + hori_slopes[y][x])/2.0 #average of two neighboring slopes
				y_diff = (vert_slopes[y-1][x] + vert_slopes[y][x])/2.0 #average of two neighboring slopes
				
				root = (x_diff**2 + y_diff**2)**0.5
				portion = root/max_dist
				
				x_coord = (x_diff * portion) * emap_layer_width_over_4 + emap_layer_width_over_2 + ((x-layer_width_over_2)/max_dimension_over_2) * emap_layer_width_over_4
				y_coord = (y_diff * portion) * emap_layer_width_over_4 + emap_layer_width_over_2 + ((y-layer_height_over_2)/max_dimension_over_2) * emap_layer_width_over_4
				x_final = int(min(emap_layer.width-1,max(0,x_coord)))
				y_final = int(min(emap_layer.width-1,max(0,y_coord)))
				
				emap_pixel = getpixel(x_final,y_final,emapRgn) + [255]
				
				setpixel(x,y,destRgn,emap_pixel)
				
		
		result_layer.update(0, 0, result_layer.width, result_layer.height)
		pdb.gimp_drawable_update(result_layer,0,0,result_layer.width,result_layer.height)
		
	pdb.gimp_image_delete(emap_img)
	#	new_display = pdb.gimp_display_new(emap_img)
	
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_reflect",                           
	"Reflect using Spherical Emap",
	"Reflect using Spherical Emap",
	"Tin Tran",
	"Tin Tran",
	"March 2018",
	"<Image>/Python-Fu/Reflect...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_FILENAME, "spherical_emap_png", "Spherical Emap (png/jpg):", "none"),
	],
	[],
	python_tt_reflect)

main()
