#!/usr/bin/env python

# reflect_emap.py Rel 2
# Mirror the image while make it 2x width 2x height while keeping the image centered.
# Created by Tin Tran http://gimplearn.net/gimplearn/
# Comments directed to http://gimplearn.net or http://gimp-forum.net or http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Not resize to image size before reflecting so that we don't get White area bug for when layer doesn't have alpha channel.
#import Image
from gimpfu import *

def python_tt_reflect_emap(image, layer):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	pdb.gimp_selection_none(image)
	layer_width = layer.width
	layer_height = layer.height
	pdb.gimp_image_resize(image,image.width*2,image.height*2,layer_width/2,layer_height/2)
	#pdb.gimp_layer_resize_to_image_size(layer)
	#mirror horizontally
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,layer_copy,None,0)
	flipped_copy = pdb.gimp_item_transform_flip_simple(layer_copy,ORIENTATION_HORIZONTAL,True,0)
	if pdb.gimp_layer_is_floating_sel(flipped_copy):
		pdb.gimp_floating_sel_to_layer(flipped_copy)
	pdb.gimp_layer_translate(flipped_copy,-layer_width+1,0)
	
	layer_copy2 = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,layer_copy2,None,0)
	flipped_copy2 = pdb.gimp_item_transform_flip_simple(layer_copy2,ORIENTATION_HORIZONTAL,True,0)
	if pdb.gimp_layer_is_floating_sel(flipped_copy2):
		pdb.gimp_floating_sel_to_layer(flipped_copy2)
	pdb.gimp_layer_translate(flipped_copy2,+layer_width-1,0)
	
	#merge down to have one layer, make layer the result with horizontal mirrored images
	result_layer = pdb.gimp_image_merge_down(image,layer_copy2,EXPAND_AS_NECESSARY)
	layer = pdb.gimp_image_merge_down(image,result_layer,EXPAND_AS_NECESSARY)
	
	#mirror vertically
	layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,layer_copy,None,0)
	flipped_copy = pdb.gimp_item_transform_flip_simple(layer_copy,ORIENTATION_VERTICAL,True,0)
	if pdb.gimp_layer_is_floating_sel(flipped_copy):
		pdb.gimp_floating_sel_to_layer(flipped_copy)
	pdb.gimp_layer_translate(flipped_copy,0,-layer_height+1)
	
	layer_copy2 = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,layer_copy2,None,0)
	flipped_copy2 = pdb.gimp_item_transform_flip_simple(layer_copy2,ORIENTATION_VERTICAL,True,0)
	if pdb.gimp_layer_is_floating_sel(flipped_copy2):
		pdb.gimp_floating_sel_to_layer(flipped_copy2)
	pdb.gimp_layer_translate(flipped_copy2,0,layer_height-1,)
	
	result_layer = pdb.gimp_image_merge_down(image,layer_copy2,EXPAND_AS_NECESSARY)
	layer = pdb.gimp_image_merge_down(image,result_layer,EXPAND_AS_NECESSARY)
	
	#done we resize to image size to get rid of excess area
	pdb.gimp_layer_resize_to_image_size(layer)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_reflect_emap",                           
	"Creates Reflect Emaps by using mirrored images of input layer",
	"Creates Reflect Emaps by using mirrored images of input layer",
	"Tin Tran",
	"Tin Tran",
	"March 2018",
	"<Image>/Python-Fu/Reflect emap...",             #Menu path
	"RGB*, GRAY*", 
	[
	#(PF_FILENAME, "spherical_emap_png", "Spherical Emap (png/jpg):", "none"),
	],
	[],
	python_tt_reflect_emap)

main()