#!/usr/bin/env python
# retro_text.py Release 2
# Created by Tin Tran 
# Refer to the below video for instructions on how to create this effect:
# https://www.youtube.com/watch?v=401re7GlOLM
#
# Comments directed to http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Use gradient[1] instead of hardcoded English value only.
from gimpfu import *
import random

def python_retro_text_tt(image, layer, retro_color, grid_color):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#get gradient list to work with array instead of English.
	num_gradients,gradient_list = pdb.gimp_gradients_get_list("")
	for l in image.layers:
		l.visible = False
	layer.visible = True
	pdb.gimp_layer_resize_to_image_size(layer)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	text_layer_position = pdb.gimp_image_get_item_position(image,layer)
	
	#blend text layer rgb
	pdb.gimp_context_set_foreground((255,255,255))
	pdb.gimp_context_set_background(retro_color)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	pdb.gimp_edit_blend(layer,FG_BG_RGB_MODE,NORMAL_MODE,GRADIENT_LINEAR,100,0,REPEAT_NONE,False, \
		False,3,0,False,image.width/2.0, y1 + (y2-y1)/2.0, \
		image.width/2.0, y1)
		
	#create 1/2 of duplicate and blend hsv	
	pdb.gimp_selection_none(image)
	copy_layer = pdb.gimp_layer_new_from_drawable(layer,image)
	pdb.gimp_image_insert_layer(image,copy_layer,None,text_layer_position)
	pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,x1,y1,x2-x1,(y2-y1)/2.0)
	pdb.gimp_edit_clear(copy_layer)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,copy_layer)
	pdb.gimp_context_set_gradient(gradient_list[1])
	pdb.gimp_edit_blend(copy_layer,CUSTOM_MODE,NORMAL_MODE,GRADIENT_LINEAR,100,0,REPEAT_NONE,False, \
		False,3,0,False,image.width/2.0, y2, \
		image.width/2.0, (y2-y1)/2.0 + y1)
	
	#outline layer
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	outline_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"outline",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,outline_layer,None,text_layer_position+2)	
	#calculate diagonals to have everything looking in proportion to tutorial.
	std_diagonal = ((1920**2)+(1080**2))**0.5
	our_diagonal = ((image.width**2)+(image.height**2))**0.5
	factor = our_diagonal/std_diagonal
	pdb.gimp_selection_grow(image,int(factor*6))
	pdb.gimp_edit_blend(outline_layer,CUSTOM_MODE,NORMAL_MODE,GRADIENT_LINEAR,100,0,REPEAT_NONE,False, \
		False,3,0,False,image.width/2.0,y1 + (y2-y1)/4, \
		image.width/2.0, ((y1 + (y2-y1)/4*3)))
	
	#glow layer
	pdb.gimp_selection_grow(image,int(factor*6))
	glow_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"retro text",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,glow_layer,None,text_layer_position+3)
	pdb.gimp_edit_fill(glow_layer,BACKGROUND_FILL)
	pdb.gimp_selection_none(image)
	pdb.plug_in_gauss(image,glow_layer,factor*150,factor*150,1)
	
	#merge all our text to one layer
	merged = copy_layer
	merged = pdb.gimp_image_merge_down(image,merged,EXPAND_AS_NECESSARY)
	merged = pdb.gimp_image_merge_down(image,merged,EXPAND_AS_NECESSARY)
	merged = pdb.gimp_image_merge_down(image,merged,EXPAND_AS_NECESSARY)
	merged_layer_position = pdb.gimp_image_get_item_position(image,merged)
	
	#grid layer
	grid_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"grid",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,grid_layer,None,merged_layer_position+1)
	pdb.plug_in_grid(image,grid_layer, \
		int(factor*10),int(factor*120),0,grid_color, 255, \
		int(factor*10),int(factor*120),0,grid_color, 255, \
		0,0,0,grid_color, 255)
		
	#transform grid layer
	grid_y1 = image.height * 0.65
	diff_x = 700.0/1080 * image.height
	grid_x1 = -diff_x
	grid_x2 = image.width+diff_x
	trans_layer = pdb.gimp_item_transform_perspective(grid_layer, \
		0, grid_y1, \
		image.width,grid_y1, \
		grid_x1,image.height, \
		grid_x2,image.height)
	if pdb.gimp_layer_is_floating_sel(trans_layer):
			pdb.gimp_floating_sel_anchor(trans_layer)
	pdb.gimp_selection_none(image)
	pdb.gimp_layer_resize_to_image_size(trans_layer)
	pdb.gimp_layer_resize_to_image_size(grid_layer)
	#mask grid layer
	mask = pdb.gimp_layer_create_mask(grid_layer,ADD_WHITE_MASK)
	pdb.gimp_layer_add_mask(grid_layer,mask)
	pdb.gimp_context_set_default_colors()
	pdb.gimp_context_swap_colors()
	pdb.gimp_edit_blend(mask,CUSTOM_MODE,NORMAL_MODE,GRADIENT_LINEAR,100,0,REPEAT_NONE,False, \
		False,3,0,False,image.width/2.0, image.height, \
		image.width/2.0, grid_y1)
	
	#stars layer
	stars_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"stars",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,stars_layer,None,merged_layer_position+2)
	pdb.gimp_edit_fill(stars_layer,BACKGROUND_FILL)
	pdb.plug_in_hsv_noise(image,stars_layer,1,0,255,255)
	pdb.gimp_desaturate(stars_layer)
	pdb.plug_in_gauss(image,stars_layer,factor*5,factor*5,1)
	pdb.gimp_levels(stars_layer,HISTOGRAM_VALUE,0,43,0.1,0,255)
	pdb.gimp_hue_saturation(stars_layer,ALL_HUES,33,0,0)
	#mask star layer
	mask = pdb.gimp_layer_create_mask(stars_layer,ADD_WHITE_MASK)
	pdb.gimp_layer_add_mask(stars_layer,mask)
	pdb.gimp_edit_blend(mask,CUSTOM_MODE,NORMAL_MODE,GRADIENT_LINEAR,100,0,REPEAT_NONE,False, \
		False,3,0,False,image.width/2, 0, \
		image.width/2, grid_y1)
	
	#lens flare layer
	lens_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"lens flare",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,lens_layer,None,merged_layer_position)
	pdb.gimp_edit_fill(lens_layer,BACKGROUND_FILL)
	pdb.plug_in_flarefx(image,lens_layer,image.width/1920.0*1353,\
		image.height/1080.0*646)
	pdb.gimp_desaturate(lens_layer)
	pdb.gimp_layer_set_mode(lens_layer,SCREEN_MODE)
	pdb.gimp_layer_set_opacity(lens_layer,57)
	
	#black bg layer
	bg_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"black bg",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,bg_layer,None,merged_layer_position+4)
	pdb.gimp_edit_fill(bg_layer,BACKGROUND_FILL)
	
	#visible layer
	visible_layer = pdb.gimp_layer_new_from_visible(image,image,"visible")
	pdb.gimp_image_insert_layer(image,visible_layer,None,merged_layer_position)
	
	#apply curves
	pdb.gimp_curves_spline(visible_layer,HISTOGRAM_VALUE,6,[0,24,24,27,255,255])
	pdb.gimp_curves_spline(visible_layer,HISTOGRAM_BLUE,4,[0,38,255,255])
	pdb.gimp_curves_spline(visible_layer,HISTOGRAM_RED,6,[0,0,118,179,255,255])
	
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return
register(
	"python_fu_retro_text_tt",                           
	"Creates retro text",
	"Creates retro text",
	"Tin Tran",
	"Tin Tran",
	"June 2018",
	"<Image>/Python-Fu/Retro Text...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_COLOR, "retro_color",  "Text Color:",(36,0,255)),
	(PF_COLOR, "grid_color",  "Grid Color:",(255,0,131)),
	],
	[],
	python_retro_text_tt)
main()
