#!/usr/bin/env python

# sand_drop.py Release 2
# drop colored pixels like sand of the whole active layer.
# Created by Tin Tran http://gimplearn.net
# Comments directed to http://gimplearn.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: More realistic sand slope by checking wider base.
#import string
#import Image
import random
from gimpfu import *
#from array import array
#import sys

def python_tt_sand_drop(image, layer, min_g, max_g):
	pdb.gimp_progress_init("Reading Grains of Sand (pixels)...",None)
	width,height = layer.width,layer.height
	#Read colors of each pixel as grain of sand
	pixels = []
	for y in range(0,height):
		pdb.gimp_progress_update(float(y)/height)
		for x in range(0,width):
			num_channels,pixel = pdb.gimp_drawable_get_pixel(layer,x,y)
			pixels.append([pixel[0],pixel[1],pixel[2],255])
			
	#Create new image to work with
	width += 4 #make it 2 pixels wider to contain the sand
	height *= 3 #make it higher than actual height so that we have room to drop sand
	width = int(width)
	height = int(height)
	new_image = pdb.gimp_image_new(width,height,RGB)
	new_display = pdb.gimp_display_new(new_image)
	work_layer = pdb.gimp_layer_new(new_image,width,height,RGBA_IMAGE,"Sand Drop",100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(new_image,work_layer,None,0)

	s = [] #our sand array
	#make s array blanks
	for y in range(0,height):
		row = []
		for x in range(0,width):
			row.append(0)
		s.append(row)
	#make left/right most column 1's
	for y in range(0,len(s)):
		s[y][0] = 1
		s[y][1] = 1
		s[y][len(s[y])-1] = 1
		s[y][len(s[y])-2] = 1
	#make bottommost row 1's
	for x in range(0,len(s[0])):
		#s[0][x] = 1
		#s[len(s)-1][x] = 1
		s[len(s)-1][x] = 1
	total_pixels = len(pixels)	
	sands = []
	x_location = random.randrange(2,width-1)
	x = random.randrange(max(2,x_location-25),min(width-2,x_location+25))
	y = 0
	#take a color and remove it from list of pixels
	color = pixels[-1]
	del pixels[-1]
	sands.append([x,y,color,0])
	s[y][x] = 2

	moving = True #still moving sand
	pdb.gimp_progress_init("Dropping Sand...",None)
	looped = 1
	dropped = random.randrange(min_g,max_g)
	while moving == True:
		looped += 1
		moving = False
		for i in range(len(sands)-1,-1,-1): #loop backward so that we can delete it as we go
			x,y,color,not_moved = sands[i][0],sands[i][1],sands[i][2],sands[i][3]
			nx,ny = x,y
			s[y][x] = 0
			moved = True
			if s[y+1][x] == 0:
				ny+=1
			elif s[y+1][x+1] == 0 and s[y+1][x-1] > 0 and s[y][x+1] == 0: #tumble right
				nx+=1
			elif s[y+1][x-1] == 0 and s[y+1][x+1] > 0 and s[y][x-1] == 0 : #tumble left
				nx-=1
			elif s[y+1][x+1] == 0 and s[y+1][x-1] == 0 and s[y][x+1] == 0 and s[y][x-1] == 0: #tumble random
				if random.randrange(100)>=50:
					if s[y][x-1] == 0:
						nx-=1
				else:
					if s[y][x+1] == 0:
						nx+=1
			elif s[y+1][x-1] > 0 and (s[y+1][x-2]>0 or s[y][x-2]>0)  and s[y][x+1] == 0 and s[y+1][x+2]==0 and s[y][x+2]==0: #tumble right
				nx+=1
			elif s[y+1][x+1] > 0 and (s[y+1][x+2]>0 or s[y][x+2]>0)  and s[y][x-1] == 0 and s[y+1][x-2]==0 and s[y][x-2]==0: #tumble left
				nx-=1
			elif s[y][x+1]==0 and s[y][x-1]==0 and s[y][x-2]==0 and s[y+1][x-2]==0 and s[y][x+2]==0 and s[y+1][x+2]==0:
				if random.randrange(100)>=50:
					if s[y][x-1] == 0:
						nx-=1
				else:
					if s[y][x+1] == 0:
						nx+=1
			else:
				moved = False
			if moved:
				s[ny][nx] = 2
				sands[i] = [nx,ny,color,0]
				moving = True #set this true so we loop again and again if any sand is still moving
			else: #sand isn't moving moved, it's stable draw it
				if not_moved > 100:
					s[y][x] = 2
					pdb.gimp_drawable_set_pixel(work_layer,x,y,4,color)
				#delete it from sands because it's drawn at a stable location now
					del sands[i]
				else:
					s[ny][nx] = 2
					sands[i] = [nx,ny,color,not_moved+1]
					moving = True	
		#if there are still pixels left we keep adding to sands one by one
		pdb.gimp_progress_update(float(total_pixels-len(pixels))/total_pixels)
		if len(pixels) > 0:
			x = random.randrange(max(2,x_location-25),min(width-2,x_location+25))
			y = 0
			#take a color and remove it from list of pixels
			color = pixels[-1]
			del pixels[-1]
			sands.append([x,y,color,0])
			s[y][x] = 2
			dropped -= 1
			if dropped <= 0:
				dropped = random.randrange(min_g,max_g)
				x_location = random.randrange(1,width-1)
		if looped > 1000:
			pdb.gimp_drawable_update(work_layer,0,0,width,height)
			pdb.gimp_displays_flush()				
			looped = 1

	pdb.gimp_drawable_update(work_layer,0,0,width,height)
	pdb.gimp_displays_flush()				

	#pdb.gimp_drawable_set_pixel(work_layer,10,10,4,pixel[10])


	pdb.gimp_progress_end()	
	pdb.gimp_message("Complete!")			
register(
	"python_fu_tt_sand_drop",						   
	"Sand Drop the whole active layer as if pixels were made of sand",
	"Sand Drop the whole active layer as if pixels were made of sand",
	"Tin Tran",
	"Tin Tran",
	"June 2019",
	"<Image>/Python-Fu/Sand Drop",			 #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "min_g", "Minimum number of grains to drop at one location:", 50, (10, 100000, 1)),
	(PF_SPINNER, "max_g", "Maximum number of grains to drop at one location:", 2000, (10, 100000, 1)),
	#(PF_SPINNER, "grow", "Pixels to grow each area before intersecting with original selection:", 2, (0, 100, 1)),
	#(PF_TOGGLE, "brushit", "Edit-Blend instead of Edit-Fill (Edit-blend is faster):", 1),
	#(PF_SPINNER, "brushsize", "Brush Size:", 500, (100, 10000, 1)),
	#(PF_TOGGLE, "dodge", "Dodge filled areas (Subtract alpha to selection's selection):", 0),
	],
	[],
	python_tt_sand_drop)

main()