#!/usr/bin/env python

# seamless_cut.py Release 1
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
from gimpfu import *
#from array import array
#import sys
def python_tt_seamless_cut(image, layer, direction):
	#pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#core code goes here/below
	saved_selection = pdb.gimp_selection_save(image)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	section_width = 3
	if direction == 0: #vertical cut
		pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,0,0,section_width,image.height)
		pdb.gimp_edit_copy(layer)
		floating_sel = pdb.gimp_edit_paste(layer,TRUE)
		pdb.gimp_floating_sel_to_layer(floating_sel)
		pdb.gimp_layer_set_mode(floating_sel,LAYER_MODE_DIFFERENCE)
		isset = False
		min_difference = 0
		save_at = 0
		for i in range(x1,x2):
			pdb.gimp_layer_set_offsets(floating_sel,i,0)
			visible = pdb.gimp_layer_new_from_visible(image,image,"visible")
			pdb.gimp_image_insert_layer(image,visible,None,0)
			pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,i,0,section_width,image.height)
			r, _, _, _, _, _ = pdb.gimp_histogram(visible,HISTOGRAM_RED,0,255)
			g, _, _, _, _, _ = pdb.gimp_histogram(visible,HISTOGRAM_GREEN,0,255)
			b, _, _, _, _, _ = pdb.gimp_histogram(visible,HISTOGRAM_BLUE,0,255)
			difference = r + g + b
			if not isset:
				min_difference = difference
				save_at = i
				isset = True 
			elif difference < min_difference:
				save_at = i
				min_difference = difference
			pdb.gimp_image_remove_layer(image,visible)
		#here we should have a minimum difference and save_at
		pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,0,0,save_at,image.height)
		pdb.gimp_edit_copy(layer)
		new_image = pdb.gimp_edit_paste_as_new_image(image)
		display = pdb.gimp_display_new(new_image)
		pdb.gimp_image_remove_layer(image,floating_sel)
	else: #horizontal cut
		pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,0,0,image.width,section_width)
		pdb.gimp_edit_copy(layer)
		floating_sel = pdb.gimp_edit_paste(layer,TRUE)
		pdb.gimp_floating_sel_to_layer(floating_sel)
		pdb.gimp_layer_set_mode(floating_sel,LAYER_MODE_DIFFERENCE)
		isset = False
		min_difference = 0
		save_at = 0
		for i in range(y1,y2):
			pdb.gimp_layer_set_offsets(floating_sel,0,i)
			visible = pdb.gimp_layer_new_from_visible(image,image,"visible")
			pdb.gimp_image_insert_layer(image,visible,None,0)
			pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,0,i,image.width,section_width)
			r, _, _, _, _, _ = pdb.gimp_histogram(visible,HISTOGRAM_RED,0,255)
			g, _, _, _, _, _ = pdb.gimp_histogram(visible,HISTOGRAM_GREEN,0,255)
			b, _, _, _, _, _ = pdb.gimp_histogram(visible,HISTOGRAM_BLUE,0,255)
			difference = r + g + b
			if not isset:
				min_difference = difference
				save_at = i
				isset = True 
			elif difference < min_difference:
				save_at = i
				min_difference = difference
			pdb.gimp_image_remove_layer(image,visible)
		#here we should have a minimum difference and save_at
		pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,0,0,image.width,save_at)
		pdb.gimp_edit_copy(layer)
		new_image = pdb.gimp_edit_paste_as_new_image(image)
		display = pdb.gimp_display_new(new_image)
		pdb.gimp_image_remove_layer(image,floating_sel)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_selection) #return to original selection
	
	#UNCOMMENT BELOW WHEN DONE
	pdb.gimp_context_pop()
	#pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_tt_seamless_cut",                           
	"Make seamless vertical/horizontal cut within selection that closest match left/top edge of layer",
	"Make seamless vertical/horizontal cut within selection that closest match left/top edge of layer",
	"Tin Tran",
	"Tin Tran",
	"March 2019",
	"<Image>/Python-Fu/Create New/Seamless Cut",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_OPTION,"direction",   "Cut Direction:", 0, ["Vertical","Horizontal"]),
	#(PF_COLOR, "paper_color", "Paper color:", (224, 223, 205)),
	#(PF_TOGGLE, "colorize", "Colorize Image", 0),
	#(PF_COLOR, "colorize_color", "Colorize Color:", (137, 152, 133)),
	],
	[],
	python_tt_seamless_cut)

main()