#!/usr/bin/env python

# selection-to-gradient-border.py
# Created by Tin Tran https://gimplearn.net
# Comments directed to https://gimplearn.net
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Use "2. Hard Brush 100" to stroke selection instead of clearing with background color
#        produces smoother results.
#import Image
from gimpfu import *
#import random
#(PF_SLIDER, "p3", "SLIDER:", 0, (0, 100, 10)),import math
#import os

def python_selection_to_gradient_border(image,layer,border_radius,gradient):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#save existing selection
	saved_selection_channel = pdb.gimp_selection_save(image)
	#creates a new layer to paint gradient with
	new_layer = pdb.gimp_layer_new(image,layer.width,layer.height,RGBA_IMAGE,'gradient layer',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,new_layer,None,0)
	border_radius = int(border_radius)
	dist = []
	for i in range(border_radius-1,-1,-1):
		dist.append(float(i)/float(border_radius-1))
	num_color_samples,color_samples = pdb.gimp_gradient_get_custom_samples(gradient,border_radius,dist,FALSE)
	pdb.gimp_context_set_brush("2. Hardness 100");
	pdb.gimp_context_set_brush_hardness(1);
	pdb.gimp_context_set_brush_spacing(0.01);

	for i in range(border_radius-1,-1,-1):
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_selection_channel)
		#pdb.gimp_selection_border(image,i)
		#dist = float(i)/float(border_radius)
		#num_color_samples,color_samples = pdb.gimp_gradient_get_custom_samples(gradient,1,[dist],FALSE)
		ci = (border_radius - (i+1))*4
		color = (int(color_samples[ci+0]*255),int(color_samples[ci+1]*255),int(color_samples[ci+2]*255))
		pdb.gimp_context_set_foreground(color)

		stroke_width = i*2+1
		pdb.gimp_context_set_brush_size(stroke_width)
		pdb.gimp_drawable_edit_stroke_selection(new_layer)
		#pdb.gimp_message(str(color_samples[0]));
		#pdb.gimp_edit_clear(new_layer)
		#pdb.gimp_edit_fill(new_layer,FILL_FOREGROUND)
	#add alpha and remove anything that isn't part of hard border-radius	
	'''pdb.gimp_layer_add_alpha(new_layer)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_selection_channel)
	pdb.gimp_selection_border(image,border_radius)
	pdb.gimp_selection_invert(image)
	pdb.gimp_edit_clear(new_layer)	
	'''
	#l = new_layer #set l to be layer we want to be layer mask we copy from
	#mask = pdb.gimp_layer_create_mask(l,ADD_MASK_COPY)
	#pdb.gimp_layer_add_mask(l,mask)
	#pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,mask)


	#Now that we have the mask selected we can remove the layer. and saved channel
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_selection_channel)

	pdb.gimp_image_remove_channel(image,saved_selection_channel)
	#pdb.gimp_image_remove_layer(image,new_layer)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_selection_to_gradient_border_tt",
	"Uses any gradient to paint with border select",
	"Uses any gradient to paint with border select",
	"Tin Tran",
	"Tin Tran",
	"June 2020",
	"<Image>/Python-Fu/Selection To Gradient Border...",             #Menu path
	"RGB*, GRAY*, INDEXED*", 
	[
	(PF_SPINNER, "border_radius", "Border-radius:", 50, (1, 1000, 5)),
	(PF_GRADIENT, "gradient", "Gradient:", 0),
	#(PF_OPTION,"option",   "Mapping:", 0, ["To Circle","To Square"]),
	
	],
	[],
	python_selection_to_gradient_border)

main()
