#!/usr/bin/env python

# simplify.py Release 3
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Additive selection.
# Rel 3: Add vectors as requested by MareroQ on GC.
from gimpfu import *
import random
#from array import array
#import sys
def python_tt_simplify(image, layer, colors, simplify_size):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#core code goes here/below
	pdb.gimp_image_convert_indexed(image,CONVERT_DITHER_NONE,CONVERT_PALETTE_GENERATE,colors,FALSE,TRUE,"unused palette name")
	num_bytes,colormap = pdb.gimp_image_get_colormap(image)
	pdb.gimp_image_convert_rgb(image)
	scale = simplify_size*1.0/layer.width
	image_copy = pdb.gimp_image_duplicate(image)

	for i in range(0,int(colors)):
		working_image = pdb.gimp_image_duplicate(image_copy)
		active_layer = pdb.gimp_image_get_active_layer(working_image)
		color = (colormap[i*3],colormap[i*3+1],colormap[i*3+2])
		pdb.gimp_by_color_select_full(active_layer,color,0,CHANNEL_OP_REPLACE,TRUE,FALSE,0,0,FALSE,FALSE,SELECT_CRITERION_COMPOSITE)
		for j in range(i+1,int(colors)):
			lighter_color = (colormap[j*3],colormap[j*3+1],colormap[j*3+2])
			pdb.gimp_by_color_select_full(active_layer,lighter_color,0,CHANNEL_OP_ADD,TRUE,FALSE,0,0,FALSE,FALSE,SELECT_CRITERION_COMPOSITE)
		#pin corners
		pdb.gimp_image_select_rectangle(working_image,CHANNEL_OP_ADD,0,0,1,1)
		pdb.gimp_image_select_rectangle(working_image,CHANNEL_OP_ADD,layer.width-1,layer.height-1,1,1)
		pdb.gimp_image_scale(working_image,layer.width*scale,layer.height*scale)
		pdb.plug_in_sel2path(working_image,active_layer)
		pdb.gimp_image_scale(working_image,layer.width,layer.height)
		vectors = pdb.gimp_image_get_active_vectors(working_image)
		#copy vectors:
		vectors_copy = pdb.gimp_vectors_copy(vectors)
		num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(vectors)
		new_vectors = pdb.gimp_vectors_new(image,"simplify stroke")
		for k in range(0,int(num_strokes)):
			stroke_id = stroke_ids[k]
			type,num_points,control_points,closed = pdb.gimp_vectors_stroke_get_points(vectors,stroke_id)
			new_stroke_id = pdb.gimp_vectors_stroke_new_from_points(new_vectors,VECTORS_STROKE_TYPE_BEZIER,num_points,control_points,closed)
		pdb.gimp_image_insert_vectors(image,new_vectors,None,0)
		pdb.gimp_image_select_item(working_image,CHANNEL_OP_REPLACE,vectors)
		pdb.gimp_context_set_foreground(color)
		new_layer = pdb.gimp_layer_new(working_image,layer.width,layer.height,RGBA_IMAGE,"new color",100,LAYER_MODE_NORMAL)
		pdb.gimp_image_insert_layer(working_image,new_layer,None,0)
		pdb.gimp_edit_fill(new_layer,FILL_FOREGROUND)
		layer_copy = pdb.gimp_layer_new_from_drawable(new_layer,image)
		pdb.gimp_image_insert_layer(image,layer_copy,None,0)
		pdb.gimp_image_delete(working_image)

	pdb.gimp_image_delete(image_copy)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	#return

register(
	"python_fu_tt_simplify",                           
	"Creates simplified version of image using selections/paths",
	"Creates simplified version of image using selections/paths",
	"Tin Tran",
	"Tin Tran",
	"May 2019",
	"<Image>/Python-Fu/Simplify...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_SPINNER, "colors", "Number of colors:", 8, (2, 255, 1)),
	(PF_SPINNER, "simplify_size", "Simplify Size (Width):", 100, (10, 1000, 1)),
	#(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	#(PF_TOGGLE, "keep_aspect", "Keep Aspect Ratio of Border:", 1),
	#(PF_TOGGLE, "mirror","Mirror/Reflect Borders:", 1),
	#(PF_TOGGLE, "drop_shadow","Drop Shadow:", 1),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_blur", "Shadow Blur Radius:", 15, (0, 1000, 1)),
	],
	[],
	python_tt_simplify)

main()
