#!/usr/bin/env python

# sliced_text.py Rel 4
# based on below youtube tutorial by "Logos by Nick"
# https://www.youtube.com/watch?v=-E9BYqtMDfs
# Created by Tin Tran 
# Comments directed to http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# rel 2: when slide shift is zero pixel (blur 5px).
# Rel 3: Try to reproduce Issabella's folding effect.
# Rel 4: Get rid of partial pixel artifacts bug.
from gimpfu import *

def python_sliced_text_tt(image, layer, gradient, reverse, slice_angle, slice_shift, text_angle):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#pdb.gimp_progress_init("Processing...",None)
	#pdb.gimp_progress_update(float(intensity)/255.0)
	#create gradient background
	slice_height = int(abs(slice_shift/4.0)) #Added for issabella's folding effect
	
	bg_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"gradient bg",100,NORMAL_MODE)
	text_position = pdb.gimp_image_get_item_position(image,layer)
	pdb.gimp_image_insert_layer(image,bg_layer,None,text_position+1) #insert below text (layer)
	pdb.gimp_selection_none(image)
	pdb.gimp_context_set_gradient(gradient)
	pdb.gimp_edit_blend(bg_layer,CUSTOM_MODE,NORMAL_MODE,GRADIENT_LINEAR,100,0,REPEAT_NONE,reverse==1,False,3,.2,False, \
		0,0, \
		image.width, image.height)
	pdb.gimp_image_set_active_layer(image,layer)
	pdb.plug_in_autocrop_layer(image,layer)
	layer = pdb.gimp_item_transform_rotate(layer,slice_angle/360.0 * 3.1415 * 2,True,0,0)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,layer)
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	pdb.gimp_layer_resize_to_image_size(layer)
	
	#the join fold
	pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,0,(y1+y2)/2.0,image.width,slice_height)
	float_join_layer = pdb.gimp_selection_float(layer,0,0)
	pdb.gimp_floating_sel_to_layer(float_join_layer)
	pdb.gimp_item_transform_perspective(float_join_layer,0,(y1+y2)/2.0,image.width,(y1+y2)/2.0,\
		0+slice_shift,(y1+y2)/2.0 + slice_height,image.width+slice_shift,(y1+y2)/2.0 + slice_height)
		
	#bottom text
	pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,0,(y1+y2)/2.0 + slice_height,image.width,image.height/2.0 - slice_height)
	float_layer = pdb.gimp_selection_float(layer,0,0)
	pdb.gimp_floating_sel_to_layer(float_layer)
	pdb.gimp_layer_translate(float_layer,slice_shift,0)
	
	#create shadow 
	shadow_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"slice shadow",100,NORMAL_MODE)
	text_position = pdb.gimp_image_get_item_position(image,layer)
	pdb.gimp_image_insert_layer(image,shadow_layer,None,text_position-2)
	pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,min(x1,x1+slice_shift),(y1+y2)/2.0 - slice_shift,\
		(x2-x1)+abs(slice_shift),abs(slice_shift*2))
	pdb.gimp_context_set_foreground((0,0,0))
	if slice_shift != 0:
		pdb.gimp_edit_fill(shadow_layer,FOREGROUND_FILL)
	pdb.gimp_selection_none(image)
	pdb.plug_in_gauss(image,shadow_layer,max(5,abs(slice_shift*5)),max(5,abs(slice_shift*5)),1)
	pdb.gimp_image_select_rectangle(image,CHANNEL_OP_REPLACE,0,(y1+y2)/2.0,image.width,image.height/2.0)
	pdb.gimp_edit_clear(shadow_layer)
	pdb.gimp_selection_none(image)
	
	#join shadow
	shadow_join_layer = pdb.gimp_layer_new_from_drawable(shadow_layer,image)
	text_position = pdb.gimp_image_get_item_position(image,layer)
	pdb.gimp_image_insert_layer(image,shadow_join_layer,None,text_position-2)
	pdb.gimp_layer_translate(shadow_join_layer,-slice_shift/2.0,slice_height)
	pdb.gimp_layer_resize_to_image_size(shadow_join_layer)
	
	#make float layer as image size
	pdb.gimp_layer_resize_to_image_size(float_layer)
	pdb.gimp_layer_resize_to_image_size(float_join_layer)
	
	#rotate when complete
	angle = text_angle/360.0 * 3.1415 * 2
	pdb.gimp_item_transform_rotate(layer,angle,True,0,0)
	pdb.gimp_item_transform_rotate(float_layer,angle,True,0,0)
	pdb.gimp_item_transform_rotate(shadow_layer,angle,True,0,0)
	pdb.gimp_item_transform_rotate(shadow_join_layer,angle,True,0,0)
	pdb.gimp_item_transform_rotate(float_join_layer,angle,True,0,0)
	
	#link them so user can move them together
	pdb.gimp_item_set_linked(layer,True)
	pdb.gimp_item_set_linked(float_layer,True)
	pdb.gimp_item_set_linked(shadow_layer,True)
	pdb.gimp_item_set_linked(shadow_join_layer,True)
	pdb.gimp_item_set_linked(float_join_layer,True)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_sliced_text_tt",                           
	"Sliced text effect on text layer",
	"Sliced text effect on text layer",
	"Tin Tran",
	"Tin Tran",
	"June 2018",
	"<Image>/Python-Fu/Sliced Text...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_GRADIENT, "gradient", "Background Gradient:", 0),
	(PF_TOGGLE, "reverse",   "Reverse Gradient:", 0),
	(PF_SPINNER, "slice_angle", "Rotate angle(before slicing):", -5, (-180, 180, 1)),
	(PF_SPINNER, "slice_shift", "Slice Shift(pixels):", -30, (-100, 100, 1)),
	(PF_SPINNER, "text_angle", "Rotate angle(when complete):", -8, (-180, 180, 1)),
	],
	[],
	python_sliced_text_tt)

main()
