#!/usr/bin/env python

# straighten.py Release 1
# Tries to straighten a regular selection.
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
from gimpfu import *
import random
import math
#from array import array
#import sys
def python_tt_straighten(image, layer):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	#core code goes here/below

	#saves selection to select it later after we find minimal rectangle area.
	non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
	if non_empty:
		selection = pdb.gimp_selection_save(image)
		pdb.plug_in_sel2path(image,layer)
		vectors = pdb.gimp_image_get_active_vectors(image)
		center_x = (x1+x2)/2.0
		center_y = (y1+y2)/2.0
		min_area = 15000*15000 #find minimum area, set it to some huge area to start
		save_angle = 0
		for ang_ in range(-15,16): #half angle accuracy, -15 degrees to +15 degrees
			angle = math.radians(ang_*1)
			pdb.gimp_item_transform_rotate(vectors,angle,FALSE,center_x,center_y) #rotate to this new angle to search
			pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,vectors) #select the path to see it's area
			non_empty,x1,y1,x2,y2 = pdb.gimp_selection_bounds(image)
			current_area = (x2-x1) * (y2-y1)
			if current_area < min_area: #found a better rotation save min_area and save angle
				min_area = current_area
				save_angle = angle
			pdb.gimp_item_transform_rotate(vectors,-angle,FALSE,center_x,center_y) #return it to original angle so that next step can continue to search
		pdb.gimp_image_remove_vectors(image,vectors) #don't need the path any more so we delete it
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,selection) #select original selection
		float_sel = pdb.gimp_item_transform_rotate(layer,save_angle,FALSE,center_x,center_y) #rotate the layer with selection to the optimal saved angle.
		pdb.gimp_floating_sel_to_layer(float_sel)
	else:
		pdb.gimp_message("You must have a (rough) rectangular selection!")
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
	#return

register(
	"python_fu_tt_straighten",                           
	"Straightens a rough regular selection",
	"Straightens a rough regular selection",
	"Tin Tran",
	"Tin Tran",
	"May 2019",
	"<Image>/Python-Fu/Straighten",             #Menu path
	"RGB*, GRAY*", 
	[
	#(PF_SPINNER, "colors", "Number of colors:", 8, (2, 255, 1)),
	#(PF_SPINNER, "simplify_size", "Simplify Size (Width):", 100, (10, 1000, 1)),
	#(PF_SPINNER, "frame_percent", "Frame percentage (percent to grow by to create border):", 10, (5, 100, 1)),
	#(PF_TOGGLE, "keep_aspect", "Keep Aspect Ratio of Border:", 1),
	#(PF_TOGGLE, "mirror","Mirror/Reflect Borders:", 1),
	#(PF_TOGGLE, "drop_shadow","Drop Shadow:", 1),
	#(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 2, (-1000, 1000, 1)),
	#(PF_SPINNER, "shadow_blur", "Shadow Blur Radius:", 15, (0, 1000, 1)),
	],
	[],
	python_tt_straighten)

main()
