#!/usr/bin/env python

# Stroke_Path_With_Gradient.py Release 9
# (requested by Pegleg44 here: https://gimplearn.net/viewtopic.php?p=10403#p10403)
# Created by Tin Tran
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Added semiflatten step
# Rel 3: Added semiflatten option
# Rel 4: Changed code according to ofnuts so that it'll work on Linux and OSX as well.
# Rel 5: Added Use Custom gradient capability.
# Rel 6: Only create one .svg file in document history instead of n files (number of strokes).
# Rel 7: Generate a single .svg file that sticks around instead of generating multiple .svg and removing them.
# Rel 8: Fixed an error so that it would work for linux.
# Rel 9: Changed to use GIMP 2.10's constant names.
import math
import string
#import Image
from gimpfu import *
import re, tempfile
import os
#from array import array
#get tempfile name here so that it only does it once instead of multiple times which clogs up document history.
#tempf=tempfile.NamedTemporaryFile(mode='w', suffix='.svg', dir=gimp.directory, delete=False)
tempf=tempfile.NamedTemporaryFile(mode='w', suffix='.svg', dir=gimp.directory, delete=False)
tempf.write("whatever")
tempf.close()
def color_to_hex(color):
	whole_color = (color[0],color[1],color[2]);
	return '#%02x%02x%02x' % whole_color
def vector_to_line_stroke(image, vector, layer, color="#000000", width=1, capstyle="butt", joinstyle="miter", miterlimit=10, shaperendering="auto",first=1):
	newelements = {
			'stroke': color,
			'stroke-width': width,
			'stroke-linecap': capstyle,
			'stroke-linejoin': joinstyle,
			'stroke-miterlimit': miterlimit,
			'shape-rendering': shaperendering,
			}
	svg = pdb.gimp_vectors_export_to_string(image, vector)
    #fix width and height to be resolution (px/inch)-independent
	svg = re.sub(r'(<svg\s[^>]*\swidth\s*=\s*)\S*"', r'\1"%dpx"' % image.width, svg, flags=re.DOTALL)
	svg = re.sub(r'(<svg\s[^>]*\sheight\s*=\s*)\S*"', r'\1"%dpx"' % image.height, svg, flags=re.DOTALL)
	svg = re.sub(r'(<path\s[^>]*)\sstroke\s*=\s*"black"', r'\1', svg, flags=re.DOTALL)
	svg = re.sub(r'(<path\s[^>]*)\sstroke-width\s*=\s*"1"', r'\1', svg, flags=re.DOTALL)
	svg = re.sub(r'(<path\s)', r'\1' + ''.join([r'%s="%s" ' % i for i in newelements.items()]), svg, flags=re.DOTALL)
	if (first == 1):
		#do nothing
		pass
	else:
		#remove xml tag, doctype tag, svg tag so that it's left with just path tag.
		svg = re.sub(r'(<\?xml\s[^>]*>)(.*)(<\!DOCTYPE\s[^>]*>)(.*)(<svg\s[^>]*>)(.*)(<path\s[^>]*>)(.*)(<\/svg>)', r'\7', svg, flags=re.DOTALL)
		#pdb.gimp_message(svg);
		
	return svg;
#custom function to append the multiple paths elements into existing svg xml.
def svg_append(svg,svgpaths):
	svg = re.sub(r'(<\?xml\s[^>]*>)(.*)(<\!DOCTYPE\s[^>]*>)(.*)(<svg\s[^>]*>)(.*)(<path\s[^>]*>)(.*)(<\/svg>)', r'\1\2\3\4\5\6\7\8', svg, flags=re.DOTALL)
	svg += svgpaths;
	svg += "</svg>";
	return svg;
#write to svg file and load as layer in GIMP
def write_to_file_and_load(image,layer,svg):
	tf = open(tempf.name,"w")
	tf.write(svg)
	tf.close()
	newlayer = pdb.gimp_file_load_layer(image, tf.name)
	#os.remove(tf.name) Try not removing it so that user can open it as svg
	image.add_layer(newlayer) #needs to be added to the image to be able to copy from
	copylayer = pdb.gimp_edit_named_copy(newlayer, "stroke")
	image.remove_layer(newlayer)
	floating_sel = pdb.gimp_edit_named_paste(layer, copylayer, True)
	pdb.gimp_floating_sel_anchor(floating_sel)
#=====================================================================================================================
def python_stroke_path_with_gradient(image, layer,stroke_line_width,strokes,center_color,border_color,custom_gradient,gradient,reverse_gradient,semiflatten) :
	pdb.gimp_image_undo_group_start(image)
	
	#grab selection bounding box values
	
	#get active vectors and do some work
	active_vectors = pdb.gimp_image_get_active_vectors(image)
	num_strokes,stroke_ids = pdb.gimp_vectors_get_strokes(active_vectors)
	
	arrow_layer_name = "Gradient Stroked Path";
	width = pdb.gimp_image_width(image)
	height = pdb.gimp_image_height(image)
	arrow_layer = pdb.gimp_layer_new(image,width,height,RGBA_IMAGE,arrow_layer_name,100,LAYER_MODE_NORMAL_LEGACY)
	pdb.gimp_image_insert_layer(image,arrow_layer,None,-1)
	arrow_layer_name = pdb.gimp_item_get_name(arrow_layer)
	svgpaths = "";
	#grab colors samples if using custom gradient.
	if custom_gradient == 1:
		num_color_samples, colors = pdb.gimp_gradient_get_uniform_samples(gradient,strokes+1,reverse_gradient);
	for stroke in range(0,int(strokes+1)):
		if custom_gradient == 0:
			r = border_color[0] + int(float(center_color[0] - border_color[0])/strokes * stroke);
			g = border_color[1] + int(float(center_color[1] - border_color[1])/strokes * stroke);
			b = border_color[2] + int(float(center_color[2] - border_color[2])/strokes * stroke);
			color = (r,g,b);
		else:
			color = (colors[stroke * 4 + 0]*255,colors[stroke * 4 + 1]*255, colors[stroke * 4 + 2]*255);
			
		stroke_width = stroke_line_width - ((stroke_line_width-0.01)/strokes * stroke);
		if stroke==0:
			svg = vector_to_line_stroke(image,active_vectors,arrow_layer,color_to_hex(color),stroke_width,"square","miter",10,"auto",stroke==0)
		else:
			svgpaths += vector_to_line_stroke(image,active_vectors,arrow_layer,color_to_hex(color),stroke_width,"square","miter",10,"auto",stroke==0)
		#after it's done we just append svgpaths to svg
		svg = svg_append(svg,svgpaths)
		write_to_file_and_load(image,arrow_layer,svg)
		
	if semiflatten == 1:
		pdb.plug_in_semiflatten(image,arrow_layer)
	
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_stroke_path_with_gradient",
	"Stroke Path with Gradient",
	"Stroke Path with Gradient...",
	"Tin Tran",
	"Tin Tran",
	"Sept 2017",
	"<Image>/Python-Fu/Stroke Path With Gradient...",
	"*",      # Create a new image, don't work on an existing one
	[
	#(PF_OPTION, "arrow_side", "Arrows Ends:", SIDE_END, SIDE_NAMES),
	#(PF_TOGGLE, "arrow_close", "Arrows Close:", 0),
	(PF_SPINNER, "stroke_line_width", "Stroke Line Width:", 6.0, (1, 100, 0.5)),
	(PF_SPINNER, "strokes", "Strokes to Emulate Gradient:", 6, (1, 255, 1)),
	(PF_COLOR, "center_color",  "Center Stroke Color",  (0,0,0) ),
	(PF_COLOR, "border_color",  "Border Stroke Color",  (255,255,255)),
	(PF_TOGGLE, "custom_gradient", "Use Custom Gradient", 0),
	(PF_GRADIENT, "gradient", "Custom Gradient:", 0),
	(PF_TOGGLE, "reverse_gradient", "Reverse Custom Gradient", 0),
	(PF_TOGGLE, "semiflatten",   "Semiflatten (Replace partial transparency with the current background color):", 0),
	
	
	#(PF_COLOR, "black",  "Black point color",  (0,0,0) ),
	#(PF_COLOR, "white",  "White point color",  (255,255,255) ),
	#(PF_COLOR, "gray",  "Gray point color",  (128,128,128) )
	#(PF_FILE, "infilename", "Temp Filepath", "/Default/Path")
	#(PF_DIRNAME, "source_directory", "Source Directory", "") for some reason, on my computer when i(Tin) use PF_DIRNAME the pythonw.exe would crash
	],
	[],
	python_stroke_path_with_gradient)

main()
