#!/usr/bin/env python

# wallace_panels.py Rel 3
# This script tries to reproduce Wallace's panel painting effect: http://gimpchat.com/viewtopic.php?f=11&t=15938#p220799
# This script needs stroke_path_with_gradient.py script (version 7) found here: https://gimplearn.net/viewtopic.php?p=10409#p10409
# Created by Tin Tran
# Comments directed to http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release
# Rel 2: Add alpha channel if layer doesn't have one.
# Rel 3: Make options editable. Error in previous versions prevent user from editing options.

import math
import string
#import Image
from gimpfu import *
def python_wallace_panels(image, layer, border_width, shadow_offset_x, shadow_offset_y, shadow_blur_radius,shadow_opacity) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	if not layer.has_alpha:
		pdb.gimp_layer_add_alpha(layer)
	layer_position = pdb.gimp_image_get_item_position(image,layer)
	saved_channel = pdb.gimp_selection_save(image)
	pdb.gimp_selection_none(image)
	bump_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"bump",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,bump_layer,None,layer_position+1)
	pdb.gimp_context_set_foreground((0,0,0))
	pdb.gimp_context_set_background((255,255,255))
	pdb.gimp_edit_fill(bump_layer,FOREGROUND_FILL)
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_channel)
	pdb.gimp_edit_fill(bump_layer,BACKGROUND_FILL)
	pdb.plug_in_sel2path(image,bump_layer)
	pdb.python_fu_stroke_path_with_gradient(image,bump_layer,border_width*2,border_width*2,(0,0,0),(255,255,255),1,"FG to BG (RGB)",1,0)
	stroked_layer = pdb.gimp_image_get_layer_by_name(image,"Gradient Stroked Path")
	bump_layer = pdb.gimp_image_merge_down(image,stroked_layer,EXPAND_AS_NECESSARY)
	pdb.plug_in_bump_map(image,layer,bump_layer,135,45,3,0,0,0,0,1,0,0)
	pdb.gimp_selection_invert(image)
	pdb.gimp_edit_clear(layer)
	pdb.gimp_selection_none(image)
	#add a white background layer
	white_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,"white background",100,NORMAL_MODE)
	pdb.gimp_image_insert_layer(image,white_layer,None,layer_position+1)
	pdb.gimp_edit_fill(white_layer,BACKGROUND_FILL)
	
	#finally drop shadow.
	pdb.script_fu_drop_shadow(image,layer,shadow_offset_x,shadow_offset_y,shadow_blur_radius,(0,0,0),shadow_opacity,0)
	
	pdb.gimp_image_remove_channel(image,saved_channel)
	

	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_wallace_panels",
	"Emulates Wallace's Panel Painting effect with current selection",
	"Emulates Wallace's Panel Painting effect with current selection...",
	"Tin Tran",
	"Tin Tran",
	"Sept 2017",
	"<Image>/Python-Fu/Wallace Panels...",
	"*",      # Create a new image, don't work on an existing one
	[
	#(PF_OPTION, "arrow_side", "Arrows Ends:", SIDE_END, SIDE_NAMES),
	#(PF_TOGGLE, "arrow_close", "Arrows Close:", 0),
	(PF_SPINNER, "border_width", "Border Width (bevelled width):", 10, (0, 500, 1)),
	(PF_SPINNER, "shadow_offset_x", "Shadow Offset X:", 6, (-4096,4096,1)),
	(PF_SPINNER, "shadow_offset_y", "Shadow Offset Y:", 6, (-4096,4096,1)),
	(PF_SPINNER, "shadow_blur_radius", "Shadow Blur Radius:", 15, (0,1024,1)),
	(PF_SPINNER, "shadow_opacity", "Shadow Opacity:", 100, (0,100,1)),
	],
	[],
	python_wallace_panels)

main()
