#!/usr/bin/env python

# Zoomed Textured Colored Rel 1
# Do steps described here(Steps 1 - 7 plus created masked colored layer for user to play with threshold of layer mask): 
#            https://www.gimplearn.net/viewtopic.php/How-to-do-quot-zoomed-textured-colored-quot-effect-?p=13504#p13504
# Created by Tin Tran (https://gimplearn.net)
# Comments directed to http://gimplearn.com or http://gimp-forum.net or http://gimpchat.com or http://gimpscripts.com
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Rel 2: Less blurry.
#import math
#import string
#import Image
from gimpfu import *
#from array import array
#import sys
def create_masked_layer(new_image,layer,color,layername):
	c = pdb.gimp_layer_new_from_drawable(layer,new_image)
	pdb.gimp_image_insert_layer(new_image,c,None,0)
	c_mask = pdb.gimp_layer_create_mask(c,ADD_WHITE_MASK)
	pdb.gimp_layer_add_mask(c,c_mask)
	floating_sel = pdb.gimp_edit_paste(c_mask,True)
	pdb.gimp_floating_sel_anchor(floating_sel)
	pdb.gimp_context_set_foreground(color)
	pdb.gimp_edit_fill(c,FOREGROUND_FILL)
	pdb.gimp_item_set_visible(c,False)
	pdb.gimp_item_set_name(c,layername)
	pdb.gimp_layer_resize_to_image_size(c)
	
def python_zoom_texture(image, layer, color1,color2,color3,color4,color5):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	#creates new image
	new_image = pdb.gimp_image_new(image.width,image.height,RGB)
	pdb.gimp_display_new(new_image)
	
	#create c5 to use as mask (inverted and blurred)
	c5 = pdb.gimp_layer_new_from_drawable(layer,new_image)
	pdb.gimp_image_insert_layer(new_image,c5,None,0)
	
	#creates inverted copy and scale and crop
	layercopy = pdb.gimp_layer_new_from_drawable(c5,new_image)
	pdb.gimp_image_insert_layer(new_image,layercopy,None,0)
	pdb.gimp_invert(layercopy)
	pdb.gimp_layer_scale(layercopy,layercopy.width * 0.95,layercopy.height * 0.95,True)
	pdb.gimp_layer_set_opacity(layercopy,50)
	offsetx,offsety = pdb.gimp_drawable_offsets(layercopy)
	#pdb.gimp_image_select_item(new_image,CHANNEL_OP_REPLACE,layercopy)
	pdb.gimp_image_crop(new_image,layercopy.width,layercopy.height,offsetx,offsety)
	c5 = pdb.gimp_image_merge_visible_layers(new_image,CLIP_TO_BOTTOM_LAYER)
	pdb.gimp_image_resize_to_layers(new_image)
	
	pdb.gimp_invert(c5)
	pdb.gimp_selection_all(new_image)
	pdb.gimp_edit_copy(c5)
	pdb.gimp_item_set_name(c5,"C5")
	
	pdb.gimp_context_set_foreground(color5)
	pdb.gimp_edit_fill(c5,FOREGROUND_FILL)
	
	create_masked_layer(new_image,layer,color4,"C4")
	create_masked_layer(new_image,layer,color3,"C3")
	create_masked_layer(new_image,layer,color2,"C2")
	create_masked_layer(new_image,layer,color1,"C1")
	
	
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()
    #return

register(
	"python_fu_zoom_texture",                           
	"Creates Layers and Layer Masked to be used for zoom texture",
	"Creates Layers and Layer Masked to be used for zoom texture",
	"Tin Tran",
	"Tin Tran",
	"2018",
	"<Image>/Python-Fu/Zoom Texture...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_COLOR, "color1", "Color 1 (Darkest):", (0, 0, 0) ), # extra param is RGB triple
	(PF_COLOR, "color2", "Color 2:", (0, 18, 255) ), # extra param is RGB triple
	(PF_COLOR, "color3", "Color 3:", (96, 107, 255) ), # extra param is RGB triple
	(PF_COLOR, "color4", "Color 4:", (174, 180, 255) ), # extra param is RGB triple
	(PF_COLOR, "color5", "Color 5 (Brightest):", (255, 255, 255) ), # extra param is RGB triple
	],
	[],
	python_zoom_texture)

main()
